@@ -1,7 +1,7 @@
 ############################################################################
 #
 # TOra - An Oracle Toolkit for DBA's and developers
-# Copyright (C) 2000-2001,2001 Underscore AB
+# Copyright (C) 2003 Quest Software, Inc
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
 #      Specifically you are not permitted to link this program with the
 #      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
 #      And you are not permitted to distribute binaries compiled against
-#      these libraries without written consent from Underscore AB. Observe
-#      that this does not disallow linking to the Qt Free Edition.
+#      these libraries without written consent from Quest Software, Inc.
+#      Observe that this does not disallow linking to the Qt Free Edition.
 #
 # All trademarks belong to their respective owners.
 #
@@ -72,6 +72,7 @@ my @source=(
 	    "tosqlparse",
 	    "tosyntaxsetup",
 	    "tosyntaxsetupui",
+	    "totabwidget",
 	    "totemplateprovider",
 	    "totextview",
 	    "tothread",
@@ -79,6 +80,8 @@ my @source=(
 	    "totoolsettingui",
 	    "utils" );
 
+my @translations=("tora_se","tora_fr","tora_it");
+
 my %plugins=(
 	     "toalert"             => { "Files" => [ "toalert" ],
 					"Oracle" => 1 },
@@ -113,6 +116,7 @@ my %plugins=(
 	     "tocurrent"           => { "Files" => [ "tocurrent" ],
 					"Oracle" => 1 },
 	     "toeditextensions"    => { "Files" => [ "toeditextensions",
+						     "toeditextensiongotoui",
 						     "toeditextensionsetupui" ],
 					"Any" => 1 },
 	     "toextract"           => { "Files" => [ "toextract",
@@ -132,6 +136,7 @@ my %plugins=(
 	     "toproject"           => { "Files" => [ "toproject" ],
 					"Any" => 1 },
 	     "toresult"            => { "Files" => [ "toresultcols",
+						     "toresultcombo",
 						     "toresultconstraint",
 						     "toresultcontent",
 						     "toresultcontentfilterui",
@@ -176,6 +181,8 @@ my %plugins=(
 						     "totemplateeditui",
 						     "totemplatesetupui" ],
 					"Any" => 1 },
+ 	     "totemporary"         => { "Files" => [ "totemporary" ],
+					"Oracle" => 1 },
 	     "totuning"            => { "Files" => [ "totuning",
 						     "totuningoverviewui",
 						     "totuningsettingui" ],
@@ -183,7 +190,8 @@ my %plugins=(
 	     "towidget"            => { "Files" => [ "tochangeconnection",
 						     "tofilesize",
 						     "tosgastatement",
-						     "tovisualize" ],
+						     "tovisualize",
+						     "towaitevents" ],
 					"Any" => 1 },
 	     "toworksheet"         => { "Files" => [ "toworksheet",
 						     "toworksheetsetupui" ],
@@ -204,8 +212,23 @@ __EOMK__
 	if [ ! -d plugins ] ; then mkdir -p plugins ; fi
 	\$(GCC) -shared \$(CFLAGS) \$(LFLAGS) \$(LFLAGS_GLOB) \$(MYSQL_SHARED) -o \$@ \$^
 __EOMK__
+	     "tooraclepreload"     => { "Files" => [ "tooraclepreload" ],
+					"Oracle" => 1 },
 	     );
 
+sub addInclude {
+    my ($before,$newinc)=@_;
+    return $before if ($newinc eq "/usr/include");
+    return $before if (defined $before && $before=~/\"$newinc\"/);
+    my $inc="\"-I$newinc\"";
+    if (defined $before) {
+	$before.=" $inc";
+    } else {
+	$before=$inc;
+    }
+    return $before;
+}
+
 sub findFile {
     my ($files)=shift(@_);
     my ($check)=shift(@_);
@@ -235,9 +258,11 @@ my $InstallBin;
 my $InstallLib;
 my $Includes;
 my $CC;
-my $Libs="-lcrypt -lm -lpthread";
+my $Libs="-lm -lpthread";
 my $MOC;
 my $UIC;
+my $LRelease;
+my $LUpdate;
 my $QtDir;
 my $QtInclude;
 my $QtVersion;
@@ -264,6 +289,7 @@ my $OracleRelease;
 my $OracleShared="-lclntsh";
 my $TestDB;
 my $RPMGenerate=0;
+my $OptLevel="-O3";
 
 my $MySQLInclude;
 my $MySQLLib;
@@ -271,6 +297,9 @@ my $MySQLShared;
 my $MySQLStatic;
 my $MySQLFound=1;
 
+my $OnlyMake=0;
+my $NewCheck=1;
+
 $QtDir=$ENV{QTDIR};
 
 for (@ARGV) {
@@ -284,6 +313,10 @@ for (@ARGV) {
 	$MOC=$1;
     } elsif (/^--with-qt-uic=(.*)$/) {
 	$UIC=$1;
+    } elsif (/^--with-qt-lrelease=(.*)$/) {
+	$LRelease=$1;
+    } elsif (/^--with-qt-lupdate=(.*)$/) {
+	$LUpdate=$1;
     } elsif (/^--prefix=(.*)$/) {
 	$InstallPrefix=$1;
     } elsif (/^--with-mysql-include=(.*)$/) {
@@ -305,8 +338,14 @@ for (@ARGV) {
 	$MySQLFound=0;
     } elsif (/^--without-oracle$/) {
 	$OracleFound=0;
+	$OracleShared=""
     } elsif (/^--without-kde$/) {
 	$NoKDE=1;
+    } elsif (/^--enable-onlymake$/) {
+	$OnlyMake=1;
+    } elsif (/^--only-prepare$/) {
+	print "skipping Makefile generation.\n";
+	exit(0);
     } elsif (/^--with-static$/) {
 	$Target="tora-static";
 	$ForceTarget=1;
@@ -319,8 +358,12 @@ for (@ARGV) {
 	    $KDELibs="$1/lib";
 	    $KDEBase=$1;
 	}
+    } elsif (/^--disable-new-check$/) {
+	$NewCheck=0;
     } elsif (/^--with-rpm-contents(?:=(.*))$/) {
 	$RPMGenerate=$1;
+    } elsif (/^--enable-opt-flag(?:=(.*))$/) {
+	$OptLevel=$1;
     } elsif (/^--without-rpath$/) {
 	$NoRPath=1;
     } elsif (/^--with-kde-include=(.*)$/) {
@@ -335,33 +378,41 @@ configure [options...]
 
 Options can be any of the following:
 
---prefix             Specify base directory of install
+--prefix             Specify base directory of install (Default: /usr/local)
 --prefix-bin         Binary directory of install
 --prefix-lib         Library directory of install
 --with-qt            Specify Qt base directory
 --with-qt-moc        Specify moc command to use
 --with-qt-uic        Specify uic command to use
+--with-qt-lrelease   Specify lrelease command to use
+--with-qt-lupdate    Specify lupdate command to use
 --with-qt-include    Specify Qt include directory
 --with-qt-libs       Specify Qt library directory
 --with-gcc           Specify which GCC compiler to use
---with-mono          Force monolithic compilation
+--with-mono          Force monolithic compilation (Not enabled for linux by default)
 --with-lib           Add extra library to include (Include -l as well)
---with-static-oracle Force use of static Oracle libraries only
---with-kde           Compile as KDE application (Requires KDE 2.2 or later)
+--with-static-oracle Force use of static Oracle libraries only (Not enabled by default)
+--with-kde           Compile as KDE application (Requires KDE 3.0 or later) (Enabled if detected)
 --without-kde        Dont compile as KDE application even though KDE available.
 --with-kde-include   Where to find KDE include files
 --with-kde-libs      Where to find KDE libraries
 --with-mysql-include Where to find MySQL include files
 --with-mysql-libs    Where to find MySQL library files
---without-oracle     Compile without Oracle support
---without-mysql      Don't compile in MySQL support
---without-rpath      Compile without rpath to Oracle libraries
-
+--without-oracle     Compile without Oracle support (Not enabled by default)
+--without-mysql      Don't compile in MySQL support (MySQL enabled if detected)
+--without-rpath      Compile without rpath to Oracle libraries (Not enabled by default)
+--disable-new-check  Disable new version check globally in TOra (Not enabled by default)
+--enable-opt-flag    Set the optimize flag to use for the compile (Default -O3)
+--only-prepare       Don't generate makefile, just prepare files. Assumes a Makefile is in place.
 __USAGE__
         exit(2);
     }
 }
 
+if ($Target eq "tora-static") {
+    $NoRPath=1;
+}
+
 if (!$InstallBin) {
     $InstallBin=$InstallPrefix."/bin";
 }
@@ -389,8 +440,22 @@ __TEMP__
 	} else {
 	    print TEMP "#define OTL_ORA8I\n";
 	}
+	print TEMP "#define OTL_ANSI_CPP\n";
 
 	print TEMP <<__TEMP__;
+
+template <class T>
+T max(T a, T b)
+{
+    return a > b ? a : b ;
+}
+
+template <class T>
+T min(T a, T b)
+{
+    return a < b ? a : b ;
+}
+
 #include "otlv4.h"
 
 void testOracle(void) // Not called
@@ -426,8 +491,8 @@ int main(int,char **)
     static int GCCVersion=__GNUC__;
     static int GCCVersionMinor=__GNUC_MINOR__;
     static int QtVersion=QT_VERSION;
-    if (QtVersion<220) {
-	printf ("Requires Qt version >= 2.2 (Found %s)\\n",QT_VERSION_STR);
+    if (QtVersion<300) {
+	printf ("Requires Qt version >= 3.0 (Found %s)\\n",QT_VERSION_STR);
 	exit(2);
     }
 #if 0
@@ -451,7 +516,7 @@ void test2(void)
 {
     QWidget *parent;
     QMainWindow *main;
-#if KDE_VERSION >= 220
+#if KDE_VERSION >= 300
     new KToolBar(main,parent);
 #endif
 }
@@ -459,7 +524,8 @@ void test2(void)
 __TEMP__
     }
     close TEMP;
-    if (!system("$gcc $LFlags -I`pwd` $Includes $Libs $QtLibShared $TestDB -o$tmpName $tmpName.cpp")) {
+    if (!system("$gcc $LFlags -I`pwd` $Includes $Libs $QtLibShared $TestDB -o $tmpName $tmpName.cpp")) {
+        $ENV{"LD_LIBRARY_PATH"}=$ENV{"LD_LIBRARY_PATH"}.":".$ENV{"ORACLE_HOME"}."/lib";
 	if (!system($tmpName)) {
 	    $CC=$gcc;
 	}
@@ -478,18 +544,18 @@ __TEMP__
 {
     if (!$NoKDE||$KDEApplication) {
 	print "checking for KDE include files ... ";
-	$KDEInclude=findFile("^kapp.*\\.h\$",sub {
+	$KDEInclude=findFile("^(?:kapp.*\\.h)|(?:kdeversion\\.h)\$",sub {
 	                                     if (open(KDE,"<$_[0]")) {
 						 while(<KDE>) {
 						     if (/#define\s+KDE_VERSION_STRING\s+\"([0-9\.]+)/) {
 							 $KDEVersion=$1;
-							 if ($KDEVersion ge "2.1") {
+							 if ($KDEVersion ge "3.0") {
 							     last;
 							 }
 						     }
 						 }
 						 close KDE;
-						 if ($KDEVersion ge "2.1") {
+						 if ($KDEVersion ge "3.0") {
 						     return 1;
 						 } else {
 						     return 0;
@@ -501,19 +567,15 @@ __TEMP__
 			     $KDEInclude,
 			     $ENV{KDEDIR}."/include",
 			     "/usr/include",
-			     "/usr/include/kde",
-			     "/usr/include/kde2",
 			     "/usr/include/kde3",
-			     "/usr/local/kde/include",
-			     "/usr/local/kde2/include",
+			     "/usr/include/kde",
 			     "/usr/local/kde3/include",
-			     "/opt/kde/include",
-			     "/opt/kde2/include",
+			     "/usr/local/kde/include",
 			     "/opt/kde3/include",
+			     "/opt/kde/include",
 			     "/usr/local/include",
-			     "/usr/local/include/kde",
-			     "/usr/local/include/kde2",
-			     "/usr/local/include/kde3"
+			     "/usr/local/include/kde3",
+			     "/usr/local/include/kde"
 			     );
 
 	if (-d $KDEInclude) {
@@ -528,7 +590,7 @@ __TEMP__
 	if (!defined $KDEInclude) {
 	    print "\nKDE includes not found\n";
 	}
-	$Includes.="\"-I".$KDEInclude."\" ";
+	$Includes=&addInclude($Includes,$KDEInclude);
 
 	print "checking for KDE print support ... ";
 
@@ -546,24 +608,21 @@ __TEMP__
 
 	print "checking for KDE libraries ... ";
 
-	$KDELibs=findFile("^libDCOP.so",sub {
+	$KDELibs=findFile("^libDCOP\\.(?:s[ol]|dylib)",sub {
                                    	    return -f $_[0];
 					},
 			     $KDELibs,
 			     $ENV{KDEDIR}."/lib",
 			     "/usr/lib",
-			     "/usr/lib/kde",
-			     "/usr/lib/kde2",
 			     "/usr/lib/kde3",
+			     "/usr/lib/kde",
+			     "/usr/local/kde3/lib",
 			     "/usr/local/kde/lib",
-			     "/usr/local/kde2/lib",
-			     "/opt/kde/lib",
-			     "/opt/kde2/lib",
 			     "/opt/kde3/lib",
+			     "/opt/kde/lib",
 			     "/usr/local/lib",
-			     "/usr/local/lib/kde",
-			     "/usr/local/lib/kde2",
-			     "/usr/local/lib/kde3"
+			     "/usr/local/lib/kde3",
+			     "/usr/local/lib/kde"
 			     );
 
 	if (!-d $KDELibs) {
@@ -576,23 +635,18 @@ __TEMP__
 	print "checking for KDE base ... ";
 	for my $a ($KDEBase,$ENV{KDEDIR},
 		   "/usr",
-		   "/usr/kde",
-		   "/usr/kde2",
 		   "/usr/kde3",
-		   "/usr/lib/kde",
-		   "/usr/lib/kde2",
 		   "/usr/lib/kde3",
-		   "/usr/local/kde",
-		   "/usr/local/kde2",
 		   "/usr/local/kde3",
-		   "/opt/kde",
-		   "/opt/kde2",
 		   "/opt/kde3",
+		   "/usr/local/lib/kde3",
+		   "/usr/kde",
+		   "/usr/lib/kde",
+		   "/usr/local/kde",
+		   "/opt/kde",
 		   "/usr/local/lib",
-		   "/usr/local/lib/kde",
-		   "/usr/local/lib/kde2",
-		   "/usr/local/lib/kde3") {
-	    if (-d "$a/share/apps/kappfinder" && -d "$a/share/icons/hicolor/32x32/apps" && -d "$a/share/icons/hicolor/16x16/apps") {
+		   "/usr/local/lib/kde") {
+	    if (-d "$a/share/apps/kwin" && -d "$a/share/icons/hicolor/32x32/apps" && -d "$a/share/icons/hicolor/16x16/apps") {
 		$KDEBase=$a;
 		last;
 	    } else {
@@ -605,10 +659,7 @@ __TEMP__
 	    print "no\n";
 	}
 
-	$Libs.=" -lkdecore -lkdeui -lDCOP -lkhtml";
-	if ($KDEVersion lt "2.9") {
-	    $Libs.=" -lkfile";
-	}
+	$Libs.=" -lkdecore -lkdeui -lDCOP -lkhtml -lkparts -lkio";
 	$LFlags.="\"-L".$KDELibs."\" ";
 	if (!$NoRPath) {
 	    $LFlags.="-Xlinker \"--rpath=$KDELibs\" ";
@@ -616,7 +667,7 @@ __TEMP__
 
 	print "checking for Qt library linked to KDE ... ";
 	my $found=0;
-	if (open(LDD,"ldd $KDELibs/libkdecore.so 2>&1 |")) {
+	if (open(LDD,"ldd $KDELibs/libkdecore\\.s[ol] 2>&1 |")) {
 	    while(<LDD>) {
 		if (/\/([^\/]+qt[^\.]*)/) {
 		    $QtSearch="$1";
@@ -637,14 +688,17 @@ __TEMP__
 
     print "checking for Qt library ... ";
 
-    $QtLib=findFile("^".$QtSearch."[23]\\.so",sub {
-	                                        if (-f $_[0] && ! -l $_[0]) {
+    $QtLib=findFile("^".$QtSearch."\\.?3\\.(?:s[ol]|dylib)",sub {
+	                                        my $file=$_[0];
+	                                        if (-f $file && (!-l $file || $file=~/dylib/ )) {
 						    my $lib;
-						    ($lib,$QtLibShared)=($_[0]=~/^(.*)\/lib(qt(?:-mt)?[23]?)[^\/]*$/);
+						    ($lib,$QtLibShared)=($file=~/^(.*)\/lib(qt(?:-mt)?3?)[^\/]*$/);
 						    if (!defined $QtLibShared) {
 							return 0;
 						    }
-						    if (-f "$lib/lib$QtLibShared.so") {
+						    if (-f "$lib/lib$QtLibShared.so" || 
+							-f "$lib/lib$QtLibShared.sl" ||
+							-f "$lib/lib$QtLibShared.dylib") {
 							$QtLibShared=" -l$QtLibShared";
 							return 1;
 						    }
@@ -657,31 +711,25 @@ __TEMP__
 		    $QtLibOrig,
 		    $QtDir."/lib",
 		    "/usr/lib",
-		    "/usr/lib/qt2",
-		    "/usr/lib/qt2/lib",
 		    "/usr/lib/qt3",
 		    "/usr/lib/qt3/lib",
+		    "/usr/local/lib/qt3/lib",
+		    "/usr/local/lib/qt3",
+		    "/usr/local/qt3",
+		    "/usr/local/qt3/lib",
 		    "/usr/lib/qt",
 		    "/usr/lib/qt/lib",
 		    "/usr/local/lib",
-		    "/usr/local/lib/qt2",
-		    "/usr/local/lib/qt2/lib",
-		    "/usr/local/lib/qt3",
-		    "/usr/local/lib/qt3/lib",
 		    "/usr/local/lib/qt",
 		    "/usr/local/lib/qt/lib",
-		    "/usr/local/qt2",
-		    "/usr/local/qt2/lib",
-		    "/usr/local/qt3",
-		    "/usr/local/qt3/lib",
 		    "/usr/local/qt",
 		    "/usr/local/qt/lib"
 		    );
 
     if (! -d $QtLib) {
-	$QtLib=findFile("^$QtSearch\\.so\\.[23]",sub {
+	$QtLib=findFile("^$QtSearch\\.(?:s[ol]|dylib)\\.3",sub {
 		                                     if (-f $_[0] && ! -l $_[0]) {
-							 ($QtLibShared)=($_[0]=~/\/lib(qt(?:-mt)?[23]?)[^\/]*$/);
+							 ($QtLibShared)=($_[0]=~/\/lib(qt(?:-mt)?3?)[^\/]*$/);
 							 if (!defined $QtLibShared) {
 							     return 0;
 							 }
@@ -694,23 +742,17 @@ __TEMP__
 			$QtLibOrig,
 			$QtDir."/lib",
 			"/usr/lib",
-			"/usr/lib/qt2",
-			"/usr/lib/qt2/lib",
 			"/usr/lib/qt3",
 			"/usr/lib/qt3/lib",
+			"/usr/local/lib/qt3",
+			"/usr/local/lib/qt3/lib",
+			"/usr/local/qt3/lib",
+			"/usr/local/qt3",
 			"/usr/lib/qt",
 			"/usr/lib/qt/lib",
 			"/usr/local/lib",
-			"/usr/local/lib/qt2",
-			"/usr/local/lib/qt2/lib",
-			"/usr/local/lib/qt3",
-			"/usr/local/lib/qt3/lib",
 			"/usr/local/lib/qt",
 			"/usr/local/lib/qt/lib",
-			"/usr/local/qt2",
-			"/usr/local/qt2/lib",
-			"/usr/local/qt3",
-			"/usr/local/qt3/lib",
 			"/usr/local/qt",
 			"/usr/local/qt/lib"
 			);
@@ -724,44 +766,16 @@ __TEMP__
 
     print "checking for moc ... ";
     if (!defined $MOC || ! -x $MOC) {
-	$MOC=findFile("moc2",sub { return -x $_[0]; },
+	$MOC=findFile("moc",sub { return -x $_[0]; },
 		      $QtDir."/bin",
-		      "/usr/lib/qt2",
-		      "/usr/lib/qt2/bin",
 		      "/usr/lib/qt3",
 		      "/usr/lib/qt3/bin",
-		      "/usr/local/lib/qt2",
-		      "/usr/local/lib/qt2/bin",
 		      "/usr/local/lib/qt3",
 		      "/usr/local/lib/qt3/bin",
-		      "/usr/local/qt2",
-		      "/usr/local/qt2/bin",
 		      "/usr/local/qt3",
 		      "/usr/local/qt3/bin",
 		      "/usr/lib/qt",
-		      "/usr/bin",
-		      "/usr/local/bin",
-		      "/usr/local/lib/qt");
-	if (defined $MOC && -d $MOC) {
-	    $MOC.="/moc2";
-	}
-    }
-
-    if (!defined $MOC || ! -x $MOC) {
-	$MOC=findFile("moc",sub { return -x $_[0]; },
-		      $QtDir."/bin",
-		      "/usr/lib/qt2",
-		      "/usr/lib/qt2/bin",
-		      "/usr/lib/qt3",
-		      "/usr/lib/qt3/bin",
-		      "/usr/local/lib/qt2",
-		      "/usr/local/lib/qt2/bin",
-		      "/usr/local/lib/qt3",
-		      "/usr/local/lib/qt3/bin",
-		      "/usr/local/qt2",
-		      "/usr/local/qt2/bin",
-		      "/usr/local/qt3",
-		      "/usr/local/qt3/bin",
+		      "/usr/lib/qt/bin",
 		      "/usr/lib/qt",
 		      "/usr/bin",
 		      "/usr/local/bin",
@@ -780,16 +794,10 @@ __TEMP__
     if (!defined $UIC || ! -x $UIC) {
 	$UIC=findFile("uic",sub { return -x $_[0]; },
 		      $QtDir."/bin",
-		      "/usr/lib/qt2",
-		      "/usr/lib/qt2/bin",
 		      "/usr/lib/qt3",
 		      "/usr/lib/qt3/bin",
-		      "/usr/local/lib/qt2",
-		      "/usr/local/lib/qt2/bin",
 		      "/usr/local/lib/qt3",
 		      "/usr/local/lib/qt3/bin",
-		      "/usr/local/qt2",
-		      "/usr/local/qt2/bin",
 		      "/usr/local/qt3",
 		      "/usr/local/qt3/bin",
 		      "/usr/lib/qt",
@@ -806,19 +814,65 @@ __TEMP__
     }
     print "$UIC\n";
 
+    print "checking for lrelease ... ";
+    if (!defined $LRelease || ! -x $LRelease) {
+	$LRelease=findFile("lrelease",sub { return -x $_[0]; },
+		      $QtDir."/bin",
+		      "/usr/lib/qt3",
+		      "/usr/lib/qt3/bin",
+		      "/usr/local/lib/qt3",
+		      "/usr/local/lib/qt3/bin",
+		      "/usr/local/qt3",
+		      "/usr/local/qt3/bin",
+		      "/usr/lib/qt",
+		      "/usr/bin",
+		      "/usr/local/bin",
+		      "/usr/local/lib/qt");
+	if (defined $LRelease && -d $LRelease) {
+	    $LRelease.="/lrelease";
+	}
+    }
+    if (!-x $LRelease) {
+	$LRelease="true";
+    }
+    print "$LRelease\n";
+
+    print "checking for lupdate ... ";
+    if (!defined $LUpdate || ! -x $LUpdate) {
+	$LUpdate=findFile("lupdate",sub { return -x $_[0]; },
+		      $QtDir."/bin",
+		      "/usr/lib/qt3",
+		      "/usr/lib/qt3/bin",
+		      "/usr/local/lib/qt3",
+		      "/usr/local/lib/qt3/bin",
+		      "/usr/local/qt3",
+		      "/usr/local/qt3/bin",
+		      "/usr/lib/qt",
+		      "/usr/bin",
+		      "/usr/local/bin",
+		      "/usr/local/lib/qt");
+	if (defined $LUpdate && -d $LUpdate) {
+	    $LUpdate.="/lupdate";
+	}
+    }
+    if (!-x $LUpdate) {
+	$LUpdate="true";
+    }
+    print "$LUpdate\n";
+
     print "checking for Qt include files ... ";
     $QtInclude=findFile("^qglobal\\.h\$",sub {
 	                                     if (open(QT,"<$_[0]")) {
 						 while(<QT>) {
-						     if (/#define\s+QT_VERSION_STR\s+\"([0-9\.]+)\"/) {
+						     if (/#define\s+QT_VERSION_STR\s+\"([0-9\.]+[a-z0-9\.]+)[-"]+/i) {
 							 $QtVersion=$1;
-							 if ($QtVersion ge "2.2") {
+							 if ($QtVersion ge "3.0") {
 							     last;
 							 }
 						     }
 						 }
 						 close QT;
-						 if ($QtVersion ge "2.2") {
+						 if ($QtVersion ge "3.0") {
 						     return 1;
 						 } else {
 						     return 0;
@@ -830,21 +884,17 @@ __TEMP__
 			$QtInclude,
 			$QtDir."/include",
 			"/usr/include",
-			"/usr/include/qt2",
 			"/usr/include/qt3",
-			"/usr/lib/qt2/include",
 			"/usr/lib/qt3/include",
-			"/usr/include/qt",
-			"/usr/lib/qt/include",
-			"/usr/local/include",
-			"/usr/local/include/qt2",
 			"/usr/local/include/qt3",
-			"/usr/local/lib/qt2/include",
 			"/usr/local/lib/qt3/include",
+			"/usr/local/qt3/include",
 			"/usr/local/include/qt",
 			"/usr/local/qt/include",
-			"/usr/local/qt2/include",
-			"/usr/local/qt3/include");
+			"/usr/include/qt",
+			"/usr/lib/qt/include",
+			"/usr/local/include"
+			);
 
     if (!-d $QtInclude) {
 	print "failed!\n\nCouldn't find include files for Qt, use --with-qt-include to specify\n";
@@ -854,10 +904,7 @@ __TEMP__
 
     print "checking for Qt version ... $QtVersion\n";
     print "checking for library consistency ... ";
-    if (defined $KDEVersion &&
-	(substr($QtVersion,0,1) ne substr($KDEVersion,0,1) ||
-	 (substr($QtVersion,0,1) eq "2" && substr($KDEVersion,0,3) eq "2.9")) &&
-	!(substr($QtVersion,0,1) eq "3" && substr($KDEVersion,0,3) eq "2.9")) {
+    if (defined $KDEVersion && substr($QtVersion,0,1) ne substr($KDEVersion,0,1)) {
 	print "failed!\n\nKDE ($KDEVersion) and Qt ($QtVersion) versions doesn't match!\n";
 	print "Try specifying using --with-qt and --with-kde switches to configure.\n";
 	exit(1);
@@ -865,32 +912,26 @@ __TEMP__
     print "yes\n";
     print "checking for static Qt library ... ";
 
-    findFile("^libqt(-mt)?[23]\\.a",sub {
+    findFile("^libqt(-mt)?3\\.a",sub {
 	                                if (-f $_[0] && ! -l $_[0]) {
 					    $QtLibStatic=$_[0];
 					}
 				    },
 	     $QtDir."/lib",
 	     "/usr/lib",
-	     "/usr/lib/qt2",
-	     "/usr/lib/qt2/lib",
 	     "/usr/lib/qt3",
 	     "/usr/lib/qt3/lib",
+	     "/usr/local/lib/qt3",
+	     "/usr/local/lib/qt3/lib",
+	     "/usr/local/qt3",
+	     "/usr/local/qt3/lib",
 	     "/usr/lib/qt",
 	     "/usr/lib/qt/lib",
 	     "/usr/local/lib",
-	     "/usr/local/lib/qt2",
-	     "/usr/local/lib/qt2/lib",
-	     "/usr/local/lib/qt3",
-	     "/usr/local/lib/qt3/lib",
+	     "/usr/local/qt",
+	     "/usr/local/qt/lib",
 	     "/usr/local/lib/qt",
 	     "/usr/local/lib/qt/lib",
-	     "/usr/local/qt2",
-	     "/usr/local/qt2/lib",
-	     "/usr/local/qt3",
-	     "/usr/local/qt3/lib",
-	     "/usr/local/qt",
-	     "/usr/local/qt/lib"
 	     );
     if (!defined $QtLibStatic) {
 	findFile("^libqt(-mt)?\\.a",sub {
@@ -900,25 +941,19 @@ __TEMP__
 				    },
 		 $QtDir."/lib",
 		 "/usr/lib",
-		 "/usr/lib/qt2",
-		 "/usr/lib/qt2/lib",
 		 "/usr/lib/qt3",
 		 "/usr/lib/qt3/lib",
-		 "/usr/lib/qt",
-		 "/usr/lib/qt/lib",
-		 "/usr/local/lib",
-		 "/usr/local/lib/qt2",
-		 "/usr/local/lib/qt2/lib",
 		 "/usr/local/lib/qt3",
 		 "/usr/local/lib/qt3/lib",
-		 "/usr/local/lib/qt",
-		 "/usr/local/lib/qt/lib",
-		 "/usr/local/qt2",
-		 "/usr/local/qt2/lib",
 		 "/usr/local/qt3",
 		 "/usr/local/qt3/lib",
 		 "/usr/local/qt",
-		 "/usr/local/qt/lib"
+		 "/usr/local/qt/lib",
+		 "/usr/lib/qt",
+		 "/usr/lib/qt/lib",
+		 "/usr/local/lib/qt",
+		 "/usr/local/lib/qt/lib",
+		 "/usr/local/lib"
 		 );
 	if (-f $QtLibStatic) {
 	    print $QtLibStatic."\n";
@@ -957,19 +992,6 @@ __TEMP__
 	}
 	$TestDB.=" -lclntsh";
     }
-    if ($MySQLFound) {
-	$TestDB.=" -lmysqlclient";
-    }
-
-    findFile("^libstdc\\+\\+.*\\.a",sub {
-	                                $StdCppLibStatic=$_[0];
-					return -f $_[0];
-				    },
-	     "/usr/lib",
-	     "/usr/local/lib");
-    if (! -f $StdCppLibStatic) {
-	$StdCppLibStatic="";
-    }
 
     if ($QtVersion ge "3.0") {
 	$MySQLFound=0;
@@ -977,6 +999,22 @@ __TEMP__
 	undef $MySQLLib;
 	undef $MySQLStatic;
 	undef $MySQLInclude;
+	findFile("^libmysqlclient.*\\.a",sub {
+		                             $MySQLStatic=$_[0];
+					     return -f $_[0];
+					 },
+		 $MySQLLib,
+		 "/usr/lib",
+		 "/usr/lib/mysql",
+		 "/usr/lib/mysql/lib",
+		 "/usr/local/lib",
+		 "/usr/local/lib/mysql",
+		 "/usr/local/lib/mysql/lib");
+	if (! -f $MySQLStatic) {
+	    $MySQLStatic="";
+	} else {
+	    $MySQLLib=$MySQLStatic;
+	}
     } elsif ($MySQLFound) {
 	print "checking for MySQL include files ... ";
 
@@ -996,7 +1034,7 @@ __TEMP__
 	} else {
 	    print "$MySQLInclude\n";
 	    print "checking for MySQL library ... ";
-	    $MySQLLib=findFile("^libmysqlclient\\.so",sub {
+	    $MySQLLib=findFile("^libmysqlclient\\.(?:s[ol]|dylib)",sub {
 		                                          return -f $_[0] && ! -l $_[0];
 						      },
 			       $MySQLLib,
@@ -1041,6 +1079,10 @@ __TEMP__
 		}
 	    }
 	}
+
+	if ($MySQLFound) {
+	    $TestDB.=" -lmysqlclient";
+	}
     }
 	
     print "checking for any database ... ";
@@ -1073,32 +1115,38 @@ __TEMP__
 
     print "checking for extra libraries ... ";
 
+    my $extra="none";
+
     if (`uname`=~/linux/i) {
-	print "none\n";
+	$extra="-lcrypt";
+    } elsif (`uname`=~/bsd/i) {
+	# Noop
     } elsif (`uname`=~/sunos/i) {
 	$NoRPath=1;
 	if (`uname -r`>=8.0) {
-	    $Libs.=" -lrt";
-	    print " -lrt\n";
+	    $extra="-lcrypt -lrt";
 	} else {
-	    $Libs.=" -lposix4";
-	    print " -lposix4\n";
+	    $extra="-lcrypt -lposix4";
 	}
     } else {
 	$NoRPath=1;
-	print "none\n";
     }
+    if ($extra ne "none") {
+	$Libs.=" $extra";
+    }
+    print "$extra\n";
+
     if (!$NoRPath) {
 	$LFlags.="-Xlinker \"--rpath=".$ENV{ORACLE_HOME}."/lib\" -Xlinker \"--rpath\=$QtLib\" ";
     }
 
-    $Includes.="\"-I".$ENV{ORACLE_HOME}."/rdbms/demo\" ";
-    $Includes.="\"-I".$ENV{ORACLE_HOME}."/plsql/public\" ";
-    $Includes.="\"-I".$ENV{ORACLE_HOME}."/rdbms/public\" ";
-    $Includes.="\"-I".$ENV{ORACLE_HOME}."/network/public\" ";
-    $Includes.="\"-I".$QtInclude."\"";
+    $Includes=&addInclude($Includes,$ENV{ORACLE_HOME}."/rdbms/demo");
+    $Includes=&addInclude($Includes,$ENV{ORACLE_HOME}."/plsql/public");
+    $Includes=&addInclude($Includes,$ENV{ORACLE_HOME}."/rdbms/public");
+    $Includes=&addInclude($Includes,$ENV{ORACLE_HOME}."/network/public");
+    $Includes=&addInclude($Includes,$QtInclude);
     if (defined $MySQLInclude) {
-	$Includes.=" \"-I".$MySQLInclude."\"";
+	$Includes=&addInclude($Includes,$MySQLInclude);
     }
 
     if (!$ForceTarget) {
@@ -1112,13 +1160,15 @@ __TEMP__
 			     },
 	         "/usr/include");
 	if (-f $dlfcn && `uname`=~/linux/i ) {
-	    $Libs.=" -ldl";
 	    print "yes\n";
 	    $Target="tora-plugin";
 	} else {
 	    print "no\n";
 	}
     }
+    if (`uname`=~/linux/i ) {
+	$Libs.=" -ldl";
+    }
 
     if (!-f $CC) {
 	findFile("^(g\\+\\+|gcc|cc)\$",\&finalTest,split(/:/,$ENV{PATH}));
@@ -1143,6 +1193,29 @@ __EOT__
 	exit(2);
     }
 
+    if ($Target eq "tora-static") {
+	print "checking for static libc++ ... ";
+	my $path=`$CC -v 2>&1`;
+	if ($path=~/\s(\S+)\/specs\s/s) {
+	    $path=$1;
+	} else {
+	    undef $path;
+	}
+	findFile("^libstdc\\+\\+.*\\.a",sub {
+	                                    $StdCppLibStatic=$_[0];
+				  	    return -f $_[0];
+					},
+		 $path,
+		 "/usr/lib",
+		 "/usr/local/lib");
+	if (! -f $StdCppLibStatic) {
+	    print "failed!\n";
+	    exit(2);
+	} else {
+	    print $StdCppLibStatic."\n";
+	}
+    }
+
     print "creating Makefile\n";
 
     if (open (MAKEFILE,">Makefile")) {
@@ -1150,7 +1223,7 @@ __EOT__
 ############################################################################
 #
 # TOra - An Oracle Toolkit for DBA's and developers
-# Copyright (C) 2000-2001,2001 Underscore AB
+# Copyright (C) 2003 Quest Software, Inc
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -1174,8 +1247,8 @@ __EOT__
 #      Specifically you are not permitted to link this program with the
 #      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
 #      And you are not permitted to distribute binaries compiled against
-#      these libraries without written consent from Underscore AB. Observe
-#      that this does not disallow linking to the Qt Free Edition.
+#      these libraries without written consent from Quest Software, Inc.
+#      Observe that this does not disallow linking to the Qt Free Edition.
 #
 # All trademarks belong to their respective owners.
 #
@@ -1233,6 +1306,14 @@ __EOT__
 	print MAKEFILE "UIC=\"$UIC\"\n";
 	print MAKEFILE "\n";
 
+	print MAKEFILE "# Path to Qt translation compiler\n";
+	print MAKEFILE "LRELEASE=\"$LRelease\"\n";
+	print MAKEFILE "\n";
+
+	print MAKEFILE "# Path to Qt translation update\n";
+	print MAKEFILE "LUPDATE=\"$LUpdate\"\n";
+	print MAKEFILE "\n";
+
 	print MAKEFILE "# Additional paths to find libraries\n";
 	print MAKEFILE "LFLAGS=$LFlags\n";
 	print MAKEFILE "\n";
@@ -1270,26 +1351,38 @@ __EOT__
 	print MAKEFILE "\n";
 
 	print MAKEFILE "# Base directory of KDE installation\n";
-	print MAKEFILE "KDE_BASE=$KDEBase\n";
+	print MAKEFILE "KDE_BASE=\$(ROOT)$KDEBase\n";
 	print MAKEFILE "\n";
 
 	print MAKEFILE "# Additional defines to use while compiling, except for the normal these are available\n";
 	print MAKEFILE "#   OTL_ORA9I          - Compile for Oracle 9.x\n";
 	print MAKEFILE "#   OTL_ORA8I          - Compile for Oracle 8.1.x\n";
 	print MAKEFILE "#   OTL_ORA8           - Compile for Oracle 8.0.x\n";
+	print MAKEFILE "#   OTL_ORA_TIMESTAMP  - Include support for timestamps\n";
+	print MAKEFILE "#   TO_NEW_CHECK       - Disable new version check globally\n";
 	print MAKEFILE "#   TO_KDE             - Compile as KDE application\n";
 	
 	if ($OracleRelease =~ /^8.0/) {
-	    print MAKEFILE "DEFINES+=-DOTL_ORA8\n";
+	    print MAKEFILE "DEFINES+=-DOTL_ORA8 -DOTL_ANSI_CPP\n";
 	} elsif ($OracleRelease ge "9") {
-	    print MAKEFILE "DEFINES+=-DOTL_ORA9I\n";
+	    print MAKEFILE "DEFINES+=-DOTL_ORA9I -DOTL_ORA_TIMESTAMP -DOTL_ANSI_CPP\n";
 	} else {
-	    print MAKEFILE "DEFINES+=-DOTL_ORA8I\n";
+	    print MAKEFILE "DEFINES+=-DOTL_ORA8I -DOTL_ORA_TIMESTAMP -DOTL_ANSI_CPP\n";
+	}
+	if (!$NewCheck) {
+	    print MAKEFILE "DEFINED+=-DTO_NO_NEW_CHECK\n";
 	}
 	print MAKEFILE "DEFINES+=-D_REENTRANT -DDEFAULT_PLUGIN_DIR=\\\"\$(INSTALLLIB)/tora\\\"\n";
+	if ($QtLibShared=~/qt-mt/) {
+	    print MAKEFILE "DEFINES+=-DQT_THREAD_SUPPORT\n";
+	}
 	if ($KDEApplication) {
 	    print MAKEFILE "DEFINES+=-DTO_KDE\n";
 	}
+	if ($Target ne "tora-plugin") {
+	    print MAKEFILE "DEFINES+=-DTOMONOLITHIC\n";
+	}
+
 	print MAKEFILE "\n";
 
 	print MAKEFILE "# Where to find perl on your system\n";
@@ -1305,10 +1398,10 @@ __EOT__
 CPPFLAGS_GLOB=
 
 # Additional flags to set when linking.
-LFLAGS_GLOB=-g
+LFLAGS_GLOB=-g $OptLevel
 
 # Additional flags set when compiling.
-CFLAGS_GLOB=-g -fPIC -Wall
+CFLAGS_GLOB=-g $OptLevel -fPIC -W -Wall
 
 ############################################################################
 #
@@ -1347,6 +1440,7 @@ API=	\\
 	toresult.h		\\
 	toresultbar.h		\\
 	toresultcols.h		\\
+	toresultcombo.h         \\
 	toresultconstraint.h	\\
 	toresultcontent.h	\\
 	toresultdepend.h	\\
@@ -1366,6 +1460,7 @@ API=	\\
 	tosmtp.h                \\
 	tosql.h			\\
 	tosqlparse.h		\\
+	totabwidget.h		\\
 	totemplate.h		\\
 	totextview.h            \\
 	tothread.h		\\
@@ -1392,12 +1487,18 @@ __EOT__
 	    print MAKEFILE "include \$(ORACLE_HOME)/rdbms/lib/env_rdbms.mk\n";
 	}
 
+	print MAKEFILE "\nTRANSLATIONS=\\\n";
+	for my $t (@translations) {
+	    print MAKEFILE "\t$t.ts\\\n";
+	}
+
 	print MAKEFILE <<__EOT__;
 
 CPPFLAGS=\$(CPPFLAGS_GLOB) \$(DEFINES) \$(INCLUDES)
 CFLAGS=\$(CFLAGS_GLOB) \$(INCLUDES) \$(DEFINES)
 
 OBJECTS=\$(filter \%.o,\$(SOURCES:\%.cpp=objs/\%.o))
+TRANSOBJ=\$(filter \%.qm,\$(TRANSLATIONS:\%.ts=\%.qm))
 
 DEPENDS=\$(filter \%.d,\$(SOURCES:\%.cpp=.depends/\%.d)) .depends/main.d
 
@@ -1405,7 +1506,7 @@ vpath \%.h \$(INCLUDE)
 
 .PHONY: all clean fixmod install distclean
 
-all: \$(TARGET)
+all: \$(TARGET) lrelease
 
 #\$(OBJECTS): Makefile Makefile.common
 
@@ -1430,7 +1531,7 @@ include \$(DEPENDS)
 plugins/\%.tso:
 	\@echo Linking plugin \$\@
 	if [ ! -d plugins ] ; then mkdir -p plugins ; fi
-	\$(GCC) -shared \$(LFLAGS) \$(LFLAGS_GLOB) \$(CFLAGS) -o \$\@ \$^
+	\$(GCC) -shared \$(LFLAGS) \$(LFLAGS_GLOB) \$(CFLAGS) \$(QT_SHARED) -o \$\@ \$^
 
 objs/\%.o: \%.cpp
 	\@echo Compiling \$<
@@ -1440,23 +1541,27 @@ objs/\%.o: \%.cpp
 \%.o : objs/\%.o
 	\@echo Faulty dependency, forgot the objs/ part
 
-install-common:
+install-common: lrelease
 	if [ \\! -f \$(TARGET) ] ; then cp tora \$(TARGET) ; fi
+	mkdir -p \$(INSTALLBIN)
 	mkdir -p \$(INSTALLLIB)/tora/help
 	-cp templates/*.tpl \$(INSTALLLIB)/tora >/dev/null 2>&1
-	-cp help/* \$(INSTALLLIB)/tora/help >/dev/null 2>&1
+	-cp -r help/* \$(INSTALLLIB)/tora/help >/dev/null 2>&1
+	-cp  *.qm \$(INSTALLLIB)/tora >/dev/null 2>&1
 
 install: \$(TARGET) install-common install-kde
 	\@echo Install \$(TARGET) to \$(INSTALLBIN)
 	if [ \\! -f \$(TARGET) ] ; then cp tora \$(TARGET) ; fi
 	-strip \$(TARGET) plugins/* >/dev/null 2>&1
 	cp \$(TARGET) \$(INSTALLBIN)/tora
-	if [ -f tora-plugin ] ; then rm tora-plugin ; fi
-	mkdir -p \$(INSTALLLIB)/tora/help
+	if [ -f tora-plugin ] ; \\
+	    then cp \$(TARGET) \$(INSTALLBIN)/tora ; \\
+	    rm -f tora-plugin ; \\
+	else cp \$(TARGET) \$(INSTALLBIN)/tora.real ; \\
+	    cp rpm/tora.sh \$(INSTALLBIN)/tora ; \\
+	fi
 	rm -f \$(INSTALLLIB)/tora/*.tso
 	-cp plugins/* \$(INSTALLLIB)/tora >/dev/null 2>&1
-	-cp templates/*.tpl \$(INSTALLLIB)/tora >/dev/null 2>&1
-	-cp -r help/* \$(INSTALLLIB)/tora/help >/dev/null 2>&1
 
 install-debug: tora-mono install-common
 	\@echo Install tora with debugging symbols to \$(INSTALLBIN)
@@ -1467,9 +1572,11 @@ __EOT__
         if (defined $KDEBase) {
 	    print MAKEFILE <<__EOT__;
 install-kde:
+	mkdir -p \$(KDE_BASE)/share/icons/hicolor/32x32/apps
+	mkdir -p \$(KDE_BASE)/share/icons/hicolor/16x16/apps 
+	mkdir -p \$(KDE_BASE)/share/applnk/Development
 	cp -f icons/tora.xpm \$(KDE_BASE)/share/icons/hicolor/32x32/apps/tora.xpm
 	cp -f icons/toramini.xpm \$(KDE_BASE)/share/icons/hicolor/16x16/apps/tora.xpm
-	mkdir -p \$(KDE_BASE)/share/applnk/Development
 	cp -f rpm/tora.desktop \$(KDE_BASE)/share/applnk/Development
 
 __EOT__
@@ -1485,8 +1592,9 @@ __EOT__
         print MAKEFILE <<__EOT__;
 uninstall:
 	\@echo Uninstalling from \$(INSTALLPREFIX)
-	rm \$(INSTALLBIN)/tora
-	rm -rf \$(INSTALLLIB)/tora
+	-rm -f \$(INSTALLBIN)/tora
+	-rm -f \$(INSTALLBIN)/tora.real
+	-rm -rf \$(INSTALLLIB)/tora
 
 clean:
 	\@echo Cleaning \$(TITLE)
@@ -1496,7 +1604,7 @@ clean:
 	-rm -f *.bak >/dev/null 2>&1
 	-rm -rf plugins >/dev/null 2>&1
 	-for a in *.ui ; \\
-             do rm `\$(PERL) -e '\$\$_=shift(\@ARGV); s/\\.[^\\.]*\$\$//; print "\$\$_.h\\n\$\$_.cpp";' \$\$a`; \\
+             do rm -f `\$(PERL) -e '\$\$_=shift(\@ARGV); s/\\.[^\\.]*\$\$//; print "\$\$_.h\\n\$\$_.cpp";' \$\$a`; \\
          done
 
 distclean: clean
@@ -1505,10 +1613,10 @@ distclean: clean
 	-rm -rf icons/.xvpics >/dev/null 2>&1
 	-rm -f *.moc qtlegacy/*.moc >/dev/null 2>&1 
 	-rm -f rpmcommon rpmoracle rpmmysql >/dev/null 2>&1 
-	-rm \\#*\\# >/dev/null 2>&1
-	-rm Makefile >/dev/null 2>&1
-	-rm LICENSE.h >/dev/null 2>&1
-	-rm configure.setup >/dev/null 2>&1
+	-rm -f \\#*\\# >/dev/null 2>&1
+	-rm -f Makefile >/dev/null 2>&1
+	-rm -f LICENSE.h >/dev/null 2>&1
+	-rm -f configure.setup >/dev/null 2>&1
 
 # This pretty requires a kdoc installtion and reference files for Qt and KDE Libs.
 # configure won't try to detect those. It's up to you.
@@ -1565,13 +1673,27 @@ tora-mono: \$(OBJECTS) main.cpp
 tora-static: \$(OBJECTS) main.cpp
 	\@echo Linking \$\@
 	\$(GCC) \$(LFLAGS) \$(CFLAGS) \$(LFLAGS_GLOB) -DTOMONOLITHIC -o \$\@ \$(OBJECTS) main.cpp \\
-		\$(QT_STATIC) \$(STDCPP_STATIC) \$(ORACLE_STATIC) \$(LIBS_GLOB) \\
-		/usr/X11R6/lib/libXext.a /usr/X11R6/lib/libX11.a /usr/X11R6/lib/libGL.a \\
-		/usr/lib/mysql/libmysqlclient.a /usr/lib/libpq.a -ldl
-
+		\$(QT_STATIC) \$(STDCPP_STATIC) \$(ORACLE_SHARED) \$(LIBS_GLOB) \\
+		-L/usr/X11R6/lib -lXext -lX11 \\
+		\$(MYSQL_STATIC) /usr/lib/libpq.a /usr/lib/libssl.a /usr/lib/libcrypto.a
 
 # The binary for the pluginbased tora
 
+tora.pro: Makefile
+	echo "# Not indended to be used for anything except lupdate" > tora.pro
+	echo 'SOURCES=	license/tolicense.cpp\\' >> tora.pro
+	echo '	license/tolicenseui.cpp\\' >> tora.pro
+	echo '	main.cpp\\' >> tora.pro
+	echo '	toextratranslations.cpp\\' >> tora.pro
+	echo "	\$(SOURCES)" >> tora.pro
+	echo 'TRANSLATIONS=\$(TRANSLATIONS)' >> tora.pro
+
+lupdate: tora.pro
+	\$(LUPDATE) tora.pro
+
+lrelease: tora.pro \$(TRANSLATIONS)
+	\$(LRELEASE) tora.pro
+
 tora:\\
 __EOT__
 	print MAKEFILE "\tobjs/".join(".o \\\n\tobjs/",sort @source).
@@ -1643,3 +1765,7 @@ if ($RPMGenerate) {
         print "\nCouldn't open rpmmysql for writing\n";
     }
 }
+
+if ($OnlyMake) {
+    exit(1);
+}
