@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -54,12 +54,15 @@
 
 #include "icons/largelogo.xpm"
 
-#ifdef TO_LICENSE
+#ifdef OAS
+#include "oasabout.h"
+#else
+#  ifdef TO_LICENSE
 static const char *AboutText="<IMG SRC=largelogo.xpm><BR>\n"
-"Version %1\n"
+"Version %1 (<A HREF=http://www.globecom.se/tora>http://www.globecom.se/tora</A>)\n"
 "<P>\n"
-"&copy; 2000-2001,2001 copyright of Underscore AB\n"
-"(<A HREF=http://www.underscore.se>http://www.underscore.se/</a>).<P>\n"
+"&copy; 2003 copyright of Quest Software, Inc\n"
+"(<A HREF=http://www.quest.com>http://www.quest.com/</a>).<P>\n"
 "Written by Henrik Johnson.<P>\n"
 "Webhosting by GlobeCom AB (<A HREF=http://www.globecom.se>www.globecom.se</A>).<P>\n"
 "This program is <B>NOT FREE SOFTWARE</B> although you are allowed to use it free of "
@@ -74,18 +77,14 @@ static const char *AboutText="<IMG SRC=largelogo.xpm><BR>\n"
 "This program is distributed in the hope that it will be useful,\n"
 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
-#else
+#  else
 static const char *AboutText="<IMG SRC=largelogo.xpm><BR>\n"
-"Version %1\n"
+"Version %1 (<A HREF=http://www.globecom.se/tora>http://www.globecom.se/tora</A>)\n"
 "<P>\n"
-"&copy; 2000-2001,2001 copyright of Underscore AB\n"
-"(<A HREF=http://www.underscore.se>http://www.underscore.se/</a>).<P>\n"
+"&copy; 2003 copyright of Quest Software, Inc\n"
+"(<A HREF=http://www.quest.com>http://www.quest.com/</a>).<P>\n"
 "Written by Henrik Johnson.<P>\n"
 "Webhosting by GlobeCom AB (<A HREF=http://www.globecom.se>www.globecom.se</A>).<P>\n"
-"<b>If you find this program usefull please have a look at my Amazon wish list at\n"
-"<A HREF=http://www.globecom.se/tora/wish>"
-"http://www.globecom.se/tora/wish</A> if you feel like showing your appreciation for this\n"
-"program.</b><P>"
 "This program is available\n"
 "under the GNU General Public License.\n"
 "Uses the Qt library by TrollTech\n"
@@ -96,33 +95,45 @@ static const char *AboutText="<IMG SRC=largelogo.xpm><BR>\n"
 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 "GNU General Public License for more details."
-#endif
+#  endif
 "<HR BREAK=ALL>\n"
 "Contributors (In alphabetical order):<P>\n"
 "<UL>\n"
 "<LI><B>Alexander (Leo) Bergolth</B> - Access to Oracle 7 database.\n"
 "<LI><B>Volker Gtz</B> - Improved PostgreSQL support.\n"
+"<LI><B>Oliver Jehle</B> - Object caching to disc, SAP and much more.\n"
 "<LI><B>Sergei Kuchin</B> - Oracle Template Library.\n"
-"<LI><B>Adam vonNieda</B> - Improvements to RMAN monitoring from OracleTool\n"
-"(http:://www.oracletool.com)\n"
+"<LI><B>Michael Meskes</B> - Debian maintainer (<A HREF=http://www.debian.org/>http://www.debian.org/</A>).\n"
 "<LI><B>Selcek Ozturk</B> - Improved content editor.\n"
+"<LI><B>Jeff Price</B> - Endless feature requests and more.\n"
 "<LI><B>Stefan Rindeskr</B> - Improved PostgreSQL support.\n"
+"<LI><B>Adam vonNieda</B> - Improvements to RMAN monitoring from OracleTool (<A HREF=http://www.oracletool.com>http://www.oracletool.com</A>)\n"
+"<LI><B>Guillaume Moulard</B> - Log4PL/SQL documentation.\n"
 "</UL>\n"
 "Other contributors:<P>\n"
 "<B>Robert Cheramy</B>,\n"
+"<B>Francesco Dandrea</B>,\n"
 "<B>Ferdinand Gassauer</B>,\n"
+"<B>Rick Hall</B>,\n"
 "<B>Frits Hoogland</B>,\n"
+"<B>Mihai Ibanescu</B>,\n"
 "<B>Bo Johansson</B>,\n"
+"<B>Olof Jnsson</B>,\n"
 "<B>Norbert Kiesel</B>,\n"
+"<B>Laramie Leavitt</B>,\n"
+"<B>Paolo Magnoli</B>,\n"
 "<B>Peter Nyborg</B>,\n"
 "<B>Frank Pavageau</B>,\n"
-"<B>Jeff Price</B>,\n"
 "<B>Jan Pruner</B>,\n"
 "<B>Jonas Rathert</B>,\n"
 "<B>Christopher R. Redinger</B>,\n"
+"<B>Erik Selberg</B>,\n"
+"<B>Dennis Selsky</B>,\n"
 "<B>Roman Stepanov</B>,\n"
+"<B>Daniel Vrit</B>,\n"
 "<B>Joachim Zetzsche</B>,\n"
 "<B>Massimiliano Ziccardi</B>...\n";
+#endif
 
 static const char *QuoteText="<H3>People who think they know everything tend to irritate those of us who do.</H3>\n"
 "<DIV ALIGN=RIGHT>Oscar Wilde<DIV ALIGN=LEFT>\n"
@@ -135,7 +146,7 @@ static const char *QuoteText="<H3>People who think they know everything tend to
 "<DIV ALIGN=RIGHT>Where the streets have no name, U2<DIV ALIGN=LEFT>\n"
 "<H3>Got a letter from the government the other day.<BR>\n"
 "Opened it and read it. It said they were suckers.</H3>\n"
-"<DIV ALIGN=RIGHT>Black steel, Tricky<DIV ALIGN=LEFT>\n"
+"<DIV ALIGN=RIGHT>Black Steel In The Hour Of Chaos, Public Enemy<DIV ALIGN=LEFT>\n"
 "<H3>It's a small world and it smells funny<BR>\n"
 "I'd buy another if it wasn't for the money</H3>\n"
 "<DIV ALIGN=RIGHT>Vision Thing, Sisters of Mercy<DIV ALIGN=LEFT>\n"
@@ -238,9 +249,26 @@ static const char *QuoteText="<H3>People who think they know everything tend to
 "but none of them serious.</H3>\n"
 "<DIV ALIGN=RIGHT>Boxer Alan Minter\n"
 "<DIV ALIGN=LEFT>\n"
+"<H3>Plans are useless, but planning is indispensible.</H3>\n"
+"<DIV ALIGN=RIGHT>Dwight Eisenhower\n"
+"<DIV ALIGN=LEFT>\n"
 "<H3>Life is a sexually transmittable desease with terminal outcome.</H3>\n"
 "<DIV ALIGN=RIGHT>Unknown\n"
 "<DIV ALIGN=LEFT>\n"
+"<H3>I'm on a seafood diet.<P>\n"
+"So am I. When I see food I eat it.</H3>\n"
+"<DIV ALIGN=RIGHT>The Drew Carey Show\n"
+"<DIV ALIGN=LEFT>\n"
+"<H3>I'm bi-sexual. Whenever I want sex I have to buy it.</H3>\n"
+"<DIV ALIGN=RIGHT>Splitting Heirs\n"
+"<DIV ALIGN=LEFT>\n"
+"<H3>What do you think of western civilisation?<BR>\n"
+"I think it would be a good idea.</H3>\n"
+"<DIV ALIGN=RIGHT>Muhatma Gandhi\n"
+"<DIV ALIGN=LEFT>\n"
+"<H3>Failure is not an option, it comes bundled with the software.</H3>\n"
+"<DIV ALIGN=RIGHT>Unknown\n"
+"<DIV ALIGN=LEFT>\n"
 "<H3>If you don't know where you are going you are never lost.</H3>\n"
 "<DIV ALIGN=RIGHT>Unknown\n"
 "<DIV ALIGN=LEFT>\n"
@@ -258,6 +286,9 @@ static const char *QuoteText="<H3>People who think they know everything tend to
 "Small minds discuss people.</H3>\n"
 "<DIV ALIGN=RIGHT>Unknown\n"
 "<DIV ALIGN=LEFT>\n"
+"<H3>Don't take life to seriously, you'll never get out of it alive anyway.</H3>\n"
+"<DIV ALIGN=RIGHT>Unknown\n"
+"<DIV ALIGN=LEFT>\n"
 "<H3>Light travels faster than sound.<BR>\n"
 "That is why some people appear bright<BR>\n"
 "until you hear them speak.</H3>\n"
@@ -327,14 +358,14 @@ static const char *QuoteText="<H3>People who think they know everything tend to
 "<DIV ALIGN=RIGHT>Despair INC (http://www.despair.com/)\n"
 "<DIV ALIGN=LEFT>";
 
-#define ABOUT_CAPTION "TOra %1"
+#define ABOUT_CAPTION TOAPPNAME " %1"
 
 void toAllocLogo(void)
 {
   static bool Alloced=false;
   if (!Alloced) {
     Alloced=true;
-    QMimeSourceFactory::defaultFactory()->setPixmap("largelogo.xpm",
+    QMimeSourceFactory::defaultFactory()->setPixmap(QString::fromLatin1("largelogo.xpm"),
 						    QPixmap((const char **)largelogo_xpm));
   }
 }
@@ -348,7 +379,7 @@ toSplash::toSplash(QWidget *parent,const char *name,WFlags f)
   QLabel *logo=new QLabel(this,"Logo");
   logo->setBackgroundColor(white);
   logo->setPixmap(QPixmap((const char **)largelogo_xpm));
-  Label=new QLabel("Loading plugins",this);
+  Label=new QLabel(tr("Loading plugins"),this);
   Label->setBackgroundColor(white);
   Progress=new QProgressBar(this,"Progress");
 
@@ -363,25 +394,33 @@ toAbout::toAbout(int page,QWidget* parent,const char* name,bool modal,WFlags fl)
 
   switch (page) {
   case 2:
-    setCaption("Quotes");
-    TextView->setText(QuoteText);
+    setCaption(tr("Quotes"));
+    TextView->setText(tr(QuoteText));
     TextView->setPaper(QColor(255,255,255));
     TextView->setTextFormat(RichText);
     CancelButton->hide();
     break;
   case 1:
-    setCaption("GNU General Public License");
-    TextView->setText(LicenseText);
+#ifdef TO_LICENSE
+    setCaption(tr("End User License"));
+#else
+    setCaption(tr("GNU General Public License"));
+#endif
+    TextView->setText(tr(LicenseText));
     TextView->setPaper(QColor(255,255,255));
     TextView->setTextFormat(PlainText);
     if (parent)
       CancelButton->hide();
     break;
   default:
-    setCaption(QString(ABOUT_CAPTION).arg(TOVERSION));
-    QString buffer=QString(AboutText).arg(TOVERSION);
+    setCaption(tr(ABOUT_CAPTION).arg(QString::fromLatin1(TOVERSION)));
+    QString buffer=tr(AboutText).arg(QString::fromLatin1(TOVERSION));
     TextView->setText(buffer);
+#ifdef OAS
+    TextView->setPaper(QColor(255,255,255));
+#else
     TextView->setPaper(QColor(227,184,54));
+#endif
     TextView->setTextFormat(RichText);
     CancelButton->hide();
     break;
