@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -72,16 +72,21 @@
 
 class toChartTool : public toTool {
 protected:
+  toChartHandler *Handler;
   toChartManager *Window;
-
 public:
   toChartTool()
     : toTool(930,"Chart Manager")
   { Window=NULL; }
-  virtual QWidget *toolWindow(QWidget *parent,toConnection &connection)
+  virtual QWidget *toolWindow(QWidget *,toConnection &)
   {
-    if (!Window)
+    if (!Handler)
       return NULL;
+    if (!Window) {
+      Window=new toChartManager(toMainWidget()->workspace());
+      Window->setCaption(qApp->translate("toChartTool","Chart Manager"));
+      Window->setIcon(QPixmap((const char **)grid_xpm));
+    }
     Window->refresh();
     Window->show();
     Window->setFocus();
@@ -90,18 +95,25 @@ public:
   virtual void customSetup(int toolid)
   {
     toMainWidget()->editMenu()->insertItem(QPixmap((const char **)grid_xpm),
-					   "Chart Manager...",toolid);
+					   qApp->translate("toChartTool","Chart Manager..."),toolid);
 
-    Window=new toChartManager(toMainWidget()->workspace());
-    Window->setCaption("Chart Manager");
-    Window->setIcon(QPixmap((const char **)grid_xpm));
-    Window->hide();
+    Handler=new toChartHandler();
   }
   void closeWindow(void)
   {
     Window=NULL;
   }
-  virtual bool canHandle(toConnection &conn)
+  void closeHandler(void)
+  {
+    Handler=NULL;
+  }
+
+  toChartHandler *handler()
+  { return Handler; }
+  toChartManager *manager()
+  { return Window; }
+
+  virtual bool canHandle(toConnection &)
   { return true; }
 };
 
@@ -115,7 +127,7 @@ public:
   virtual void changeValue(int val)
   { 
     if (val==1)
-      ExtraLabel->setText("Email");
+      ExtraLabel->setText(qApp->translate("toChartAlarm","Email"));
     else
       ExtraLabel->setText(QString::null);
 
@@ -132,7 +144,7 @@ public:
 
   virtual void browseFile()
   {
-    QString str=toOpenFilename(Filename->text(),"*.csv",this);
+    QString str=toOpenFilename(Filename->text(),QString::fromLatin1("*.csv"),this);
     if (!str.isEmpty())
       Filename->setText(str);
   }
@@ -148,7 +160,7 @@ public:
     diag.Value->setValidator(new QDoubleValidator(diag.Value));
     diag.Extra->setText(alarm.Extra);
     std::list<int>::iterator sel=alarm.Columns.begin();
-    diag.Charts->addColumn("Charts");
+    diag.Charts->addColumn(qApp->translate("toChartSetup","Charts"));
     diag.Charts->setSelectionMode(QListView::Multi);
     QListViewItem *item=NULL;
     std::list<QString>::iterator lab=Chart->labels().begin();
@@ -183,17 +195,17 @@ public:
     toChartManager::chartAlarm alarm;
     QString str=modifyAlarm(alarm.toString(),alarm.Persistent);
     if (!str.isEmpty())
-      new QListViewItem(Alarms,str,alarm.Persistent?"Persistent":"Temporary");
+      new QListViewItem(Alarms,str,alarm.Persistent?qApp->translate("toChartSetup","Persistent"):qApp->translate("toChartSetup","Temporary"));
   }
   virtual void modifyAlarm()
   {
     QListViewItem *item=Alarms->selectedItem();
     if (item) {
-      bool pers=item->text(1)=="Persistent";
+      bool pers=item->text(1)==qApp->translate("toChartSetup","Persistent");
       QString str=modifyAlarm(item->text(0),pers);
       if (!str.isEmpty()) {
 	item->setText(0,str);
-	item->setText(1,pers?"Persistent":"Temporary");
+	item->setText(1,pers?qApp->translate("toChartSetup","Persistent"):qApp->translate("toChartSetup","Temporary"));
       }
     }
   }
@@ -203,7 +215,7 @@ public:
   }
 };
 
-toChartReceiver::toChartReceiver(toChartManager *parent,toLineChart *chart)
+toChartReceiver::toChartReceiver(toChartHandler *parent,toLineChart *chart)
   : QObject(parent)
 {
   Parent=parent;
@@ -225,7 +237,7 @@ QString toChartReceiver::name(void)
   if (!Result||Result->sqlName().isEmpty())
     return QString::null;
   try {
-    LastName=Result->connection().description(false)+":"+Result->sqlName();
+    LastName=Result->connection().description(false)+QString::fromLatin1(":")+QString::fromLatin1(Result->sqlName());
   } catch(...) {
   }
   return LastName;
@@ -237,44 +249,35 @@ void toChartReceiver::valueAdded(std::list<double> &value,const QString &xValues
 }
 
 toChartManager::toChartManager(QWidget *main)
-  : QVBox(main),toHelpContext("chartmanager.html")
+  : QVBox(main),toHelpContext(QString::fromLatin1("chartmanager.html"))
 {
-  loadSettings();
-
-  connect(toMainWidget(),SIGNAL(chartAdded(toLineChart *)),
-	  this,SLOT(addChart(toLineChart *)));
-  connect(toMainWidget(),SIGNAL(chartRemoved(toLineChart *)),
-	  this,SLOT(removeChart(toLineChart *)));
-  connect(toMainWidget(),SIGNAL(chartSetup(toLineChart *)),
-	  this,SLOT(setupChart(toLineChart *)));
-
-  QToolBar *toolbar=toAllocBar(this,"Chart Manager",QString::null);
+  QToolBar *toolbar=toAllocBar(this,tr("Chart Manager"));
 
   new QToolButton(QPixmap((const char **)refresh_xpm),
-		  "Refresh list",
-		  "Refresh list",
+		  tr("Refresh list"),
+		  tr("Refresh list"),
 		  this,SLOT(refresh()),
 		  toolbar);
   toolbar->addSeparator();
   new QToolButton(QPixmap((const char **)fileopen_xpm),
-		  "Open tracker file",
-		  "Open tracker file",
+		  tr("Open tracker file"),
+		  tr("Open tracker file"),
 		  this,SLOT(openChart()),
 		  toolbar);
   new QToolButton(QPixmap((const char **)grid_xpm),
-		  "Setup chart",
-		  "Setup chart",
+		  tr("Setup chart"),
+		  tr("Setup chart"),
 		  this,SLOT(setupChart()),
 		  toolbar);
 
-  toolbar->setStretchableWidget(new QLabel("",toolbar));
+  toolbar->setStretchableWidget(new QLabel(toolbar,TO_KDE_TOOLBAR_WIDGET));
 
   List=new toListView(this);
-  List->addColumn("Connection");
-  List->addColumn("Title");
-  List->addColumn("ID");
-  List->addColumn("Tracking");
-  List->addColumn("Alarms");
+  List->addColumn(tr("Connection"));
+  List->addColumn(tr("Title"));
+  List->addColumn(tr("ID"));
+  List->addColumn(tr("Tracking"));
+  List->addColumn(tr("Alarms"));
   List->setSorting(2);
   List->setSelectionMode(QListView::Single);
 
@@ -282,20 +285,19 @@ toChartManager::toChartManager(QWidget *main)
   connect(toMainWidget()->workspace(),SIGNAL(windowActivated(QWidget *)),
 	  this,SLOT(windowActivated(QWidget *)));
   connect(&Refresh,SIGNAL(timeout()),this,SLOT(refresh()));
-  connect(&Timer,SIGNAL(timeout()),this,SLOT(alarm()));
 
   setFocusProxy(List);
 }
 
 void toChartManager::windowActivated(QWidget *widget)
 {
-  if (widget==this&&!isHidden()) {
+  if (widget==this) {
     if (!ToolMenu) {
       ToolMenu=new QPopupMenu(this);
-      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),"&Refresh",
+      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),tr("&Refresh"),
 			   this,SLOT(refresh(void)),Key_F5);
 
-      toMainWidget()->menuBar()->insertItem("&Chart Manager",ToolMenu,-1,toToolMenuIndex());
+      toMainWidget()->menuBar()->insertItem(tr("&Chart Manager"),ToolMenu,-1,toToolMenuIndex());
     }
   } else {
     delete ToolMenu;
@@ -306,25 +308,26 @@ void toChartManager::windowActivated(QWidget *widget)
 toChartManager::~toChartManager()
 {
   ChartTool.closeWindow();
-  Charts.clear();
 }
 
-void toChartManager::addChart(toLineChart *chart)
+void toChartHandler::addChart(toLineChart *chart)
 {
   Charts.insert(Charts.end(),new toChartReceiver(this,chart));
-  Refresh.start(1,true);
+  toChartManager *manager=ChartTool.manager();
+  if (manager)
+    manager->Refresh.start(1,true);
 }
 
-void toChartManager::removeChart(toLineChart *chart)
+void toChartHandler::removeChart(toLineChart *chart)
 {
   for(std::list<toChartReceiver *>::iterator i=Charts.begin();i!=Charts.end();i++) {
     if ((*i)->chart()==chart) {
       QString name=(*i)->name();
       if (!name.isNull()) {
-	std::map<QString,std::list<chartAlarm> >::iterator fnda=Alarms.find(name);
+	std::map<QString,std::list<toChartManager::chartAlarm> >::iterator fnda=Alarms.find(name);
 	if (fnda!=Alarms.end()) {
 	  bool any=false;
-	  for(std::list<chartAlarm>::iterator j=(*fnda).second.begin();j!=(*fnda).second.end();) {
+	  for(std::list<toChartManager::chartAlarm>::iterator j=(*fnda).second.begin();j!=(*fnda).second.end();) {
 	    if (!(*j).Persistent) {
 	      (*fnda).second.erase(j);
 	      j=(*fnda).second.begin();
@@ -337,7 +340,7 @@ void toChartManager::removeChart(toLineChart *chart)
 	    Alarms.erase(fnda);
 	}
 
-	std::map<QString,chartTrack>::iterator fndt=Files.find(name);
+	std::map<QString,toChartManager::chartTrack>::iterator fndt=Files.find(name);
 	if (fndt!=Files.end()) {
 	  if (!(*fndt).second.Persistent)
 	    Files.erase(fndt);
@@ -346,7 +349,9 @@ void toChartManager::removeChart(toLineChart *chart)
 
       delete *i;
       Charts.erase(i);
-      Refresh.start(1,true);
+      toChartManager *manager=ChartTool.manager();
+      if (manager)
+	manager->Refresh.start(1,true);
       return;
     }
   }
@@ -377,49 +382,49 @@ toChartManager::chartAlarm::chartAlarm(const QString &inp,bool pers)
     return;
   }
 
-  QString t=oper;
-  if(t=="min")
+  QString t=QString::fromLatin1(oper);
+  if(t==QString::fromLatin1("min"))
     Operation=Min;
-  else if (t=="all")
+  else if (t==QString::fromLatin1("all"))
     Operation=All;
-  else if (t=="sum")
+  else if (t==QString::fromLatin1("sum"))
     Operation=Sum;
-  else if (t=="average")
+  else if (t==QString::fromLatin1("average"))
     Operation=Average;
-  else if (t=="max")
+  else if (t==QString::fromLatin1("max"))
     Operation=Max;
   else
     Operation=Any;
 
   t=QString::fromUtf8(cols);
   if (t.length()>2) {
-    QStringList lst=QStringList::split(",",t.mid(1,t.length()-2));
+    QStringList lst=QStringList::split(QString::fromLatin1(","),t.mid(1,t.length()-2));
     for(unsigned int i=0;i<lst.count();i++)
       Columns.insert(Columns.end(),lst[i].toInt());
     Columns.sort();
   }
 
-  t=comp;
-  if (t=="=") {
+  t=QString::fromLatin1(comp);
+  if (t==QString::fromLatin1("=")) {
     Comparison=Equal;
-  } else if (t=="!=") {
+  } else if (t==QString::fromLatin1("!=")) {
     Comparison=NotEqual;
-  } else if (t=="<") {
+  } else if (t==QString::fromLatin1("<")) {
     Comparison=Less;
-  } else if (t==">") {
+  } else if (t==QString::fromLatin1(">")) {
     Comparison=Greater;
-  } else if (t=="<=") {
+  } else if (t==QString::fromLatin1("<=")) {
     Comparison=LessEqual;
-  } else if (t==">=") {
+  } else if (t==QString::fromLatin1(">=")) {
     Comparison=GreaterEqual;
   } else {
     Comparison=Equal;
   }
 
-  t=act;
-  if (t=="StatusMessage")
+  t=QString::fromLatin1(act);
+  if (t==QString::fromLatin1("StatusMessage"))
     Action=StatusMessage;
-  else if (t=="Email")
+  else if (t==QString::fromLatin1("Email"))
     Action=Email;
   else
     Action=Ignore;
@@ -456,68 +461,68 @@ QString toChartManager::chartAlarm::toString(void)
   QString t;
   switch (Operation) {
   case Any:
-    t="any";
+    t=QString::fromLatin1("any");
     break;
   case All:
-    t="all";
+    t=QString::fromLatin1("all");
     break;
   case Sum:
-    t="sum";
+    t=QString::fromLatin1("sum");
     break;
   case Average:
-    t="average";
+    t=QString::fromLatin1("average");
     break;
   case Max:
-    t="max";
+    t=QString::fromLatin1("max");
     break;
   case Min:
-    t="min";
+    t=QString::fromLatin1("min");
     break;
   }
-  t+=" (";
+  t+=QString::fromLatin1(" (");
   bool first=true;
   for(std::list<int>::iterator i=Columns.begin();i!=Columns.end();i++) {
     if (first)
       first=false;
     else
-      t+=",";
+      t+=QString::fromLatin1(",");
     t+=QString::number(*i);
   }
-  t+=")";
+  t+=QString::fromLatin1(")");
   switch (Comparison) {
   case Equal:
-    t+=" = ";
+    t+=QString::fromLatin1(" = ");
     break;
   case NotEqual:
-    t+=" != ";
+    t+=QString::fromLatin1(" != ");
     break;
   case Less:
-    t+=" < ";
+    t+=QString::fromLatin1(" < ");
     break;
   case Greater:
-    t+=" > ";
+    t+=QString::fromLatin1(" > ");
     break;
   case LessEqual:
-    t+=" <= ";
+    t+=QString::fromLatin1(" <= ");
     break;
   case GreaterEqual:
-    t+=" >= ";
+    t+=QString::fromLatin1(" >= ");
     break;
   }
   t+=QString::number(Value);
   switch (Action) {
   case StatusMessage:
-    t+=" StatusMessage";
+    t+=QString::fromLatin1(" StatusMessage");
     break;
   case Email:
-    t+=" Email";
+    t+=QString::fromLatin1(" Email");
     break;
   case Ignore:
-    t+=" Ignore";
+    t+=QString::fromLatin1(" Ignore");
     break;
   }
   if (!Extra.isEmpty()) {
-    t+=" ";
+    t+=QString::fromLatin1(" ");
     t+=Extra;
   }
   return t;
@@ -542,7 +547,7 @@ bool toChartManager::chartAlarm::checkValue(double val)
   return false;
 }
 
-void toChartManager::chartAlarm::valueAdded(toChartManager *manager,
+void toChartManager::chartAlarm::valueAdded(toChartHandler *handler,
 					    const QString &str,
 					    std::list<double> &value,
 					    const QString &xValue)
@@ -596,7 +601,7 @@ void toChartManager::chartAlarm::valueAdded(toChartManager *manager,
   case Max:
     {
       bool first=true;
-      double val;
+      double val=0;
       for(std::list<double>::iterator i=vals.begin();i!=vals.end();i++) {
 	if (first) {
 	  first=false;
@@ -610,7 +615,7 @@ void toChartManager::chartAlarm::valueAdded(toChartManager *manager,
   case Min:
     {
       bool first=true;
-      double val;
+      double val=0;
       for(std::list<double>::iterator i=vals.begin();i!=vals.end();i++) {
 	if (first) {
 	  first=false;
@@ -625,22 +630,22 @@ void toChartManager::chartAlarm::valueAdded(toChartManager *manager,
   if (sig) {
     if (Signal)
       return;
-    manager->SignalAlarms.insert(manager->SignalAlarms.end(),
+    handler->SignalAlarms.insert(handler->SignalAlarms.end(),
 				 alarmSignal(Action,xValue,str,toString(),Extra));
-    manager->Timer.start(1,true);
+    handler->Timer.start(1,true);
     Signal=true;
   } else
     Signal=false;
 }
 
-void toChartManager::saveSettings(void)
+void toChartHandler::saveSettings(void)
 {
   {
     int num=0;
-    for(std::map<QString,chartTrack>::iterator i=Files.begin();i!=Files.end();i++) {
+    for(std::map<QString,toChartManager::chartTrack>::iterator i=Files.begin();i!=Files.end();i++) {
       if ((*i).second.Persistent) {
 	num++;
-	QString name="Files:"+QString::number(num);
+	QCString name=QCString("Files:")+QString::number(num).latin1();
 	ChartTool.setConfig(name+":Name",(*i).first);
 	ChartTool.setConfig(name+":Spec",(*i).second.File.name());
       }
@@ -649,12 +654,12 @@ void toChartManager::saveSettings(void)
   }
   {
     int num=0;
-    for(std::map<QString,std::list<chartAlarm> >::iterator i=Alarms.begin();
+    for(std::map<QString,std::list<toChartManager::chartAlarm> >::iterator i=Alarms.begin();
 	i!=Alarms.end();i++) {
-      for(std::list<chartAlarm>::iterator j=(*i).second.begin();j!=(*i).second.end();j++) {
+      for(std::list<toChartManager::chartAlarm>::iterator j=(*i).second.begin();j!=(*i).second.end();j++) {
 	if ((*j).Persistent) {
 	  num++;
-	  QString name="Alarms:"+QString::number(num);
+	  QCString name=QCString("Alarms:")+QString::number(num).latin1();
 	  ChartTool.setConfig(name+":Name",(*i).first);
 	  ChartTool.setConfig(name+":Spec",(*j).toString());
 	}
@@ -665,58 +670,55 @@ void toChartManager::saveSettings(void)
   toTool::saveConfig();
 }
 
-void toChartManager::loadSettings(void)
+void toChartHandler::loadSettings(void)
 {
   {
     for(int num=ChartTool.config("FilesCount","0").toInt();num>0;num--) {
-      QString name="Files:"+QString::number(num);
-      QString t=ChartTool.config(name+":Name",QString::null);
-      QString s=ChartTool.config(name+":Spec",QString::null);
+      QCString name=QCString("Files:")+QString::number(num).latin1();
+      QString t=ChartTool.config(name+":Name","");
+      QString s=ChartTool.config(name+":Spec","");
       if (!t.isEmpty()&&!s.isEmpty())
-	Files[t]=chartTrack(s,true);
+	Files[t]=toChartManager::chartTrack(s,true);
     }
   }
   {
     for(int num=ChartTool.config("AlarmCount","0").toInt();num>0;num--) {
-      QString name="Alarms:"+QString::number(num);
-      QString t=ChartTool.config(name+":Name",QString::null);
-      QString s=ChartTool.config(name+":Spec",QString::null);
+      QCString name=QCString("Alarms:")+QString::number(num).latin1();
+      QString t=ChartTool.config(name+":Name","");
+      QString s=ChartTool.config(name+":Spec","");
       if (!t.isEmpty()&&!s.isEmpty())
-	Alarms[t].insert(Alarms[t].end(),chartAlarm(s,true));
+	Alarms[t].insert(Alarms[t].end(),toChartManager::chartAlarm(s,true));
     }
   }
 }
 
-void toChartManager::alarm(void)
+void toChartHandler::alarm(void)
 {
   while(SignalAlarms.size()>0) {
-    alarmSignal signal=toShift(SignalAlarms);
-    if (signal.Action==StatusMessage)
-      toStatusMessage("ALARM:"+signal.Chart+": "+signal.Alarm+": "+signal.xValue);
-    else if (signal.Action==Email)
-      new toSMTP("Toolkit for Oracle <noreply@localhost>",
+    toChartManager::alarmSignal signal=toShift(SignalAlarms);
+    if (signal.Action==toChartManager::StatusMessage)
+      toStatusMessage(tr("ALARM:")+signal.Chart+QString::fromLatin1(": ")+signal.Alarm+QString::fromLatin1(": ")+signal.xValue);
+    else if (signal.Action==toChartManager::Email)
+      new toSMTP(QString::fromLatin1(TOAPPNAME " <noreply@localhost>"),
 		 signal.Extra,
-		 "TOra alert: "+signal.Chart,
-		 "A defined alert value was detected:\n\n"+
-		 signal.Alarm+
-		 "\n\nAt: "+
-		 signal.xValue);
+		 tr("TOra alert:")+" "+signal.Chart,
+		 tr("A defined alert value was detected:\n\n%1\n\nAt: %2").arg(signal.Alarm).arg(signal.xValue));
   }
 }
 
-void toChartManager::valueAdded(toLineChart *chart,
+void toChartHandler::valueAdded(toLineChart *chart,
 				const QString &chartName,
 				std::list<double> &value,
 				const QString &xValue)
 {
-  std::map<QString,std::list<chartAlarm> >::iterator fnda=Alarms.find(chartName);
+  std::map<QString,std::list<toChartManager::chartAlarm> >::iterator fnda=Alarms.find(chartName);
   if (fnda!=Alarms.end()) {
-    std::list<chartAlarm> &alarms=(*fnda).second;
-    for(std::list<chartAlarm>::iterator i=alarms.begin();i!=alarms.end();i++)
+    std::list<toChartManager::chartAlarm> &alarms=(*fnda).second;
+    for(std::list<toChartManager::chartAlarm>::iterator i=alarms.begin();i!=alarms.end();i++)
       (*i).valueAdded(this,chartName,value,xValue);
   }
 
-  std::map<QString,chartTrack>::iterator fndt=Files.find(chartName);
+  std::map<QString,toChartManager::chartTrack>::iterator fndt=Files.find(chartName);
   if (fndt!=Files.end()) {
     QFile &file=(*fndt).second.File;
     bool header=false;
@@ -726,23 +728,23 @@ void toChartManager::valueAdded(toLineChart *chart,
       file.open(IO_Raw|IO_WriteOnly|IO_Append);
     }
     if (file.isOpen()) {
-      static QRegExp quote("\"");
+      static QRegExp quote(QString::fromLatin1("\""));
       QCString out="\"";
       if (header) {
 	QString t=chart->title();
-	t.replace(quote,"\"\"");
+	t.replace(quote,QString::fromLatin1("\"\""));
 	out+=t.utf8();
 	std::list<QString> labels=chart->labels();
 	for(std::list<QString>::iterator i=labels.begin();i!=labels.end();i++) {
 	  out+="\";\"";
 	  QString t=(*i);
-	  t.replace(quote,"\"\"");
+	  t.replace(quote,QString::fromLatin1("\"\""));
 	  out+=t.utf8();
 	}
 	out+="\"\n\"";
       }
       QString t=xValue;
-      t.replace(quote,"\"\"");
+      t.replace(quote,QString::fromLatin1("\"\""));
       out+=t.utf8();
       for(std::list<double>::iterator i=value.begin();i!=value.end();i++) {
 	out+="\";\"";
@@ -758,7 +760,7 @@ static QString ReadCSV(const QCString &data,unsigned int &pos,bool &nl)
 {
   QCString ret;
   if (data.at(pos)!='\"')
-    throw QString("Initial value didn't start with \" in CSV file");
+    throw qApp->translate("toChartManager","Initial value didn't start with \" in CSV file");
   pos++;
   while(pos<data.length()&&
 	(data.at(pos)!='\"'||(pos+1<data.length()&&
@@ -769,7 +771,7 @@ static QString ReadCSV(const QCString &data,unsigned int &pos,bool &nl)
     pos++;
   }
   if (pos>=data.length())
-    throw QString("Missing closing \" in CSV file");
+    throw qApp->translate("toChartManager","Missing closing \" in CSV file");
   pos++;
   nl=false;
   while(pos<data.length()&&(isspace(data.at(pos))||data.at(pos)==','||data.at(pos)==';')) {
@@ -782,7 +784,7 @@ static QString ReadCSV(const QCString &data,unsigned int &pos,bool &nl)
 
 void toChartManager::openChart(void)
 {
-  QString name=toOpenFilename(QString::null,"*.csv",this);
+  QString name=toOpenFilename(QString::null,QString::fromLatin1("*.csv"),this);
   if (!name.isEmpty()) {
     try {
       QCString data=toReadFile(name);
@@ -790,9 +792,11 @@ void toChartManager::openChart(void)
       
       toLineChart *chart;
       switch(TOMessageBox::information(toMainWidget(),
-				       "Chart format",
-				       "Select format of the chart to display",
-				       "Barchart","Linechart","Cancel")) {
+				       tr("Chart format"),
+				       tr("Select format of the chart to display"),
+				       tr("Barchart"),
+				       tr("Linechart"),
+				       tr("Cancel"))) {
       case 0:
 	chart=new toBarChart(toMainWidget()->workspace(),NULL,
 			     WDestructiveClose);
@@ -801,7 +805,7 @@ void toChartManager::openChart(void)
 	chart=new toLineChart(toMainWidget()->workspace(),NULL,
 			      WDestructiveClose);
 	break;
-      case 2:
+      default:
 	return;
       }
       chart->show();
@@ -826,45 +830,47 @@ void toChartManager::openChart(void)
 void toChartManager::setupChart(void)
 {
   toChartReceiver *chart=selectedChart();
-  if (chart)
-    setupChart(chart->chart());
+  if (chart&&ChartTool.handler())
+    ChartTool.handler()->setupChart(chart->chart());
 }
 
 toChartReceiver *toChartManager::selectedChart(void)
 {
-  QListViewItem *item=List->selectedItem();
-  if (item) {
-    for(std::list<toChartReceiver *>::iterator i=Charts.begin();i!=Charts.end();i++) {
-      toResult *result=(*i)->result();
-      if (result) {
-	if (item->text(0)==result->connection().description(false)&&
-	    item->text(2)==result->sqlName())
-	  return *i;
+  try {
+    QListViewItem *item=List->selectedItem();
+    if (item&&ChartTool.handler()) {
+      for(std::list<toChartReceiver *>::iterator i=ChartTool.handler()->Charts.begin();i!=ChartTool.handler()->Charts.end();i++) {
+	toResult *result=(*i)->result();
+	if (result) {
+	  if (item->text(0)==result->connection().description(false)&&
+	      item->text(2)==result->sqlName())
+	    return *i;
+	}
       }
     }
-  }
+  } TOCATCH
   return NULL;
 }
 
-void toChartManager::setupChart(toLineChart *chart)
+void toChartHandler::setupChart(toLineChart *chart)
 {
   for(std::list<toChartReceiver *>::iterator i=Charts.begin();i!=Charts.end();i++) {
     if ((*i)->chart()==chart) {
       QString name=(*i)->name();
       if (!name.isNull()) {
-	std::list<chartAlarm> alarm;
-	std::map<QString,std::list<chartAlarm> >::iterator fnda=Alarms.find(name);
+	std::list<toChartManager::chartAlarm> alarm;
+	std::map<QString,std::list<toChartManager::chartAlarm> >::iterator fnda=Alarms.find(name);
 	if (fnda!=Alarms.end())
 	  alarm=(*fnda).second;
 
-	chartTrack file;
-	std::map<QString,chartTrack>::iterator fndt=Files.find(name);
+	toChartManager::chartTrack file;
+	std::map<QString,toChartManager::chartTrack>::iterator fndt=Files.find(name);
 	if (fndt!=Files.end())
 	  file=(*fndt).second;
 
-	toChartSetup setup(chart,this,NULL,true);
-	setup.Alarms->addColumn("Alarms");
-	setup.Alarms->addColumn("Persistent");
+	toChartSetup setup(chart,toMainWidget(),NULL,true);
+	setup.Alarms->addColumn(tr("Alarms"));
+	setup.Alarms->addColumn(tr("Persistent"));
 	setup.Alarms->setSorting(0);
 	setup.Alarms->setSelectionMode(QListView::Single);
 	setup.Persistent->setChecked(file.Persistent);
@@ -879,26 +885,26 @@ void toChartManager::setupChart(toLineChart *chart)
 				     QSizePolicy::Preferred));
 	t->showLegend(false);
 	t->showAxisLegend(false);
-	for(std::list<chartAlarm>::iterator j=alarm.begin();j!=alarm.end();j++)
+	for(std::list<toChartManager::chartAlarm>::iterator j=alarm.begin();j!=alarm.end();j++)
 	  new QListViewItem(setup.Alarms,
 			    (*j).toString(),
-			    (*j).Persistent?"Persistent":"Temporary");
+			    (*j).Persistent?tr("Persistent"):tr("Temporary"));
 
 	if (setup.exec()) {
 	  if (setup.Filename->text().isEmpty()||!setup.Enabled->isChecked()) {
 	    if (fndt!=Files.end())
 	      Files.erase(fndt);
 	  } else {
-	    Files[name]=chartTrack(setup.Filename->text(),
-				   setup.Persistent->isChecked());
+	    Files[name]=toChartManager::chartTrack(setup.Filename->text(),
+						   setup.Persistent->isChecked());
 	  }
 	  if (fnda!=Alarms.end())
 	    Alarms.erase(fnda);
 	  alarm.clear();
 	  for(QListViewItem *item=setup.Alarms->firstChild();
 	      item;item=item->nextSibling()) {
-	    alarm.insert(alarm.end(),chartAlarm(item->text(0),
-						item->text(1)=="Persistent"));
+	    alarm.insert(alarm.end(),toChartManager::chartAlarm(item->text(0),
+								item->text(1)==tr("Persistent")));
 	  }
 	  if (alarm.size()>0)
 	    Alarms[name]=alarm;
@@ -912,32 +918,52 @@ void toChartManager::setupChart(toLineChart *chart)
 
 void toChartManager::refresh(void)
 {
-  List->clear();
-  for(std::list<toChartReceiver *>::iterator i=Charts.begin();i!=Charts.end();i++) {
-    toResult *result=(*i)->result();
-    if (result) {
-      toResultViewItem *item=new toResultViewMLine(List,
-						   NULL,
-						   result->connection().description(false));
-      item->setText(1,(*i)->chart()->title());
-      item->setText(2,result->sqlName());
-      QString name=(*i)->name();
-      if (!name.isNull()) {
-	std::map<QString,std::list<chartAlarm> >::iterator fnda=Alarms.find(name);
-	if (fnda!=Alarms.end()) {
-	  QString t;
-	  for(std::list<chartAlarm>::iterator j=(*fnda).second.begin();j!=(*fnda).second.end();j++) {
-	    t+=(*j).toString();
-	    t+="\n";
+  if (!ChartTool.handler())
+    return;
+  try {
+    List->clear();
+    for(std::list<toChartReceiver *>::iterator i=ChartTool.handler()->Charts.begin();i!=ChartTool.handler()->Charts.end();i++) {
+      toResult *result=(*i)->result();
+      if (result) {
+	toResultViewItem *item=new toResultViewMLine(List,
+						     NULL,
+						     result->connection().description(false));
+	item->setText(1,(*i)->chart()->title());
+	item->setText(2,result->sqlName());
+	QString name=(*i)->name();
+	if (!name.isNull()) {
+	  std::map<QString,std::list<chartAlarm> >::iterator fnda=ChartTool.handler()->Alarms.find(name);
+	  if (fnda!=ChartTool.handler()->Alarms.end()) {
+	    QString t;
+	    for(std::list<chartAlarm>::iterator j=(*fnda).second.begin();j!=(*fnda).second.end();j++) {
+	      t+=(*j).toString();
+	      t+=QString::fromLatin1("\n");
+	    }
+	    if (t.length()>0)
+	      item->setText(4,t.mid(0,t.length()-1));
 	  }
-	  if (t.length()>0)
-	    item->setText(4,t.mid(0,t.length()-1));
-	}
 
-	std::map<QString,chartTrack>::iterator fndt=Files.find(name);
-	if (fndt!=Files.end())
-	  item->setText(3,(*fndt).second.File.name());
+	  std::map<QString,chartTrack>::iterator fndt=ChartTool.handler()->Files.find(name);
+	  if (fndt!=ChartTool.handler()->Files.end())
+	    item->setText(3,(*fndt).second.File.name());
+	}
       }
     }
-  }
+  } TOCATCH
+}
+
+toChartHandler::toChartHandler()
+{
+  connect(toMainWidget(),SIGNAL(chartAdded(toLineChart *)),
+	  this,SLOT(addChart(toLineChart *)));
+  connect(toMainWidget(),SIGNAL(chartRemoved(toLineChart *)),
+	  this,SLOT(removeChart(toLineChart *)));
+  connect(toMainWidget(),SIGNAL(chartSetup(toLineChart *)),
+	  this,SLOT(setupChart(toLineChart *)));
+  connect(&Timer,SIGNAL(timeout()),this,SLOT(alarm()));
+}
+
+toChartHandler::~toChartHandler()
+{
+  ChartTool.closeHandler();
 }
