@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -40,65 +40,83 @@
 #include "tosql.h"
 #include "totool.h"
 
+#include <string>
+#include <time.h>
+
 #include <qapplication.h>
+#include <qdir.h>
 #include <qprogressdialog.h>
+#include <qtextstream.h>
 #include <qwidget.h>
 
+// A little magic to get lrefresh to work and get a check on qApp
+
+#undef QT_TRANSLATE_NOOP
+#define QT_TRANSLATE_NOOP(x,y) QTRANS(x,y)
+
+#define TO_DEBUGOUT(x) fprintf(stderr,(const char *)x);
+
 // Connection provider implementation
 
-std::map<QString,toConnectionProvider *> *toConnectionProvider::Providers;
-std::map<QString,toConnectionProvider *> *toConnectionProvider::Types;
+std::map<QCString,toConnectionProvider *> *toConnectionProvider::Providers;
+std::map<QCString,toConnectionProvider *> *toConnectionProvider::Types;
+
+static int NumberFormat;
+static int NumberDecimals;
 
 void toConnectionProvider::checkAlloc(void)
 {
   if (!Providers)
-    Providers=new std::map<QString,toConnectionProvider *>;
+    Providers=new std::map<QCString,toConnectionProvider *>;
 }
 
-void toConnectionProvider::addProvider(const QString &provider)
+void toConnectionProvider::addProvider(const QCString &provider)
 {
   checkAlloc();
   Provider=provider; (*Providers)[Provider]=this;
 }
 
-toConnectionProvider::toConnectionProvider(const QString &provider,bool add)
+toConnectionProvider::toConnectionProvider(const QCString &provider,bool add)
 {
   Provider=provider;
   if (add)
     addProvider(provider);
   if (!Types)
-    Types=new std::map<QString,toConnectionProvider *>;
+    Types=new std::map<QCString,toConnectionProvider *>;
   (*Types)[provider]=this;
 }
 
-void toConnectionProvider::removeProvider(const QString &provider)
+void toConnectionProvider::removeProvider(const QCString &provider)
 {
-  std::map<QString,toConnectionProvider *>::iterator i=Providers->find(Provider);
+  std::map<QCString,toConnectionProvider *>::iterator i=Providers->find(provider);
   if (i!=Providers->end())
     Providers->erase(i);
 }
 
 toConnectionProvider::~toConnectionProvider()
 {
-  if (!Provider.isEmpty())
-    removeProvider(Provider);
-  std::map<QString,toConnectionProvider *>::iterator i=Types->find(Provider);
-  if (i!=Types->end())
-    Types->erase(i);
+  try {
+    if (!Provider.isEmpty())
+      removeProvider(Provider);
+    std::map<QCString,toConnectionProvider *>::iterator i=Types->find(Provider);
+    if (i!=Types->end())
+      Types->erase(i);
+  } catch(...) {
+  }
 }
 
-std::list<QString> toConnectionProvider::providedHosts(const QString &)
+std::list<QString> toConnectionProvider::providedHosts(const QCString &)
 {
   std::list<QString> ret;
   return ret;
 }
 
-std::list<QString> toConnectionProvider::providers()
+std::list<QCString> toConnectionProvider::providers()
 {
-  std::list<QString> ret;
+  std::list<QCString> ret;
   if (!Providers)
     return ret;
-  for(std::map<QString,toConnectionProvider *>::iterator i=Providers->begin();i!=Providers->end();i++)
+  for(std::map<QCString,toConnectionProvider *>::iterator i=Providers->begin();i!=Providers->end();i++)
     ret.insert(ret.end(),(*i).first);
   return ret;
 }
@@ -106,70 +124,75 @@ std::list<QString> toConnectionProvider::providers()
 void toConnectionProvider::initializeAll(void)
 {
   if (Types)
-    for(std::map<QString,toConnectionProvider *>::iterator i=Types->begin();
+    for(std::map<QCString,toConnectionProvider *>::iterator i=Types->begin();
 	i!=Types->end();i++)
       (*i).second->initialize();
 }
 
-std::list<QString> toConnectionProvider::providedModes(const QString &)
+std::list<QString> toConnectionProvider::providedModes(const QCString &)
 {
   std::list<QString> ret;
-  ret.insert(ret.end(),"Normal");
+  ret.insert(ret.end(),QString::fromLatin1("Normal"));
   return ret;
 }
 
-toConnectionProvider &toConnectionProvider::fetchProvider(const QString &provider)
+toConnectionProvider &toConnectionProvider::fetchProvider(const QCString &provider)
 {
   checkAlloc();
-  std::map<QString,toConnectionProvider *>::iterator i=Providers->find(provider);
+  std::map<QCString,toConnectionProvider *>::iterator i=Providers->find(provider);
   if (i==Providers->end())
-    throw QString("Tried to fetch unknown provider %1").arg(provider);
+    throw QT_TRANSLATE_NOOP("toConnectionProvider","Tried to fetch unknown provider %1").arg(provider);
   return *((*i).second);
 }
 
-std::list<QString> toConnectionProvider::modes(const QString &provider)
+std::list<QString> toConnectionProvider::modes(const QCString &provider)
 {
   return fetchProvider(provider).providedModes(provider);
 }
 
-QWidget *toConnectionProvider::configurationTab(const QString &provider,QWidget *parent)
+QWidget *toConnectionProvider::configurationTab(const QCString &provider,QWidget *parent)
 {
   return fetchProvider(provider).providerConfigurationTab(provider,parent);
 }
 
-toConnection::connectionImpl *toConnectionProvider::connection(const QString &provider,
+toConnection::connectionImpl *toConnectionProvider::connection(const QCString &provider,
 							       toConnection *conn)
 {
   return fetchProvider(provider).provideConnection(provider,conn);
 }
 
-std::list<QString> toConnectionProvider::hosts(const QString &provider)
+std::list<QString> toConnectionProvider::hosts(const QCString &provider)
 {
   return fetchProvider(provider).providedHosts(provider);
 }
 
-std::list<QString> toConnectionProvider::databases(const QString &provider,const QString &host,
+std::list<QString> toConnectionProvider::databases(const QCString &provider,const QString &host,
 						   const QString &user,const QString &pwd)
 {
   return fetchProvider(provider).providedDatabases(provider,host,user,pwd);
 }
 
-const QString &toConnectionProvider::config(const QString &tag,const QString &def)
+const QString &toConnectionProvider::config(const QCString &tag,const QCString &def)
 {
-  QString str=Provider;
+  QCString str=Provider;
   str.append(":");
   str.append(tag);
   return toTool::globalConfig(str,def);
 }
 
-void toConnectionProvider::setConfig(const QString &tag,const QString &def)
+void toConnectionProvider::setConfig(const QCString &tag,const QCString &def)
 {
-  QString str=Provider;
+  QCString str=Provider;
   str.append(":");
   str.append(tag);
   toTool::globalSetConfig(str,def);
 }
 
+QWidget *toConnectionProvider::providerConfigurationTab(const QCString &,QWidget *)
+{
+  return NULL;
+}
+
 // Query value implementation
 
 toQValue::toQValue(int i)
@@ -282,13 +305,30 @@ QCString toQValue::utf8Value(void) const
   case doubleType:
     {
       QCString ret;
-      ret.setNum(Value.Double);
+      if (Value.Double!=int(Value.Double)) {
+	ret.setNum(Value.Double);
+	return ret;
+      }
+      char buf[100];
+      switch(NumberFormat) {
+      default:
+	ret.setNum(Value.Double);
+	break;
+      case 1:
+	sprintf(buf,"%E",Value.Double);
+	ret=buf;
+	break;
+      case 2:
+	sprintf(buf,"%0.*f",NumberDecimals,Value.Double);
+	ret=buf;
+	break;
+      }
       return ret;
     }
   case stringType:
     return Value.String->utf8();
   }
-  throw QString("Unknown type of query value");
+  throw qApp->translate("toQValue","Unknown type of query value");
 }
 
 int toQValue::toInt(void) const
@@ -303,7 +343,7 @@ int toQValue::toInt(void) const
   case stringType:
     return Value.String->toInt();
   }
-  throw QString("Unknown type of query value");
+  throw qApp->translate("toQValue","Unknown type of query value");
 }
 
 double toQValue::toDouble(void) const
@@ -318,7 +358,45 @@ double toQValue::toDouble(void) const
   case stringType:
     return Value.String->toDouble();
   }
-  throw QString("Unknown type of query value");
+  throw qApp->translate("toQValue","Unknown type of query value");
+}
+
+void toQValue::setNumberFormat(int format,int decimals)
+{
+  NumberFormat=format;
+  NumberDecimals=decimals;
+}
+
+QString toQValue::formatNumber(double number)
+{
+  if (number==int(number))
+    return QString::number(number);
+  switch(NumberFormat) {
+  case 1:
+    {
+      char buf[100];
+      sprintf(buf,"%E",number);
+      return buf;
+    }
+  case 2:
+    {
+      char buf[100];
+      sprintf(buf,"%0.*f",NumberDecimals,number);
+      return buf;
+    }
+  default:
+    return QString::number(number);
+  }
+}
+
+int toQValue::numberFormat(void)
+{
+  return NumberFormat;
+}
+
+int toQValue::numberDecimals(void)
+{
+  return NumberDecimals;
 }
 
 toQValue::operator QString() const
@@ -326,14 +404,14 @@ toQValue::operator QString() const
   switch(Type) {
   case nullType:
     return QString::null;
+  case doubleType:
+    return formatNumber(Value.Double);
   case intType:
     return QString::number(Value.Int);
-  case doubleType:
-    return QString::number(Value.Double);
   case stringType:
     return *Value.String;
   }
-  throw QString("Unknown type of query value");
+  throw qApp->translate("toQValue","Unknown type of query value");
 }
 
 // toQuery implementation
@@ -399,6 +477,7 @@ toQuery::toQuery(toConnection &conn,const toSQL &sql,
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,const QString &sql,
@@ -462,6 +541,7 @@ toQuery::toQuery(toConnection &conn,const QString &sql,
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,const toSQL &sql,const toQList &params)
@@ -479,6 +559,7 @@ toQuery::toQuery(toConnection &conn,const toSQL &sql,const toQList &params)
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,const QString &sql,const toQList &params)
@@ -496,6 +577,7 @@ toQuery::toQuery(toConnection &conn,const QString &sql,const toQList &params)
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,queryMode mode,const toSQL &sql,const toQList &params)
@@ -529,6 +611,7 @@ toQuery::toQuery(toConnection &conn,queryMode mode,const toSQL &sql,const toQLis
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,queryMode mode,const QString &sql,const toQList &params)
@@ -562,6 +645,7 @@ toQuery::toQuery(toConnection &conn,queryMode mode,const QString &sql,const toQL
     Connection.freeConnection(ConnectionSub);
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 toQuery::toQuery(toConnection &conn,queryMode mode)
@@ -591,6 +675,7 @@ toQuery::toQuery(toConnection &conn,queryMode mode)
     Query=NULL;
     throw;
   }
+  ConnectionSub->setQuery(this);
 }
 
 void toQuery::execute(const toSQL &sql,const toQList &params)
@@ -613,7 +698,12 @@ toQuery::~toQuery()
 {
   toBusy busy;
   delete Query;
-  Connection.freeConnection(ConnectionSub);
+  try {
+    if (ConnectionSub->query()==this)
+      ConnectionSub->setQuery(NULL);
+    Connection.freeConnection(ConnectionSub);
+  } catch(...) {
+  }
 }
 
 bool toQuery::eof(void)
@@ -752,7 +842,7 @@ toQValue toQuery::readValue(void)
 {
   toBusy busy;
   if (Connection.Abort)
-    throw QString("Query aborted");
+    throw qApp->translate("toQuery","Query aborted");
   if (Mode==All)
     eof();
   return toNull(Query->readValue());
@@ -762,7 +852,7 @@ toQValue toQuery::readValueNull(void)
 {
   toBusy busy;
   if (Connection.Abort)
-    throw QString("Query aborted");
+    throw qApp->translate("toQuery","Query aborted");
   if (Mode==All)
     eof();
   return Query->readValue();
@@ -789,7 +879,7 @@ void toConnection::addConnection(void)
   }
 }
 
-toConnection::toConnection(const QString &provider,
+toConnection::toConnection(const QCString &provider,
 			   const QString &user,const QString &password,
 			   const QString &host,const QString &database,
 			   const QString &mode,bool cache)
@@ -805,8 +895,10 @@ toConnection::toConnection(const QString &provider,
   if (cache) {
     if (toTool::globalConfig(CONF_OBJECT_CACHE,"0").toInt()==1)
       readObjects();
-  } else
+  } else {
     ReadingValues.up();
+    ReadingValues.up();
+  }
 }
 
 toConnection::toConnection(const toConnection &conn)
@@ -823,10 +915,29 @@ toConnection::toConnection(const toConnection &conn)
   addConnection();
   Version=Connection->version(mainConnection());
   ReadingValues.up();
+  ReadingValues.up();
   ReadingCache=false;
   NeedCommit=Abort=false;
 }
 
+std::list<QString> toConnection::running(void)
+{
+  toBusy busy;
+  toLocker lock(Lock);
+  std::list<QString> ret;
+  toConnectionSub *sub=(*(Connections.begin()));
+  if (sub&&sub->query())
+    ret.insert(ret.end(),sub->query()->sql());
+  if (BackgroundConnection&&BackgroundConnection->query())
+    ret.insert(ret.end(),BackgroundConnection->query()->sql());
+  for(std::list<toConnectionSub *>::const_iterator i=Running.begin();i!=Running.end();i++) {
+    sub=*i;
+    if (sub&&sub->query())
+      ret.insert(ret.end(),sub->query()->sql());
+  }
+  return ret;
+}
+
 void toConnection::cancelAll(void)
 {
   toBusy busy;
@@ -838,23 +949,31 @@ void toConnection::cancelAll(void)
 toConnection::~toConnection()
 {
   toBusy busy;
+
   {
     toLocker lock(Lock);
     {
-      for (std::list<QWidget *>::iterator i=Widgets.begin();i!=Widgets.end();i=Widgets.begin()) {
+      for (std::list<QObject *>::iterator i=Widgets.begin();i!=Widgets.end();i=Widgets.begin()) {
 	delete (*i);
       }
     }
     {
       for(std::list<toConnectionSub *>::iterator i=Running.begin();i!=Running.end();i++)
-	(*i)->cancel();
+	try {
+	  (*i)->cancel();
+	} catch (...) {
+	}
     }
   }
   Abort=true;
   if (ReadingCache) {
     ReadingValues.down();
-    for(std::list<toConnectionSub *>::iterator i=Connections.begin();i!=Connections.end();i++) {
+    ReadingValues.down();
+  }
+  for(std::list<toConnectionSub *>::iterator i=Connections.begin();i!=Connections.end();i++) {
+    try {
       Connection->closeConnection(*i);
+    } catch (...) {
     }
   }
   delete Connection;
@@ -862,12 +981,18 @@ toConnection::~toConnection()
 
 toConnectionSub *toConnection::mainConnection()
 {
-  toLocker lock(Lock);
-  return (*(Connections.begin()));
+  if (Connection->handleMultipleQueries()) {
+    toLocker lock(Lock);
+    return (*(Connections.begin()));
+  } else {
+    return longConnection();
+  }
 }
 
 toConnectionSub *toConnection::backgroundConnection()
 {
+  if (!Connection->handleMultipleQueries())
+    return longConnection();
   if (toTool::globalConfig(CONF_BKGND_CONNECT,"").isEmpty())
     return mainConnection();
   Lock.lock();
@@ -877,8 +1002,9 @@ toConnectionSub *toConnection::backgroundConnection()
     Lock.lock();
   
     BackgroundConnection=tmp;
-    BackgroundCount++;
+    BackgroundCount=0;
   }
+  BackgroundCount++;
   Lock.unlock();
   return BackgroundConnection;
 }
@@ -886,14 +1012,17 @@ toConnectionSub *toConnection::backgroundConnection()
 toConnectionSub *toConnection::longConnection()
 {
   Lock.lock();
-  if (Connections.size()==1) {
+  bool multiple=Connection->handleMultipleQueries();
+  if ((multiple&&Connections.size()==1)||
+      (!multiple&&Connections.size()==0)) {
     Lock.unlock();
     addConnection();
   } else
     Lock.unlock();
   toLocker lock(Lock);
   std::list<toConnectionSub *>::iterator i=Connections.begin();
-  i++;
+  if (multiple)
+    i++;
   toConnectionSub *ret=(*i);
   Connections.erase(i);
   Running.insert(Running.end(),ret);
@@ -956,9 +1085,9 @@ void toConnection::rollback(void)
   setNeedCommit(false);
 }
 
-void toConnection::delWidget(QWidget *widget)
+void toConnection::delWidget(QObject *widget)
 {
-  for (std::list<QWidget *>::iterator i=Widgets.begin();i!=Widgets.end();i++) {
+  for (std::list<QObject *>::iterator i=Widgets.begin();i!=Widgets.end();i++) {
     if ((*i)==widget) {
       Widgets.erase(i);
       break;
@@ -968,12 +1097,16 @@ void toConnection::delWidget(QWidget *widget)
 
 bool toConnection::closeWidgets(void)
 {
-  for (std::list<QWidget *>::iterator i=Widgets.begin();i!=Widgets.end();i=Widgets.begin()) {
-    if (!(*i)->close(true))
-      return false;
-    std::list<QWidget *>::iterator nextI=Widgets.begin();
+  for (std::list<QObject *>::iterator i=Widgets.begin();i!=Widgets.end();i=Widgets.begin()) {
+    if ((*i)->inherits("QWidget")) {
+      QWidget *widget=static_cast<QWidget *>(*i);
+      if (!widget->close(true))
+	return false;
+    } else
+      delete *i;
+    std::list<QObject *>::iterator nextI=Widgets.begin();
     if (i==nextI)
-      throw QString("All tool widgets need to have autodelete flag set");
+      throw qApp->translate("toConnection","All tool widgets need to have autodelete flag set");
   }
   return true;
 }
@@ -981,18 +1114,18 @@ bool toConnection::closeWidgets(void)
 QString toConnection::description(bool version) const
 {
   QString ret(User);
-  ret+="@";
+  ret+=QString::fromLatin1("@");
   ret+=Database;
   if (!Host.isEmpty()&&Host!="SQL*Net") {
-    ret+=".";
+    ret+=QString::fromLatin1(".");
     ret+=Host;
   }
 
   if (version) {
     if (!Version.isEmpty()) {
-      ret+=" [";
-      ret+=Version;
-      ret+="]";
+      ret+=QString::fromLatin1(" [");
+      ret+=QString::fromLatin1(Version);
+      ret+=QString::fromLatin1("]");
     }
   }
   return ret;
@@ -1017,6 +1150,18 @@ void toConnection::delInit(const QString &sql)
   }
 }
 
+void toConnection::parse(const QString &sql)
+{
+  toBusy busy;
+  Connection->parse(mainConnection(),sql.utf8());
+}
+
+void toConnection::parse(const toSQL &sql)
+{
+  toBusy busy;
+  Connection->parse(mainConnection(),toSQL::sql(sql,*this));
+}
+
 void toConnection::execute(const toSQL &sql,toQList &params)
 {
   toBusy busy;
@@ -1273,19 +1418,149 @@ void toConnection::allExecute(const QString &sql,
   }
 }
 
-const QString &toConnection::provider(void) const
+const QCString &toConnection::provider(void) const
 {
   return Provider;
 }
 
+bool toConnection::loadDiskCache() 
+{
+   if (toTool::globalConfig(CONF_CACHE_DISK,"No") == "No")
+   	return false;
+
+   toConnection::objectName *cur;
+   int objCounter = 0;
+   int synCounter = 0;
+   
+   QString dbname (Database.stripWhiteSpace());
+   QString home=QDir::homeDirPath();
+   QString dirname = toTool::globalConfig(CONF_CACHE_DIR, "");
+
+   if (dirname == "")
+       dirname = QString(home+"/.tora_cache");
+
+   QString filename(dirname+"/"+dbname);
+   QFile file(filename.stripWhiteSpace());
+
+   if (!QFile::exists(filename.stripWhiteSpace())) 
+     return false;
+
+   QFileInfo fi(file);
+   QDateTime today;
+   if (fi.lastModified().addDays(toTool::globalConfig(CONF_CACHE_TIMEOUT,DEFAULT_CACHE_TIMEOUT).toInt())<today)
+     return false;
+  
+   /** read in all data
+    */
+
+   if (!file.open(IO_ReadOnly)) 
+     return false;
+
+   QString data = file.readAll();
+
+   /** build cache lists
+    */
+   
+   QStringList records = QStringList::split("\x1D",data,true);
+   for ( QStringList::Iterator i = records.begin(); i != records.end(); i++) {
+     objCounter++;
+     QStringList record = QStringList::split("\x1E",(*i),true);
+     QStringList::Iterator rec = record.begin();
+     cur = new objectName;
+     (*cur).Owner = (*rec);
+     rec++;
+     (*cur).Name =  (*rec);
+     rec++;
+     (*cur).Type =  (*rec);
+     rec++;
+     (*cur).Comment = (*rec);
+     rec++;
+     QStringList slist = QStringList::split("\x1F",(*rec),false);
+     for (QStringList::Iterator s = slist.begin(); s != slist.end(); s++) {
+       SynonymMap[(*s)] = (*cur);
+       (*cur).Synonyms.insert((*cur).Synonyms.end(),(*s));
+       synCounter++;
+     }
+     ObjectNames.insert(ObjectNames.end(),(*cur));
+   }
+   return true;
+}
+
+void toConnection::writeDiskCache() 
+{
+  QString text;
+  long objCounter = 0;
+  long synCounter = 0;
+
+  if (toTool::globalConfig(CONF_CACHE_DISK,"No") == "No")
+   	return;
+
+
+  QString dbname (Database.stripWhiteSpace());
+  QString home=QDir::homeDirPath();
+  QString dirname = toTool::globalConfig(CONF_CACHE_DIR, "");
+  if (dirname == "")
+    dirname = QString(home+"/.tora_cache");
+  QString filename(dirname+"/"+dbname);
+	
+  /** check pathnames and create
+   */
+
+  QDir dir;
+  dir.setPath(dirname);
+
+  if (!dir.exists(dirname)) 
+    dir.mkdir(dirname);
+  
+ 
+  /** build record to write out
+   */
+   
+  QStringList record;  
+  QStringList records;
+  QStringList recordSynonym;  
+  for(std::list<objectName>::iterator i=ObjectNames.begin();i!=ObjectNames.end();i++) {
+    record.clear();
+    record.append((*i).Owner);
+    record.append((*i).Name);
+    record.append((*i).Type);
+    record.append((*i).Comment);
+    for(std::list<QString>::iterator s=(*i).Synonyms.begin();s!=(*i).Synonyms.end();s++) {
+     recordSynonym.append((*s));
+     synCounter++;
+    }
+    record.append(recordSynonym.join("\x1F"));
+    recordSynonym.clear();
+    objCounter++;
+    records.append(record.join("\x1E"));
+  }
+  /** open file
+   */  
+  QFile file(filename.stripWhiteSpace());	
+  file.open( IO_ReadWrite  | IO_Truncate );
+  QTextStream t (&file);
+  t << records.join("\x1D");
+  file.flush();
+  file.close();
+}
+
 void toConnection::cacheObjects::run()
 {
+  bool diskloaded=false;
   try {
-    Connection.ObjectNames=Connection.Connection->objectNames();
+    diskloaded=Connection.loadDiskCache();
+    if (!diskloaded) {
+      Connection.ObjectNames=Connection.Connection->objectNames();
+    }	    
     Connection.ObjectNames.sort();
-    if (Connection.ObjectNames.size()>0)
+    Connection.ReadingValues.up();
+    if (!diskloaded) {
       Connection.SynonymMap=Connection.Connection->synonymMap(Connection.ObjectNames);
+      Connection.writeDiskCache();
+    }
   } catch(...) {
+    if (Connection.ReadingValues.getValue()==0)
+      Connection.ReadingValues.up();
   }
   Connection.ReadingValues.up();
 }
@@ -1293,6 +1568,11 @@ void toConnection::cacheObjects::run()
 
 void toConnection::readObjects(void)
 {
+  if (toTool::globalConfig(CONF_OBJECT_CACHE,"0").toInt()==3) {
+    ReadingCache=false;
+    return;
+  } 
+
   if (!ReadingCache) {
     ReadingCache=true;
     try {
@@ -1305,16 +1585,41 @@ void toConnection::readObjects(void)
 
 void toConnection::rereadCache(void)
 {
-  if(ReadingValues.getValue()==0&&ReadingCache) {
-    toStatusMessage("Not done caching objects, can not clear unread cache");
+
+  if (toTool::globalConfig(CONF_OBJECT_CACHE,"0").toInt()==3) {
+    ColumnCache.clear();
     return;
   }
+
+  if(ReadingValues.getValue() < 2 && ReadingCache) {
+    toStatusMessage(qApp->translate("toConnection",
+				    "Not done caching objects, can not clear unread cache"));
+    return;
+  }
+
+
   ReadingCache=false;
   while(ReadingValues.getValue()>0)
     ReadingValues.down();
+
   ObjectNames.clear();
   ColumnCache.clear();
   SynonymMap.clear();
+
+  /** delete cache file to force reload
+   */
+
+  QString home=QDir::homeDirPath();
+  QString dirname = toTool::globalConfig(CONF_CACHE_DIR, "");
+  QString dbname (Database.stripWhiteSpace());
+
+  if (dirname == "")
+    dirname = QString(home+"/.tora_cache");
+  QString filename(dirname+"/"+dbname);
+
+  if (QFile::exists(filename.stripWhiteSpace())) 
+    QFile::remove(filename.stripWhiteSpace());
+ 
   readObjects();
 }
 
@@ -1328,8 +1633,11 @@ QString toConnection::unQuote(const QString &name)
   return Connection->unQuote(name);
 }
 
-bool toConnection::cacheAvailable(bool block,bool need)
+bool toConnection::cacheAvailable(bool synonyms,bool block,bool need)
 {
+  if (toTool::globalConfig(CONF_OBJECT_CACHE,"0").toInt()==3)
+      return true;
+
   if (!ReadingCache) {
     if (!need)
       return true;
@@ -1338,30 +1646,37 @@ bool toConnection::cacheAvailable(bool block,bool need)
     readObjects();
     toMainWidget()->checkCaching();
   }
-  if (ReadingValues.getValue()==0) {
+  if (ReadingValues.getValue()==0||(ReadingValues.getValue()==1&&synonyms==true)) {
     if (block) {
       toBusy busy;
       if (toThread::mainThread()) {
-	QProgressDialog waiting("Waiting for object caching to be completed.\n"
-				"Canceling this dialog will probably leave some list of\n"
-				"database objects empty.",
-				"&Cancel",
+	QProgressDialog waiting(qApp->translate("toConnection",
+						"Waiting for object caching to be completed.\n"
+						"Canceling this dialog will probably leave some list of\n"
+						"database objects empty."),
+				qApp->translate("toConnection","Cancel"),
 				10,
 				toMainWidget(),
 				"progress",
 				true);
-	waiting.setCaption("Waiting for object cache");
+	waiting.setCaption(qApp->translate("toConnection","Waiting for object cache"));
 	int num=1;
+
+	int waitVal=(synonyms?2:1);
 	do {
 	  qApp->processEvents();
 	  toThread::msleep(100);
 	  waiting.setProgress((++num)%10);
 	  if (waiting.wasCancelled())
 	    return false;
-	} while(ReadingValues.getValue()<1);
+	} while(ReadingValues.getValue()<waitVal);
       }
 
       ReadingValues.down();
+      if (synonyms) {
+	ReadingValues.down();
+	ReadingValues.up();
+      }
       ReadingValues.up();
     } else
       return false;
@@ -1371,8 +1686,8 @@ bool toConnection::cacheAvailable(bool block,bool need)
 
 std::list<toConnection::objectName> &toConnection::objects(bool block)
 {
-  if (!cacheAvailable(block)) {
-    toStatusMessage("Not done caching objects",false,false);
+  if (!cacheAvailable(false,block)) {
+    toStatusMessage(qApp->translate("toConnection","Not done caching objects"),false,false);
     static std::list<objectName> ret;
     return ret;
   }
@@ -1382,8 +1697,8 @@ std::list<toConnection::objectName> &toConnection::objects(bool block)
 
 void toConnection::addIfNotExists(toConnection::objectName &obj)
 {
-  if (!cacheAvailable(false)) {
-    toStatusMessage("Not done caching objects",false,false);
+  if (!cacheAvailable(true,false)) {
+    toStatusMessage(qApp->translate("toConnection","Not done caching objects"),false,false);
     return;
   }
   std::list<toConnection::objectName>::iterator i=ObjectNames.begin();
@@ -1396,8 +1711,8 @@ void toConnection::addIfNotExists(toConnection::objectName &obj)
 
 std::map<QString,toConnection::objectName> &toConnection::synonyms(bool block)
 {
-  if (!cacheAvailable(block)) {
-    toStatusMessage("Not done caching objects",false,false);
+  if (!cacheAvailable(true,block)) {
+    toStatusMessage(qApp->translate("toConnection","Not done caching objects"),false,false);
     static std::map<QString,objectName> ret;
     return ret;
   }
@@ -1409,20 +1724,23 @@ const toConnection::objectName &toConnection::realName(const QString &object,
 						       QString &synonym,
 						       bool block)
 {
-  if (!cacheAvailable(block))
-    throw QString("Not done caching objects");
+  if (!cacheAvailable(true,block))
+    throw qApp->translate("toConnection","Not done caching objects");
 
   QString name;
   QString owner;
 
+  QString q=QString::fromLatin1("\"");
+  QString c=QString::fromLatin1(".");
+
   bool quote=false;
   for (unsigned int pos=0;pos<object.length();pos++) {
-    if (object.at(pos)=="\"") {
+    if (object.at(pos)==q) {
       quote=!quote;
     } else {
-      if (!quote&&object.at(pos)=='.') {
+      if (!quote&&object.at(pos)==c) {
 	owner=name;
-	name="";
+	name=QString::null;
       } else
 	name+=object.at(pos);
     }
@@ -1452,7 +1770,7 @@ const toConnection::objectName &toConnection::realName(const QString &object,
       return (*i).second;
     }
   }
-  throw QString("Object %1 not available for %2").arg(object).arg(user());
+  throw qApp->translate("toConnection","Object %1 not available for %2").arg(object).arg(user());
 }
 
 const toConnection::objectName &toConnection::realName(const QString &object,bool block)
@@ -1464,8 +1782,9 @@ const toConnection::objectName &toConnection::realName(const QString &object,boo
 toQDescList &toConnection::columns(const objectName &object,bool nocache)
 {
   std::map<objectName,toQDescList>::iterator i=ColumnCache.find(object);
-  if (i==ColumnCache.end()||nocache)
+  if (i==ColumnCache.end()||nocache) {
     ColumnCache[object]=Connection->columnDesc(object);
+  }
 
   return ColumnCache[object];
 }
@@ -1507,3 +1826,8 @@ toQDescList toConnection::connectionImpl::columnDesc(const objectName &)
   toQDescList ret;
   return ret;
 }
+
+void toConnection::connectionImpl::parse(toConnectionSub *,const QCString &)
+{
+  throw qApp->translate("toConnection","Parse only not implemented for this type of connection");
+}
