@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -47,6 +47,7 @@ class QWidget;
 class toConnection;
 class toConnectionProvider;
 class toSQL;
+class toQuery;
 
 /** This class is an abstract definition of an actual connection to a database.
  * Each @ref toConnection object can have one or more actual connections to the
@@ -56,15 +57,24 @@ class toSQL;
  */
 
 class toConnectionSub {
+  toQuery *Query;
 public:
   /** Create connection to database.
    */
   toConnectionSub()
-  { }
+  { Query=NULL; }
   /** Close connection.
    */
   virtual ~toConnectionSub()
   { }
+  /** Query current running on connection or NULL.
+   */
+  toQuery *query()
+  { return Query; }
+  /** Set query currently running on connection. NULL means none.
+   */
+  void setQuery(toQuery *query)
+  { Query=query; }
   /** Cancel anything running on this sub.
    */
   virtual void cancel(void)
@@ -397,14 +407,14 @@ typedef std::list<toQDescribe> toQDescList;
  * executed in their own connection to make sure the interface doesn't lock up for a long time.
  */
 class toConnection {
-  QString Provider;
+  QCString Provider;
   QString User;
   QString Password;
   QString Host;
   QString Database;
   QString Mode;
-  QString Version;
-  std::list<QWidget *> Widgets;
+  QCString Version;
+  std::list<QObject *> Widgets;
   std::list<QCString> InitStrings;
   toLock Lock;
   std::list<toConnectionSub *> Connections;
@@ -452,6 +462,9 @@ public:
     /** Comment about this object
      */
     QString Comment;
+    /** synonyms (used for faster disk caching...)
+    */
+    std::list <QString> Synonyms;
 
     bool operator < (const objectName &) const;
     bool operator == (const objectName &) const;
@@ -485,6 +498,12 @@ public:
      */
     virtual void rollback(toConnectionSub *) = 0;
 
+    /** If not true can never run more than one query per connection sub and TOra will
+     * work around this limitation by opening more connections if needed.
+     */
+    virtual bool handleMultipleQueries()
+    { return true; }
+
     /** Create a new connection to the database.
      */
     virtual toConnectionSub *createConnection(void) = 0;
@@ -494,7 +513,7 @@ public:
 
     /** Get the version of the database connected to.
      */
-    virtual QString version(toConnectionSub *) = 0;
+    virtual QCString version(toConnectionSub *) = 0;
 
     /** Return a string representation to address an object.
      * @param name The name to be quoted.
@@ -540,6 +559,12 @@ public:
      * @param params Parameters to pass to query.
      */
     virtual void execute(toConnectionSub *conn,const QCString &sql,toQList &params) = 0;
+    /** Parse a query on an actual connection and report any syntax problems encountered.
+     * Defaults to not implemented.
+     * @param conn Connection to execute on.
+     * @param sql SQL to parse
+     */
+    virtual void parse(toConnectionSub *conn,const QCString &sql);
   };
 
 private:
@@ -583,7 +608,7 @@ public:
    * @param mode Mode to connect to the database with.
    * @param cache Enable object cache for this connection.
    */
-  toConnection(const QString &provider,const QString &user,const QString &password,
+  toConnection(const QCString &provider,const QString &user,const QString &password,
 	       const QString &host,const QString &database,const QString &mode=QString::null,
 	       bool cache=true);
   /** Create a copy of a connection. Will not cache objects, so objects will never be available
@@ -607,6 +632,10 @@ public:
    */
   const QString &password() const
   { return Password; }
+  /** Change password of connection.
+   */
+  void setPassword(const QString &pwd)
+  { Password=pwd; }
   /** Get host of connection.
    */
   const QString &host() const
@@ -621,11 +650,11 @@ public:
   { return Mode; }
   /** Get version of connection.
    */
-  const QString &version() const
+  const QCString &version() const
   { return Version; }
   /** Get provider of connection.
    */
-  const QString &provider() const;
+  const QCString &provider() const;
 
   /** Get a description of this connection.
    * @version Include version in returned string.
@@ -652,6 +681,19 @@ public:
    */
   virtual void rollback(void);
 
+  /** Parse a query on an actual connection and report any syntax problems encountered.
+   * Defaults to not implemented.
+   * @param conn Connection to execute on.
+   * @param sql SQL to parse
+   */
+  void parse(const QString &sql);
+  /** Parse a query on an actual connection and report any syntax problems encountered.
+   * Defaults to not implemented.
+   * @param conn Connection to execute on.
+   * @param sql SQL to parse
+   */
+  void parse(const toSQL &sql);
+
   /** Execute a statement without caring about the result.
    * @param sql SQL to execute
    * @param params Parameters to pass to query.
@@ -719,13 +761,13 @@ public:
 		  const QString &arg9=QString::null);
 
   /**
-   * Add a widget that uses this connection. This is needed to ensure that all widgets
+   * Add a object that uses this connection. This is needed to ensure that all widgets
    * that make use of a connection are destroyed when the connection is closed. Usually
    * tool windows need to call this function.
    *
    * @param widget The widget to add to the connection.
    */
-  void addWidget(QWidget *widget)
+  void addWidget(QObject *widget)
   { Widgets.insert(Widgets.end(),widget); }
   /**
    * Remove a widget from this connection. Should be called by the destructor of the
@@ -734,7 +776,7 @@ public:
    * @see addWidget
    * @param widget Widget to remove from the widget list.
    */
-  void delWidget(QWidget *widget);
+  void delWidget(QObject *widget);
 
   /**
    * Add a statement to be run uppon making new connections.
@@ -802,16 +844,33 @@ public:
    */
   const objectName &realName(const QString &object,QString &synonym,bool block);
   /** Check if cache is available or not.
+   * @param synonyms If synonyms are needed or not.
    * @param block Block until cache is done.
    * @param true True if you need the cache, or just checking.
    * @return True if cache is available.
    */
-  bool cacheAvailable(bool block=false,bool need=true);
+  bool cacheAvailable(bool synonyms,bool block=false,bool need=true);
 
   /** Try to stop all running queries.
    */
   void cancelAll(void);
 
+  /** load disk cache
+   */
+
+  bool loadDiskCache(void);
+
+  /** write disk cache
+   */
+
+  void writeDiskCache(void);
+
+  /** Get a list of currently running SQL.
+   */
+  std::list<QString> running(void);
+
+  void logMessage(QString text);
+  
   friend class toQuery;
 };
 
@@ -819,28 +878,28 @@ public:
  */
 
 class toConnectionProvider {
-  static std::map<QString,toConnectionProvider *> *Providers;
-  static std::map<QString,toConnectionProvider *> *Types;
-  QString Provider;
+  static std::map<QCString,toConnectionProvider *> *Providers;
+  static std::map<QCString,toConnectionProvider *> *Types;
+  QCString Provider;
   static void checkAlloc(void);
 
   /** Get the provider object for a given provider name.
    * @param provider Name of provider.
    * @return Reference to provider object.
    */
-  static toConnectionProvider &fetchProvider(const QString &provider);
+  static toConnectionProvider &fetchProvider(const QCString &provider);
 protected:
   /** Add a provider to the list that this provider can handle.
    */
-  void addProvider(const QString &provider);
+  void addProvider(const QCString &provider);
   /** Remove a provider from the list that this provider can handle.
    */
-  void removeProvider(const QString &provider);
+  void removeProvider(const QCString &provider);
 public:
   /** Create a new provider with the specified name.
    * @param provider Name of the provider.
    */
-  toConnectionProvider(const QString &provider,bool add=true);
+  toConnectionProvider(const QCString &provider,bool add=true);
   /** Destructor.
    */
   virtual ~toConnectionProvider();
@@ -848,23 +907,23 @@ public:
   /** Create an implementation of a connection to this database.
    * @return A connection implementation created with new.
    */
-  virtual toConnection::connectionImpl *provideConnection(const QString &provider,
+  virtual toConnection::connectionImpl *provideConnection(const QCString &provider,
 							  toConnection *conn)=0;
   /** List the available modes for this database.
    * @return A list of modes that the connection implementation understands.
    */
-  virtual std::list<QString> providedModes(const QString &provider);
+  virtual std::list<QString> providedModes(const QCString &provider);
   /** List the available hosts this database provider knows about.
    * @return A list of hosts.
    */
-  virtual std::list<QString> providedHosts(const QString &provider);
+  virtual std::list<QString> providedHosts(const QCString &provider);
   /** List the available databases this provider knows about for a given host.
    * @param host Host to return connections for.
    * @param user That might be needed.
    * @param password That might be needed.
    * @return A list of databases available for a given host.
    */
-  virtual std::list<QString> providedDatabases(const QString &provider,
+  virtual std::list<QString> providedDatabases(const QCString &provider,
 					       const QString &host,
 					       const QString &user,
 					       const QString &pwd)=0;
@@ -881,26 +940,25 @@ public:
    * @return A pointer to the widget containing the setup tab for this tool or NULL of
    * no settings are available.
    */
-  virtual QWidget *providerConfigurationTab(const QString &provider,QWidget *parent)
-  { return NULL; }
+  virtual QWidget *providerConfigurationTab(const QCString &provider,QWidget *parent);
 
   /** Get a list of names for providers.
    */
-  static QWidget *configurationTab(const QString &provider,QWidget *parent);
+  static QWidget *configurationTab(const QCString &provider,QWidget *parent);
   /** Get a list of names for providers.
    */
-  static std::list<QString> providers();
+  static std::list<QCString> providers();
   /** Get a list of modes for a given provider.
    */
-  static std::list<QString> modes(const QString &provider);
+  static std::list<QString> modes(const QCString &provider);
   /** Implement a connection for a given provider.
    * @param provider Provider to implement.
    * @param conn Connection to create implementation for.
    */
-  static toConnection::connectionImpl *connection(const QString &provider,toConnection *conn);
+  static toConnection::connectionImpl *connection(const QCString &provider,toConnection *conn);
   /** Get a list of hosts this provider knows about.
    */
-  static std::list<QString> hosts(const QString &provider);
+  static std::list<QString> hosts(const QCString &provider);
   /** Get a list of databases for a given provider and host.
    * @param provider Provider to fetch databases for.
    * @param host Host to fetch databases for.
@@ -908,7 +966,7 @@ public:
    * @param password That might be needed.
    * @return List of known databases.
    */
-  static std::list<QString> databases(const QString &provider,const QString &host,
+  static std::list<QString> databases(const QCString &provider,const QString &host,
 				      const QString &user,const QString &pwd);
   /**
    * Get connection specific settings.
@@ -921,7 +979,7 @@ public:
    * @param tag The name of the configuration setting.
    * @param def Contents of this setting.
    */
-  const QString &config(const QString &tag,const QString &def);
+  const QString &config(const QCString &tag,const QCString &def);
   /**
    * Change connectionspecific setting. Depending on the implementation this can change the
    * contents on disk or not.
@@ -934,7 +992,7 @@ public:
    * @param tag The name of the configuration setting.
    * @param def Default value of the setting, if it is not available.
    */
-  void setConfig(const QString &tag,const QString &value);
+  void setConfig(const QCString &tag,const QCString &value);
 
   /** Call all initializers
    */
