@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -53,6 +53,7 @@
 
 #include "toeditextensions.moc"
 #include "toeditextensionsetupui.moc"
+#include "toeditextensiongotoui.moc"
 
 #include "icons/deindent.xpm"
 #include "icons/indent.xpm"
@@ -65,6 +66,9 @@ static int ObfuscateBlock;
 static int ObfuscateBuffer;
 static int ReverseSearch;
 static int IncrementalSearch;
+static int UpperCase;
+static int LowerCase;
+static int GotoLine;
 
 #define CONF_EXPAND_SPACES	"ExpandSpaces"
 #define CONF_COMMA_BEFORE	"CommaBefore"
@@ -90,14 +94,18 @@ void toEditExtensions::receivedFocus(toEditWidget *widget)
 
   bool enable=Current&&!Current->isReadOnly();
 
-  toMainWidget()->menuBar()->setItemEnabled(IndentIndex,enable);
-  toMainWidget()->menuBar()->setItemEnabled(DeindentIndex,enable);
-  toMainWidget()->menuBar()->setItemEnabled(AutoIndentBlock,enable);
-  toMainWidget()->menuBar()->setItemEnabled(ObfuscateBlock,enable);
-  toMainWidget()->menuBar()->setItemEnabled(AutoIndentBuffer,Current);
-  toMainWidget()->menuBar()->setItemEnabled(ObfuscateBuffer,Current);
-  toMainWidget()->menuBar()->setItemEnabled(IncrementalSearch,Current);
-  toMainWidget()->menuBar()->setItemEnabled(ReverseSearch,Current);
+  toMainWidget()->editMenu()->setItemEnabled(IndentIndex,enable);
+  toMainWidget()->editMenu()->setItemEnabled(DeindentIndex,enable);
+  toMainWidget()->editMenu()->setItemEnabled(AutoIndentBlock,enable);
+  toMainWidget()->editMenu()->setItemEnabled(ObfuscateBlock,enable);
+  toMainWidget()->editMenu()->setItemEnabled(AutoIndentBuffer,Current);
+  toMainWidget()->editMenu()->setItemEnabled(ObfuscateBuffer,Current);
+  toMainWidget()->editMenu()->setItemEnabled(IncrementalSearch,Current);
+  toMainWidget()->editMenu()->setItemEnabled(ReverseSearch,Current);
+  toMainWidget()->editMenu()->setItemEnabled(UpperCase,Current);
+  toMainWidget()->editMenu()->setItemEnabled(LowerCase,Current);
+  toMainWidget()->editMenu()->setItemEnabled(GotoLine,Current);
+
   if(IndentButton)
     IndentButton->setEnabled(enable);
   if(DeindentButton)
@@ -113,6 +121,15 @@ void toEditExtensions::lostFocus(toEditWidget *widget)
   }
 }
 
+void toEditExtensions::gotoLine()
+{
+  if (Current) {
+    toEditExtensionGoto dialog(Current);
+    if (dialog.exec())
+      dialog.gotoLine();
+  }
+}
+
 void toEditExtensions::intIndent(int delta)
 {
   int line1,col1,line2,col2;
@@ -157,21 +174,23 @@ void toEditExtensions::indentBlock(void)
 void toEditExtensions::autoIndentBlock(void)
 {
   if (Current) {
-    int line1,col1,line2,col2;
-    if (Current->getMarkedRegion(&line1,&col1,&line2,&col2)) {
-      QString t=Current->textLine(line1).mid(0,col1);
-      t+="a";
-      int chars=0;
-      QString ind=toSQLParse::indentString(toSQLParse::countIndent(t,chars));
-      QString mrk=Current->markedText();
-      QString res=toSQLParse::indent(ind+mrk);
-      t=Current->textLine(line2);
-      unsigned int l=res.length()-ind.length();
-      if (col2==int(t.length())&&t.length()>0) // Strip last newline if on last col of line
-	l--;
-      res=res.mid(ind.length(),l); // Strip indent.
-      Current->insert(res,true);
-    }
+    try {
+      int line1,col1,line2,col2;
+      if (Current->getMarkedRegion(&line1,&col1,&line2,&col2)) {
+	QString t=Current->textLine(line1).mid(0,col1);
+	t+=QString::fromLatin1("a");
+	int chars=0;
+	QString ind=toSQLParse::indentString(toSQLParse::countIndent(t,chars));
+	QString mrk=Current->markedText();
+	QString res=toSQLParse::indent(ind+mrk);
+	t=Current->textLine(line2);
+	unsigned int l=res.length()-ind.length();
+	if (col2==int(t.length())&&t.length()>0) // Strip last newline if on last col of line
+	  l--;
+	res=res.mid(ind.length(),l); // Strip indent.
+	Current->insert(res,true);
+      }
+    } TOCATCH
   }
 }
 
@@ -184,13 +203,33 @@ void toEditExtensions::autoIndentBuffer(void)
       pos++;
     }
     Current->selectAll();
-    Current->insert(toSQLParse::indent(text.mid(pos)));
+    try {
+      Current->insert(toSQLParse::indent(text.mid(pos)));
+    } TOCATCH
+  }
+}
+
+void toEditExtensions::upperCase(void)
+{
+  if (Current) {
+    QString text=Current->markedText().upper();
+    if (!text.isEmpty())
+      Current->insert(text,true);
+  }
+}
+
+void toEditExtensions::lowerCase(void)
+{
+  if (Current) {
+    QString text=Current->markedText().lower();
+    if (!text.isEmpty())
+      Current->insert(text,true);
   }
 }
 
 static int CountLine(const QString &str)
 {
-  int found=str.findRev("\n");
+  int found=str.findRev(QString::fromLatin1("\n"));
   if (found<0)
     return str.length();
   else
@@ -204,14 +243,14 @@ static void ObfuscateStat(toSQLParse::statement &stat,QString &ret)
       toIsIdent(ret.at(ret.length()-1))&&
       toIsIdent(stat.String.at(0))) {
     if (CountLine(ret)<60)
-      ret+=" ";
+      ret+=QString::fromLatin1(" ");
     else
-      ret+="\n";
+      ret+=QString::fromLatin1("\n");
   }
   ret+=stat.String;
   if (!stat.Comment.isEmpty()) {
     ret+=stat.Comment;
-    ret+="\n";
+    ret+=QString::fromLatin1("\n");
   }
   for(std::list<toSQLParse::statement>::iterator i=stat.subTokens().begin();
       i!=stat.subTokens().end();
@@ -276,23 +315,25 @@ public:
     CommentColumn->setValue(Current.CommentColumn);
     AutoIndent->setChecked(!toTool::globalConfig(CONF_AUTO_INDENT_RO,"Yes").isEmpty());
     Ok=false;
-    Example->setText(toSQLParse::indent("CREATE OR REPLACE procedure spTuxGetAccData (oRet OUT  NUMBER)\n"
-					"AS\n"
-					"  vYear  CHAR(4);\n"
-					"BEGIN\n"
-					"select a.TskCod TskCod, -- A Comment\n"
-					"       count(1) Tot\n"
-					"  from (select * from EssTsk where PrsID >= '1940');\n"
-					"having count(a.TspActOprID) > 0;\n"
-					"    DECLARE\n"
-					"      oTrdStt NUMBER;\n"
-					"    BEGIN\n"
-					"      oTrdStt := 0;\n"
-					"    END;\n"
-					"    EXCEPTION\n"
-					"        WHEN VALUE_ERROR THEN\n"
-					"	    oRet := 3;\n"
-					"END;"));
+    try {
+      Example->setText(toSQLParse::indent(QString::fromLatin1("CREATE OR REPLACE procedure spTuxGetAccData (oRet OUT  NUMBER)\n"
+							      "AS\n"
+							      "  vYear  CHAR(4);\n"
+							      "BEGIN\n"
+							      "select a.TskCod TskCod, -- A Comment\n"
+							      "       count(1) Tot\n"
+							      "  from (select * from EssTsk where PrsID >= '1940');\n"
+							      "having count(a.TspActOprID) > 0;\n"
+							      "    DECLARE\n"
+							      "      oTrdStt NUMBER;\n"
+							      "    BEGIN\n"
+							      "      oTrdStt := 0;\n"
+							      "    END;\n"
+							      "    EXCEPTION\n"
+							      "        WHEN VALUE_ERROR THEN\n"
+							      "	    oRet := 3;\n"
+							      "END;")));
+    } TOCATCH
     Started=true;
   }
   virtual ~toEditExtensionSetup()
@@ -317,7 +358,9 @@ public:
   {
     if (Started) {
       saveCurrent();
-      Example->setText(toSQLParse::indent(Example->text()));
+      try {
+	Example->setText(toSQLParse::indent(Example->text()));
+      } TOCATCH
     }
   }
   virtual void saveSetting(void);
@@ -354,11 +397,11 @@ public:
     cur.CommentColumn=config(CONF_COMMENT_COLUMN,DEFAULT_COMMENT_COLUMN).toInt();
     toSQLParse::setSetting(cur);
   }
-  virtual QWidget *toolWindow(QWidget *parent,toConnection &connection)
+  virtual QWidget *toolWindow(QWidget *,toConnection &)
   {
     return NULL; // Has no tool window
   }
-  virtual void customSetup(int toolid)
+  virtual void customSetup(int)
   {
     toMainWidget()->editMenu()->insertSeparator();
 
@@ -366,48 +409,59 @@ public:
 
     QPopupMenu *menu=new QPopupMenu(toMainWidget());
     
-    IncrementalSearch=menu->insertItem("Forward",&EditExtensions,SLOT(searchForward()),
+    IncrementalSearch=menu->insertItem(qApp->translate("toEditExtensionTool","Forward"),&EditExtensions,SLOT(searchForward()),
 				       CTRL+Key_S);
-    ReverseSearch=menu->insertItem("Backward",&EditExtensions,SLOT(searchBackward()),
+    ReverseSearch=menu->insertItem(qApp->translate("toEditExtensionTool","Backward"),&EditExtensions,SLOT(searchBackward()),
 				   CTRL+Key_R);
 
-    toMainWidget()->editMenu()->insertItem("Incremental Search",menu,-1,(idx>=0?idx+1:0));
+    toMainWidget()->editMenu()->insertItem(qApp->translate("toEditExtensionTool","Incremental Search"),menu,-1,(idx>=0?idx+1:0));
     
     menu=new QPopupMenu(toMainWidget());
-    AutoIndentBlock=menu->insertItem("Selection",
+    AutoIndentBlock=menu->insertItem(qApp->translate("toEditExtensionTool","Selection"),
 				     &EditExtensions,
 				     SLOT(autoIndentBlock()),
 				     ALT+CTRL+Key_I);
-    AutoIndentBuffer=menu->insertItem("Editor",
+    AutoIndentBuffer=menu->insertItem(qApp->translate("toEditExtensionTool","Editor"),
 				      &EditExtensions,
 				      SLOT(autoIndentBuffer()),
 				      ALT+CTRL+SHIFT+Key_I);
     menu->insertSeparator();
-    ObfuscateBlock=menu->insertItem("Obfuscate Selection",
+    ObfuscateBlock=menu->insertItem(qApp->translate("toEditExtensionTool","Obfuscate Selection"),
 				    &EditExtensions,
 				    SLOT(obfuscateBlock()));
-    ObfuscateBuffer=menu->insertItem("Obfuscate Editor",
+    ObfuscateBuffer=menu->insertItem(qApp->translate("toEditExtensionTool","Obfuscate Editor"),
 				     &EditExtensions,
 				     SLOT(obfuscateBuffer()));
-    toMainWidget()->editMenu()->insertItem("Auto Indent",menu);
+    toMainWidget()->editMenu()->insertItem(qApp->translate("toEditExtensionTool","Auto Indent"),menu);
+
+    menu=new QPopupMenu(toMainWidget());
+    UpperCase=menu->insertItem(qApp->translate("toEditExtensionTool","Upper"),
+			       &EditExtensions,
+			       SLOT(upperCase()));
+    LowerCase=menu->insertItem(qApp->translate("toEditExtensionTool","Lower"),
+			       &EditExtensions,
+			       SLOT(lowerCase()));
+    toMainWidget()->editMenu()->insertItem(qApp->translate("toEditExtensionTool","Modify Case"),menu);
 
     IndentIndex=toMainWidget()->editMenu()->insertItem(QPixmap((const char **)indent_xpm),
-						       "Indent Block",&EditExtensions,
+						       qApp->translate("toEditExtensionTool","Indent Block"),&EditExtensions,
 						       SLOT(indentBlock()),
 						       ALT+Key_Right);
     DeindentIndex=toMainWidget()->editMenu()->insertItem(QPixmap((const char **)deindent_xpm),
-							 "De-indent Block",&EditExtensions,
+							 qApp->translate("toEditExtensionTool","De-indent Block"),&EditExtensions,
 							 SLOT(deindentBlock()),
 							 ALT+Key_Left);
+    GotoLine=toMainWidget()->editMenu()->insertItem(qApp->translate("toEditExtensionTool","Goto Line"),&EditExtensions,
+						    SLOT(gotoLine()));
 
     IndentButton=new QToolButton(QPixmap((const char **)indent_xpm),
-				 "Indent block in editor",
-				 "Indent block in editor",
+				 qApp->translate("toEditExtensionTool","Indent block in editor"),
+				 qApp->translate("toEditExtensionTool","Indent block in editor"),
 				 &EditExtensions,SLOT(indentBlock()),
 				 toMainWidget()->editToolbar());
     DeindentButton=new QToolButton(QPixmap((const char **)deindent_xpm),
-				   "De-indent block in editor",
-				   "De-indent block in editor",
+				   qApp->translate("toEditExtensionTool","De-indent block in editor"),
+				   qApp->translate("toEditExtensionTool","De-indent block in editor"),
 				   &EditExtensions,SLOT(deindentBlock()),
 				   toMainWidget()->editToolbar());
 
@@ -436,3 +490,21 @@ void toEditExtensionSetup::saveSetting(void)
 }
 
 static toEditExtensionTool EditExtensionTool;
+
+toEditExtensionGoto::toEditExtensionGoto(toMarkedText *editor)
+  :toEditExtensionGotoUI(editor,"GotoLine",true),Editor(editor)
+{
+  toHelp::connectDialog(this);
+  Line->setMaxValue(Editor->numLines());
+  Line->setMinValue(1);
+  {
+    int curline,curcol;
+    Editor->getCursorPosition(&curline,&curcol);
+    Line->setValue(curline);
+  }
+}
+
+void toEditExtensionGoto::gotoLine()
+{
+  Editor->setCursorPosition(Line->value()-1,0,false);
+}
