@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -38,7 +38,9 @@
 #include <list>
 #include <map>
 
+#include <qfile.h>
 #include <qstring.h>
+#include <qtextstream.h>
 #include <qvariant.h>
 
 class QWidget;
@@ -96,11 +98,11 @@ public:
      * toExtract::setBlocksize).
      * @param ext Extractor to generate script.
      */
-    virtual void initialize(toExtract &ext) const
-    { }
+    virtual void initialize(toExtract &ext) const;
 
     /** Called to generate a script to recreate a database object.
      * @param ext Extractor to generate script.
+     * @param stream Stream to write script to.
      * @param type Type of object to recreate.
      * @param schema Specify the schema of the output script or description. If empty
      *               don't specify any object. If the string "1" use same object as input.
@@ -109,12 +111,12 @@ public:
      * @param name Name of database object.
      * @return A string containing a script to recreate an object.
      */
-    virtual QString create(toExtract &ext,
-			   const QString &type,
-			   const QString &schema,
-			   const QString &owner,
-			   const QString &name) const
-    { return QString::null; }
+    virtual void create(toExtract &ext,
+			QTextStream &stream,
+			const QString &type,
+			const QString &schema,
+			const QString &owner,
+			const QString &name) const;
     /** Called to describe a database object.
      * @param ext Extractor to generate script.
      * @param lst List of descriptions for the object. Should be appended.
@@ -130,8 +132,7 @@ public:
 			  const QString &type,
 			  const QString &schema,
 			  const QString &owner,
-			  const QString &name) const 
-    { }
+			  const QString &name) const;
     /** Called to generate a script to migrate a database object from one description to
      * another description.
      * @param ext Extractor to generate script.
@@ -140,11 +141,12 @@ public:
      * @param dst Destination description list.
      * @return A script to change the src database object to dst.
      */
-    virtual QString migrate(toExtract &ext,
-			    const QString &type,
-			    std::list<QString> &src,
-			    std::list<QString> &dst) const
-    { return QString::null; }
+    virtual void migrate(toExtract &ext,
+			 QTextStream &stream,
+			 const QString &type,
+			 std::list<QString> &src,
+			 std::list<QString> &dst) const;
+
     /** Called to generate a script to drop an object.
      * @param ext Extractor to generate script.
      * @param type Type of object to recreate.
@@ -155,12 +157,12 @@ public:
      * @param name Name of database object.
      * @return A string containing a script to recreate an object.
      */
-    virtual QString drop(toExtract &ext,
-			 const QString &type,
-			 const QString &schema,
-			 const QString &owner,
-			 const QString &name) const
-    { return QString::null; }
+    virtual void drop(toExtract &ext,
+		      QTextStream &stream,
+		      const QString &type,
+		      const QString &schema,
+		      const QString &owner,
+		      const QString &name) const;
   };
 
 private:
@@ -183,6 +185,8 @@ private:
   bool Storage;
   bool Initialized;
 
+  int CommitDistance;
+
   // Database info
   int BlockSize;
   std::list<QString> Initial;
@@ -190,7 +194,7 @@ private:
   std::list<QString> Limit;
 
   // Context, can be used by the extractor to save context
-  std::map<QString,QVariant> Context;
+  std::map<QCString,QVariant> Context;
 
   // Stuff to handle extractors
   static std::map<QString,extractor *> *Extractors;
@@ -239,7 +243,21 @@ public:
    *               TRIGGER, TYPE, USER, USER GRANTS for Oracle databases.
    * @return A string containing a script to recreate the specified objects.
    */
-  QString create(std::list<QString> &object);
+  QString create(std::list<QString> &object)
+  { QString ret; QTextStream s(&ret,IO_WriteOnly); create(s,object); return ret; }
+  /** Create script to recreate list of objects.
+   * @param stream Stream to write result to.
+   * @param object List of object. This has the format {type}:{schema}.{object}.
+   *               The type is database dependent but can as an example be of
+   *               CONSTRAINT, DATABASE LINK, EXCHANGE INDEX,
+   *               EXCHANGE TABLE, FUNCTION, INDEX, MATERIALIZED VIEW,
+   *               MATERIALIZED VIEW LOG, PACKAGE, PACKAGE BODY, PROCEDURE,
+   *               PROFILE, ROLE, ROLE GRANTS, ROLLBACK SEGMENT, SEQUENCE,
+   *               SNAPSHOT, SNAPSHOT LOG, SYNONYM, TABLE, TABLE FAMILY,
+   *               TABLE CONTENTS, TABLE REFERENCES, TABLESPACE, TRIGGER,
+   *               TRIGGER, TYPE, USER, USER GRANTS for Oracle databases.
+   */
+  void create(QTextStream &stream,std::list<QString> &object);
 
   /** Create a description of objects.
    * @param object List of object. This has the format {type}:{schema}.{object}.
@@ -269,26 +287,53 @@ public:
    *               TRIGGER, TYPE, USER, USER GRANTS for Oracle databases.
    * @return A string containing a script to drop the specified objects.
    */
-  QString drop(std::list<QString> &object);
+  QString drop(std::list<QString> &object)
+  { QString ret; QTextStream s(&ret,IO_WriteOnly); drop(s,object); return ret; }
+
+  /** Create script to drop a list of objects.
+   * @param stream Stream to write result to.
+   * @param object List of object. This has the format {type}:{schema}.{object}.
+   *               The type is database dependent but can as an example be of
+   *               CONSTRAINT, DATABASE LINK, EXCHANGE INDEX,
+   *               EXCHANGE TABLE, FUNCTION, INDEX, MATERIALIZED VIEW,
+   *               MATERIALIZED VIEW LOG, PACKAGE, PACKAGE BODY, PROCEDURE,
+   *               PROFILE, ROLE, ROLE GRANTS, ROLLBACK SEGMENT, SEQUENCE,
+   *               SNAPSHOT, SNAPSHOT LOG, SYNONYM, TABLE, TABLE FAMILY,
+   *               TABLE CONTENTS, TABLE REFERENCES, TABLESPACE, TRIGGER,
+   *               TRIGGER, TYPE, USER, USER GRANTS for Oracle databases.
+   * @return A string containing a script to drop the specified objects.
+   */
+  void drop(QTextStream &stream,std::list<QString> &object);
 
   /** Called to generate a script to migrate a database object from one description to
    * another description.
+   * @param stream Stream to write result to.
    * @param src Source description list, generated by describes for the same database.
    * @param dst Destination description list, generated by describes for the same database.
    * @return A script to change the src database object to dst.
    */
-  QString migrate(std::list<QString> &drpLst,std::list<QString> &crtLst);
+  QString migrate(std::list<QString> &drpLst,std::list<QString> &crtLst)
+  { QString ret; QTextStream s(&ret,IO_WriteOnly); migrate(s,drpLst,crtLst); return ret; }
+
+  /** Called to generate a script to migrate a database object from one description to
+   * another description.
+   * @param stream Stream to write result to.
+   * @param src Source description list, generated by describes for the same database.
+   * @param dst Destination description list, generated by describes for the same database.
+   * @return A script to change the src database object to dst.
+   */
+  void migrate(QTextStream &stream,std::list<QString> &drpLst,std::list<QString> &crtLst);
 
   /** Set a context for this extractor.
    * @param name Name of this context
    * @param val Value of this context
    */
-  void setState(const QString &name,const QVariant &val);
+  void setState(const QCString &name,const QVariant &val);
   /** Get the value of a context for the current extractor.
    * @param name Name of the context to extract.
    * @return The value of the context.
    */
-  QVariant state(const QString &name);
+  QVariant state(const QCString &name);
 
   /** Set the schema of the extraction.
    * @param schema Specify the schema of the output script or description. If empty
@@ -347,9 +392,10 @@ public:
   { Partition=val; }
   /** Include contents of tables in scripts.
    * @param val Include contents of tables.
+   * @param commitdistance The commit distance of the contents 0 means whole tables.
    */
-  void setContents(bool val)
-  { Contents=val; }
+  void setContents(bool val,int commitdistance)
+  { Contents=val; CommitDistance=commitdistance; }
   /** Include comments in extraction.
    * @param val Include indexes.
    */
@@ -423,6 +469,11 @@ public:
    */
   bool getContents(void)
   { return Contents; }
+  /** Get the distance of the commits when content is generated.
+   * @return Commit distance.
+   */
+  int getCommitDistance(void)
+  { return CommitDistance; }
   /** Check if comments are generated.
    * @return If comments are generated.
    */
@@ -491,8 +542,19 @@ public:
    */
   static std::list<QString> splitDescribe(const QString &str);
 
+  /** Get the part of a description indicated by the level
+   * @param str Description string
+   * @param level The level to get (0 is the first level)
+   */
   static QString partDescribe(const QString &str,int level);
 
+  /** Get the context of a given level.
+   * @param str Description string
+   * @param level The level to get the context for (1 is the first to not return empty string.
+   * @return The context, if not enough parts are available null is returned.
+   */
+  static QString contextDescribe(const QString &str,int level);
+
   friend class extractor;
 };
 
