@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -101,13 +101,13 @@ std::list<toSyntaxAnalyzer::highlightInfo> toSyntaxAnalyzer::analyzeLine(const Q
   }
 
   char c;
-  char nc=str[0];
+  char nc=str[0].latin1();
   for (int i=0;i<int(str.length());i++) {
     std::list<posibleHit>::iterator j=search.begin();
 
     c=nc;
     if (int(str.length())>i)
-      nc=str[i+1];
+      nc=str[i+1].latin1();
     else
       nc=' ';
 
@@ -207,7 +207,7 @@ bool toSyntaxAnalyzer::reservedWord(const QString &str)
   if(str.length()==0)
     return false;
   QString t=str.upper();
-  std::list<const char *> &curKey=Keywords[(unsigned char)char(str[0])];
+  std::list<const char *> &curKey=Keywords[(unsigned char)char(str[0].latin1())];
   for(std::list<const char *>::iterator i=curKey.begin();i!=curKey.end();i++)
     if (t==(*i))
       return true;
@@ -275,6 +275,10 @@ void toHighlightedText::setStatusMessage(void)
     toStatusMessage((*err).second,true);
 }
 
+#if QT_VERSION >= 0x030000 && QT_VERSION < 0x030200
+#  define FORCE_BREAK 20
+#endif
+
 void toHighlightedText::paintCell(QPainter *painter,int row,int col)
 {
   if (!Highlight) {
@@ -282,6 +286,8 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
     return;
   }
 
+  painter->setBackgroundMode(OpaqueMode);
+
   QString str=textLine(row);
 
   int line1,col1,line2,col2;
@@ -301,12 +307,14 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
 	LineInput.erase(LineInput.find(i));
       else
 	LineInput[i]=out;
+#if 0
       if (i<=lastRowVisible()) {
 	painter->translate(0,cellHeight(i-1));
 	painter->setClipping(false);
         paintCell(painter,i,0);
 	break;
       }
+#endif
       Analyzer->analyzeLine(textLine(i),out,out);
       i++;
     }
@@ -343,9 +351,9 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
 
     QString c;
 
-#if QT_VERSION >= 300
+#ifdef FORCE_BREAK
     int lastcol=0;
-    int MAXCOL=20;
+    int MAXCOL=FORCE_BREAK;
 #endif
 
     for (int i=0;i<=int(str.length())&&posx<width;i++) {
@@ -382,11 +390,11 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
       }
 
       if (wasMarked!=marked||col!=wasCol||str[i]=='\t'||(curline==row&&curcol==i)
-#if QT_VERSION >= 300
-	  ||(lastcol+MAXCOL>i)
+#ifdef FORCE_BREAK
+	  ||(lastcol+MAXCOL<i)
 #endif
 	  ) {
-#if QT_VERSION >= 300
+#ifdef FORCE_BREAK
 	lastcol=i;
 #endif
 	QChar nc;
@@ -396,29 +404,31 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
 	}
 
 	if (c.length()>0) {
-	  rect=painter->boundingRect(0,0,width,height,AlignLeft|AlignTop,c);
-	  int left=posx;
-	  int cw=rect.right()+1;
-	  if (i==int(str.length()))
-	    cw+=hMargin()-1;
-	  if (i==int(c.length())) {
-	    cw+=left;
-	    left=LeftIgnore;
-	  }
 	  if (wasMarked) {
-	    if (Completion&&Completion->count()==0) {
-	      painter->setBrush(Analyzer->getColor(toSyntaxAnalyzer::ErrorBkg));
-	      painter->fillRect(left,0,cw,height,painter->brush());
-	      painter->setBrush(cp.active().highlight());
-	    } else
-	      painter->fillRect(left,0,cw,height,painter->brush());
+	    if (Completion&&Completion->count()==0)
+	      painter->setBackgroundColor(Analyzer->getColor(toSyntaxAnalyzer::ErrorBkg));
+	    else
+	      painter->setBackgroundColor(cp.active().highlight());
 	    painter->setPen(cp.active().highlightedText());
 	  } else {
 	    painter->setPen(wasCol);
-	    painter->fillRect(left,0,cw,height,bkg);
+	    painter->setBackgroundColor(bkg);
 	  }
 
 	  painter->drawText(posx,0,width-posx,height,AlignLeft|AlignTop,c,c.length(),&rect);
+
+	  if (i==int(c.length()))
+	    painter->fillRect(LeftIgnore,0,posx-LeftIgnore,height,painter->backgroundColor());
+
+#if 1 && defined (WIN32)
+	  rect.setWidth(rect.width()-2);
+#endif
+#if 0 && defined (WIN32)
+	  rect.setHeight(rect.height()-1);
+#endif
+	  if (rect.height()<=height)
+	    painter->fillRect(posx,rect.height(),rect.width(),height-rect.height(),painter->backgroundColor());
+
 	  posx=rect.right()+1;
 	}
 	wasCol=col;
@@ -429,7 +439,10 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
 	cursorx=posx;
 
       if (c=="\t") {
-	int tab=painter->fontMetrics().width("xxxxxxxx");;
+	QString t;
+	for(int i=0;i<defaultTabStop();i++)
+	  t+="x";
+	int tab=painter->fontMetrics().width(t);;
 	int nx=((posx-hMargin()+1)/tab+1)*tab+hMargin()-1;
 	int left=(posx==hMargin()-1?LeftIgnore:posx);
 	painter->fillRect(left,0,nx-left,height,marked?painter->brush():bkg);
@@ -442,6 +455,7 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
   } else
     painter->fillRect(LeftIgnore,0,width-LeftIgnore,height,bkg);
   painter->setPen(cp.active().text());
+  painter->setBackgroundColor(Analyzer->getColor(toSyntaxAnalyzer::NormalBkg));
 
   if (hasFocus()) {
     if (row==curline) {
@@ -455,7 +469,7 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
     }
 
     if (row==curline) {
-      bool check;
+      bool check=false;
       if (LastRow!=curline||LastCol!=curcol) {
 	Cursor=0;
 	if (Completion&&!KeepCompletion) {
@@ -466,8 +480,19 @@ void toHighlightedText::paintCell(QPainter *painter,int row,int col)
       }
       if (Cursor<2) {
 	if (!isReadOnly()) {
-	  painter->drawLine(cursorx-1,0,cursorx-1,
-			    painter->fontMetrics().ascent()+painter->fontMetrics().descent());
+	  painter->drawLine(
+#if 1 && defined (WIN32)
+			    cursorx+1,
+#else
+			    cursorx-1,
+#endif
+			    0,
+#if 1 && defined (WIN32)
+			    cursorx+1,
+#else
+			    cursorx-1,
+#endif
+			    painter->fontMetrics().height());
 	}
 	if (Completion&&!KeepCompletion) {
 	  int x=cursorPoint().x()-xOffset();
@@ -645,47 +670,52 @@ void toHighlightedText::checkComplete(void)
     if (toTool::globalConfig(CONF_CODE_COMPLETION,"Yes").isEmpty())
       return;
 
-    QString name=toSQLParse::getToken(this,curline,curcol,false);
+    toSQLParse::editorTokenizer tokens(this,curcol,curline);
+    QString name=tokens.getToken(false);
     QString owner;
     if (name==".")
-      name=toSQLParse::getToken(this,curline,curcol,false);
+      name=tokens.getToken(false);
 
-    QString token=toSQLParse::getToken(this,curline,curcol,false);
+    QString token=tokens.getToken(false);
     if (token==".")
-      owner=toSQLParse::getToken(this,curline,curcol,false);
+      owner=tokens.getToken(false);
     else {
       QString cmp=UpperIdent(name);
-      while ((invalidToken(curline,curcol+token.length())||UpperIdent(token)!=cmp)&&
+      QString lastToken;
+      while ((invalidToken(tokens.line(),tokens.offset()+token.length())||UpperIdent(token)!=cmp||lastToken==".")&&
 	     token!=";"&&!token.isEmpty()) {
-	token=toSQLParse::getToken(this,curline,curcol,false);
+	lastToken=token;
+	token=tokens.getToken(false);
       }
 
       if (token==";"||token.isEmpty()) {
-	getCursorPosition (&curline,&curcol);
-	token=toSQLParse::getToken(this,curline,curcol);
-	while ((invalidToken(curline,curcol)||UpperIdent(token)!=cmp)&&
+	tokens.setLine(curline);
+	tokens.setOffset(curcol);
+	token=tokens.getToken();
+	while ((invalidToken(tokens.line(),tokens.offset())||UpperIdent(token)!=cmp&&lastToken!=".")&&
 	       token!=";"&&!token.isEmpty())
-	  token=toSQLParse::getToken(this,curline,curcol);
-	toSQLParse::getToken(this,curline,curcol,false);
+	  token=tokens.getToken();
+	lastToken=token;
+	tokens.getToken(false);
       }
       if (token!=";"&&!token.isEmpty()) {
-	token=toSQLParse::getToken(this,curline,curcol,false);
+	token=tokens.getToken(false);
 	if (token!="TABLE"&&
 	    token!="UPDATE"&&
 	    token!="FROM"&&
 	    token!="INTO"&&
 	    (toIsIdent(token[0])||token[0]=='\"')) {
 	  name=token;
-	  token=toSQLParse::getToken(this,curline,curcol,false);
+	  token=tokens.getToken(false);
 	  if (token==".")
-	    owner=toSQLParse::getToken(this,curline,curcol,false);
+	    owner=tokens.getToken(false);
 	} else if (token==")") {
 	  return;
 	}
       }
     }
     if (!owner.isEmpty()) {
-      name=owner+"."+name;
+      name=owner+QString::fromLatin1(".")+name;
     }
     if (!name.isEmpty()) {
       try {
@@ -694,9 +724,16 @@ void toHighlightedText::checkComplete(void)
 	std::list<QString> complete;
 	for (toQDescList::iterator i=desc.begin();
 	     i!=desc.end();i++) {
-	  QString t=conn.quote((*i).Name);
+	  QString t;
+	  int ind=(*i).Name.find("(");
+	  if (ind <0)
+	    ind=(*i).Name.find("RETURNING")-1;//it could be a function or procedure without parameters. -1 to remove the space
+	  if (ind>=0)
+	    t=conn.quote((*i).Name.mid(0,ind))+(*i).Name.mid(ind);
+	  else
+	    t=conn.quote((*i).Name);
 	  if (!(*i).Comment.isEmpty()) {
-	    t+=" - ";
+	    t+=QString::fromLatin1(" - ");
 	    t+=(*i).Comment;
 	  }
 	  complete.insert(complete.end(),t);
@@ -783,7 +820,7 @@ void toHighlightedText::keyPressEvent(QKeyEvent *e)
 	e->accept();
 	return;
       }
-    } else if (e->key()==Key_Return&&e->state()==0&&CompleteItem>=0) {
+    } else if ((e->key()==Key_Return||e->key()==Key_Enter)&&e->state()==0&&CompleteItem>=0) {
       if (CompleteItem>=0) {
 	QString tmp=Completion->text(CompleteItem);
 	unsigned int pos=0;
@@ -862,7 +899,7 @@ void toHighlightedText::keyPressEvent(QKeyEvent *e)
 	  if ((*i).second.Type=="TABLE"||(*i).second.Type=="VIEW") {
 	    QString add=conn.quote((*i).first);
 	    if(!(*i).second.Comment.isEmpty()) {
-	      add+=" - ";
+	      add+=QString::fromLatin1(" - ");
 	      add+=(*i).second.Comment;
 	    }
 	    complete.insert(complete.end(),add);
@@ -877,7 +914,7 @@ void toHighlightedText::keyPressEvent(QKeyEvent *e)
 	      if (synonyms.find((*i).Owner)==synonyms.end()) {
 		QString add=conn.quote((*i).Name);
 		if(!(*i).Comment.isEmpty()) {
-		  add+=" - ";
+		  add+=QString::fromLatin1(" - ");
 		  add+=(*i).Comment;
 		}
 		complete.insert(complete.end(),add);
@@ -886,10 +923,10 @@ void toHighlightedText::keyPressEvent(QKeyEvent *e)
 	  } else {
 	    QString add=conn.quote((*i).Owner);
 	    if (!add.isEmpty())
-	      add+=".";
+	      add+=QString::fromLatin1(".");
 	    add+=conn.quote((*i).Name);
 	    if(!(*i).Comment.isEmpty()) {
-	      add+=" - ";
+	      add+=QString::fromLatin1(" - ");
 	      add+=(*i).Comment;
 	    }
 	    complete.insert(complete.end(),add);
@@ -948,6 +985,7 @@ void toHighlightedText::focusOutEvent(QFocusEvent *e)
 {
   delete Completion;
   Completion=NULL;
+  toMarkedText::focusOutEvent(e);
 }
 
 toHighlightedText::~toHighlightedText()
@@ -970,44 +1008,43 @@ void toHighlightedText::tableAtCursor(QString &owner,QString &table,bool mark)
     getCursorPosition (&curline,&curcol);
 
     QString token=textLine(curline);
+    toSQLParse::editorTokenizer tokens(this,curcol,curline);
     if (curcol>0&&toIsIdent(token[curcol-1]))
-      token=toSQLParse::getToken(this,curline,curcol,false);
+      token=tokens.getToken(false);
     else
       token=QString::null;
 
-    int lastline=curline;
-    int lastcol=curcol;
-
-    token=toSQLParse::getToken(this,curline,curcol,false);
+    toSQLParse::editorTokenizer lastTokens(this,tokens.offset(),tokens.line());
+    token=tokens.getToken(false);
     if (token==".") {
-      lastline=curline;
-      lastcol=curcol;
-      owner=conn.unQuote(toSQLParse::getToken(this,curline,curcol,false));
-      toSQLParse::getToken(this,lastline,lastcol,true);
-      table+=conn.unQuote(toSQLParse::getToken(this,lastline,lastcol,true));
+      lastTokens.setLine(tokens.line());
+      lastTokens.setOffset(tokens.offset());
+      owner=conn.unQuote(tokens.getToken(false));
+      lastTokens.getToken(true);
+      table+=conn.unQuote(lastTokens.getToken(true));
     } else {
-      curline=lastline;
-      curcol=lastcol;
-      owner=conn.unQuote(toSQLParse::getToken(this,lastline,lastcol,true));
-      int tmplastline=lastline;
-      int tmplastcol=lastcol;
-      token=toSQLParse::getToken(this,lastline,lastcol,true);
+      tokens.setLine(lastTokens.line());
+      tokens.setOffset(lastTokens.offset());
+      owner=conn.unQuote(lastTokens.getToken(true));
+      int tmplastline=lastTokens.line();
+      int tmplastcol=lastTokens.offset();
+      token=lastTokens.getToken(true);
       if (token==".")
-	table=conn.unQuote(toSQLParse::getToken(this,lastline,lastcol,true));
+	table=conn.unQuote(lastTokens.getToken(true));
       else {
-	lastline=tmplastline;
-	lastcol=tmplastcol;
+	lastTokens.setLine(tmplastline);
+	lastTokens.setOffset(tmplastcol);
 	table=owner;
 	owner=QString::null;
       }
     }
     if (mark) {
-      setCursorPosition(curline,curcol,false);
-      if (lastline>=numLines()) {
-	lastline=numLines()-1;
-	lastcol=textLine(lastline).length();
+      setCursorPosition(tokens.line(),tokens.offset(),false);
+      if (lastTokens.line()>=numLines()) {
+	lastTokens.setLine(numLines()-1);
+	lastTokens.setOffset(textLine(numLines()-1).length());
       }
-      setCursorPosition(lastline,lastcol,true);
+      setCursorPosition(lastTokens.line(),lastTokens.offset(),true);
     }
   } catch(...) {
   }
