@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -53,6 +53,7 @@
 
 #include <qcheckbox.h>
 #include <qfile.h>
+#include <qinputdialog.h>
 #include <qlineedit.h>
 #include <qpushbutton.h>
 #include <qregexp.h>
@@ -85,7 +86,7 @@ static toSQL SQLMembers("toOracleConnection:Members",
 			"  FROM sys.All_Arguments\n"
 			" WHERE Owner = :f1<char[100]>\n"
 			"   AND Package_Name = :f2<char[100]>\n"
-			" ORDER BY object_name,overload,sequence",
+			" ORDER BY object_name,overload,DECODE(argument_name,NULL,9999,sequence)",
 			"Get list of package members");
 
 static toSQL SQLListObjects("toOracleConnection:ListObjects",
@@ -106,20 +107,31 @@ static toSQL SQLListSynonyms("toOracleConnection:ListSynonyms",
 
 static void ThrowException(const otl_exception &exc)
 {
-  toConnection::exception ret=QString::fromUtf8((const char *)exc.msg);
-#if 1
-  if (exc.stm_text&&strlen(exc.stm_text)) {
-    ret+="\n";
-    QString sql=QString::fromUtf8((const char *)exc.stm_text);
-    if (exc.errorofs>=0) {
-      QString t=QString::fromUtf8((const char *)exc.stm_text,exc.errorofs);
-      ret.setOffset(t.length());
-      sql.insert(t.length(),"<ERROR>");
+  if (exc.code==24344)
+    throw toConnection::exception(QString::fromLatin1("ORA-24344 success with compilation error"));
+  else {
+    toConnection::exception ret(QString::fromUtf8((const char *)exc.msg));
+
+    if (ret.isEmpty()) {
+      if (exc.code!=0)
+	ret=QString::fromLatin1("ORA-")+QString::number(exc.code)+QString::fromLatin1(" missing error description");
+      else if (ret.isEmpty())
+	ret=QString::fromLatin1("Missing error description, could occur if you have several ORACLE_HOME and the binary directory\n"
+				"of the one which is not active is before the active ORACLE_HOME in your path");
     }
-    ret+=sql;
+
+    if (exc.stm_text&&strlen(exc.stm_text)) {
+      ret+=QString::fromLatin1("\n");
+      QString sql=QString::fromUtf8((const char *)exc.stm_text);
+      if (exc.errorofs>=0) {
+	QString t=QString::fromUtf8((const char *)exc.stm_text,exc.errorofs);
+	ret.setOffset(t.length());
+	sql.insert(t.length(),QString::fromLatin1("<ERROR>"));
+      }
+      ret+=sql;
+    }
+    throw ret;
   }
-#endif
-  throw ret;
 }
 
 class toOracleProvider : public toConnectionProvider {
@@ -146,21 +158,56 @@ public:
     { toThread *thread=new toThread(new connectionDeleter(Connection)); thread->start(); }
     virtual void cancel(void)
     { Connection->cancel(); }
+    virtual void throwExtendedException(toConnection &conn,const otl_exception &exc)
+    {
+      if (conn.version()<"8.0"&&exc.errorofs==0) {
+	// Serious OCI voodoo to get the Parse error location on Oracle7 servers
+
+	Lda_Def lda;
+	Cda_Def cda;
+	if (OCISvcCtxToLda(Connection->connect_struct.svchp,
+			   Connection->connect_struct.errhp,
+			   &lda)==OCI_SUCCESS) {
+	  if (oopen(&cda,
+		    &lda,
+		    (OraText *)0,-1,
+		    -1,
+		    (OraText *)0,-1)==OCI_SUCCESS) {
+	    cda.peo=0;
+	    oparse(&cda,(OraText *)exc.stm_text,-1,FALSE,1);
+	    ((otl_exception &)exc).errorofs=cda.peo;
+	    oclose(&cda);
+	  }
+	  OCILdaToSvcCtx(&Connection->connect_struct.svchp,
+			 Connection->connect_struct.errhp,
+			 &lda);
+	}
+      }
+      ThrowException(exc);
+    }
   };
 
   class oracleQuery : public toQuery::queryImpl {
     bool Cancel;
     bool Running;
+    bool SaveInPool;
     otl_stream *Query;
   public:
-    oracleQuery(toQuery *query,oracleSub *conn)
+    oracleQuery(toQuery *query,oracleSub *)
       : toQuery::queryImpl(query)
     {
       Running=Cancel=false;
+      SaveInPool=false;
       Query=NULL;
     }
     virtual ~oracleQuery()
-    { delete Query; }
+    {
+#ifdef OTL_STREAM_POOLING_ON
+      if (!SaveInPool&&Query)
+	Query->close(false);
+#endif
+      delete Query;
+    }
     virtual void execute(void);
 
     virtual toQValue readValue(void)
@@ -168,15 +215,16 @@ public:
       char *buffer=NULL;
       otl_var_desc *dsc=Query->describe_next_out_var();
       if (!dsc)
-	throw QString("Couldn't get description of next column to read");
+	throw QString::fromLatin1("Couldn't get description of next column to read");
 
       oracleSub *conn=dynamic_cast<oracleSub *>(query()->connectionSub());
       if (!conn)
-	throw QString("Internal error, not oracle sub connection");
+	throw QString::fromLatin1("Internal error, not oracle sub connection");
       conn->Lock.down();
       if (Cancel)
-	throw QString("Cancelled while waiting to read value");
+	throw QString::fromLatin1("Cancelled while waiting to read value");
       Running=true;
+      SaveInPool=true;
       try {
 	toQValue null;
 	switch (dsc->ftype) {
@@ -221,7 +269,7 @@ public:
 	    if (!str.len())
 	      return null;
 	    QString buf(QString::fromUtf8(buffer));
-	    delete buffer;
+	    delete[] buffer;
 	    return buf;
 	  }
 	case otl_var_clob:
@@ -247,16 +295,27 @@ public:
 	    lob>>data;
 	    if (!lob.eof()) {
 	      otl_long_string sink(10000);
-	      while(!lob.eof())
+	      do {
 		lob>>sink;
+	      } while(!lob.eof()&&sink.len()>0);
 	      if (toThread::mainThread())
-		toStatusMessage("Data exists past length of LOB");
+		toStatusMessage(QString::fromLatin1("Data exists past length of LOB"));
 	      else
 		printf("Data exists past length of LOB in thread\n");
 	    }
 	    buffer[data.len()]=0;
-	    QString buf(QString::fromUtf8(buffer));
-	    delete buffer;
+	    QString buf;
+	    if (dsc->ftype==otl_var_clob)
+	      buf=QString::fromUtf8(buffer);
+	    else {
+	      buf.fill('0',data.len()*2);
+	      QString t;
+	      for(int i=0;i<data.len();i++) {
+		t.sprintf("%02x",((unsigned int)buffer[i])%0xff);
+		buf.replace(i*2,2,t);
+	      }
+	    }
+	    delete[] buffer;
 	    Running=false;
 	    conn->Lock.up();
 	    return buf;
@@ -273,11 +332,11 @@ public:
 	    Running=false;
 	    conn->Lock.up();
 	    if (Query->is_null()) {
-	      delete buffer;
+	      delete[] buffer;
 	      return null;
 	    }
 	    QString buf(QString::fromUtf8(buffer));
-	    delete buffer;
+	    delete[] buffer;
 	    return buf;
 	  }
 	  break;
@@ -285,12 +344,12 @@ public:
       } catch (const otl_exception &exc) {
 	Running=false;
 	conn->Lock.up();
-	delete buffer;
-	ThrowException(exc);
+	delete[] buffer;
+	conn->throwExtendedException(query()->connection(),exc);
       } catch (...) {
 	Running=false;
 	conn->Lock.up();
-	delete buffer;
+	delete[] buffer;
 	throw;
       }
       // Never get here
@@ -331,7 +390,7 @@ public:
 	case 5:
 	case 9:
 	case 155:
-	  desc.Datatype="VARCHAR2";
+	  desc.Datatype=QString::fromLatin1("VARCHAR2");
 	  break;
 	case 2:
 	case 3:
@@ -339,63 +398,63 @@ public:
 	case 6:
 	case 68:
 	  desc.AlignRight=true;
-	  desc.Datatype="NUMBER";
+	  desc.Datatype=QString::fromLatin1("NUMBER");
 	  break;
 	case 8:
 	case 94:
 	case 95:
-	  desc.Datatype="LONG";
+	  desc.Datatype=QString::fromLatin1("LONG");
 	  break;
 	case 11:
 	case 104:
-	  desc.Datatype="ROWID";
+	  desc.Datatype=QString::fromLatin1("ROWID");
 	  break;
 	case 12:
 	case 156:
 	  desc.AlignRight=true;
-	  desc.Datatype="DATE";
+	  desc.Datatype=QString::fromLatin1("DATE");
 	  break;
 	case 15:
 	case 23:
 	case 24:
-	  desc.Datatype="RAW";
+	  desc.Datatype=QString::fromLatin1("RAW");
 	  break;
 	case 96:
 	case 97:
-	  desc.Datatype="CHAR";
+	  desc.Datatype=QString::fromLatin1("CHAR");
 	  break;
 	case 108:
-	  desc.Datatype="NAMED DATA TYPE";
+	  desc.Datatype=QString::fromLatin1("NAMED DATA TYPE");
 	  break;
 	case 110:
-	  desc.Datatype="REF";
+	  desc.Datatype=QString::fromLatin1("REF");
 	  break;
 	case 112:
-	  desc.Datatype="CLOB";
+	  desc.Datatype=QString::fromLatin1("CLOB");
 	  break;
 	case 113:
 	case 114:
-	  desc.Datatype="BLOB";
+	  desc.Datatype=QString::fromLatin1("BLOB");
 	  break;
 	default:
-	  desc.Datatype="UNKNOWN";
+	  desc.Datatype=QString::fromLatin1("UNKNOWN");
           break;
 	}
 
-	if (desc.Datatype=="NUMBER") {
+	if (desc.Datatype==QString::fromLatin1("NUMBER")) {
 	  if (description[i].prec) {
-	    desc.Datatype.append(" (");
+	    desc.Datatype.append(QString::fromLatin1(" ("));
 	    desc.Datatype.append(QString::number(description[i].prec));
 	    if (description[i].scale!=0) {
-	      desc.Datatype.append(",");
+	      desc.Datatype.append(QString::fromLatin1(","));
 	      desc.Datatype.append(QString::number(description[i].scale));
 	    }
-	    desc.Datatype.append(")");
+	    desc.Datatype.append(QString::fromLatin1(")"));
 	  }
 	} else {
-	  desc.Datatype.append(" (");
+	  desc.Datatype.append(QString::fromLatin1(" ("));
 	  desc.Datatype.append(QString::number(description[i].dbsize));
-	  desc.Datatype.append(")");
+	  desc.Datatype.append(QString::fromLatin1(")"));
 	}
 	desc.Null=description[i].nullok;
 
@@ -410,10 +469,10 @@ public:
     {
       QCString ret;
       ret=connection().user().utf8();
-      ret+="/";
+      ret+=QString::fromLatin1("/");
       ret+=connection().password().utf8();
       if (!connection().host().isEmpty()) {
-	ret+="@";
+	ret+=QString::fromLatin1("@");
 	ret+=connection().database().utf8();
       }
       return ret;
@@ -422,7 +481,7 @@ public:
     {
       oracleSub *conn=dynamic_cast<oracleSub *>(sub);
       if (!conn)
-	throw QString("Internal error, not oracle sub connection");
+	throw QString::fromLatin1("Internal error, not oracle sub connection");
       return conn;
     }
   public:
@@ -436,14 +495,19 @@ public:
      */
     virtual QString quote(const QString &name)
     {
-      if (name.upper()==name)
+      bool ok=true;
+      for(unsigned int i=0;i<name.length();i++) {
+	if (name.at(i).upper()!=name.at(i)||!toIsIdent(name.at(i)))
+	  ok=false;
+      }
+      if (ok)
 	return name.lower();
       else
-	return "\""+name+"\"";
+	return QString::fromLatin1("\"")+name+QString::fromLatin1("\"");
     }
     virtual QString unQuote(const QString &str)
     {
-      if (str.at(0)=='\"'&&str.at(str.length()-1)=='\"')
+      if (str.at(0).latin1()=='\"'&&str.at(str.length()-1).latin1()=='\"')
 	return str.left(str.length()-1).right(str.length()-2);
       return str.upper();
     }
@@ -471,7 +535,7 @@ public:
       std::map<QString,toConnection::objectName> ret;
 
       toConnection::objectName cur;
-      cur.Type="A";
+      cur.Type=QString::fromLatin1("A");
       std::list<toQValue> par;
       par.insert(par.end(),toQValue(connection().user().upper()));
       toQuery synonyms(connection(),toQuery::Long,
@@ -485,8 +549,10 @@ public:
 	  i++;
 	if (i==objects.end())
 	  break;
-	if (cur.Name==(*i).Name&&cur.Owner==(*i).Owner)
+	if (cur.Name==(*i).Name&&cur.Owner==(*i).Owner) {
 	  ret[synonym]=(*i);
+          (*i).Synonyms.insert((*i).Synonyms.end(),synonym);
+        }
       }
 
       return ret;
@@ -494,34 +560,47 @@ public:
     virtual toQDescList columnDesc(const toConnection::objectName &table)
     {
       toBusy busy;
-      if(table.Type=="PACKAGE") {
+      if(table.Type==QString::fromLatin1("PACKAGE")) {
 	toQDescList ret;
 	try {
 	  toQuery::queryDescribe desc;
-	  desc.Datatype="MEMBER";
+	  desc.Datatype=("MEMBER");
 	  desc.Null=false;
 	  QString lastName;
 	  QString lastOver;
 	  toQuery member(connection(),SQLMembers,table.Owner,table.Name);
+	  bool hasArgs=false;
 	  while(!member.eof()) {
 	    QString name = member.readValue();
 	    QString overld = member.readValue();
 	    QString arg = member.readValueNull();
 	    QString type = member.readValueNull();
 	    if (lastName!=name||overld!=lastOver) {
-	      if (desc.Name.contains("("))
+	      if (hasArgs)
 		desc.Name+=")";
 	      if (!desc.Name.isEmpty())
 		ret.insert(ret.end(),desc);
 	      desc.Name=name;
 	      lastName=name;
 	      lastOver=overld;
-	      if (!arg.isEmpty())
-		desc.Name+=" (";
-	    } else
-	      desc.Name+=", ";
-	    desc.Name+=arg;
-	    desc.Name+=" ";
+	      hasArgs=false;
+	    }
+	    if (arg.isEmpty()) {
+	      if (hasArgs) {
+		desc.Name+=")";
+		hasArgs=false;
+	      }
+	      desc.Name+=" RETURNING ";
+	    } else {
+	      if (hasArgs)
+		desc.Name+=", ";
+	      else {
+		desc.Name+="(";
+		hasArgs=true;
+	      }
+	      desc.Name+=arg;
+	      desc.Name+=" ";
+	    }
 	    desc.Name+=type;
 	  }
 	  if (desc.Name.contains("("))
@@ -544,12 +623,22 @@ public:
       }
 
       try {
-	QString SQL="SELECT * FROM \"";
+	toQuery query(connection());
+#ifdef OTL_STREAM_POOLING_ON
+	// Need to clear the stream cache first.
+	oracleSub *sub=dynamic_cast<oracleSub *>(query.connectionSub());
+	sub->Lock.down();
+	sub->Connection->set_stream_pool_size(max(toTool::globalConfig(CONF_OPEN_CURSORS,
+								       DEFAULT_OPEN_CURSORS).toInt(),1));
+	sub->Lock.up();
+#endif
+	QString SQL=QString::fromLatin1("SELECT * FROM \"");
 	SQL+=table.Owner;
-	SQL+="\".\"";
+	SQL+=QString::fromLatin1("\".\"");
 	SQL+=table.Name;
-	SQL+="\" WHERE NULL=NULL";
-	toQuery query(connection(),SQL);
+	SQL+=QString::fromLatin1("\" WHERE NULL=NULL");
+	toQList par;
+	query.execute(SQL,par);
 	toQDescList desc=query.describe();
 	for(toQDescList::iterator j=desc.begin();j!=desc.end();j++)
 	  (*j).Comment=comments[(*j).Name];
@@ -588,15 +677,15 @@ public:
       delete conn;
     }
 
-    virtual QString version(toConnectionSub *sub)
+    virtual QCString version(toConnectionSub *sub)
     {
       oracleSub *conn=oracleConv(sub);
       try {
 	otl_stream version(1,
 			   "SELECT banner FROM v$version",
 			   *(conn->Connection));
-	QRegExp verre("[0-9]\\.[0-9\\.]+[0-9]");
-	QRegExp orare("^oracle",false);
+	QRegExp verre(QString::fromLatin1("[0-9]\\.[0-9\\.]+[0-9]"));
+	QRegExp orare(QString::fromLatin1("^(\\S+ )?oracle"),false);
 	while(!version.eof()) {
 	  char buffer[1024];
 	  version>>buffer;
@@ -606,13 +695,13 @@ public:
 	    int len;
 	    pos=verre.match(ver,0,&len);
 	    if (pos>=0)
-	      return ver.mid(pos,len);
+	      return ver.mid(pos,len).latin1();
 	  }
 	}
       } catch (...) {
 	// Ignore any errors here
       }
-      return QString::null;
+      return "";
     }
 
     virtual toQuery::queryImpl *createQuery(toQuery *query,toConnectionSub *sub)
@@ -625,39 +714,53 @@ public:
 	try {
 	  otl_cursor::direct_exec(*(conn->Connection),sql);
 	} catch (const otl_exception &exc) {
-	  ThrowException(exc);
+	  conn->throwExtendedException(connection(),exc);
 	}
       } else
-	toQuery query(connection(),sql,params);
+	toQuery query(connection(),QString::fromUtf8(sql),params);
+    }
+    virtual void parse(toConnectionSub *sub,const QCString &sql)
+    {
+      oracleSub *conn=oracleConv(sub);
+      try {
+	otl_cursor::parse(*(conn->Connection),sql);
+      } catch (const otl_exception &exc) {
+	ThrowException(exc);
+      }
     }
   };
 
   toOracleProvider(void)
-    : toConnectionProvider("Oracle")
+    : toConnectionProvider("Oracle",false)
   {
-    toMaxLong=toTool::globalConfig(CONF_MAX_LONG,
-				   QString::number(DEFAULT_MAX_LONG)).toInt();
-    otl_connect::otl_initialize(1);
   }
 
-  virtual toConnection::connectionImpl *provideConnection(const QString &,toConnection *conn)
+  virtual void initialize(void)
+  {
+    toMaxLong=toTool::globalConfig(CONF_MAX_LONG,
+				   QString::number(DEFAULT_MAX_LONG).latin1()).toInt();
+    if (otl_connect::otl_initialize(1))
+      addProvider("Oracle");
+  }
+
+  virtual toConnection::connectionImpl *provideConnection(const QCString &,toConnection *conn)
   { return new oracleConnection(conn); }
-  virtual std::list<QString> providedModes(const QString &)
+  virtual std::list<QString> providedModes(const QCString &)
   {
     std::list<QString> ret;
-    ret.insert(ret.end(),"Normal");
-    ret.insert(ret.end(),"SYS_OPER");
-    ret.insert(ret.end(),"SYS_DBA");
+    ret.insert(ret.end(),QString::fromLatin1("Normal"));
+    ret.insert(ret.end(),QString::fromLatin1("SYS_OPER"));
+    ret.insert(ret.end(),QString::fromLatin1("SYS_DBA"));
     return ret;
   }
-  virtual std::list<QString> providedHosts(const QString &)
+  virtual std::list<QString> providedHosts(const QCString &)
   {
     std::list<QString> ret;
     ret.insert(ret.end(),QString::null);
-    ret.insert(ret.end(),"SQL*Net");
+    ret.insert(ret.end(),QString::fromLatin1("SQL*Net"));
     return ret;
   }
-  virtual std::list<QString> providedDatabases(const QString &,const QString &host,const QString &,const QString &)
+  virtual std::list<QString> providedDatabases(const QCString &,const QString &host,const QString &,const QString &)
   {
     QString str;
 #ifdef WIN32
@@ -691,15 +794,15 @@ public:
     }
 #else
     if (!getenv("ORACLE_HOME"))
-      throw QString("ORACLE_HOME environment variable not set");
+      throw QString::fromLatin1("ORACLE_HOME environment variable not set");
     if (getenv("TNS_ADMIN")) {
       str=getenv("TNS_ADMIN");
     } else {
       str=getenv("ORACLE_HOME");
-      str.append("/network/admin");
+      str.append(QString::fromLatin1("/network/admin"));
     }
 #endif
-    str.append("/tnsnames.ora");
+    str.append(QString::fromLatin1("/tnsnames.ora"));
 
 
     std::list<QString> ret;
@@ -738,8 +841,8 @@ public:
       } else if (buf[pos]==')') {
 	if (parambeg>=0&&host.isEmpty()) {
 	  QString tmp=QString::fromLatin1(buf+parambeg,pos-parambeg);
-	  tmp.replace(QRegExp("\\s+"),"");
-	  if (tmp.lower().startsWith("sid="))
+	  tmp.replace(QRegExp(QString::fromLatin1("\\s+")),QString::null);
+	  if (tmp.lower().startsWith(QString::fromLatin1("sid=")))
 	    ret.insert(ret.end(),tmp.mid(4));
 	}
 	begname=-1;
@@ -753,7 +856,7 @@ public:
     delete[] buf;
     return ret;
   }
-  virtual QWidget *providerConfigurationTab(const QString &provider,QWidget *parent);
+  virtual QWidget *providerConfigurationTab(const QCString &provider,QWidget *parent);
 };
 
 static toOracleProvider OracleProvider;
@@ -762,26 +865,33 @@ void toOracleProvider::oracleQuery::execute(void)
 {
   oracleSub *conn=dynamic_cast<oracleSub *>(query()->connectionSub());
   if (!conn)
-    throw QString("Internal error, not oracle sub connection");
+    throw QString::fromLatin1("Internal error, not oracle sub connection");
   try {
     delete Query;
     Query=NULL;
 
     while (conn->Lock.getValue()>1) {
       conn->Lock.down();
-      toStatusMessage("Too high value on connection lock semaphore");
+      toStatusMessage(QString::fromLatin1("Too high value on connection lock semaphore"));
     }
 
     conn->Lock.down();
     if (Cancel)
-      throw QString("Query aborted before started");
+      throw QString::fromLatin1("Query aborted before started");
     Running=true;
     try {
+      static QRegExp stripnl("\r");
       Query=new otl_stream;
       Query->set_commit(0);
-      Query->set_all_column_types(otl_all_num2str|otl_all_date2str);
+      if (toQValue::numberFormat()==0)
+	Query->set_all_column_types(otl_all_num2str|otl_all_date2str);
+      else
+	Query->set_all_column_types(otl_all_date2str);
+
+      QCString sql=query()->sql();
+      sql.replace(stripnl,"");
       Query->open(1,
-		  query()->sql(),
+		  sql,
 		  *(conn->Connection));
     } catch(...) {
       conn->Lock.up();
@@ -789,7 +899,7 @@ void toOracleProvider::oracleQuery::execute(void)
     }
   } catch (const otl_exception &exc) {
     Running=false;
-    ThrowException(exc);
+    conn->throwExtendedException(query()->connection(),exc);
   }
   try {
     otl_null null;
@@ -798,20 +908,24 @@ void toOracleProvider::oracleQuery::execute(void)
 	(*Query)<<null;
       else {
 	otl_var_desc *next=Query->describe_next_in_var();
-	switch(next->ftype) {
-	case otl_var_double:
-	case otl_var_float:
-	  (*Query)<<(*i).toDouble();
-	  break;
-	case otl_var_int:
-	case otl_var_unsigned_int:
-	case otl_var_short:
-	case otl_var_long_int:
-	  (*Query)<<(*i).toInt();
-	  break;
-	default:
-	  (*Query)<<QString(*i).utf8();
-	  break;
+	if (next) {
+	  switch(next->ftype) {
+	  case otl_var_double:
+	  case otl_var_float:
+	    (*Query)<<(*i).toDouble();
+	    break;
+	  case otl_var_int:
+	  case otl_var_unsigned_int:
+	  case otl_var_short:
+	  case otl_var_long_int:
+	    (*Query)<<(*i).toInt();
+	    break;
+	  default:
+	    (*Query)<<QString(*i).utf8();
+	    break;
+	  }
+	} else {
+	  printf("ERROR: More parameters than binds\n");
 	}
       }
     }
@@ -820,7 +934,7 @@ void toOracleProvider::oracleQuery::execute(void)
   } catch (const otl_exception &exc) {
     Running=false;
     conn->Lock.up();
-    ThrowException(exc);
+    conn->throwExtendedException(query()->connection(),exc);
   }
 }
 
@@ -828,7 +942,7 @@ void toOracleProvider::oracleQuery::cancel(void)
 {
   oracleSub *conn=dynamic_cast<oracleSub *>(query()->connectionSub());
   if (!conn)
-    throw QString("Internal error, not oracle sub connection");
+    throw QString::fromLatin1("Internal error, not oracle sub connection");
   if (Running)
     conn->Connection->cancel();
   else {
@@ -849,20 +963,59 @@ toConnectionSub *toOracleProvider::oracleConnection::createConnection(void)
   try {
     QString mode=connection().mode();
     int session_mode=OCI_DEFAULT;
-    if (mode=="SYS_OPER")
+    if (mode==QString::fromLatin1("SYS_OPER"))
       session_mode=OCI_SYSOPER;
-    else if (mode=="SYS_DBA")
+    else if (mode==QString::fromLatin1("SYS_DBA"))
       session_mode=OCI_SYSDBA;
-    conn=new otl_connect;
-    conn->set_stream_pool_size(max(toTool::globalConfig(CONF_OPEN_CURSORS,
-							DEFAULT_OPEN_CURSORS).toInt(),1));
-    if(!sqlNet)
-      conn->server_attach();
-    else
-      conn->server_attach(connection().database().utf8());
-    QCString user=connection().user().utf8();
-    QCString pass=connection().password().utf8();
-    conn->session_begin(user.isEmpty()?"":(const char *)user,pass.isEmpty()?"":(const char *)pass,0,session_mode);
+    do {
+      conn=new otl_connect;
+#ifdef OTL_STREAM_POOLING_ON
+      conn->set_stream_pool_size(max(toTool::globalConfig(CONF_OPEN_CURSORS,
+							  DEFAULT_OPEN_CURSORS).toInt(),1));
+#endif
+      if(!sqlNet)
+	conn->server_attach();
+      else
+	conn->server_attach(connection().database().utf8());
+      QCString user=connection().user().utf8();
+      QCString pass=connection().password().utf8();
+      try {
+	conn->session_begin(user.isEmpty()?"":(const char *)user,pass.isEmpty()?"":(const char *)pass,0,session_mode);
+      } catch(const otl_exception &exc) {
+	if (toThread::mainThread()&&exc.code==28001) {
+	  bool ok=false;
+	  QString newpass=QInputDialog::getText(qApp->translate("toOracleConnection","Password expired"),
+						qApp->translate("toOracleConnection","Enter new password"),
+						QLineEdit::Password,
+						QString::null,
+						&ok,
+						toMainWidget());
+	  if (ok) {
+	    QString newpass2=QInputDialog::getText(qApp->translate("toOracleConnection","Password expired"),
+						   qApp->translate("toOracleConnection","Enter password again for confirmation"),
+						   QLineEdit::Password,
+						   QString::null,
+						   &ok,
+						   toMainWidget());
+	    if (ok) {
+	      if (newpass2!=newpass)
+		throw qApp->translate("toOracleConnection","The two passwords doesn't match");
+	      QCString nputf=newpass.utf8();
+	      conn->change_password(user.isEmpty()?"":(const char *)user,
+				    pass.isEmpty()?"":(const char *)pass,
+				    newpass.isEmpty()?"":(const char *)nputf);
+	      connection().setPassword(newpass);
+	      delete conn;
+	      conn=NULL;
+	    } else
+	      throw exc;
+	  } else {
+	    throw exc;
+	  }
+	} else
+	  throw exc;
+      }
+    } while(!conn);
   } catch (const otl_exception &exc) {
     if (!sqlNet) {
       if (oldSid.isNull())
@@ -883,20 +1036,25 @@ toConnectionSub *toOracleProvider::oracleConnection::createConnection(void)
   
   try {
     {
-      QString str="ALTER SESSION SET NLS_DATE_FORMAT = '";
+      QString str=QString::fromLatin1("ALTER SESSION SET NLS_DATE_FORMAT = '");
       str+=toTool::globalConfig(CONF_DATE_FORMAT,DEFAULT_DATE_FORMAT);
-      str+="'";
+      str+=QString::fromLatin1("'");
       otl_stream date(1,str.utf8(),*conn);
     }
     {
       otl_stream info(1,
 		      "BEGIN\n"
-		      "  SYS.DBMS_APPLICATION_INFO.SET_CLIENT_INFO('TOra (http://www.globecom.se/tora)');\n"
+		      "  SYS.DBMS_APPLICATION_INFO.SET_CLIENT_INFO('" TOAPPNAME
+#ifndef OAS
+		      " (http://www.globecom.se/tora)"
+#endif
+		      "');\n"
+		      "  SYS.DBMS_APPLICATION_INFO.SET_MODULE('" TOAPPNAME "','Access Database');\n"
 		      "END;",
 		      *conn);
     }
   } catch(...) {
-    toStatusMessage("Failed to set new default date format for session");
+    toStatusMessage(QString::fromLatin1("Failed to set new default date format for session"));
   }
   return new oracleSub(conn);
 }
@@ -946,7 +1104,7 @@ public:
 					       DEFAULT_OPEN_CURSORS).toInt());
     KeepPlans->setChecked(!toTool::globalConfig(CONF_KEEP_PLANS,"").isEmpty());
     int len=toTool::globalConfig(CONF_MAX_LONG,
-				 QString::number(DEFAULT_MAX_LONG)).toInt();
+				 QString::number(DEFAULT_MAX_LONG).latin1()).toInt();
     if (len>=0) {
       MaxLong->setText(QString::number(len));
       MaxLong->setValidator(new QIntValidator(MaxLong));
@@ -968,7 +1126,7 @@ public:
     toTool::globalSetConfig(CONF_OPEN_CURSORS,QString::number(OpenCursors->value()));
     if (Unlimited->isChecked()) {
       toMaxLong=-1;
-      toTool::globalSetConfig(CONF_MAX_LONG,"-1");
+      toTool::globalSetConfig(CONF_MAX_LONG,QString::fromLatin1("-1"));
     } else {
       toTool::globalSetConfig(CONF_MAX_LONG,MaxLong->text());
       toMaxLong=MaxLong->text().toInt();
@@ -984,7 +1142,7 @@ public:
   }
 };
 
-QWidget *toOracleProvider::providerConfigurationTab(const QString &,QWidget *parent)
+QWidget *toOracleProvider::providerConfigurationTab(const QCString &,QWidget *parent)
 {
   return new toOracleSetting(parent);
 }
