@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -68,6 +68,7 @@
 
 #include "icons/addrecord.xpm"
 #include "icons/canceledit.xpm"
+#include "icons/duplicaterecord.xpm"
 #include "icons/filter.xpm"
 #include "icons/forward.xpm"
 #include "icons/next.xpm"
@@ -78,6 +79,9 @@
 #include "icons/single.xpm"
 #include "icons/trash.xpm"
 
+std::map<QCString,QString> toResultContentEditor::Criteria;
+std::map<QCString,QString> toResultContentEditor::Order;
+
 toResultContentEditor *toResultContentMemo::contentEditor()
 {
   return dynamic_cast<toResultContentEditor *>(parent());
@@ -166,15 +170,19 @@ toResultContentEditor::contentItem::contentItem(QTable *table,const QString &tex
 
 QString toResultContentEditor::contentItem::key(void) const
 {
-  static QRegExp number("^\\d*\\.?\\d+E?-?\\d*.?.?$");
+  static QRegExp number(QString::fromLatin1("^\\d*\\.?\\d+E?-?\\d*.?.?$"));
 
-  QString val=text();
-  if (number.match(val)>=0) {
-    static char buf[100];
-    sprintf(buf,"%015.5f",text().toFloat());
-    return buf;
+  QString txt=text();
+  if (number.match(txt)>=0) {
+    QString ret;
+    double val=txt.toFloat();
+    if(val<0)
+      ret.sprintf("\x01%015.5f",val);
+    else
+      ret.sprintf("%015.5f",val);
+    return ret;
   }
-  return val;
+  return txt;
 }
 
 void toResultContentEditor::setText(int row,int col,const QString &text)
@@ -186,11 +194,6 @@ void toResultContentEditor::setText(int row,int col,const QString &text)
     setItem(row,col,new contentItem(this,text));
 }
 
-void toResultContentEditor::editSearch(toSearchReplace *search)
-{
-  search->setTarget(this);
-}
-
 void toResultContentEditor::setCurrentCellFocus(int row, int col)
 {
   QTable::setCurrentCell(row,col);
@@ -275,7 +278,10 @@ toResultContentEditor::toResultContentEditor(QWidget *parent,const char *name)
 		 false,false,false,
 		 true,true,true)
 {
+  CurrentEditor=NULL;
+  SearchStart=SearchEnd=0;
   NoUseReturning=false;
+  AllFilter=false;
   MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
   Query=NULL;
   SingleEdit=NULL;
@@ -290,22 +296,22 @@ toResultContentEditor::toResultContentEditor(QWidget *parent,const char *name)
   MenuColumn=MenuRow=-1;
 
   Menu=new QPopupMenu(this);
-  Menu->insertItem("&Display in editor...",TORESULT_MEMO);
+  Menu->insertItem(tr("&Display in editor..."),TORESULT_MEMO);
   Menu->insertSeparator();
-  Menu->insertItem("&Copy field",TORESULT_COPY_FIELD);
-  Menu->insertItem("&Paste field",TORESULT_PASTE);
+  Menu->insertItem(tr("&Copy field"),TORESULT_COPY_FIELD);
+  Menu->insertItem(tr("&Paste field"),TORESULT_PASTE);
   Menu->insertSeparator();
-  Menu->insertItem("Copy selection",TORESULT_COPY_SEL);
-  Menu->insertItem("Copy selection with header",TORESULT_COPY_SEL_HEAD);
-  Menu->insertItem("Copy transposed",TORESULT_COPY_TRANS);
+  Menu->insertItem(tr("Copy selection"),TORESULT_COPY_SEL);
+  Menu->insertItem(tr("Copy selection with header"),TORESULT_COPY_SEL_HEAD);
+  Menu->insertItem(tr("Copy transposed"),TORESULT_COPY_TRANS);
   Menu->insertSeparator();
-  Menu->insertItem("&Delete record",TORESULT_DELETE);
+  Menu->insertItem(tr("&Delete record"),TORESULT_DELETE);
   Menu->insertSeparator();
-  Menu->insertItem("Select all",TORESULT_SELECT_ALL);
+  Menu->insertItem(tr("Select all"),TORESULT_SELECT_ALL);
   Menu->setAccel(CTRL+Key_A,TORESULT_SELECT_ALL);
   Menu->insertSeparator();
-  Menu->insertItem("Export to file...",TORESULT_EXPORT);
-  Menu->insertItem("Read all",TORESULT_READ_ALL);
+  Menu->insertItem(tr("Export to file..."),TORESULT_EXPORT);
+  Menu->insertItem(tr("Read all"),TORESULT_READ_ALL);
   connect(Menu,SIGNAL(activated(int)),this,SLOT(menuCallback(int)));
 
   QString str=toTool::globalConfig(CONF_LIST,"");
@@ -326,7 +332,7 @@ toResultContentEditor::~toResultContentEditor()
 
 void toResultContentEditor::wrongUsage(void)
 {
-  throw QString("Can't use these on toResultContent");
+  toStatusMessage(tr("Can't use these on toResultContent"));
 }
 
 void toResultContentEditor::changeSort(int col)
@@ -339,18 +345,22 @@ void toResultContentEditor::changeSort(int col)
   sortColumn(SortRow,SortRowAsc,true);
 }
 
-void toResultContentEditor::changeParams(const QString &Param1,const QString &Param2)
+void toResultContentEditor::query(const QString &,const toQList &params)
 {
-  if (Param1==Owner&&Param2==Table&&!toTool::globalConfig(CONF_DONT_REREAD,"Yes").isEmpty())
-    return;
+  if (params.size()!=2)
+    wrongUsage();
+  toQList::const_iterator par=params.begin();
+  QString Param1=*par;
+  par++;
+  QString Param2=*par;
 
   Owner=Param1;
   Table=Param2;
   if (AllFilter)
-    FilterName="";
+    FilterName=("");
   else {
     FilterName=Owner;
-    FilterName+=".";
+    FilterName+=(".");
     FilterName+=Table;
   }
 
@@ -364,38 +374,52 @@ void toResultContentEditor::changeParams(const QString &Param1,const QString &Pa
   GotoEnd=false;
 
   try {
-    SQL="SELECT * FROM ";
+    SQL=("SELECT * FROM ");
     SQL+=table();
-    if (!Criteria[FilterName].isEmpty()) {
-      SQL+=" WHERE (";
-      SQL+=Criteria[FilterName];
+    bool where=false;
+    if (!Criteria[FilterName.utf8()].isEmpty()) {
+      SQL+=(" WHERE ");
+      SQL+=Criteria[FilterName.utf8()];
+      SQL+=(" ");
+      where=true;
     }
-
-    SkipNumber=toTool::globalConfig(CONF_MAX_CONTENT,DEFAULT_MAX_CONTENT).toInt();
-    
-    if (!Order[FilterName].isEmpty()) {
-      SQL+=" ORDER BY ";
-      SQL+=Order[FilterName];
-    }
-
-    QString init;
-    if (SkipNumber>0)
-      init="SELECT * FROM ("+SQL+") WHERE ROWNUM <= "+QString::number(SkipNumber);
-    else
-      init=SQL;
+    emit filterEnabled(where);
 
     toQList par;
 
-    if (connection().provider()=="Oracle")
-      Query=new toNoBlockQuery(connection(),toQuery::Background,init,par);
-    else
-      Query=new toNoBlockQuery(connection(),toQuery::Background,SQL,par);
+    QString order;
+    if (!Order[FilterName.utf8()].isEmpty()) {
+      order=(" ORDER BY ");
+      order+=Order[FilterName.utf8()];
+    }
+
+    QString init=SQL;
+    SkipNumber=toTool::globalConfig(CONF_MAX_CONTENT,DEFAULT_MAX_CONTENT).toInt();
+
+    if (SkipNumber>0) {
+      if (connection().provider()=="Oracle")
+	init="SELECT * FROM ("+SQL+order+") WHERE ROWNUM <= "+QString::number(SkipNumber);
+      else if (connection().provider()=="MySQL")
+	init=SQL+order+" LIMIT "+QString::number(SkipNumber);
+      else if (connection().provider()=="SapDB") {
+	init=SQL;
+	if (where)
+	  init+=" AND ";
+	else
+	  init+=" WHERE ";
+	init+="ROWNO <= "+QString::number(SkipNumber)+order;
+      }
+    }
+    SQL+=order;
+    Query=new toNoBlockQuery(connection(),toQuery::Background,init,par);
     Poll.start(100);
     OrigValues.clear();
     CurrentRow=-1;
   } TOCATCH
 }
 
+#define STOP_RESIZE_ROW 200
+
 void toResultContentEditor::poll(void)
 {
   try {
@@ -418,43 +442,54 @@ void toResultContentEditor::poll(void)
       }
       
       std::list<QString> data;
+      int dataSize=0;
 
       for (int j=Row;(j<MaxNumber||MaxNumber<0)&&Query->poll()&&!Query->eof();j++) {
 	for (int k=0;k<numCols();k++)
-	  if (SkipNumber==0||j<SkipNumber)
-	    data.insert(data.end(),Query->readValueNull());
-	  else
+	  if (SkipNumber<=0||j<SkipNumber) {
+	    toQValue val=Query->readValueNull();
+	    if (val.isDouble())
+	      data.insert(data.end(),QString::number(val.toDouble()));
+	    else
+	      data.insert(data.end(),val);
+	    dataSize++;
+	  } else
 	    Query->readValueNull();
-	if (SkipNumber!=0&&j>=SkipNumber)
+	if (SkipNumber>0&&j>=SkipNumber)
 	  SkipNumber--;
       }
 
-      int rows=Row+data.size()/numCols()+1;
+      int rows=Row+dataSize/numCols()+1;
       if (numRows()!=rows) {
+	setUpdatesEnabled(false);
 	setNumRows(rows);
 
 #if QT_VERSION >= 300
 	int origRow=Row;
 #endif
-	while(data.size()>0) {
+	while(dataSize>0) {
 	  verticalHeader()->setLabel(Row,QString::number(Row+1));
-	  for(int j=0;j<numCols();j++)
+	  for(int j=0;j<numCols();j++) {
 	    setText(Row,j,toShift(data));
+	    dataSize--;
+	  }
 	  Row++;
 	}
 #if QT_VERSION >= 300
-	for(int j=0;j<numCols();j++) {
-	  int width=columnWidth(j);
-	  for(int k=origRow;k<Row;k++) {
-	    QRect bounds=fontMetrics().boundingRect(text(k,j));
-	    int cw=min(bounds.width(),MaxColDisp);
-	    if (cw>width)
-	      width=cw;
+	if (numRows()<STOP_RESIZE_ROW)
+	  for(int j=0;j<numCols();j++) {
+	    int width=columnWidth(j);
+	    for(int k=origRow;k<Row;k++) {
+	      QRect bounds=fontMetrics().boundingRect(text(k,j));
+	      int cw=min(bounds.width(),MaxColDisp);
+	      if (cw>width)
+		width=cw;
+	    }
+	    if (width!=columnWidth(j))
+	      setColumnWidth(j,width);
 	  }
-	  if (width!=columnWidth(j))
-	    setColumnWidth(j,width);
-	}
 #endif
+	setUpdatesEnabled(true);
       }
       if (first&&SingleEdit) {
 	SingleEdit->changeSource(this);
@@ -549,10 +584,10 @@ void toResultContentEditor::paintCell(QPainter *p,int row,int col,const QRect &c
   }
 
   QString txt=text(row,col);
-  int nl=txt.find("\n");
+  int nl=txt.find(("\n"));
   if (nl>=0) {
     txt=txt.mid(0,nl);
-    txt+="...";
+    txt+=("...");
   }
 
   toQDescList::iterator i=Description.begin();
@@ -573,18 +608,28 @@ void toResultContentEditor::paintCell(QPainter *p,int row,int col,const QRect &c
 
 QWidget *toResultContentEditor::beginEdit(int row,int col,bool replace)
 {
+  SearchStart=SearchEnd=0;
   if (CurrentRow!=row)
-    toStatusMessage("Unsaved data in contents, select other row to store",true);
+    toStatusMessage(tr("Unsaved data in contents, select other row to store"),true);
   saveRow(row);
 
   QString txt=text(row,col);
-  if (txt.contains("\n")) {
+  if (txt.contains(("\n"))) {
     toMemoEditor *edit=new toResultContentMemo(this,txt,row,col);
     connect(edit,SIGNAL(changeData(int,int,const QString &)),
 	    this,SLOT(changeData(int,int,const QString &)));
   }
 
-  return QTable::beginEdit(row,col,replace);
+  QWidget *ret=QTable::beginEdit(row,col,replace);
+  if (ret->isA("QLineEdit"))
+    CurrentEditor=(QLineEdit *)ret;
+  return ret;
+}
+
+void toResultContentEditor::endEdit(int row,int col,bool accept,bool replace)
+{
+  CurrentEditor=NULL;
+  QTable::endEdit(row,col,accept,replace);
 }
 
 void toResultContentEditor::gotoLastRecord()
@@ -594,7 +639,7 @@ void toResultContentEditor::gotoLastRecord()
     setNumRows(Row+1);
     setCurrentCellFocus(Row-1,currentColumn());
   } else {
-    toStatusMessage("Reading all values, moving cursor to end when done",false,false);
+    toStatusMessage(tr("Reading all values, moving cursor to end when done"),false,false);
     GotoEnd=true;
   }
 }
@@ -633,7 +678,7 @@ void toResultContentEditor::addRecord()
       swapRows(row,row-1);
 
     for (int i=0;i<numCols();i++)
-      setText(crow,i,"");
+      setText(crow,i,(""));
 
     NewRecordRow = crow;
     setNumRows(numRows());
@@ -641,6 +686,21 @@ void toResultContentEditor::addRecord()
   }
 }
 
+void toResultContentEditor::duplicateRecord()
+{
+  if(CurrentRow>0 || NewRecordRow>0) {
+    return;
+  }
+
+  int fromrow=currentRow();
+  addRecord();
+  if (currentRow()+1<numRows())
+    fromrow++;
+  if (fromrow!=currentRow())
+    for(int i=0;i<numCols();i++)
+      setText(currentRow(),i,text(fromrow,i));
+}
+
 void toResultContentEditor::cancelEdit()
 {
 
@@ -670,7 +730,7 @@ void toResultContentEditor::cancelEdit()
     setCurrentCellFocus(crow,!currentColumn()); // Must change position
   else
     setCurrentCellFocus(crow,0);
-  toStatusMessage("Edit cancelled",false,false);
+  toStatusMessage(tr("Edit cancelled"),false,false);
 }
 
 void toResultContentEditor::deleteCurrent()
@@ -681,42 +741,47 @@ void toResultContentEditor::deleteCurrent()
   }
   saveUnsaved();
   if (currentRow()<Row) {
-    QString sql="DELETE FROM ";
-    sql+=table();
-    sql+=" WHERE ";
+    try {
+      QString sql=("DELETE FROM ");
+      sql+=table();
+      sql+=(" WHERE ");
     
-    QHeader *head=horizontalHeader();
-    toQDescList::iterator di=Description.begin();
-    bool where=false;
-    toConnection &conn=connection();
+      QHeader *head=horizontalHeader();
+      toQDescList::iterator di=Description.begin();
+      bool where=false;
+      toConnection &conn=connection();
       
-    for(int i=0;i<numCols();i++) {
-      if (!(*di).Datatype.startsWith("LONG")&&!(*di).Datatype.contains("LOB")) {
-	if (where)
-	  sql+=" AND ";
-	else
-	  where=true;
-	sql+=conn.quote(head->label(i));
-	if (!text(currentRow(),i))
-	  sql+=" IS NULL";
-	else {
-	  sql+="= :c";
-	  sql+=QString::number(i);
-	  sql+="<char[4000]>";
+      bool oracle=(connection().provider()=="Oracle");
+      {
+        for(int i=0;i<numCols();i++) {
+	  if (!oracle||(!(*di).Datatype.startsWith(("LONG"))&&
+			!(*di).Datatype.contains(("LOB")))) {
+	    if (where)
+	      sql+=(" AND ");
+	    else
+	      where=true;
+	    sql+=conn.quote(head->label(i));
+	    if (!text(currentRow(),i))
+	      sql+=(" IS NULL");
+	    else {
+	      sql+=("= :c");
+	      sql+=QString::number(i);
+	      sql+=("<char[4000]>");
+	    }
+	  }
+	  di++;
 	}
       }
-      di++;
-    }
-    if (!where) {
-      toStatusMessage("This table contains only LOB/LONG columns and can not be edited");
-      return;
-    }
-    try {
+      if (!where) {
+	toStatusMessage(tr("This table contains only LOB/LONG columns and can not be edited"));
+	return;
+      }
       toQList args;
-      toQDescList::iterator di=Description.begin();
+      di=Description.begin();
       for(int i=0;i<numCols();i++) {
 	QString str=text(currentRow(),i);
-	if (!str.isNull()&&!(*di).Datatype.startsWith("LONG")&&!(*di).Datatype.contains("LOB"))
+	if (!str.isNull()&&(!oracle||(!(*di).Datatype.startsWith(("LONG"))&&
+				      !(*di).Datatype.contains(("LOB")))))
 	  toPush(args,toQValue(str));
 	di++;
       }
@@ -759,145 +824,162 @@ void toResultContentEditor::saveUnsaved(void)
       if (*j!=text(CurrentRow,i))
 	break;
     if (j==OrigValues.end()) {
-      toStatusMessage("No changes made",false,false);
+      toStatusMessage(tr("No changes made"),false,false);
       return;
     }
 
-    toConnection &conn = connection();
+    bool oracle=false;
     QString rowid;
-    bool oracle=(connection().provider()=="Oracle"&&!NoUseReturning);
-    toStatusMessage("Saved row",false,false);
-    if (CurrentRow>=Row || CurrentRow==NewRecordRow) {
-      QString sql="INSERT INTO ";
-      sql+=table();
-      sql+=" VALUES (";
-      for (int i=0;i<numCols();i++) {
-	sql+=":f";
-	sql+=QString::number(i);
-	sql+="<char[4000],in>";
-	if (i+1<numCols())
-	  sql+=",";
-      }
-      sql+=")";
-      if(oracle) 
-	sql+=" RETURNING ROWID INTO :r<char[101],out>";
-	
-      try {
-	toQList args;
-	toQValue null;
+    try {
+      toConnection &conn = connection();
+      oracle=(connection().provider()=="Oracle");
+      toStatusMessage(tr("Saved row"),false,false);
+      if (CurrentRow>=Row || CurrentRow==NewRecordRow) {
+	QString sql=("INSERT INTO ");
+	sql+=table();
+	sql+=" (";
+	int num=0;
+	QHeader *head=horizontalHeader();
 	for (int i=0;i<numCols();i++) {
-	  QString str=text(CurrentRow,i);
-	  if (str.isNull())
-	    toPush(args,null);
-	  else
-	    toPush(args,toQValue(str));
-	}
-	toQuery q(conn,sql,args);
-	if(oracle)
-	  rowid = q.readValueNull();
-	Row++;
-	setNumRows(Row+1);
-	if (!toTool::globalConfig(CONF_AUTO_COMMIT,"").isEmpty())
-	  conn.commit();
-	else
-	  toMainWidget()->setNeedCommit(conn);
-      } catch (const QString &str) {
-	cancelEdit();
-	toStatusMessage(str);
-	oracle = false;
-      }
-    } else {
-      QString sql="UPDATE ";
-      sql+=table();
-      sql+=" SET ";
-      QHeader *head=horizontalHeader();
-      std::list<QString>::iterator k=OrigValues.begin();
-      bool first=false;
-      for(int i=0;i<numCols();i++,k++) {
-	QString fld=text(CurrentRow,i);
-	if (*k!=fld) {
-	  if (!first)
-	    first=true;
-	  else
-	    sql+=", ";
-	  sql+=conn.quote(head->label(i));
-	  if (fld.isNull())
-	    sql+=" = NULL";
-	  else {
-	    sql+="= :f";
-	    sql+=QString::number(i);
-	    sql+="<char[4000],in>";
+	  if (!text(CurrentRow,i).isNull()) {
+	    if (num>0)
+	      sql+=",";
+	    sql+=conn.quote(head->label(i));
+	    num++;
 	  }
 	}
-      }
-      if (first) {
-	sql+=" WHERE ";
-	int col=0;
-	bool where=false;
-	toQDescList::iterator di=Description.begin();
-	for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++) {
-	  if (!(*di).Datatype.startsWith("LONG")&&!(*di).Datatype.contains("LOB")) {
-	    if (where)
-	      sql+=" AND ";
-	    else
-	      where=true;
-	    sql+=conn.quote((*di).Name);
-	    if ((*j).isNull())
-	      sql+=" IS NULL";
-	    else {
-	      sql+="= :c";
-	      sql+=QString::number(col);
-	      sql+="<char[4000],in>";
-	    }
+	sql+=") VALUES (";
+	num=0;
+	for (int i=0;i<numCols();i++) {
+	  if (!text(CurrentRow,i).isNull()) {
+	    if (num>0)
+	      sql+=(",");
+	    sql+=(":f");
+	    sql+=QString::number(num+1);
+	    sql+=("<char[4000],in>");
+	    num++;
 	  }
-	  di++;
 	}
-	if (!where) {
-	  toStatusMessage("This table contains only LOB/LONG columns and can not be edited");
-	  return;
-	}
-	if(oracle)
-	  sql+=" RETURNING ROWID INTO :r<char[101],out>";
+	sql+=(")");
+	if(oracle&&!NoUseReturning) 
+	  sql+=(" RETURNING ROWID INTO :r<char[101],out>");
+	
 	try {
 	  toQList args;
-
-	  std::list<QString>::iterator k=OrigValues.begin();
-	  for (int i=0;i<numCols();i++,k++) {
+	  for (int i=0;i<numCols();i++) {
 	    QString str=text(CurrentRow,i);
-	    if (str!=*k&&!str.isNull())
+	    if (!str.isNull())
 	      toPush(args,toQValue(str));
 	  }
-
-	  toQDescList::iterator di=Description.begin();
-	  for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++) {
-	    QString str=(*j);
-	    if (!str.isNull()&&!(*di).Datatype.startsWith("LONG")&&!(*di).Datatype.contains("LOB"))
-	      toPush(args,toQValue(str));
-	    di++;
-	  }
 	  toQuery q(conn,sql,args);
-	  if(oracle)
+	  if(oracle&&!NoUseReturning)
 	    rowid = q.readValueNull();
+	  Row++;
+	  setNumRows(Row+1);
 	  if (!toTool::globalConfig(CONF_AUTO_COMMIT,"").isEmpty())
 	    conn.commit();
 	  else
 	    toMainWidget()->setNeedCommit(conn);
 	} catch (const QString &str) {
-	  int col=0;
-	  for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++)
-	    setText(CurrentRow,col,*j);
+	  cancelEdit();
 	  toStatusMessage(str);
 	  oracle = false;
 	}
+      } else {
+	QString sql=("UPDATE ");
+	sql+=table();
+	sql+=(" SET ");
+	QHeader *head=horizontalHeader();
+	std::list<QString>::iterator k=OrigValues.begin();
+	bool first=false;
+	for(int i=0;i<numCols();i++,k++) {
+	  QString fld=text(CurrentRow,i);
+	  if (*k!=fld) {
+	    if (!first)
+	      first=true;
+	    else
+	      sql+=(", ");
+	    sql+=conn.quote(head->label(i));
+	    if (fld.isNull())
+	      sql+=(" = NULL");
+	    else {
+	      sql+=("= :f");
+	      sql+=QString::number(i);
+	      sql+=("<char[4000],in>");
+	    }
+	  }
+	}
+	if (first) {
+	  sql+=(" WHERE ");
+	  int col=0;
+	  bool where=false;
+	  toQDescList::iterator di=Description.begin();
+	  for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++) {
+	    if (!oracle||(!(*di).Datatype.startsWith(("LONG"))&&
+			  !(*di).Datatype.contains(("LOB")))) {
+	      if (where)
+		sql+=(" AND ");
+	      else
+		where=true;
+	      sql+=conn.quote((*di).Name);
+	      if ((*j).isNull())
+		sql+=(" IS NULL");
+	      else {
+		sql+=("= :c");
+		sql+=QString::number(col);
+		sql+=("<char[4000],in>");
+	      }
+	    }
+	    di++;
+	  }
+	  if (!where) {
+	    toStatusMessage(tr("This table contains only LOB/LONG columns and can not be edited"));
+	    return;
+	  }
+	  if(oracle&&!NoUseReturning)
+	    sql+=(" RETURNING ROWID INTO :r<char[101],out>");
+	  try {
+	    toQList args;
+
+	    std::list<QString>::iterator k=OrigValues.begin();
+	    for (int i=0;i<numCols();i++,k++) {
+	      QString str=text(CurrentRow,i);
+	      if (str!=*k&&!str.isNull())
+		toPush(args,toQValue(str));
+	    }
+
+	    toQDescList::iterator di=Description.begin();
+	    for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++) {
+	      QString str=(*j);
+	      if (!str.isNull()&&(!oracle||(!(*di).Datatype.startsWith(("LONG"))&&
+					    !(*di).Datatype.contains(("LOB")))))
+		toPush(args,toQValue(str));
+	      di++;
+	    }
+	    toQuery q(conn,sql,args);
+	    if(oracle&&!NoUseReturning)
+	      rowid = q.readValueNull();
+	    if (!toTool::globalConfig(CONF_AUTO_COMMIT,"").isEmpty())
+	      conn.commit();
+	    else
+	      toMainWidget()->setNeedCommit(conn);
+	  } catch (const QString &str) {
+	    int col=0;
+	    for(std::list<QString>::iterator j=OrigValues.begin();j!=OrigValues.end();j++,col++)
+	      setText(CurrentRow,col,*j);
+	    toStatusMessage(str);
+	    oracle = false;
+	  }
+	}
       }
-    }
+    } TOCATCH
     OrigValues.clear();
     if(oracle) {
       try {
 	QString sql;
-	sql="SELECT * FROM ";
+	sql=("SELECT * FROM ");
 	sql+=table();
-	sql+=" WHERE rowid = :r<char[101]>";
+	sql+=(" WHERE rowid = :r<char[101]>");
 	toQuery q(connection(),sql,rowid);
 	for (int j=0;j<numCols()&&!q.eof();j++)
 	  setText(CurrentRow,j,q.readValueNull());
@@ -910,6 +992,7 @@ void toResultContentEditor::saveUnsaved(void)
 
 void toResultContentEditor::changePosition(int row,int col)
 {
+  SearchStart=SearchEnd=0;
   if (CurrentRow!=row&&CurrentRow>=0)
     saveUnsaved();
 
@@ -939,10 +1022,7 @@ void toResultContentEditor::editPrint(void)
 {
   toResultView print(false,true,this);
   print.hide();
-  QString name="Content of ";
-  name+=Owner;
-  name+=".";
-  name+=Table;
+  QString name=tr("Content of %1.%2").arg(Owner).arg(Table);
   print.setSQLName(name);
   print.query(SQL);
   print.editPrint();
@@ -952,10 +1032,7 @@ bool toResultContentEditor::editSave(bool ask)
 {
   toResultView list(false,true,this);
   list.hide();
-  QString name="Content of ";
-  name+=Owner;
-  name+=".";
-  name+=Table;
+  QString name=tr("Content of %1.%2").arg(Owner).arg(Table);
   list.setSQLName(name);
   list.query(SQL);
   list.editReadAll();
@@ -1088,72 +1165,140 @@ void toResultContentEditor::menuCallback(int cmd)
 
 QString toResultContentEditor::table(void)
 {
-  QString sql;
-  if (connection().provider()!="PostgreSQL") {
-    sql=connection().quote(Owner);
-    sql+=".";
+  try {
+    QString sql;
+    if (connection().provider()!="PostgreSQL") {
+      sql=connection().quote(Owner);
+      sql+=(".");
+    }
+    sql+=connection().quote(Table);
+    return sql;
+  } catch(const QString &str) {
+    toStatusMessage(str);
+    return Table;
   }
-  sql+=connection().quote(Table);
-  return sql;
+}
+
+bool toResultContentEditor::searchNext(toSearchReplace *search)
+{
+  int row=currentRow();
+  int col=currentColumn();
+  while(row<numRows()) {
+    int pos=SearchEnd;
+    int endPos;
+
+    if (search->findString(text(row,col),pos,endPos)) {
+      setCurrentCell(row,col);
+
+#if QT_VERSION >= 300
+      editCell(row,col);
+      if (CurrentEditor)
+	CurrentEditor->setSelection(pos,endPos-pos);
+#endif
+
+      SearchEnd=endPos;
+      SearchStart=pos;
+      return true;
+    }
+    col++;
+    if (col>=numCols()) {
+      row++;
+      col=0;
+    }
+    SearchEnd=0;
+  }
+  return false;
+}
+
+void toResultContentEditor::searchReplace(const QString &newData)
+{
+  if (SearchEnd>0) {
+    int row=currentRow();
+    int col=currentColumn();
+    QTableItem *itm=item(row,col);
+    if (itm) {
+      QString txt=itm->text();
+      txt.replace(SearchStart,SearchEnd-SearchStart,newData);
+      saveRow(row);
+      itm->setText(txt);
+    }
+  }
+}
+
+bool toResultContentEditor::searchCanReplace(bool all)
+{
+  if (all||(SearchEnd>0))
+    return true;
+  return false;
 }
 
 toResultContent::toResultContent(QWidget *parent,const char *name)
   : QVBox(parent,name)
 {
-  QToolBar *toolbar=toAllocBar(this,"Content editor",connection().description());
+  QToolBar *toolbar=toAllocBar(this,tr("Content editor"));
   Editor=new toResultContentEditor(this,name);
 
-  new QToolButton(QPixmap((const char **)filter_xpm),
-		  "Define filter for editor",
-		  "Define filter for editor",
-		  this,SLOT(changeFilter()),toolbar);
+  QToolButton *btn=new QToolButton(QPixmap((const char **)filter_xpm),
+				   tr("Define filter for editor"),
+				   tr("Define filter for editor"),
+				   this,SLOT(changeFilter()),toolbar);
+  connect(Editor,
+	  SIGNAL(filterEnabled(bool)),
+	  btn,
+	  SLOT(setOn(bool)));
+  btn->setToggleButton(true);
+
   new QToolButton(QPixmap((const char **)nofilter_xpm),
-		  "Remove any filter",
-		  "Remove any filter",
+		  tr("Remove any filter"),
+		  tr("Remove any filter"),
 		  this,SLOT(removeFilter()),toolbar);
   toolbar->addSeparator();
   new QToolButton(QPixmap((const char **)addrecord_xpm),
-		  "Add a new record",
-		  "Add a new record",
+		  tr("Add a new record"),
+		  tr("Add a new record"),
 		  Editor,SLOT(addRecord()),toolbar);
+  new QToolButton(QPixmap((const char **)duplicaterecord_xpm),
+		  tr("Duplicate an existing record"),
+		  tr("Duplicate an existing record"),
+		  Editor,SLOT(duplicateRecord()),toolbar);
   new QToolButton(QPixmap((const char **)saverecord_xpm),
-		  "Save changes",
-		  "Save changes",
+		  tr("Save changes"),
+		  tr("Save changes"),
 		  Editor,SLOT(saveUnsaved()),toolbar);
   new QToolButton(QPixmap((const char **)canceledit_xpm),
-		  "Discard changes",
-		  "Discard changes",
+		  tr("Discard changes"),
+		  tr("Discard changes"),
 		  Editor,SLOT(cancelEdit()),toolbar);
   new QToolButton(QPixmap((const char **)trash_xpm),
-		  "Delete current record from table",
-		  "Delete current record from table",
+		  tr("Delete current record from table"),
+		  tr("Delete current record from table"),
 		  Editor,SLOT(deleteCurrent()),toolbar);
   toolbar->addSeparator();
   new QToolButton(QPixmap((const char **)rewind_xpm),
-		  "Go to first row",
-		  "Go to first row",
+		  tr("Go to first row"),
+		  tr("Go to first row"),
 		  Editor,SLOT(gotoFirstRecord()),toolbar);
   new QToolButton(QPixmap((const char **)previous_xpm),
-		  "Go to previous row",
-		  "Go to previous row",
+		  tr("Go to previous row"),
+		  tr("Go to previous row"),
 		  Editor,SLOT(gotoPreviousRecord()),toolbar);
   new QToolButton(QPixmap((const char **)next_xpm),
-		  "Go to next row",
-		  "Go to next row",
+		  tr("Go to next row"),
+		  tr("Go to next row"),
 		  Editor,SLOT(gotoNextRecord()),toolbar);
   new QToolButton(QPixmap((const char **)forward_xpm),
-		  "Go to last row",
-		  "Go to last row",
+		  tr("Go to last row"),
+		  tr("Go to last row"),
 		  Editor,SLOT(gotoLastRecord()),toolbar);
   toolbar->addSeparator();
 
-  QToolButton *btn=new QToolButton(toolbar);
+  btn=new QToolButton(toolbar);
   btn->setToggleButton(true);
   btn->setIconSet(QIconSet(QPixmap((const char **)single_xpm)));
   connect(btn,SIGNAL(toggled(bool)),Editor,SLOT(singleRecordForm(bool)));
-  QToolTip::add(btn,"Toggle between table or single record editing");
+  QToolTip::add(btn,tr("Toggle between table or single record editing"));
   
-  toolbar->setStretchableWidget(new QLabel(toolbar));
+  toolbar->setStretchableWidget(new QLabel(toolbar,TO_KDE_TOOLBAR_WIDGET));
   connect(toMainWidget(),SIGNAL(willCommit(toConnection &,bool)),
 	  this,SLOT(saveUnsaved(toConnection &,bool)));
 }
@@ -1162,8 +1307,8 @@ void toResultContent::changeFilter(void)
 {
   toResultContentFilterUI filter(this,"FilterSetup",true);
   filter.AllTables->setChecked(Editor->allFilter());
-  filter.Order->setText(Editor->Order[Editor->FilterName]);
-  filter.Criteria->setText(Editor->Criteria[Editor->FilterName]);
+  filter.Order->setText(Editor->Order[Editor->FilterName.utf8()]);
+  filter.Criteria->setText(Editor->Criteria[Editor->FilterName.utf8()]);
   filter.Columns->changeParams(Editor->Owner,Editor->Table);
   if(filter.exec())
     Editor->changeFilter(filter.AllTables->isChecked(),
@@ -1176,29 +1321,31 @@ void toResultContentEditor::changeFilter(bool all,const QString &crit,const QStr
   AllFilter=all;
   QString nam;
   if (AllFilter)
-    nam="";
+    nam=("");
   else {
     nam=Owner;
-    nam+=".";
+    nam+=(".");
     nam+=Table;
   }
-  Criteria[nam]=crit;
-  Order[nam]=ord;
+  Criteria[nam.utf8()]=crit;
+  Order[nam.utf8()]=ord;
   saveUnsaved();
-  
+
   QString t=Owner;
   Owner=QString::null;
   changeParams(t,Table);
 }
 
-void toResultContent::saveUnsaved(toConnection &conn,bool cmt)
+void toResultContent::saveUnsaved(toConnection &conn,bool)
 {
-  toConnection &mycon=connection();
-  if (&mycon==&conn) // Is this same connection
-    saveUnsaved();
+  try {
+    toConnection &mycon=connection();
+    if (&mycon==&conn) // Is this same connection
+      saveUnsaved();
+  } TOCATCH
 }
 
-bool toResultContent::canHandle(toConnection &conn)
+bool toResultContent::canHandle(toConnection &)
 {
   return true;
 }
@@ -1206,9 +1353,9 @@ bool toResultContent::canHandle(toConnection &conn)
 void toResultContent::removeFilter(void)
 {
   if (!Editor->allFilter()) {
-    switch(TOMessageBox::information(this,"Remove filter",
-				     "Remove the filter for this table only or for all tables.",
-				     "&All","&This","&Cancel",0)) {
+    switch(TOMessageBox::information(this,tr("Remove filter"),
+				     tr("Remove the filter for this table only or for all tables."),
+				     tr("&All"),tr("&This"),tr("Cancel"),0)) {
     case 0:
       Editor->Criteria.clear();
       Editor->Order.clear();
@@ -1249,7 +1396,7 @@ void toResultContentEditor::singleRecordForm(bool display)
   }
 }
 
-void toResultContentEditor::exportData(std::map<QString,QString> &data,const QString &prefix)
+void toResultContentEditor::exportData(std::map<QCString,QString> &data,const QCString &prefix)
 {
   if (AllFilter)
     data[prefix+":All"]="Yes";
@@ -1257,9 +1404,9 @@ void toResultContentEditor::exportData(std::map<QString,QString> &data,const QSt
   toMapExport(data,prefix+":Order",Order);
 }
 
-void toResultContentEditor::importData(std::map<QString,QString> &data,const QString &prefix)
+void toResultContentEditor::importData(std::map<QCString,QString> &data,const QCString &prefix)
 {
-  AllFilter=!data[prefix+":All"].isEmpty();
+  AllFilter=!(data[prefix+":All"].isEmpty());
   toMapImport(data,prefix+":Crit",Criteria);
   toMapImport(data,prefix+":Order",Order);
 }
@@ -1272,7 +1419,7 @@ toListView *toResultContentEditor::copySelection(bool header)
     int row;
     int col;
     if (header) {
-      lst->addColumn("#");
+      lst->addColumn(("#"));
       lst->setColumnAlignment(0,AlignRight);
     }
     for (col=sel.leftCol();col<=sel.rightCol();col++) {
@@ -1296,10 +1443,7 @@ toListView *toResultContentEditor::copySelection(bool header)
 		      text(row,col));
       }
     }
-    QString name="Content of ";
-    name+=Owner;
-    name+=".";
-    name+=Table;
+    QString name=tr("Content of %1.%2").arg(Owner).arg(Table);
     lst->setSQLName(name);
     return lst;
   }
@@ -1329,11 +1473,11 @@ void toResultContentSingle::changeSource(QTable *table)
     new QLabel(head->label(i),Container);
     QLineEdit *edit=new QLineEdit(Container,QString::number(i));
     edit->setFixedWidth(300);
-    QCheckBox *box=new QCheckBox("NULL",Container);
+    QCheckBox *box=new QCheckBox(("NULL"),Container);
     connect(box,SIGNAL(toggled(bool)),edit,SLOT(setDisabled(bool)));
 
     toParamGetButton *btn=new toParamGetButton(i,Container);
-    btn->setText("Edit");
+    btn->setText(tr("Edit"));
     btn->setSizePolicy(QSizePolicy(QSizePolicy::Maximum,QSizePolicy::Fixed));
     connect(btn,SIGNAL(clicked(int)),this,SLOT(showMemo(int)));
     connect(box,SIGNAL(toggled(bool)),btn,SLOT(setDisabled(bool)));
@@ -1374,7 +1518,7 @@ void toResultContentSingle::saveRow(QTable *table,int row)
       val++;
     }
   } else
-    toStatusMessage("Internal error, save different row than current in content editor");
+    toStatusMessage(tr("Internal error, save different row than current in content editor"));
 }
 
 void toResultContentSingle::showMemo(int row)
