@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -37,6 +37,7 @@
 #include "toconf.h"
 #include "toconnection.h"
 #include "tonoblockquery.h"
+#include "toresultlong.h"
 #include "toresultstorage.h"
 #include "tosql.h"
 #include "totool.h"
@@ -125,22 +126,22 @@ toResultStorage::toResultStorage(bool available,QWidget *parent,const char *name
   setAllColumnsShowFocus(true);
   setSorting(0);
   setRootIsDecorated(true);
-  addColumn("Name");
-  addColumn("Status");
-  addColumn("Information");
-  addColumn("Contents");
-  addColumn("Logging");
-  addColumn(QString("Size (%1)").arg(Unit));
-  addColumn(QString("Free (%1)").arg(Unit));
-  addColumn(QString("Autoextend (%1)").arg(Unit));
+  addColumn(tr("Name"));
+  addColumn(tr("Status"));
+  addColumn(tr("Information"));
+  addColumn(tr("Contents"));
+  addColumn(tr("Logging"));
+  addColumn(tr("Size (%1)").arg(Unit));
+  addColumn(tr("Free (%1)").arg(Unit));
+  addColumn(tr("Autoextend (%1)").arg(Unit));
   if (available)
-    addColumn("Used/Free/Autoextend");
+    addColumn(tr("Used/Free/Autoextend"));
   else
-    addColumn("Available");
-  addColumn("Coalesced");
-  addColumn(QString("Max free (%1)").arg(Unit));
-  addColumn("Free fragments");
-  setSQLName("toResultStorage");
+    addColumn(tr("Available"));
+  addColumn(tr("Coalesced"));
+  addColumn(tr("Max free (%1)").arg(Unit));
+  addColumn(tr("Free fragments"));
+  setSQLName(tr("toResultStorage"));
 
   setColumnAlignment(5,AlignRight);
   setColumnAlignment(6,AlignRight);
@@ -212,7 +213,7 @@ static toSQL SQLShowCoalesced("toResultStorage:ShowCoalesced",
 static toSQL SQLShowCoalesced8("toResultStorage:ShowCoalesced",
                                "SELECT d.tablespace_name, \n"
                                "       d.status,\n"
-			       "       d.extent_management,\n"
+			       "       ' ',\n"
                                "       d.contents,\n"
                                "       d.logging,\n"
                                "       TO_CHAR(ROUND(NVL(a.bytes / b.unit, 0),2)),\n"
@@ -229,7 +230,7 @@ static toSQL SQLShowCoalesced8("toResultStorage:ShowCoalesced",
                                " WHERE d.tablespace_name = a.tablespace_name(+)\n"
                                "   AND d.tablespace_name = f.tablespace_name(+)\n"
                                " ORDER BY d.tablespace_name",
-                               QString::null,
+                               "",
                                "8.0");
 
 static toSQL SQLShowCoalesced7("toResultStorage:ShowCoalesced",
@@ -252,7 +253,7 @@ static toSQL SQLShowCoalesced7("toResultStorage:ShowCoalesced",
                                " WHERE d.tablespace_name = a.tablespace_name(+)\n"
                                "   AND d.tablespace_name = f.tablespace_name(+)\n"
                                " ORDER BY d.tablespace_name",
-                               QString::null,
+                               "",
                                "7.3");
 
 static toSQL SQLNoShowCoalesced("toResultStorage:NoCoalesced",
@@ -303,7 +304,7 @@ static toSQL SQLNoShowCoalesced("toResultStorage:NoCoalesced",
 static toSQL SQLNoShowCoalesced8("toResultStorage:NoCoalesced",
                                  "SELECT d.tablespace_name, \n"
                                  "       d.status,\n"
-				 "       d.extent_management,\n"
+				 "       ' ',\n"
                                  "       d.contents,\n"
                                  "       d.logging,\n"
                                  "       TO_CHAR(ROUND(NVL(a.bytes / b.unit, 0),2)),\n"
@@ -320,7 +321,7 @@ static toSQL SQLNoShowCoalesced8("toResultStorage:NoCoalesced",
                                  " WHERE d.tablespace_name = a.tablespace_name(+)\n"
                                  "   AND d.tablespace_name = f.tablespace_name(+)\n"
                                  " ORDER BY d.tablespace_name",
-                                 QString::null,
+                                 "",
                                  "8.0");
 
 static toSQL SQLNoShowCoalesced7("toResultStorage:NoCoalesced",
@@ -343,7 +344,7 @@ static toSQL SQLNoShowCoalesced7("toResultStorage:NoCoalesced",
                                  " WHERE d.tablespace_name = a.tablespace_name(+)\n"
                                  "   AND d.tablespace_name = f.tablespace_name(+)\n"
                                  " ORDER BY d.tablespace_name",
-                                 QString::null,
+                                 "",
                                  "7.3");
 
 static toSQL SQLDatafile("toResultStorage:Datafile",
@@ -416,7 +417,7 @@ static toSQL SQLDatafile8("toResultStorage:Datafile",
                           "       (select :unt<int> unit from sys.dual) b\n"
                           " WHERE (s.file_id (+)= d.file_id)\n"
                           "   AND (d.file_name = v.name)",
-                          QString::null,
+                          "",
                           "8.0");
 
 static toSQL SQLDatafile7("toResultStorage:Datafile",
@@ -441,7 +442,7 @@ static toSQL SQLDatafile7("toResultStorage:Datafile",
 			  "        (select :unt<int> unit from sys.dual) b\n"
 			  " WHERE  (s.file_id (+)= d.file_id)\n"
 			  "   AND  (d.file_name = v.name)",
-			  QString::null,
+			  "",
 			  "7.3");
 
 #define FILECOLUMNS 14
@@ -467,15 +468,15 @@ void toResultStorage::query(void)
   if (!handled()||Tablespaces||Files)
     return;
 
-  saveSelected();
-  clear();
-
-  toConnection &conn=connection();
-
-  toQList args;
-  toPush(args,toQValue(toSizeDecode(Unit)));
-
   try {
+    saveSelected();
+    clear();
+
+    toConnection &conn=connection();
+
+    toQList args;
+    toPush(args,toQValue(toSizeDecode(Unit)));
+
     TablespaceValues.clear();
     FileValues.clear();
 
@@ -520,7 +521,7 @@ void toResultStorage::updateList(void)
 	if (Files)
 	  break;
 	else
-	  throw QString("Couldn't find tablespace parent %1 for datafile").arg(name);
+	  throw tr("Couldn't find tablespace parent %1 for datafile").arg(name);
       }
       file=new toResultStorageItem(AvailableGraph,tablespace,NULL);
     }
@@ -585,14 +586,14 @@ QString toResultStorage::currentTablespace(void)
 {
   QListViewItem *item=selectedItem();
   if (!item)
-    throw QString("No tablespace selected");
+    throw tr("No tablespace selected");
   QString name;
   if (item->parent()||OnlyFiles)
     name=item->text(COLUMNS);
   else
     name=item->text(0);
   if (name.isEmpty())
-    throw QString("Weird, empty tablespace name");
+    throw tr("Weird, empty tablespace name");
   return name;
 }
 
@@ -600,7 +601,7 @@ QString toResultStorage::currentFilename(void)
 {
   QListViewItem *item=selectedItem();
   if (!item||(!item->parent()&&!OnlyFiles))
-    throw QString("No file selected");
+    throw tr("No file selected");
   QString name=item->text(0);
   return name;
 }
@@ -609,7 +610,7 @@ void toResultStorage::setOnlyFiles(bool only)
 {
   saveSelected();
   if (only&&!OnlyFiles) {
-    addColumn("Tablespace");
+    addColumn(tr("Tablespace"));
     setRootIsDecorated(false);
   } else if (!only&&OnlyFiles) {
     removeColumn(columns()-1);
@@ -618,3 +619,350 @@ void toResultStorage::setOnlyFiles(bool only)
   OnlyFiles=only;
   updateList();
 }
+
+toStorageExtent::extentName::extentName(const QString &owner,const QString &table,
+					const QString &partition,int size)
+  : Owner(owner),Table(table),Partition(partition)
+{
+  Size=size;
+}
+
+bool toStorageExtent::extentName::operator < (const toStorageExtent::extentName &ext) const
+{
+  if (Owner<ext.Owner)
+    return true;
+  if (Owner>ext.Owner)
+    return false;
+  if (Table<ext.Table)
+    return true;
+  if (Table>ext.Table)
+    return false;
+  if (Partition<ext.Partition)
+    return true;
+  return false;
+}
+
+bool toStorageExtent::extentName::operator == (const toStorageExtent::extentName &ext) const
+{
+  return Owner==ext.Owner&&Table==ext.Table&&(Partition==ext.Partition||ext.Partition.isNull());
+}
+
+toStorageExtent::extent::extent(const QString &owner,const QString &table,
+				const QString &partition,
+				int file,int block,int size)
+  : extentName(owner,table,partition,size)
+{
+  File=file;
+  Block=block;
+}
+
+bool toStorageExtent::extent::operator < (const toStorageExtent::extent &ext) const
+{
+  if (File<ext.File)
+    return true;
+  if (File>ext.File)
+    return false;
+  if (Block<ext.Block)
+    return true;
+  return false;
+}
+
+bool toStorageExtent::extent::operator == (const toStorageExtent::extent &ext) const
+{
+  return Owner==ext.Owner&&Table==ext.Table&&Partition==ext.Partition&&
+    File==ext.File&&Block==ext.Block&&Size==ext.Size;
+}
+
+toStorageExtent::toStorageExtent(QWidget *parent,const char *name)
+  : QWidget(parent,name)
+{
+  setBackgroundColor(Qt::white);
+}
+
+void toStorageExtent::highlight(const QString &owner,const QString &table,
+				const QString &partition)
+{
+  Highlight.Owner=owner;
+  Highlight.Table=table;
+  Highlight.Partition=partition;
+  update();
+}
+
+static toSQL SQLObjectsFile("toStorageExtent:ObjectsFile",
+			    "SELECT owner,\n"
+			    "       segment_name,\n"
+			    "       partition_name,\n"
+			    "       file_id,\n"
+			    "       block_id,\n"
+			    "       blocks\n"
+			    "  FROM sys.dba_extents\n"
+			    " WHERE tablespace_name = :tab<char[101]>\n"
+			    "   AND file_id = :fil<int>",
+			    "Get objects in a datafile, must have same columns and binds",
+			    "8.0");
+
+static toSQL SQLObjectsFile7("toStorageExtent:ObjectsFile",
+			     "SELECT owner,\n"
+			     "       segment_name,\n"
+			     "       NULL,\n"
+			     "       file_id,\n"
+			     "       block_id,\n"
+			     "       blocks\n"
+			     "  FROM sys.dba_extents\n"
+			     " WHERE tablespace_name = :tab<char[101]>\n"
+			     "   AND file_id = :fil<int>",
+			     "",
+			     "7.3");
+
+static toSQL SQLObjectsTablespace("toStorageExtent:ObjectsTablespace",
+				  "SELECT owner,\n"
+				  "       segment_name,\n"
+				  "       partition_name,\n"
+				  "       file_id,\n"
+				  "       block_id,\n"
+				  "       blocks\n"
+				  "  FROM sys.dba_extents WHERE tablespace_name = :tab<char[101]>",
+				  "Get objects in a tablespace, must have same columns and binds",
+				  "8.0");
+
+static toSQL SQLObjectsTablespace7("toStorageExtent:ObjectsTablespace",
+				   "SELECT owner,\n"
+				   "       segment_name,\n"
+				   "       NULL,\n"
+				   "       file_id,\n"
+				   "       block_id,\n"
+				   "       blocks\n"
+				   "  FROM sys.dba_extents WHERE tablespace_name = :tab<char[101]>",
+				   "",
+				   "7.3");
+
+static toSQL SQLTableTablespace("toStorageExtent:TableTablespace",
+				"SELECT MAX(tablespace_name)\n"
+				"  FROM (SELECT tablespace_name FROM sys.all_all_tables\n"
+				"         WHERE owner = :own<char[101]> AND table_name = :tab<char[101]>\n"
+				"        UNION\n"
+				"        SELECT i.tablespace_name\n"
+				"          FROM sys.all_indexes i,\n"
+				"               sys.all_constraints c,\n"
+				"               sys.all_all_tables t\n"
+				"         WHERE t.owner = :own<char[101]> AND t.table_name = :tab<char[101]>\n"
+				"           AND i.table_name = t.table_name AND i.owner = t.owner\n"
+				"           AND c.constraint_name = i.index_name AND c.owner = i.owner\n"
+				"           AND c.constraint_type = 'P')",
+				"Get the tablespace of a table");
+
+static toSQL SQLTableTablespace7("toStorageExtent:TableTablespace",
+				 "SELECT tablespace_name FROM sys.all_tables\n"
+				 " WHERE owner = :own<char[101]> AND table_name = :tab<char[101]>",
+				 "",
+				 "7.3");
+
+static toSQL SQLFileBlocks("toStorageExtent:FileSize",
+			   "SELECT file_id,blocks FROM sys.dba_data_files\n"
+			   " WHERE tablespace_name = :tab<char[101]>\n"
+			   "   AND file_id = :fil<int>\n"
+			   " ORDER BY file_id",
+			   "Get blocks for datafiles, must have same columns and binds");
+
+static toSQL SQLTablespaceBlocks("toStorageExtent:TablespaceSize",
+				 "SELECT file_id,blocks FROM sys.dba_data_files\n"
+				 " WHERE tablespace_name = :tab<char[101]>"
+				 " ORDER BY file_id",
+				 "Get blocks for tablespace datafiles, must have same columns and binds");
+
+void toStorageExtent::setTablespace(const QString &tablespace)
+{
+  try {
+    if (Tablespace==tablespace)
+      return;
+    Tablespace=tablespace;
+    toBusy busy;
+    Extents.clear();
+    FileOffset.clear();
+    toQuery query(toCurrentConnection(this),SQLObjectsTablespace,tablespace);
+    extent cur;
+    while(!query.eof()) {
+      cur.Owner=query.readValueNull();
+      cur.Table=query.readValueNull();
+      cur.Partition=query.readValueNull();
+      cur.File=query.readValueNull().toInt();
+      cur.Block=query.readValueNull().toInt();
+      cur.Size=query.readValueNull().toInt();
+      toPush(Extents,cur);
+    }
+    toQuery blocks(toCurrentConnection(this),SQLTablespaceBlocks,tablespace);
+    Total=0;
+    while(!blocks.eof()) {
+      int id=blocks.readValueNull().toInt();
+      FileOffset[id]=Total;
+      Total+=blocks.readValueNull().toInt();
+    }
+  } TOCATCH
+  Extents.sort();
+  update();
+}
+
+void toStorageExtent::setFile(const QString &tablespace,int file)
+{
+  try {
+    toBusy busy;
+    Extents.clear();
+    FileOffset.clear();
+    toQuery query(toCurrentConnection(this),SQLObjectsFile,tablespace,QString::number(file));
+    extent cur;
+    while(!query.eof()) {
+      cur.Owner=query.readValueNull();
+      cur.Table=query.readValueNull();
+      cur.Partition=query.readValueNull();
+      cur.File=query.readValueNull().toInt();
+      cur.Block=query.readValueNull().toInt();
+      cur.Size=query.readValueNull().toInt();
+      toPush(Extents,cur);
+    }
+    toQuery blocks(toCurrentConnection(this),SQLFileBlocks,tablespace,QString::number(file));
+    Total=0;
+    while(!blocks.eof()) {
+      int id=blocks.readValueNull().toInt();
+      FileOffset[id]=Total;
+      Total+=blocks.readValueNull().toInt();
+    }
+  } TOCATCH
+  Extents.sort();
+  update();
+}
+
+void toStorageExtent::paintEvent(QPaintEvent *)
+{
+  QPainter paint(this);
+  if (FileOffset.begin()==FileOffset.end())
+    return;
+  QFontMetrics fm=paint.fontMetrics();
+
+  int offset=2*fm.lineSpacing();
+  double lineblocks=Total/(height()-offset-FileOffset.size()+1);
+
+  paint.fillRect(0,0,width(),offset,colorGroup().background());
+  paint.drawText(0,0,width(),offset,AlignLeft|AlignTop,tr("Files: %1").arg(FileOffset.size()));
+  paint.drawText(0,0,width(),offset,AlignRight|AlignTop,tr("Extents: %1").arg(Extents.size()));
+  if (Tablespace)
+    paint.drawText(0,0,width(),offset,AlignCenter|AlignTop,tr("Tablespace: %1").arg(Tablespace));
+  paint.drawText(0,0,width(),offset,AlignLeft|AlignBottom,tr("Blocks: %1").arg(Total));
+  paint.drawText(0,0,width(),offset,AlignRight|AlignBottom,tr("Blocks/line: %1").arg(int(lineblocks)));
+
+  for(std::list<extent>::iterator i=Extents.begin();i!=Extents.end();i++) {
+    QColor col=Qt::green;
+    if (extentName(*i)==Highlight)
+      col=Qt::red;
+    int fileo=0;
+    for (std::map<int,int>::iterator j=FileOffset.begin();j!=FileOffset.end();j++,fileo++)
+      if ((*j).first==(*i).File)
+	break;
+    int block=FileOffset[(*i).File]+(*i).Block;
+
+    int y1=int(block/lineblocks);
+    int x1=int((block/lineblocks-y1)*width());
+    block+=(*i).Size;
+    int y2=int(block/lineblocks);
+    int x2=int((block/lineblocks-y2)*width());
+    paint.setPen(col);
+    if (y1!=y2) {
+      paint.drawLine(x1,y1+offset+fileo,width()-1,y1+offset+fileo);
+      paint.drawLine(0,y2+offset+fileo,x2-1,y2+offset+fileo);
+      if (y1+1!=y2)
+	paint.fillRect(0,y1+1+offset+fileo,width(),y2-y1-1,col);
+    } else
+      paint.drawLine(x1,y1+offset+fileo,x2,y2+offset+fileo);
+  }
+  std::map<int,int>::iterator j=FileOffset.begin();
+  j++;
+  paint.setPen(Qt::black);
+  int fileo=offset;
+  while(j!=FileOffset.end()) {
+    int block=(*j).second;
+    int y1=int(block/lineblocks);
+    int x1=int((block/lineblocks-y1)*width());
+    paint.drawLine(x1,y1+fileo,width()-1,y1+fileo);
+    if (x1!=0)
+      paint.drawLine(0,y1+1+fileo,x1-1,y1+fileo+1);
+    j++;
+    fileo++;
+  }
+}
+
+
+std::list<toStorageExtent::extentTotal> toStorageExtent::objects(void)
+{
+  std::list<extentTotal> ret;
+
+  for(std::list<extent>::iterator i=Extents.begin();i!=Extents.end();i++) {
+    bool dup=false;
+    for (std::list<extentTotal>::iterator j=ret.begin();j!=ret.end();j++) {
+      if ((*j)==(*i)) {
+	(*j).Size+=(*i).Size;
+	(*j).Extents++;
+	dup=true;
+	break;
+      }
+    }
+    if (!dup)
+      toPush(ret,extentTotal((*i).Owner,(*i).Table,(*i).Partition,(*i).Size));
+  }
+
+  ret.sort();
+
+  return ret;
+}
+
+static toSQL SQLListExtents("toResultStorage:ListExtents",
+			    "SELECT * \n"
+			    "  FROM SYS.DBA_EXTENTS WHERE OWNER = :f1<char[101]> AND SEGMENT_NAME = :f2<char[101]>\n"
+			    " ORDER BY extent_id",
+			    "List the extents of a table in a schema.",
+			    "" ,
+			    "Oracle");
+
+toResultExtent::toResultExtent(QWidget *parent,const char *name)
+  : QSplitter(Vertical,parent,name)
+{
+  Graph=new toStorageExtent(this);
+  List=new toResultLong(this);
+  List->setSQL(SQLListExtents);
+
+  QValueList<int> sizes=QSplitter::sizes();
+  sizes[0]=400;
+  sizes[1]=200;
+  setSizes(sizes);
+}
+
+bool toResultExtent::canHandle(toConnection &conn)
+{
+  return toIsOracle(conn);
+}
+
+void toResultExtent::query(const QString &sql,const toQList &params)
+{
+  try {
+    if (!handled())
+      return;
+
+    if (!setSQLParams(sql,params))
+      return;
+
+    toQList::const_iterator i=params.begin();
+    if (i==params.end())
+      return;
+    QString owner=(*i);
+
+    i++;
+    if (i==params.end())
+      return;
+    QString table=(*i);
+
+    List->changeParams(owner,table);
+
+    toQList res=toQuery::readQueryNull(connection(),SQLTableTablespace,owner,table);
+    
+    Graph->setTablespace(toShift(res));
+    Graph->highlight(owner,table,QString::null);
+  } TOCATCH
+}
