@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -57,6 +57,7 @@
 #include <qdragobject.h>
 #include <qfiledialog.h>
 #include <qheader.h>
+#include <qlineedit.h>
 #include <qmessagebox.h>
 #include <qpaintdevicemetrics.h>
 #include <qpainter.h>
@@ -64,6 +65,7 @@
 #include <qpopupmenu.h>
 #include <qprinter.h>
 #include <qregexp.h>
+#include <qstyle.h>
 #include <qtooltip.h>
 #include <qworkspace.h>
 
@@ -74,11 +76,11 @@ static int MaxColDisp;
 
 void toResultViewMLine::setText (int col,const QString &text)
 {
-  QListViewItem::setText(col,text);
+  toResultViewItem::setText(col,text);
   int pos=0;
   int lines=0;
   do {
-    pos=text.find("\n",pos);
+    pos=text.find(QString::fromLatin1("\n"),pos);
     lines++;
     pos++;
   } while(pos>0);
@@ -86,15 +88,22 @@ void toResultViewMLine::setText (int col,const QString &text)
     Lines=lines;
 }
 
-void toResultViewMLine::setup(void)
+void toResultViewMLine::setText (int col,const toQValue &text)
 {
-  QListViewItem::setup();
-  int margin=listView()->itemMargin()*2+1;
-  setHeight((height()-margin)*Lines+margin);
+  if (text.isDouble())
+    toResultViewItem::setText(col,text);
+  setText(col,QString(text));
 }
 
-void toResultViewMLine::paintCell (QPainter *pnt,const QColorGroup & cg,
-				   int column,int width,int alignment)
+void toResultViewMLine::setup(void)
+{
+  toResultViewItem::setup();
+  int margin=listView()->itemMargin()*2+1;
+  setHeight((listView()->fontMetrics().height()+1)*Lines+margin);
+}
+
+void toResultViewMLine::paintCell(QPainter *pnt,const QColorGroup & cg,
+				  int column,int width,int alignment)
 {
   toResultViewItem::paintCell(pnt,cg,column,
 			      max(QListViewItem::width(pnt->fontMetrics(),listView(),column),width),
@@ -106,10 +115,9 @@ static int TextWidth(const QFontMetrics &fm,const QString &str)
   int lpos=0;
   int pos=0;
   int maxWidth=0;
-  const char *strtext=str;
   do {
-    pos=str.find("\n",lpos);
-    QRect bounds=fm.boundingRect(strtext+lpos,pos-lpos);
+    pos=str.find(QString::fromLatin1("\n"),lpos);
+    QRect bounds=fm.boundingRect(str.mid(lpos,pos-lpos));
     if (bounds.width()>maxWidth)
       maxWidth=bounds.width();
     lpos=pos+1;
@@ -117,18 +125,44 @@ static int TextWidth(const QFontMetrics &fm,const QString &str)
   return maxWidth;
 }
 
-int toResultViewMLine::width(const QFontMetrics &fm, const QListView *top, int column) const
+int toResultViewMLine::realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const
 {
   if (!MaxColDisp)
     MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
-  return min(TextWidth(fm,text(column)),MaxColDisp)+top->itemMargin()*2+2;
+  QString t=text(column);
+  if (t.isNull())
+    t=txt;
+  return min(TextWidth(fm,t),MaxColDisp)+top->itemMargin()*2+2;
 }
 
-int toResultViewItem::width(const QFontMetrics &fm, const QListView *top, int column) const
+QString toResultViewItem::firstText(int col) const
+{
+  if (col>=ColumnCount)
+    return QString::null;
+  QString txt=ColumnData[col].Data;
+  int pos=txt.find('\n');
+  if (pos!=-1)
+    return txt.mid(0,pos)+"...";
+  return txt;
+}
+
+QString toResultViewItem::text(int col) const
+{
+  if (col>=ColumnCount)
+    return QString::null;
+  if (ColumnData[col].Type==keyData::Number)
+    return QListViewItem::text(col);
+  return firstText(col);
+}
+
+int toResultViewItem::realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const
 {
   if (!MaxColDisp)
     MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
-  QRect bounds=fm.boundingRect(text(column));
+  QString t=text(column);
+  if (t.isNull())
+    t=txt;
+  QRect bounds=fm.boundingRect(t);
   return min(bounds.width(),MaxColDisp)+top->itemMargin()*2+2;
 }
 
@@ -140,43 +174,151 @@ void toResultViewItem::paintCell(QPainter * p,const QColorGroup & cg,int column,
     view->addItem();
 }
 
-QString toResultViewItem::text(int col) const
+#define ALLOC_SIZE 10
+
+void toResultViewItem::setText (int col,const QString &txt)
 {
-  QString text=QListViewItem::text(col);
-  int pos=text.find('\n');
-  if (pos!=-1) {
-    text.remove(pos,text.length());
-    text.append("...");
+  if (txt!=text(col)) {
+    if (col>=ColumnCount||!ColumnData) {
+      int ns=(col+ALLOC_SIZE)/ALLOC_SIZE*ALLOC_SIZE;
+      keyData *nd=new keyData[ns];
+      int i;
+      for(i=0;i<ColumnCount;i++)
+	nd[i]=ColumnData[i];
+      while(i<ns) {
+	nd[i].Width=0;
+	nd[i].Type=keyData::String;
+	i++;
+      }
+      delete[] ColumnData;
+      ColumnData=nd;
+      ColumnCount=ns;
+    }
+
+    static QRegExp number(QString::fromLatin1("^\\d*\\.?\\d+E?-?\\d*.?.?$"));
+
+    ColumnData[col].Data=txt;
+
+    if(txt=="N/A") {
+      ColumnData[col].Type=keyData::String;
+      ColumnData[col].KeyAsc="\xff";
+      ColumnData[col].KeyDesc="\x00";
+    } else if(number.match(txt)>=0) {
+      ColumnData[col].Type=keyData::Number;
+
+      static char buf[100];
+      double val=txt.toFloat();
+      if(val<0)
+	sprintf(buf,"\x01%015.5f",val);
+      else
+	sprintf(buf,"%015.5f",val);
+      ColumnData[col].KeyAsc=ColumnData[col].KeyDesc=QString::fromLatin1(buf);
+    } else {
+      ColumnData[col].Type=keyData::String;
+      ColumnData[col].KeyAsc=ColumnData[col].KeyDesc=ColumnData[col].Data;
+    }
+    ColumnData[col].Width=realWidth(listView()->fontMetrics(),listView(),col,txt);
   }
-  
-  return text;
+  QListViewItem::setText(col,firstText(col));
 }
 
-QString toResultViewItem::key(int col,bool asc) const
+void toResultViewItem::setText (int col,const toQValue &text)
 {
-  static QRegExp number("^\\d*\\.?\\d+E?-?\\d*.?.?$");
+  setText(col,QString(text));
+  if (text.isDouble())
+    ColumnData[col].Data=QString::number(text.toDouble());
+}
 
-  QString val=text(col);
-  if (number.match(val)>=0) {
-    static char buf[100];
-    sprintf(buf,"%015.5f",text(col).toFloat());
-    return buf;
-  } else if (val=="N/A") {
-    if (asc)
-      return "\xff";
-    else
-      return "\001";
+toResultViewCheck::toResultViewCheck(QListView *parent,QListViewItem *after,const QString &text,QCheckListItem::Type type)
+#if QT_VERSION >= 0x030100
+  : QCheckListItem(parent,after,QString::null,type)
+{
+#else
+  : QCheckListItem(parent,QString::null,type)
+{
+  moveItem(after);
+#endif
+  ColumnData=NULL;
+  ColumnCount=0;
+  if (!text.isNull())
+    setText(0,text);
+}
+
+toResultViewCheck::toResultViewCheck(QListViewItem *parent,QListViewItem *after,const QString &text,QCheckListItem::Type type)
+#if QT_VERSION >= 0x030100
+  : QCheckListItem(parent,after,QString::null,type)
+{
+#else
+  : QCheckListItem(parent,QString::null,type)
+{
+  moveItem(after);
+#endif
+  ColumnData=NULL;
+  ColumnCount=0;
+  if (!text.isNull())
+    setText(0,text);
+}
+
+void toResultViewCheck::setText (int col,const QString &txt)
+{
+  if (txt!=text(col)) {
+    if (col>=ColumnCount||!ColumnData) {
+      int ns=(col+ALLOC_SIZE)/ALLOC_SIZE*ALLOC_SIZE;
+      keyData *nd=new keyData[ns];
+      int i;
+      for(i=0;i<ColumnCount;i++)
+	nd[i]=ColumnData[i];
+      while(i<ns) {
+	nd[i].Width=0;
+	nd[i].Type=keyData::String;
+	i++;
+      }
+      delete[] ColumnData;
+      ColumnData=nd;
+      ColumnCount=ns;
+    }
+
+    static QRegExp number(QString::fromLatin1("^\\d*\\.?\\d+E?-?\\d*.?.?$"));
+
+    ColumnData[col].Data=txt;
+
+    if(txt=="N/A") {
+      ColumnData[col].Type=keyData::String;
+      ColumnData[col].KeyAsc="\xff";
+      ColumnData[col].KeyDesc="\x00";
+    } else if(number.match(txt)>=0) {
+      ColumnData[col].Type=keyData::Number;
+
+      static char buf[100];
+      double val=txt.toFloat();
+      if(val<0)
+	sprintf(buf,"\x01%015.5f",val);
+      else
+	sprintf(buf,"%015.5f",val);
+      ColumnData[col].KeyAsc=ColumnData[col].KeyDesc=QString::fromLatin1(buf);
+    } else {
+      ColumnData[col].Type=keyData::String;
+      ColumnData[col].KeyAsc=ColumnData[col].KeyDesc=ColumnData[col].Data;
+    }
+    ColumnData[col].Width=realWidth(listView()->fontMetrics(),listView(),col,txt);
   }
-  return val;
+  QCheckListItem::setText(col,firstText(col));
+}
+
+void toResultViewCheck::setText (int col,const toQValue &text)
+{
+  setText(col,QString(text));
+  if (text.isDouble())
+    ColumnData[col].Data=QString::number(text.toDouble());
 }
 
 void toResultViewMLCheck::setText (int col,const QString &text)
 {
-  QCheckListItem::setText(col,text);
+  toResultViewCheck::setText(col,text);
   int pos=0;
   int lines=0;
   do {
-    pos=text.find("\n",pos);
+    pos=text.find(QString::fromLatin1("\n"),pos);
     lines++;
     pos++;
   } while(pos>0);
@@ -184,34 +326,61 @@ void toResultViewMLCheck::setText (int col,const QString &text)
     Lines=lines;
 }
 
+void toResultViewMLCheck::setText (int col,const toQValue &text)
+{
+  if (text.isDouble())
+    toResultViewCheck::setText(col,text);
+  setText(col,QString(text));
+}
+
 void toResultViewMLCheck::setup(void)
 {
-  QCheckListItem::setup();
-  int margin=listView()->itemMargin()*2+1;
-  setHeight((height()-margin)*Lines+margin);
+  toResultViewCheck::setup();
+  int margin=listView()->itemMargin()*2;
+  setHeight((listView()->fontMetrics().height()+1)*Lines+margin);
 }
 
 void toResultViewMLCheck::paintCell (QPainter *pnt,const QColorGroup & cg,
 				   int column,int width,int alignment)
 {
   toResultViewCheck::paintCell(pnt,cg,column,
-			      max(QCheckListItem::width(pnt->fontMetrics(),listView(),column),width),
-			      alignment);
+			       max(QCheckListItem::width(pnt->fontMetrics(),listView(),column),width),
+			       alignment);
 }
 
-int toResultViewMLCheck::width(const QFontMetrics &fm, const QListView *top, int column) const
+int toResultViewMLCheck::realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const
 {
   if (!MaxColDisp)
     MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
-  return min(TextWidth(fm,text(column)),MaxColDisp)+top->itemMargin()*2+2;
+  QString t=text(column);
+  if (t.isNull())
+    t=txt;
+  int wx=top->itemMargin()*2+2;
+  if (column==0)
+#if QT_VERSION >= 0x030100
+    wx+=top->style().pixelMetric(QStyle::PM_CheckListButtonSize)+4+top->itemMargin();
+#else
+    wx+=20;
+#endif
+  return min(TextWidth(fm,t),MaxColDisp)+wx;
 }
 
-int toResultViewCheck::width(const QFontMetrics &fm, const QListView *top, int column) const
+int toResultViewCheck::realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const
 {
   if (!MaxColDisp)
     MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
-  QRect bounds=fm.boundingRect(text(column));
-  return min(bounds.width(),MaxColDisp)+top->itemMargin()*2+2;
+  QString t=text(column);
+  if (t.isNull())
+    t=txt;
+  QRect bounds=fm.boundingRect(t);
+  int wx=top->itemMargin()*2+2;
+  if (column==0)
+#if QT_VERSION >= 0x030100
+    wx+=top->style().pixelMetric(QStyle::PM_CheckListButtonSize)+4+top->itemMargin();
+#else
+    wx+=20;
+#endif
+  return min(bounds.width(),MaxColDisp)+wx;
 }
 
 
@@ -225,31 +394,22 @@ void toResultViewCheck::paintCell(QPainter * p,const QColorGroup & cg,int column
 
 QString toResultViewCheck::text(int col) const
 {
-  QString text=QCheckListItem::text(col);
-  int pos=text.find('\n');
-  if (pos!=-1) {
-    text.remove(pos,text.length());
-    text.append("...");
-  }
-  
-  return text;
+  if (col>=ColumnCount)
+    return QString::null;
+  if (ColumnData[col].Type==keyData::Number)
+    return QCheckListItem::text(col);
+  return firstText(col);
 }
 
-QString toResultViewCheck::key(int col,bool asc) const
+QString toResultViewCheck::firstText(int col) const
 {
-  static QRegExp number("^\\d*\\.?\\d+$");
-  QString val=text(col);
-  if (number.match(val)>=0) {
-    static char buf[100];
-    sprintf(buf,"%015f",text(col).toFloat());
-    return buf;
-  } else if (val=="N/A") {
-    if (asc)
-      return "\xff";
-    else
-      return "\001";
-  }
-  return val;
+  if (col>=ColumnCount)
+    return QString::null;
+  QString txt=ColumnData[col].Data;
+  int pos=txt.find('\n');
+  if (pos!=-1)
+    return txt.mid(0,pos)+"...";
+  return txt;
 }
 
 class toListTip : public QToolTip {
@@ -300,6 +460,7 @@ toListView::toListView(QWidget *parent,const char *name)
 		 false,false,false,
 		 true,false,false)
 {
+  FirstSearch=false;
   setTreeStepSize(15);
   setSelectionMode(Extended);
   setAllColumnsShowFocus(true);
@@ -411,8 +572,7 @@ QListViewItem *toListView::printPage(TOPrinter *printer,QPainter *painter,QListV
   double mheight=metrics.height()/wpscaley;
   double x=0;
   if (paint) {
-    QString numPage("Page: ");
-    numPage+=QString::number(pageNo);
+    QString numPage(tr("Page: %1").arg(pageNo));
     painter->drawText(0,int(metrics.height()/wpscaley)-header()->height(),int(metrics.width()/wpscalex),
 		      header()->height(),
 		      SingleLine|AlignRight|AlignVCenter,
@@ -511,8 +671,9 @@ void toListView::editPrint(void)
 {
   TOPrinter printer;
   printer.setMinMax(1,1000);
+  printer.setFromTo(1,1000);
   if (printer.setup()) {
-    printer.setCreator("TOra");
+    printer.setCreator(tr(TOAPPNAME));
     QPainter painter(&printer);
 
     QListViewItem *item=firstChild();
@@ -527,15 +688,20 @@ void toListView::editPrint(void)
       printer.newPage();
       painter.resetXForm();
       qApp->processEvents();
-      QString str("Printing page ");
-      str+=QString::number(page);
+      QString str=tr("Printing page %1").arg(page);
       toStatusMessage(str,false,false);
     }
     painter.end();
-    toStatusMessage("Done printing",false,false);
+    toStatusMessage(tr("Done printing"),false,false);
   }
 }
 
+void toListView::setDisplayMenu(QPopupMenu *m) {
+	if (Menu)
+		delete Menu;
+	Menu=m;
+}
+
 #define TORESULT_COPY_FIELD     1
 #define TORESULT_COPY_SEL	2
 #define TORESULT_COPY_SEL_HEAD	3
@@ -545,30 +711,41 @@ void toListView::editPrint(void)
 #define TORESULT_EXPORT   	7
 #define TORESULT_SELECT_ALL	8
 #define TORESULT_COPY_TRANS	9
+#define TORESULT_LEFT		10
+#define TORESULT_CENTER		11
+#define TORESULT_RIGHT		12
 
 void toListView::displayMenu(QListViewItem *item,const QPoint &p,int col)
 {
   if (item) {
     if (!Menu) {
       Menu=new QPopupMenu(this);
-      Menu->insertItem("Display in editor...",TORESULT_MEMO);
+      Menu->insertItem(tr("Display in editor..."),TORESULT_MEMO);
+
+      QPopupMenu *just=new QPopupMenu(this);
+      just->insertItem(tr("Left"),TORESULT_LEFT);
+      just->insertItem(tr("Center"),TORESULT_CENTER);
+      just->insertItem(tr("Right"),TORESULT_RIGHT);
+      Menu->insertItem(tr("Alignment"),just);
+      connect(just,SIGNAL(activated(int)),this,SLOT(menuCallback(int)));
+
       Menu->insertSeparator();
-      Menu->insertItem("&Copy field",TORESULT_COPY_FIELD);
+      Menu->insertItem(tr("&Copy field"),TORESULT_COPY_FIELD);
       if (selectionMode()==Multi||selectionMode()==Extended) {
-	Menu->insertItem("Copy selection",TORESULT_COPY_SEL);
-	Menu->insertItem("Copy selection with header",TORESULT_COPY_SEL_HEAD);
+	Menu->insertItem(tr("Copy selection"),TORESULT_COPY_SEL);
+	Menu->insertItem(tr("Copy selection with header"),TORESULT_COPY_SEL_HEAD);
       }
-      Menu->insertItem("Copy transposed",TORESULT_COPY_TRANS);
+      Menu->insertItem(tr("Copy transposed"),TORESULT_COPY_TRANS);
       if (selectionMode()==Multi||selectionMode()==Extended) {
 	Menu->insertSeparator();
-	Menu->insertItem("Select all",TORESULT_SELECT_ALL);
+	Menu->insertItem(tr("Select all"),TORESULT_SELECT_ALL);
 	Menu->setAccel(CTRL+Key_A,TORESULT_SELECT_ALL);
       }
       Menu->insertSeparator();
-      Menu->insertItem("Export to file...",TORESULT_EXPORT);
+      Menu->insertItem(tr("Export to file..."),TORESULT_EXPORT);
       if (!Name.isEmpty()) {
 	Menu->insertSeparator();
-	Menu->insertItem("Edit SQL...",TORESULT_SQL);
+	Menu->insertItem(tr("Edit SQL..."),TORESULT_SQL);
       }
       connect(Menu,SIGNAL(activated(int)),this,SLOT(menuCallback(int)));
       addMenues(Menu);
@@ -597,14 +774,29 @@ void toListView::menuCallback(int cmd)
     break;
   case TORESULT_COPY_SEL:
     try {
+      QString str=exportAsText(false,true);
       QClipboard *clip=qApp->clipboard();
-      clip->setText(exportAsText(false,true));
+      QTextDrag *drag=new QTextDrag(str);
+      drag->setSubtype("html");
+      clip->setData(drag);
     } TOCATCH
     break;
+  case TORESULT_LEFT:
+    setColumnAlignment(MenuColumn,AlignLeft);
+    break;
+  case TORESULT_CENTER:
+    setColumnAlignment(MenuColumn,AlignCenter);
+    break;
+  case TORESULT_RIGHT:
+    setColumnAlignment(MenuColumn,AlignRight);
+    break;
   case TORESULT_COPY_SEL_HEAD:
     try {
+      QString str=exportAsText(true,true);
       QClipboard *clip=qApp->clipboard();
-      clip->setText(exportAsText(true,true));
+      QTextDrag *drag=new QTextDrag(str);
+      drag->setSubtype("html");
+      clip->setData(drag);
     } TOCATCH
     break;
   case TORESULT_SELECT_ALL:
@@ -647,15 +839,58 @@ void toListView::focusInEvent (QFocusEvent *e)
 
 static QString QuoteString(const QString &str)
 {
-  static QRegExp quote("\"");
+  static QRegExp quote(QString::fromLatin1("\""));
   QString t=str;
-  t.replace(quote,"\"\"");
+  t.replace(quote,QString::fromLatin1("\"\""));
   return t;
 }
 
-void toListView::editSearch(toSearchReplace *search)
+bool toListView::searchNext(toSearchReplace *search)
 {
-  search->setTarget(this);
+  QListViewItem *item=currentItem();
+
+  bool first=FirstSearch;
+  FirstSearch=false;
+
+  for (QListViewItem *next=NULL;item;item=next) {
+    if (!first)
+      first=true;
+    else {
+      for (int i=0;i<columns();i++) {
+	int pos=0;
+	int endPos=0;
+
+	toResultViewItem *resItem=dynamic_cast<toResultViewItem *>(item);
+	toResultViewCheck *chkItem=dynamic_cast<toResultViewCheck *>(item);
+	QString txt;
+	if (resItem)
+	  txt=resItem->allText(i);
+	else if (chkItem)
+	  txt=chkItem->allText(i);
+	else if (item)
+	  txt=item->text(i);
+	
+	if (search->findString(item->text(0),pos,endPos)) {
+	  setCurrentItem(item);
+	  return true;
+	}
+      }
+    }
+
+    if (item->firstChild())
+      next=item->firstChild();
+    else if (item->nextSibling())
+      next=item->nextSibling();
+    else {
+      next=item;
+      do {
+	next=next->parent();
+      } while(next&&!next->nextSibling());
+      if (next)
+	next=next->nextSibling();
+    }
+  }
+  return false;
 }
 
 toListView *toListView::copyTransposed(void)
@@ -702,10 +937,12 @@ toListView *toListView::copyTransposed(void)
   return lst;
 }
 
-bool toListView::editSave(bool ask)
+bool toListView::editSave(bool)
 {
   try {
-    int type=exportType();
+    QString delimiter;
+    QString separator;
+    int type=exportType(separator,delimiter);
 
     QString nam;
     switch(type) {
@@ -726,33 +963,44 @@ bool toListView::editSave(bool ask)
     if (filename.isEmpty())
       return false;
   
-    return toWriteFile(filename,exportAsText(true,false,type));
+    return toWriteFile(filename,exportAsText(true,false,type,separator,delimiter));
   } TOCATCH
   return false;
 }
 
-int toListView::exportType(void)
+void toListView::addMenues(QPopupMenu *)
 {
-  toResultListFormatUI format(this,NULL,true);
-  format.Format->insertItem("Text");
-  format.Format->insertItem("Tab delimited");
-  format.Format->insertItem("CSV");
-  format.Format->insertItem("HTML");
-  format.Format->setCurrentItem(toTool::globalConfig(CONF_DEFAULT_FORMAT,"").toInt());
+}
 
+bool toListView::searchCanReplace(bool)
+{
+  return false;
+}
+
+int toListView::exportType(QString &separator,QString &delimiter)
+{
+  toResultListFormat format(this,NULL);
   if (!format.exec())
     return -1;
 
+  format.saveDefault();
+
+  separator=format.Separator->text();
+  delimiter=format.Delimiter->text();
+
   return format.Format->currentItem();
 
 }
 
-QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
+QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type,
+				 const QString &sep,const QString &del)
 {
+  QString separator=sep;
+  QString delimiter=del;
   if (type<0)
-    type=exportType();
+    type=exportType(separator,delimiter);
   if (type<0)
-    throw QString("");
+    throw QString::null;
 
   int *sizes=NULL;
   try {
@@ -807,7 +1055,7 @@ QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
 
     QString output;
     if (type==3) {
-      output=QString("<HTML><HEAD><TITLE>%1</TITLE></HEAD><BODY><TABLE CELLSPACING=0 BORDER=0>").
+      output=QString::fromLatin1("<HTML><HEAD><TITLE>%1</TITLE></HEAD><BODY><TABLE CELLSPACING=0 BORDER=0>").
 	arg(sqlName());
     }
 
@@ -816,41 +1064,47 @@ QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
     QString bgcolor;
     if (includeHeader) {
       if (bgcolor.isEmpty())
-	bgcolor="nonull";
+	bgcolor=QString::fromLatin1("nonull");
       else
-	bgcolor="";
+	bgcolor=QString::null;
       if (type==3)
-	output+="<TR BGCOLOR=#7f7f7f>";
+	output+=QString::fromLatin1("<TR BGCOLOR=#7f7f7f>");
       for (int j=0;j<columns();j++)
 	switch(type) {
 	case 0:
-	  output+=QString("%1 ").arg(header()->label(j),-sizes[j]);
+	  output+=QString::fromLatin1("%1 ").arg(header()->label(j),-sizes[j]);
 	  break;
 	case 1:
-	  output+=QString("%1\t").arg(header()->label(j));
+	  output+=QString::fromLatin1("%1\t").arg(header()->label(j));
 	  break;
 	case 2:
-	  output+=QString("\"%1\";").arg(QuoteString(header()->label(j)));
+	  output+=QString::fromLatin1("%1%2%3%4").
+	    arg(delimiter).
+	    arg(QuoteString(header()->label(j))).
+	    arg(delimiter).
+	    arg(separator);
 	  break;
 	case 3:
-	  output+="<TH ALIGN=LEFT BGCOLOR=#cfcfcf>";
+	  output+=QString::fromLatin1("<TH ALIGN=LEFT BGCOLOR=#cfcfcf>");
 	  output+=header()->label(j);
-	  output+="</TH>";
+	  output+=QString::fromLatin1("</TH>");
 	  break;
 	}
-      if (output.length()>0&&type!=3)
+      if (output.length()>0&&type==2)
+	output=output.left(output.length()-separator.length());
+      else if (output.length()>0&&type!=3)
 	output=output.left(output.length()-1);
-      if (type==3&&includeHeader)
-	output+="</TR>";
-      output+="\n";
+      else if (type==3&&includeHeader)
+	output+=QString::fromLatin1("</TR>");
+      output+=QString::fromLatin1("\n");
       if (type==0) {
 	for (int k=0;k<columns();k++) {
 	  for (int l=0;l<sizes[k];l++)
-	    output+="=";
+	    output+=QString::fromLatin1("=");
 	  if (k!=columns()-1)
-	    output+=" ";
+	    output+=QString::fromLatin1(" ");
 	}
-	output+="\n";
+	output+=QString::fromLatin1("\n");
       }
     }
 
@@ -863,12 +1117,12 @@ QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
 	toResultViewCheck *chkItem=dynamic_cast<toResultViewCheck *>(item);
 
 	if (bgcolor.isEmpty())
-	  bgcolor=" BGCOLOR=#cfcfff";
+	  bgcolor=QString::fromLatin1(" BGCOLOR=#cfcfff");
 	else
-	  bgcolor="";
+	  bgcolor=QString::null;
 	QString line;
 	if (type==3)
-	  line=QString("<TR%1>").arg(bgcolor);      
+	  line=QString::fromLatin1("<TR%1>").arg(bgcolor);      
 
 	for (int i=0;i<columns();i++) {
 	  QString text;
@@ -883,38 +1137,54 @@ QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
 	  switch(type) {
 	  case 0:
 	    line+=indent;
-	    line+=QString("%1 ").arg(text,(i==0?indent.length():0)-sizes[i]);
+	    line+=QString::fromLatin1("%1 ").arg(text,(i==0?indent.length():0)-sizes[i]);
 	    break;
 	  case 1:
 	    line+=indent;
-	    line+=QString("%1\t").arg(text);
+	    line+=QString::fromLatin1("%1\t").arg(text);
 	    break;
 	  case 2:
 	    line+=indent;
-	    line+=QString("\"%1\";").arg(QuoteString(text));
+	    line+=QString::fromLatin1("%1%2%3%4").
+	      arg(delimiter).
+	      arg(QuoteString(text)).
+	      arg(delimiter).
+	      arg(separator);
+
 	    break;
 	  case 3:
-	    line+=QString("<TD%1>").arg(bgcolor);
+	    line+=QString::fromLatin1("<TD%1>").arg(bgcolor);
 	    if (i==0)
 	      line+=indent;
+#if QT_VERSION >= 0x030100
+	    text.replace('&',"&amp;");
+	    text.replace('<',"&lt;");
+	    text.replace('>',"&gt");
+#else
+	    text.replace(QRegExp("&"),"&amp;");
+	    text.replace(QRegExp("<"),"&lt;");
+	    text.replace(QRegExp(">"),"&gt");
+#endif
 	    line+=text;
-	    line+="</TD>";
+	    line+=QString::fromLatin1("</TD>");
 	    break;
 	  }
 	}
 	if (type==3)
-	  line+="</TR>";
-	else
+	  line+=QString::fromLatin1("</TR>");
+	else if (type==2)
+	  line=line.left(line.length()-separator.length());
+	else 
 	  line=line.left(line.length()-1);
-	line+="\n";
+	line+=QString::fromLatin1("\n");
 	output+=line;
       }
 
       if (item->firstChild()) {
 	if (type!=3)
-	  indent+=" ";
+	  indent+=QString::fromLatin1(" ");
 	else
-	  indent+="&nbsp;";
+	  indent+=QString::fromLatin1("&nbsp;");
 	next=item->firstChild();
       } else if (item->nextSibling())
 	next=item->nextSibling();
@@ -932,40 +1202,40 @@ QString toListView::exportAsText(bool includeHeader,bool onlySelection,int type)
       }
     }
     if (type==3)
-      output+="</TABLE></BODY></HTML>";
-    delete sizes;
+      output+=QString::fromLatin1("</TABLE></BODY></HTML>");
+    delete[] sizes;
     return output;
   } catch(...) {
-    delete sizes;
+    delete[] sizes;
     throw;
   }
 }
 
-void toListView::exportData(std::map<QString,QString> &ret,const QString &prefix)
+void toListView::exportData(std::map<QCString,QString> &ret,const QCString &prefix)
 {
   int id=0;
   for(int i=0;i<columns();i++) {
     id++;
-    ret[prefix+":Labels:"+QString::number(id)]=header()->label(i);
+    ret[prefix+":Labels:"+QString::number(id).latin1()]=header()->label(i);
   }
   std::map<QListViewItem *,int> itemMap;
   QListViewItem *next=NULL;
   id=0;
   if (rootIsDecorated())
-    ret[prefix+":Decorated"]="Yes";
+    ret[prefix+":Decorated"]=QString::fromLatin1("Yes");
   for (QListViewItem *item=firstChild();item;item=next) {
     id++;
-    QString nam=prefix;
+    QCString nam=prefix;
     nam+=":Items:";
-    nam+=QString::number(id);
+    nam+=QString::number(id).latin1();
     nam+=":";
     itemMap[item]=id;
     if (item->parent())
       ret[nam+"Parent"]=QString::number(itemMap[item->parent()]);
     else
-      ret[nam+"Parent"]="0";
+      ret[nam+"Parent"]=QString::fromLatin1("0");
     if (item->isOpen())
-      ret[nam+"Open"]="Yes";
+      ret[nam+"Open"]=QString::fromLatin1("Yes");
     for(int i=0;i<columns();i++) {
       toResultViewItem *resItem=dynamic_cast<toResultViewItem *>(item);
       toResultViewCheck *chkItem=dynamic_cast<toResultViewCheck *>(item);
@@ -976,7 +1246,7 @@ void toListView::exportData(std::map<QString,QString> &ret,const QString &prefix
 	val=resItem->allText(i);
       else
 	val=item->text(i);
-      ret[nam+QString::number(i)]=val;
+      ret[nam+QString::number(i).latin1()]=val;
     }
 
     if (item->firstChild())
@@ -994,14 +1264,14 @@ void toListView::exportData(std::map<QString,QString> &ret,const QString &prefix
   }
 }
 
-void toListView::importData(std::map<QString,QString> &ret,const QString &prefix)
+void toListView::importData(std::map<QCString,QString> &ret,const QCString &prefix)
 {
   int id;
-  std::map<QString,QString>::iterator i;
+  std::map<QCString,QString>::iterator i;
   clear();
 
   id=1;
-  while((i=ret.find(prefix+":Labels:"+QString::number(id)))!=ret.end()) {
+  while((i=ret.find(prefix+":Labels:"+QString::number(id).latin1()))!=ret.end()) {
     addColumn((*i).second);
     id++;
   }
@@ -1012,8 +1282,8 @@ void toListView::importData(std::map<QString,QString> &ret,const QString &prefix
   std::map<int,QListViewItem *> itemMap;
 
   id=1;
-  while((i=ret.find(prefix+":Items:"+QString::number(id)+":Parent"))!=ret.end()) {
-    QString nam=prefix+":Items:"+QString::number(id)+":";
+  while((i=ret.find(prefix+":Items:"+QString::number(id).latin1()+":Parent"))!=ret.end()) {
+    QCString nam=prefix+":Items:"+QString::number(id).latin1()+":";
     int parent=(*i).second.toInt();
     toResultViewItem *item;
     if (parent)
@@ -1024,7 +1294,7 @@ void toListView::importData(std::map<QString,QString> &ret,const QString &prefix
       item->setOpen(true);
     itemMap[id]=item;
     for(int j=0;j<columns();j++)
-      item->setText(j,ret[nam+QString::number(j)]);
+      item->setText(j,ret[nam+QString::number(j).latin1()]);
     id++;
   }
 }
@@ -1036,7 +1306,11 @@ bool toResultView::eof(void)
 
 QString toResultView::middleString()
 {
-  return connection().description();
+  try {
+    return connection().description();
+  } catch(...) {
+    return QString::null;
+  }
 }
 
 void toResultView::setup(bool readable,bool dispCol)
@@ -1044,13 +1318,16 @@ void toResultView::setup(bool readable,bool dispCol)
   Query=NULL;
   ReadableColumns=readable;
   NumberColumn=dispCol;
+  SortConnected=false;
   if (NumberColumn) {
-    addColumn("#");
+    addColumn(QString::fromLatin1("#"));
     setColumnAlignment(0,AlignRight);
   }
   Filter=NULL;
   readAllEnabled(true);
   ReadAll=false;
+  SortColumn=-2;
+  SortAscending=true;
 }
 
 toResultView::toResultView(bool readable,bool dispCol,QWidget *parent,const char *name)
@@ -1065,6 +1342,17 @@ toResultView::toResultView(QWidget *parent,const char *name)
   setup(false,true);
 }
 
+#define STOP_RESIZE_ROW 500
+
+QListViewItem *toResultView::createItem(QListViewItem *last,const QString &str)
+{
+  if (childCount()==STOP_RESIZE_ROW&&resizeMode()==QListView::NoColumn)
+    for (int i=0;i<columns();i++)
+      setColumnWidthMode(i,Manual);
+
+  return new toResultViewItem(this,last,str);
+}
+
 void toResultView::addItem(void)
 {
   MaxColDisp=toTool::globalConfig(CONF_MAX_COL_DISP,DEFAULT_MAX_COL_DISP).toInt();
@@ -1074,7 +1362,7 @@ void toResultView::addItem(void)
       RowNumber++;
       int disp=0;
       QListViewItem *last=LastItem;
-      LastItem=createItem(LastItem,NULL);
+      LastItem=createItem(LastItem,QString::null);
       if (NumberColumn) {
 	LastItem->setText(0,QString::number(RowNumber));
 	disp=1;
@@ -1109,7 +1397,7 @@ void toResultView::query(const QString &sql,const toQList &param)
     removeColumn(0);
   }
   if (NumberColumn) {
-    addColumn("#");
+    addColumn(QString::fromLatin1("#"));
     setColumnAlignment(0,AlignRight);
   }
 
@@ -1127,18 +1415,20 @@ void toResultView::query(const QString &sql,const toQList &param)
 
       if (name.length()>0&&name.at(0)!=' ') {
 	if (hidden)
-	  throw QString("Can only hide last column in query");
+	  throw tr("Can only hide last column in query");
 	if (name.at(0)=='-') {
-	  addColumn(name.right(name.length()-1));
+	  addColumn(toTranslateMayby(sqlName(),name.right(name.length()-1)));
 	  setColumnAlignment(columns()-1,AlignRight);
 	} else {
-	  addColumn(name);
+	  addColumn(toTranslateMayby(sqlName(),name));
 	  if ((*i).AlignRight)
 	    setColumnAlignment(columns()-1,AlignRight);
 	}
       } else
 	hidden=true;
     }
+    if (resizeMode()!=QListView::NoColumn)
+      setResizeMode(resizeMode());
 
     if (NumberColumn)
       setSorting(0);
@@ -1150,12 +1440,6 @@ void toResultView::query(const QString &sql,const toQList &param)
       addItem();
     if (ReadAll||MaxNumber<0)
       editReadAll();
-
-    char buffer[100];
-    if (Query->rowsProcessed()>0)
-      sprintf(buffer,"%d rows processed",(int)Query->rowsProcessed());
-    else
-      sprintf(buffer,"Query executed");
   } TOCATCH
   updateContents();
 }
@@ -1163,7 +1447,7 @@ void toResultView::query(const QString &sql,const toQList &param)
 void toResultView::editReadAll(void)
 {
   if (!ReadAll)
-    toStatusMessage("Reading all entries",false,false);
+    toStatusMessage(tr("Reading all entries"),false,false);
   int i=0;
   while(!eof()) {
     addItem();
@@ -1197,7 +1481,7 @@ void toResultView::keyPressEvent(QKeyEvent *e)
 void toResultView::addMenues(QPopupMenu *menu)
 {
   menu->insertSeparator();
-  menu->insertItem("Read All",TORESULT_READ_ALL);
+  menu->insertItem(tr("Read All"),TORESULT_READ_ALL);
 }
 
 void toResultView::menuCallback(int cmd)
@@ -1216,3 +1500,80 @@ int toResultView::queryColumns(void) const
   return Query?Query->columns():0;
 }
 
+void toResultView::setSorting(int col,bool asc)
+{
+  if (col==SortColumn&&asc==SortAscending)
+    return;
+  SortColumn=col;
+  SortAscending=asc;
+  if (((col==0&&NumberColumn)||(col==columns()&&!NumberColumn))&&asc==true) {
+    col=-1;
+    toListView::setSorting(0,true);
+    sort();
+    QTimer::singleShot(1,this,SLOT(checkHeading()));
+  } else if (SortConnected) {
+    SortConnected=false;
+    disconnect(header(),SIGNAL(clicked(int)),this,SLOT(headingClicked(int)));
+  }
+  toListView::setSorting(col,asc);
+}
+
+void toResultView::headingClicked(int col)
+{
+  if (col==SortColumn)
+    setSorting(col,!SortAscending);
+  else
+    setSorting(col,true);
+}
+
+void toResultView::checkHeading(void)
+{
+  SortConnected=true;
+  connect(header(),SIGNAL(clicked(int)),this,SLOT(headingClicked(int)));
+}
+
+void toResultView::refresh(void)
+{
+  int lastSort=SortColumn;
+  bool lastAsc=SortAscending;
+  toResult::refresh();
+  if (lastSort>=0)
+    setSorting(lastSort,lastAsc);
+}
+
+toResultListFormat::toResultListFormat(QWidget *parent,const char *name)
+  : toResultListFormatUI(parent,name,true)
+{
+  Format->insertItem(tr("Text"));
+  Format->insertItem(tr("Tab delimited"));
+  Format->insertItem(tr("CSV"));
+  Format->insertItem(tr("HTML"));
+  int num=toTool::globalConfig(CONF_DEFAULT_FORMAT,"").toInt();
+  Format->setCurrentItem(num);
+  formatChanged(num);
+
+  Delimiter->setText(toTool::globalConfig(CONF_CSV_DELIMITER,DEFAULT_CSV_DELIMITER));
+  Separator->setText(toTool::globalConfig(CONF_CSV_SEPARATOR,DEFAULT_CSV_SEPARATOR));
+}
+
+void toResultListFormat::formatChanged(int pos)
+{
+  Separator->setEnabled(pos==2);
+  Delimiter->setEnabled(pos==2);
+}
+
+void toResultListFormat::saveDefault(void)
+{
+  toTool::globalSetConfig(CONF_CSV_DELIMITER,Delimiter->text());
+  toTool::globalSetConfig(CONF_CSV_SEPARATOR,Separator->text());
+  toTool::globalSetConfig(CONF_DEFAULT_FORMAT,QString::number(Format->currentItem()));
+}
+
+void toResultFilter::exportData(std::map<QCString,QString> &,const QCString &)
+{
+}
+
+void toResultFilter::importData(std::map<QCString,QString> &,const QCString &)
+{
+}
+
