@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -37,6 +37,7 @@
 
 #include "toeditwidget.h"
 #include "toresult.h"
+#include "toresultlistformatui.h"
 
 #include <qlistview.h>
 
@@ -75,14 +76,12 @@ public:
    * @param data A map that can be used to recreate the data of a chart.
    * @param prefix Prefix to add to the map.
    */
-  virtual void exportData(std::map<QString,QString> &data,const QString &prefix)
-  { }
+  virtual void exportData(std::map<QCString,QString> &data,const QCString &prefix);
   /** Import data
    * @param data Data to read from a map.
    * @param prefix Prefix to read data from.
    */
-  virtual void importData(std::map<QString,QString> &data,const QString &prefix)
-  { }
+  virtual void importData(std::map<QCString,QString> &data,const QCString &prefix);
 };
 
 /** An item to display in a toListView or toResultView. They differ from normal
@@ -90,6 +89,18 @@ public:
  * than is displayed in the cell of the listview.
  */
 class toResultViewItem : public QListViewItem {
+  struct keyData {
+    QString Data;
+    QString KeyAsc;
+    QString KeyDesc;
+    int Width;
+    enum { String,Number } Type;
+  };
+  int ColumnCount;
+  keyData *ColumnData;
+  QString firstText(int col) const;
+protected:
+  virtual int realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const;
 public:
   /** Create a new item.
    * @param parent Parent list view.
@@ -98,7 +109,7 @@ public:
    */
   toResultViewItem(QListView *parent,QListViewItem *after,const QString &buf=QString::null)
     : QListViewItem(parent,after,QString::null)
-  { if (buf) setText(0,buf); }
+  { ColumnData=NULL; ColumnCount=0; if (buf) setText(0,buf); }
   /** Create a new item.
    * @param parent Parent to this item.
    * @param after Insert after this item.
@@ -106,7 +117,17 @@ public:
    */
   toResultViewItem(QListViewItem *parent,QListViewItem *after,const QString &buf=QString::null)
     : QListViewItem(parent,after,QString::null)
-  { if (buf) setText(0,buf); }
+  { ColumnData=NULL; ColumnCount=0; if (buf) setText(0,buf); }
+  /** Reimplemented for internal reasons.
+   */
+  virtual ~toResultViewItem()
+  { delete[] ColumnData; }
+  /** Reimplemented for internal reasons.
+   */
+  virtual void setText (int col,const QString &txt);
+  /** Set from database.
+   */
+  virtual void setText (int col,const toQValue &val);
   /** Reimplemented for internal reasons.
    */
   virtual void paintCell(QPainter * p,const QColorGroup & cg,int column,int width,int align);
@@ -118,16 +139,18 @@ public:
    * @param col Column
    * @param asc Wether to sort ascending or not.
    */
-  virtual QString key(int col,bool asc) const;
+  virtual QString key(int col,bool asc) const
+  { if (col>=ColumnCount) return QString::null; return asc?ColumnData[col].KeyAsc:ColumnData[col].KeyDesc; }
   /** Reimplemented for internal reasons.
    */
-  virtual int width(const QFontMetrics &fm, const QListView *top, int column) const;
+  virtual int width(const QFontMetrics &, const QListView *, int col) const
+  { if (col>=ColumnCount) return 0; return ColumnData[col].Width; }
   /** Get all text for this item. This is used for copying, drag & drop and memo editing etc.
    * @param col Column.
    * @return All of the text.
    */
   virtual QString allText(int col) const
-  { return QListViewItem::text(col); }
+  { if (col>=ColumnCount) return QString::null; return ColumnData[col].Data; }
   /** Get the text to be displayed as tooltip for this item.
    * @param col Column.
    * @return The text to display as tooltip.
@@ -143,6 +166,8 @@ private:
   /** Number of lines in the largest row.
    */
   int Lines;
+protected:
+  virtual int realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const;
 public:
   /** Create a new item.
    * @param parent Parent list view.
@@ -163,20 +188,16 @@ public:
   /** Reimplemented for internal reasons.
    */
   virtual void setText (int,const QString &);
+  /** Set from database.
+   */
+  virtual void setText (int col,const toQValue &val);
   /** Reimplemented for internal reasons.
    */
   virtual void setup(void);
   /** Reimplemented for internal reasons.
    */
   virtual QString text(int col) const
-  { return QListViewItem::text(col); }
-  /** Reimplemented for internal reasons.
-   */
-  virtual QString allText(int col) const
-  { return QListViewItem::text(col); }
-  /** Reimplemented for internal reasons.
-   */
-  virtual int width(const QFontMetrics &fm, const QListView *top, int column) const;
+  { return toResultViewItem::allText(col); }
   /** Reimplemented for internal reasons.
    */
   virtual void paintCell (QPainter *pnt,const QColorGroup & cg,int column,int width,int alignment);
@@ -187,6 +208,18 @@ public:
  * than is displayed in the cell of the listview.
  */
 class toResultViewCheck : public QCheckListItem {
+  struct keyData {
+    QString Data;
+    QString KeyAsc;
+    QString KeyDesc;
+    int Width;
+    enum { String,Number } Type;
+  };
+  int ColumnCount;
+  keyData *ColumnData;
+protected:
+  virtual int realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const;
+  QString firstText(int col) const;
 public:
   /** Create a new item.
    * @param parent Parent list view.
@@ -195,7 +228,7 @@ public:
    */
   toResultViewCheck(QListView *parent,const QString &text,QCheckListItem::Type type=Controller)
     : QCheckListItem(parent,QString::null,type)
-  { if (!text.isNull()) setText(0,text); }
+  { ColumnData=NULL; ColumnCount=0; if (!text.isNull()) setText(0,text); }
   /** Create a new item.
    * @param parent Parent item.
    * @param text Text of first column.
@@ -203,7 +236,31 @@ public:
    */
   toResultViewCheck(QListViewItem *parent,const QString &text,QCheckListItem::Type type=Controller)
     : QCheckListItem(parent,QString::null,type)
-  { if (!text.isNull()) setText(0,text); }
+  { ColumnData=NULL; ColumnCount=0; if (!text.isNull()) setText(0,text); }
+  /** Create a new item.
+   * @param parent Parent list view.
+   * @param after After last item.
+   * @param text Text of first column.
+   * @param type Type of check on this item.
+   */
+  toResultViewCheck(QListView *parent,QListViewItem *after,const QString &text,QCheckListItem::Type type=Controller);
+  /** Create a new item.
+   * @param parent Parent item.
+   * @param after After last item.
+   * @param text Text of first column.
+   * @param type Type of check on this item.
+   */
+  toResultViewCheck(QListViewItem *parent,QListViewItem *after,const QString &text,QCheckListItem::Type type=Controller);
+  /** Reimplemented for internal reasons.
+   */
+  virtual ~toResultViewCheck()
+  { delete[] ColumnData; }
+  /** Reimplemented for internal reasons.
+   */
+  virtual void setText (int col,const QString &txt);
+  /** Set from database.
+   */
+  virtual void setText (int col,const toQValue &val);
   /** Reimplemented for internal reasons.
    */
   virtual void paintCell(QPainter * p,const QColorGroup & cg,int column,int width,int align);
@@ -215,16 +272,23 @@ public:
    * @param col Column
    * @param asc Wether to sort ascending or not.
    */
-  virtual QString key(int col,bool) const;
+  /** String to sort the data on. This is reimplemented so that numbers are sorted as numbers
+   * and not as strings.
+   * @param col Column
+   * @param asc Wether to sort ascending or not.
+   */
+  virtual QString key(int col,bool asc) const
+  { if (col>=ColumnCount) return QString::null; return asc?ColumnData[col].KeyAsc:ColumnData[col].KeyDesc; }
   /** Reimplemented for internal reasons.
    */
-  virtual int width(const QFontMetrics &fm, const QListView *top, int column) const;
+  virtual int width(const QFontMetrics &, const QListView *, int col) const
+  { if (col>=ColumnCount) return 0; return ColumnData[col].Width; }
   /** Get all text for this item. This is used for copying, drag & drop and memo editing etc.
    * @param col Column.
    * @return All of the text.
    */
   virtual QString allText(int col) const
-  { return QCheckListItem::text(col); }
+  { if (col>=ColumnCount) return QString::null; return ColumnData[col].Data; }
   /** Get the text to be displayed as tooltip for this item.
    * @param col Column.
    * @return The text to display as tooltip.
@@ -240,6 +304,8 @@ private:
   /** Number of lines in the largest row.
    */
   int Lines;
+protected:
+  virtual int realWidth(const QFontMetrics &fm, const QListView *top, int column,const QString &txt) const;
 public:
   /** Create a new item.
    * @param parent Parent list view.
@@ -263,17 +329,13 @@ public:
   /** Reimplemented for internal reasons.
    */
   virtual void setText (int,const QString &);
+  /** Set from database.
+   */
+  virtual void setText (int col,const toQValue &val);
   /** Reimplemented for internal reasons.
    */
   virtual QString text(int col) const
-  { return QListViewItem::text(col); }
-  /** Reimplemented for internal reasons.
-   */
-  virtual QString allText(int col) const
-  { return QListViewItem::text(col); }
-  /** Reimplemented for internal reasons.
-   */
-  virtual int width(const QFontMetrics &fm, const QListView *top, int column) const;
+  { return toResultViewCheck::allText(col); }
   /** Reimplemented for internal reasons.
    */
   virtual void paintCell (QPainter *pnt,const QColorGroup & cg,int column,int width,int alignment);
@@ -288,6 +350,8 @@ public:
 class toListView : public QListView,public toEditWidget {
   Q_OBJECT
 
+  bool FirstSearch;
+
   /** Name of this list, used primarily when printing. Also used to be able to edit
    * SQL displayed in @ref toResultView.
    */
@@ -333,7 +397,7 @@ class toListView : public QListView,public toEditWidget {
    */
   virtual QListViewItem *printPage(TOPrinter *printer,QPainter *painter,QListViewItem *top,
 				   int &column,int &level,int pageNo,bool paint=true);
-  int exportType(void);
+  int exportType(QString &separator,QString &delimiter);
 public:
   /** Create new list view.
    * @param parent Parent of list.
@@ -356,9 +420,6 @@ public:
   /** Print this list
    */
   virtual void editPrint(void);
-  /** Reimplemented for internal reasons.
-   */
-  virtual void editSearch(toSearchReplace *);
   /** Reimplemented for internal reasons.
    */
   virtual void focusInEvent (QFocusEvent *e);
@@ -370,33 +431,51 @@ public:
   /** Adds option to add menues to the popup menu before it is displayed.
    * @param menu Menu to add entries to.
    */
-  virtual void addMenues(QPopupMenu *menu)
-  { }
+  virtual void addMenues(QPopupMenu *menu);
   /** Export list as a string.
    * @param includeHeader Include header.
    * @param onlySelection Only include selection.
    * @param type Format of exported list.
+   * @param separator Separator for CSV format.
+   * @param delimiter Delimiter for CSV format.
    */
-  virtual QString exportAsText(bool includeHeader,bool onlySelection,int type=-1);
+  virtual QString exportAsText(bool includeHeader,bool onlySelection,int type=-1,const QString &separator=";",const QString &delimiter="\"");
   /** Export list as file.
    */
   virtual bool editSave(bool ask);
 
+  /** Move to top of data
+   */
+  virtual void searchTop(void)
+  { if (firstChild()) setCurrentItem(firstChild()); FirstSearch=true; }
+  /** Search for next entry
+   * @return True if found, should select the found text.
+   */
+  virtual bool searchNext(toSearchReplace *search);
+  /** Check if data can be modified by search
+   * @param all If true can replace all, otherwise can replace right now.
+   */
+  virtual bool searchCanReplace(bool all);
+
   /** Export data to a map.
    * @param data A map that can be used to recreate the data of a chart.
    * @param prefix Prefix to add to the map.
    */
-  virtual void exportData(std::map<QString,QString> &data,const QString &prefix);
+  virtual void exportData(std::map<QCString,QString> &data,const QCString &prefix);
   /** Import data
    * @param data Data to read from a map.
    * @param prefix Prefix to read data from.
    */
-  virtual void importData(std::map<QString,QString> &data,const QString &prefix);
+  virtual void importData(std::map<QCString,QString> &data,const QCString &prefix);
   /** Create transposed copy of list
    * @return Pointer to newly allocated transposed listview.
    */
   virtual toListView *copyTransposed(void);
 public slots:
+  /** set the popup menu --> see displayMenu()
+   * @param item Item to display.
+   */
+  virtual void setDisplayMenu(QPopupMenu *item);
   /** Display the menu at the given point and column.
    * @param item Item to display.
    * @param pnt Point to display menu at.
@@ -424,6 +503,10 @@ protected slots:
 class toResultView : public toListView, public toResult {
   Q_OBJECT
 
+  int SortColumn;
+  bool SortAscending;
+  bool SortConnected;
+
   /** Reimplemented for internal reasons.
    */
   virtual void keyPressEvent (QKeyEvent * e);
@@ -493,6 +576,11 @@ public:
    */
   int queryColumns() const;
 
+  /** Get the query used to execute this.
+   */
+  toQuery *query()
+  { return Query; }
+
   /** Set a filter to this list.
    * @param filter The new filter or NULL if no filter is to be used.
    */
@@ -533,8 +621,7 @@ public:
    * @param str String to set first column to.
    * @return Allocated item.
    */
-  virtual QListViewItem *createItem(QListViewItem *last,const QString &str)
-  { return new toResultViewItem(this,last,str); }
+  virtual QListViewItem *createItem(QListViewItem *last,const QString &str);
 
   /** Reimplemented for internal reasons.
    */
@@ -592,11 +679,17 @@ public:
   /** Reimplemented for internal reasons.
    */
   virtual void addMenues(QPopupMenu *);
+  /** Reimplemented for internal reasons.
+   */
+  virtual void setSorting(int col,bool asc=true);
+  /** Reimplemented for internal reasons.
+   */
+  virtual int sortColumn() const
+  { return SortColumn; }
 public slots:
   /** Reimplemented for internal reasons.
    */
-  virtual void refresh(void)
-  { toResult::refresh(); }
+  virtual void refresh(void);
   /** Reimplemented for internal reasons.
    */
   virtual void changeParams(const QString &Param1)
@@ -616,10 +709,27 @@ public slots:
    */
   virtual bool canHandle(toConnection &)
   { return true; }
+private slots:
+  void headingClicked(int col);
+  void checkHeading(void); 
 protected slots:
   /** Reimplemented for internal reasons.
    */
   virtual void menuCallback(int);
 };
 
+/***
+ * Used internally by toListView.
+ * @internal
+ */
+
+class toResultListFormat : public toResultListFormatUI {
+  Q_OBJECT
+public:
+  toResultListFormat(QWidget *parent,const char *name);
+  void saveDefault(void);
+public slots:
+  virtual void formatChanged(int pos);
+};
+
 #endif
