@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -95,13 +95,16 @@
 #define CONF_NEED_READ  "NeedRead"
 #define CONF_NEED_TWO   "NeedTwo"
 #define CONF_ALIGN_LEFT "AlignLeft"
+#define CONF_OLD_ENABLE "OldEnable"
 
 class toRollbackPrefs : public QGroupBox,public toSettingTab
 { 
+  QCheckBox *OldEnable;
   QCheckBox *NoExec;
   QCheckBox *NeedRead;
   QCheckBox *NeedTwo;
   QCheckBox *AlignLeft;
+
   toTool *Tool;
 
 public:
@@ -112,24 +115,38 @@ public:
 toRollbackPrefs::toRollbackPrefs(toTool *tool,QWidget* parent,const char* name)
   : QGroupBox(1,Horizontal,parent,name),toSettingTab("rollback.html#options"),Tool(tool)
 {
-  setTitle(tr("Rollback Tool" ));
+  setTitle(qApp->translate("toRollbackPrefs","Rollback Tool" ));
 
-  NoExec=new QCheckBox(this,"NoCopy");
-  NoExec->setText(tr("&Restart reexecuted statements" ));
-  QToolTip::add(NoExec,tr("Start statements again that have been reexecuted."));
-  
-  NeedRead=new QCheckBox(this,"Needread");
-  NeedRead->setText(tr("&Must read buffers" ));
-  QToolTip::add(NeedRead,tr("Don't display statements that have not read buffers."));
-  
-  NeedTwo=new QCheckBox(this,"NeedTwo");
-  NeedTwo->setText(tr("&Exclude first appearance" ));
-  QToolTip::add(NeedTwo,tr("A statement must be visible at least two consecutive polls to be displayed."));
+  OldEnable=new QCheckBox(this,"OldEnable");
+  OldEnable->setText(qApp->translate("toRollbackPrefs","&Enable snapshot too old detection." ));
+  QToolTip::add(OldEnable,qApp->translate("toRollbackPrefs","Enable snapshot too old detection, will put load on large databases."));
   
   AlignLeft=new QCheckBox(this,"AlignLeft");
-  AlignLeft->setText(tr("&Disregard start extent." ));
-  QToolTip::add(AlignLeft,tr("Always start from the left border when displaying extent usage."));
+  AlignLeft->setText(qApp->translate("toRollbackPrefs","&Disregard start extent." ));
+  AlignLeft->setEnabled(false);
+  connect(OldEnable,SIGNAL(toggled(bool)),AlignLeft,SLOT(setEnabled(bool)));
+  QToolTip::add(AlignLeft,qApp->translate("toRollbackPrefs","Always start from the left border when displaying extent usage."));
   
+  NoExec=new QCheckBox(this,"NoCopy");
+  NoExec->setText(qApp->translate("toRollbackPrefs","&Restart reexecuted statements" ));
+  NoExec->setEnabled(false);
+  connect(OldEnable,SIGNAL(toggled(bool)),NoExec,SLOT(setEnabled(bool)));
+  QToolTip::add(NoExec,qApp->translate("toRollbackPrefs","Start statements again that have been reexecuted."));
+  
+  NeedRead=new QCheckBox(this,"Needread");
+  NeedRead->setText(qApp->translate("toRollbackPrefs","&Must read buffers" ));
+  NeedRead->setEnabled(false);
+  connect(OldEnable,SIGNAL(toggled(bool)),NeedRead,SLOT(setEnabled(bool)));
+  QToolTip::add(NeedRead,qApp->translate("toRollbackPrefs","Don't display statements that have not read buffers."));
+  
+  NeedTwo=new QCheckBox(this,"NeedTwo");
+  NeedTwo->setText(qApp->translate("toRollbackPrefs","&Exclude first appearance" ));
+  NeedTwo->setEnabled(false);
+  connect(OldEnable,SIGNAL(toggled(bool)),NeedTwo,SLOT(setEnabled(bool)));
+  QToolTip::add(NeedTwo,qApp->translate("toRollbackPrefs","A statement must be visible at least two consecutive polls to be displayed."));
+  
+  if (!tool->config(CONF_OLD_ENABLE,"").isEmpty())
+    OldEnable->setChecked(true);
   if (!tool->config(CONF_NO_EXEC,"Yes").isEmpty())
     NoExec->setChecked(true);
   if (!tool->config(CONF_NEED_READ,"Yes").isEmpty())
@@ -146,6 +163,7 @@ void toRollbackPrefs::saveSetting(void)
   Tool->setConfig(CONF_NEED_READ,NeedRead->isChecked()?"Yes":"");
   Tool->setConfig(CONF_NEED_TWO,NeedTwo->isChecked()?"Yes":"");
   Tool->setConfig(CONF_ALIGN_LEFT,AlignLeft->isChecked()?"Yes":"");
+  Tool->setConfig(CONF_OLD_ENABLE,OldEnable->isChecked()?"Yes":"");
 }
 
 class toRollbackTool : public toTool {
@@ -191,24 +209,24 @@ toRollbackDialog::toRollbackDialog(toConnection &Connection,QWidget* parent,cons
 
   Storage=new toStorageDefinition(DialogTabs);
   Storage->forRollback(true);
-  DialogTabs->addTab(Storage,"Storage");
+  DialogTabs->addTab(Storage,tr("Storage"));
 
   OkButton->setEnabled(false);
 }
 
 std::list<QString> toRollbackDialog::sql(void)
 {
-  QString str("CREATE ");
+  QString str(QString::fromLatin1("CREATE "));
   if (Public->isChecked())
-    str.append("PUBLIC ");
-  str.append("ROLLBACK SEGMENT \"");
+    str.append(QString::fromLatin1("PUBLIC "));
+  str.append(QString::fromLatin1("ROLLBACK SEGMENT \""));
   str.append(Name->text());
-  str.append("\" TABLESPACE \"");
+  str.append(QString::fromLatin1("\" TABLESPACE \""));
   str.append(Tablespace->currentText());
-  str.append("\"");
+  str.append(QString::fromLatin1("\""));
   std::list<QString> lst=Storage->sql();
   for(std::list<QString>::iterator i=lst.begin();i!=lst.end();i++) {
-    str+=" ";
+    str+=QString::fromLatin1(" ");
     str+=*i;
   }
   std::list<QString> ret;
@@ -222,16 +240,16 @@ void toRollbackDialog::displaySQL(void)
   QString res;
   for(std::list<QString>::iterator i=lines.begin();i!=lines.end();i++) {
     res+=*i;
-    res+=";\n";
+    res+=QString::fromLatin1(";\n");
   }
   if (res.length()>0) {
     toMemoEditor memo(this,res,-1,-1,true,true);
     memo.exec();
   } else
-    toStatusMessage("No changes made",false,false);
+    toStatusMessage(tr("No changes made"),false,false);
 }
 
-void toRollbackDialog::valueChanged(const QString &str)
+void toRollbackDialog::valueChanged(const QString &)
 {
   if (Name->text().isEmpty()||
       Tablespace->currentText().isEmpty())
@@ -241,7 +259,7 @@ void toRollbackDialog::valueChanged(const QString &str)
 }
 
 #define MIN_HEIGHT 4
-#define TRANSCOL 18
+#define TRANSCOL 17
 
 static bool BarsAlignLeft=true;
 
@@ -294,34 +312,33 @@ static void PaintBars(QListViewItem *item,QPainter *p,const QColorGroup & cg,
 }
 
 static toSQL SQLRollback("toRollback:Information",
-			 "select a.segment_name \"Segment\",\n"
+			 "SELECT a.segment_name \"Segment\",\n"
 			 "       a.owner \"Owner\",\n"
 			 "       a.tablespace_name \"Tablespace\",\n"
 			 "       a.status \"Status\",\n"
 			 "       b.xacts \"-Transactions\",\n"
-			 "       ROUND(a.initial_extent/unit,3) \"-Initial (MB)\",\n"
-			 "       ROUND(a.next_extent/unit,3) \"-Next (MB)\",\n"
+			 "       ROUND ( a.initial_extent / :unit<char[100]>,\n"
+			 "	       3 ) \"-Initial (MB)\",\n"
+			 "       ROUND ( a.next_extent / :unit<char[100]>,\n"
+			 "	       3 ) \"-Next (MB)\",\n"
 			 "       a.pct_increase \"-PCT Increase\",\n"
-			 "       ROUND(b.rssize/unit,3) \"-Current (MB)\",\n"
-			 "       ROUND(b.optsize/unit,3) \"-Optimal (MB)\",\n"
-			 "       ROUND(b.aveactive/unit,3) \"-Used (MB)\",\n"
-			 "       b.Extents \"-Extents\",\n"
+			 "       ROUND ( b.rssize / :unit<char[100]>,\n"
+			 "	       3 ) \"-Current (MB)\",\n"
+			 "       ROUND ( b.optsize / :unit<char[100]>,\n"
+			 "	       3 ) \"-Optimal (MB)\",\n"
+			 "       ROUND ( b.aveactive / :unit<char[100]>,\n"
+			 "	       3 ) \"-Used (MB)\",\n"
+			 "       b.EXTENTS \"-Extents\",\n"
 			 "       b.CurExt \"-Current\",\n"
 			 "       b.CurBlk \"-Block\",\n"
-			 "       c.Blocks \"-Blocks\",\n"
 			 "       b.gets \"-Reads\",\n"
-			 "       ROUND((b.gets-b.waits)*100/b.gets,2)||'%' \"-Hitrate\",\n"
+			 "       ROUND ( ( b.gets - b.waits ) * 100 / b.gets,\n"
+			 "	       2 ) || '%' \"-Hitrate\",\n"
 			 "       a.segment_id \" USN\"\n"
-			 "  from sys.dba_rollback_segs a,\n"
-			 "       v$rollstat b,\n"
-			 "       sys.dba_extents c,\n"
-			 "       (select :unit<char[100]> unit from sys.dual) d\n"
-			 " where a.segment_id = b.usn(+)\n"
-			 "   and (a.owner = c.owner or a.owner = 'PUBLIC')\n"
-			 "   and a.segment_name = c.segment_name\n"
-			 "   and c.segment_type = 'ROLLBACK'\n"
-			 "   and (c.extent_id = b.CurExt or (b.curext is null and c.extent_id = 0))\n"
-			 " order by a.segment_name",
+			 "  FROM sys.dba_rollback_segs a,\n"
+			 "       v$rollstat b\n"
+			 " WHERE a.segment_id = b.usn ( + )\n"
+			 " ORDER BY a.segment_name",
 			 "Get information about rollback segments.");
 
 static toSQL SQLStartExt("toRollback:StartExtent",
@@ -346,7 +363,7 @@ public:
 	std::list<double> maxExt;
 	for (int i=TRANSCOL;!text(i).isEmpty();i++) {
 	  items.insert(items.end(),text(i).toDouble());
-	  curExt.insert(curExt.end(),text(12).toDouble()+text(13).toDouble()/text(14).toDouble());
+	  curExt.insert(curExt.end(),text(12).toDouble());
 	  maxExt.insert(maxExt.end(),text(11).toDouble());
 	}
 	PaintBars(this,pnt,cg,width,items,
@@ -377,17 +394,17 @@ public:
   {
     setSQL(SQLRollback);
   }
-  virtual void query(const QString &sql,const toQList &param)
+  virtual void query(const QString &sql,const toQList &)
   {
     QString unit=toTool::globalConfig(CONF_SIZE_UNIT,DEFAULT_SIZE_UNIT);
     
     toQList par;
     par.insert(par.end(),QString::number(toSizeDecode(unit)));
     toResultView::query(sql,(const toQList &)par);
-    QRegExp repl("(MB)");
-    QString res="(";
+    QRegExp repl(QString::fromLatin1("(MB)"));
+    QString res=QString::fromLatin1("(");
     res+=unit;
-    res+=")";
+    res+=QString::fromLatin1(")");
     for (int i=0;i<columns();i++) {
       QString str=header()->label(i);
       str.replace(repl,res);
@@ -489,15 +506,15 @@ public:
   toRollbackOpen(QWidget *parent)
     : toResultView(false,false,parent)
   {
-    addColumn("Started");
-    addColumn("User");
-    addColumn("Snapshot");
-    addColumn("SQL");
+    addColumn(qApp->translate("toRollbackOpen","Started"));
+    addColumn(qApp->translate("toRollbackOpen","User"));
+    addColumn(qApp->translate("toRollbackOpen","Snapshot"));
+    addColumn(qApp->translate("toRollbackOpen","SQL"));
     setSorting(0);
     NumExtents=0;
-    setSQLName("toRollbackOpen");
+    setSQLName(QString::fromLatin1("toRollbackOpen"));
   }
-  virtual void query(const QString &sql,const toQList &param)
+  virtual void query(const QString &,const toQList &)
   {
     try {
       clear();
@@ -506,7 +523,7 @@ public:
       toQuery sql(conn,SQLStatementInfo);
       QListViewItem *last=NULL;
       while(!sql.eof()) {
-	QListViewItem *item=createItem(last,NULL);
+	QListViewItem *item=createItem(last,QString::null);
 	last=item;
 	item->setText(0,sql.readValue());
 	item->setText(1,sql.readValue());
@@ -600,48 +617,55 @@ toSQL SQLTransactionUsers("toRollback:TransactionUsers",
 toRollback::toRollback(QWidget *main,toConnection &connection)
   : toToolWidget(RollbackTool,"rollback.html",main,connection)
 {
-  QToolBar *toolbar=toAllocBar(this,"Rollback analyzer",connection.description());
+  QToolBar *toolbar=toAllocBar(this,tr("Rollback analyzer"));
 
   new QToolButton(QPixmap((const char **)refresh_xpm),
-		  "Update segment list",
-		  "Update segment list",
+		  tr("Update segment list"),
+		  tr("Update segment list"),
 		  this,SLOT(refresh(void)),
 		  toolbar);
   toolbar->addSeparator();
 
+  QToolButton *enableOld=new QToolButton(toolbar);
+  enableOld->setToggleButton(true);
+  enableOld->setIconSet(QIconSet(QPixmap((const char **)torollback_xpm)));
+  connect(enableOld,SIGNAL(toggled(bool)),this,SLOT(enableOld(bool)));
+  QToolTip::add(enableOld,tr("Enable snapshot too old detection."));
+  toolbar->addSeparator();
+
   OnlineButton=new QToolButton(QPixmap((const char **)online_xpm),
-			       "Take segment online",
-			       "Take segment online",
+			       tr("Take segment online"),
+			       tr("Take segment online"),
 			       this,SLOT(online(void)),
 			       toolbar);
   OnlineButton->setEnabled(false);
   OfflineButton=new QToolButton(QPixmap((const char **)offline_xpm),
-				"Take segment offline",
-				"Take segment offline",
+				tr("Take segment offline"),
+				tr("Take segment offline"),
 				this,SLOT(offline(void)),
 				toolbar);
   OfflineButton->setEnabled(false);
   toolbar->addSeparator();
 
   new QToolButton(QPixmap((const char **)addrollback_xpm),
-		  "Create new rollback segment",
-		  "Create new rollback segment",
+		  tr("Create new rollback segment"),
+		  tr("Create new rollback segment"),
 		  this,SLOT(addSegment(void)),
 		  toolbar);
   DropButton=new QToolButton(QPixmap((const char **)trash_xpm),
-			     "Drop segment",
-			     "Drop segment",
+			     tr("Drop segment"),
+			     tr("Drop segment"),
 			     this,SLOT(dropSegment(void)),
 			     toolbar);
   DropButton->setEnabled(false);
   toolbar->addSeparator();
 
-  new QLabel("Refresh ",toolbar);
-  connect(Refresh=toRefreshCreate(toolbar),
+  new QLabel(tr("Refresh")+" ",toolbar,TO_KDE_TOOLBAR_WIDGET);
+  connect(Refresh=toRefreshCreate(toolbar,TO_KDE_TOOLBAR_WIDGET),
 	  SIGNAL(activated(const QString &)),this,SLOT(changeRefresh(const QString &)));
 
-  toolbar->setStretchableWidget(new QLabel("",toolbar));
-  new toChangeConnection(toolbar);
+  toolbar->setStretchableWidget(new QLabel(toolbar,TO_KDE_TOOLBAR_WIDGET));
+  new toChangeConnection(toolbar,TO_KDE_TOOLBAR_WIDGET);
 
   QSplitter *splitter=new QSplitter(Vertical,this);
   Segments=new toRollbackView(splitter);
@@ -650,26 +674,33 @@ toRollback::toRollback(QWidget *main,toConnection &connection)
 	  this,SLOT(changeItem(QListViewItem *)));
 
   QTabWidget *tab=new QTabWidget(splitter,"TabWidget");
-  QSplitter *horsplit=new QSplitter(Horizontal,splitter);
-  tab->addTab(horsplit,"Open Cursors");
-  
   TransactionUsers=new toResultLong(false,false,toQuery::Background,tab);
-  tab->addTab(TransactionUsers,"Transaction Users");
+  tab->addTab(TransactionUsers,tr("Transaction Users"));
   TransactionUsers->setSQL(SQLTransactionUsers);
 
+  QSplitter *horsplit=new QSplitter(Horizontal,splitter);
+  tab->addTab(horsplit,tr("Open Cursors"));
+  
   Statements=new toRollbackOpen(horsplit);
   Statements->setSelectionMode(QListView::Single);
   connect(Statements,SIGNAL(selectionChanged(QListViewItem *)),
 	  this,SLOT(changeStatement(QListViewItem *)));
   CurrentStatement=new toSGAStatement(horsplit);
 
-  connect(timer(),SIGNAL(timeout(void)),this,SLOT(refresh(void)));
-  toRefreshParse(timer(),toTool::globalConfig(CONF_REFRESH,DEFAULT_REFRESH));
+  try {
+    connect(timer(),SIGNAL(timeout(void)),this,SLOT(refresh(void)));
+    toRefreshParse(timer(),toTool::globalConfig(CONF_REFRESH,DEFAULT_REFRESH));
+  } TOCATCH
 
   ToolMenu=NULL;
   connect(toMainWidget()->workspace(),SIGNAL(windowActivated(QWidget *)),
 	  this,SLOT(windowActivated(QWidget *)));
 
+  if (!RollbackTool.config(CONF_OLD_ENABLE,"").isEmpty())
+    enableOld->setOn(true);
+  else
+    Statements->setEnabled(false);
+
   refresh();
 
   setFocusProxy(Segments);
@@ -685,25 +716,25 @@ void toRollback::windowActivated(QWidget *widget)
   if (widget==this) {
     if (!ToolMenu) {
       ToolMenu=new QPopupMenu(this);
-      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),"&Refresh",
+      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),tr("&Refresh"),
 			   this,SLOT(refresh(void)),Key_F5);
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem(QPixmap((const char **)online_xpm),"Online",
+      ToolMenu->insertItem(QPixmap((const char **)online_xpm),tr("Online"),
 			   this,SLOT(online(void)),0,TO_ID_ONLINE);
-      ToolMenu->insertItem(QPixmap((const char **)offline_xpm),"Offline",
+      ToolMenu->insertItem(QPixmap((const char **)offline_xpm),tr("Offline"),
 			   this,SLOT(offline(void)),0,TO_ID_OFFLINE);
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem(QPixmap((const char **)addrollback_xpm),"Create segment...",
+      ToolMenu->insertItem(QPixmap((const char **)addrollback_xpm),tr("Create segment..."),
 			   this,SLOT(addSegment(void)),0,TO_ID_CREATE);
-      ToolMenu->insertItem(QPixmap((const char **)trash_xpm),"Drop segment",
+      ToolMenu->insertItem(QPixmap((const char **)trash_xpm),tr("Drop segment"),
 			   this,SLOT(dropSegment(void)),0,TO_ID_DROP);
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem("&Change Refresh",Refresh,SLOT(setFocus(void)),
+      ToolMenu->insertItem(tr("&Change Refresh"),Refresh,SLOT(setFocus(void)),
 			   Key_R+ALT);
-      toMainWidget()->menuBar()->insertItem("&Rollback",ToolMenu,-1,toToolMenuIndex());
-      toMainWidget()->menuBar()->setItemEnabled(TO_ID_ONLINE,OnlineButton->isEnabled());
-      toMainWidget()->menuBar()->setItemEnabled(TO_ID_OFFLINE,OfflineButton->isEnabled());
-      toMainWidget()->menuBar()->setItemEnabled(TO_ID_DROP,DropButton->isEnabled());
+      toMainWidget()->menuBar()->insertItem(tr("&Rollback"),ToolMenu,-1,toToolMenuIndex());
+      ToolMenu->setItemEnabled(TO_ID_ONLINE,OnlineButton->isEnabled());
+      ToolMenu->setItemEnabled(TO_ID_OFFLINE,OfflineButton->isEnabled());
+      ToolMenu->setItemEnabled(TO_ID_DROP,DropButton->isEnabled());
     }
   } else {
     delete ToolMenu;
@@ -726,18 +757,20 @@ void toRollback::refresh(void)
 	Segments->setSelected(item,true);
 	break;
       }
-  item=Statements->selectedItem();
-  if (item)
-    current=item->text(4);
-  else
-    current="";
-  Statements->refresh();
-  if (!current.isEmpty())
-    for (item=Statements->firstChild();item;item=item->nextSibling())
-      if (item->text(4)==current) {
-	Statements->setSelected(item,true);
-	break;
-      }
+  if (Statements->isEnabled()) {
+    item=Statements->selectedItem();
+    if (item)
+      current=item->text(4);
+    else
+      current="";
+    Statements->refresh();
+    if (!current.isEmpty())
+      for (item=Statements->firstChild();item;item=item->nextSibling())
+	if (item->text(4)==current) {
+	  Statements->setSelected(item,true);
+	  break;
+	}
+  }
   TransactionUsers->refresh();
 }
 
@@ -750,11 +783,11 @@ void toRollback::changeStatement(QListViewItem *item)
 void toRollback::changeItem(QListViewItem *item)
 {
   if (item) {
-    if (item->text(3)=="OFFLINE")
+    if (item->text(3)==QString::fromLatin1("OFFLINE"))
       OfflineButton->setEnabled(false);
     else
       OfflineButton->setEnabled(true);
-    if (item->text(3)=="ONLINE")
+    if (item->text(3)==QString::fromLatin1("ONLINE"))
       OnlineButton->setEnabled(false);
     else
       OnlineButton->setEnabled(true);
@@ -763,44 +796,46 @@ void toRollback::changeItem(QListViewItem *item)
     OfflineButton->setEnabled(false);
   }
   DropButton->setEnabled(item);
-  toMainWidget()->menuBar()->setItemEnabled(TO_ID_ONLINE,OnlineButton->isEnabled());
-  toMainWidget()->menuBar()->setItemEnabled(TO_ID_OFFLINE,OfflineButton->isEnabled());
-  toMainWidget()->menuBar()->setItemEnabled(TO_ID_DROP,DropButton->isEnabled());
+  ToolMenu->setItemEnabled(TO_ID_ONLINE,OnlineButton->isEnabled());
+  ToolMenu->setItemEnabled(TO_ID_OFFLINE,OfflineButton->isEnabled());
+  ToolMenu->setItemEnabled(TO_ID_DROP,DropButton->isEnabled());
 }
 
 void toRollback::changeRefresh(const QString &str)
 {
-  toRefreshParse(timer(),str);
+  try {
+    toRefreshParse(timer(),str);
+  } TOCATCH
 }
 
 QString toRollback::currentSegment(void)
 {
   QListViewItem *item=Segments->selectedItem();
   if (!item)
-    throw QString("No segment selected");
+    throw tr("No segment selected");
   return item->text(0);
 }
 
 void toRollback::addSegment(void)
 {
-  toRollbackDialog newSegment(connection(),this);
-  if (newSegment.exec()) {
-    try {
+  try {
+    toRollbackDialog newSegment(connection(),this);
+    if (newSegment.exec()) {
       std::list<QString> sql=newSegment.sql();
       for(std::list<QString>::iterator i=sql.begin();i!=sql.end();i++)
 	connection().execute(*i);
       refresh();
-    } TOCATCH
-  }
+    }
+  } TOCATCH
 }
 
 void toRollback::offline(void)
 {
   try {
     QString str;
-    str="ALTER ROLLBACK SEGMENT \"";
+    str=QString::fromLatin1("ALTER ROLLBACK SEGMENT \"");
     str.append(currentSegment());
-    str.append("\" OFFLINE");
+    str.append(QString::fromLatin1("\" OFFLINE"));
     connection().execute(str);
     refresh();
   } TOCATCH
@@ -810,14 +845,14 @@ void toRollback::dropSegment(void)
 {
   try {
     if (TOMessageBox::warning(this,
-			      "Drop rollback segment",
-			      QString("Are you sure you want to drop the segment %1.").arg(currentSegment()),
-			      "&Drop",
-			      "Cancel")==0) {
+			      tr("Drop rollback segment"),
+			      tr("Are you sure you want to drop the segment %1.").arg(currentSegment()),
+			      tr("&Drop"),
+			      tr("Cancel"))==0) {
       QString str;
-      str="DROP ROLLBACK SEGMENT \"";
+      str=QString::fromLatin1("DROP ROLLBACK SEGMENT \"");
       str.append(currentSegment());
-      str.append("\"");
+      str.append(QString::fromLatin1("\""));
       connection().execute(str);
       refresh();
     }
@@ -828,10 +863,15 @@ void toRollback::online(void)
 {
   try {
     QString str;
-    str="ALTER ROLLBACK SEGMENT \"";
+    str=QString::fromLatin1("ALTER ROLLBACK SEGMENT \"");
     str.append(currentSegment());
-    str.append("\" ONLINE");
+    str.append(QString::fromLatin1("\" ONLINE"));
     connection().execute(str);
     refresh();
   } TOCATCH
 }
+
+void toRollback::enableOld(bool ena)
+{
+  Statements->setEnabled(ena);
+}
