@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -34,6 +34,7 @@
 
 #include "utils.h"
 
+#include "toconf.h"
 #include "toextract.h"
 #include "tofilesize.h"
 #include "tohighlightedtext.h"
@@ -47,14 +48,21 @@
 #include "totool.h"
 #include "toworksheet.h"
 
+#ifdef TO_HAS_KPRINT
+#include <kfiledialog.h>
+#endif
+
 #include <qcheckbox.h>
 #include <qcombobox.h>
+#include <qdir.h>
+#include <qfiledialog.h>
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qlineedit.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
 #include <qregexp.h>
+#include <qspinbox.h>
 #include <qsplitter.h>
 #include <qtabwidget.h>
 #include <qtextview.h>
@@ -120,7 +128,7 @@ static toSQL SQLUserObjects("toScript:UserObjects",
 			    "        SELECT object_type,object_name\n"
 			    "          FROM sys.all_objects\n"
 			    "         WHERE object_type IN ('VIEW','TYPE','SEQUENCE','PACKAGE',\n"
-			    "                               'PACKAGE BODY','FUNCTION','PROCEDURE')\n"
+			    "                               'PACKAGE BODY','FUNCTION','PROCEDURE','TRIGGER')\n"
 			    "           AND owner = :own<char[101]>\n"
 			    "         UNION\n"
 			    "        SELECT 'TABLE',table_name\n"
@@ -143,7 +151,7 @@ static toSQL SQLUserObjects7("toScript:UserObjects",
 			     "        SELECT object_type,object_name\n"
 			     "          FROM sys.all_objects\n"
 			     "         WHERE object_type IN ('VIEW','TYPE','SEQUENCE','PACKAGE',\n"
-			     "                               'PACKAGE BODY','FUNCTION','PROCEDURE')\n"
+			     "                               'PACKAGE BODY','FUNCTION','PROCEDURE','TRIGGER')\n"
 			     "           AND owner = :own<char[101]>\n"
 			     "         UNION\n"
 			     "        SELECT 'TABLE',table_name\n"
@@ -155,7 +163,7 @@ static toSQL SQLUserObjects7("toScript:UserObjects",
 			     "          FROM sys.all_mviews\n"
 			     "         WHERE owner = :own<char[101]>)\n"
 			     " ORDER BY 1,2",
-			     QString::null,
+			     "",
 			     "7.3");
 
 static toSQL SQLUserObjectList("toScript:UserExtractObject",
@@ -174,14 +182,14 @@ static toSQL SQLSchemas("toScript:ExtractSchema",
 toScript::toScript(QWidget *parent,toConnection &connection)
   : toToolWidget(ScriptTool,"script.html",parent,connection)
 {
-  QToolBar *toolbar=toAllocBar(this,"SQL worksheet",connection.description());
+  QToolBar *toolbar=toAllocBar(this,tr("Extraction and Compare"));
 
   new QToolButton(QPixmap((const char **)execute_xpm),
-		  "Perform defined extraction",
-		  "Perform defined extraction",
+		  tr("Perform defined extraction"),
+		  tr("Perform defined extraction"),
 		  this,SLOT(execute(void)),
 		  toolbar);
-  toolbar->setStretchableWidget(new QLabel("",toolbar));
+  toolbar->setStretchableWidget(new QLabel(toolbar,TO_KDE_TOOLBAR_WIDGET));
 
   ScriptUI=new toScriptUI(this);
 
@@ -189,7 +197,7 @@ toScript::toScript(QWidget *parent,toConnection &connection)
   QVBox *box=new QVBox(ScriptUI->ResultTab);
   Worksheet=new toWorksheet(box,connection);
   SearchList=new toListView(box);
-  SearchList->addColumn("Search result");
+  SearchList->addColumn(tr("Search result"));
   SearchList->setRootIsDecorated(true);
   SearchList->setSorting(0);
   SearchList->hide();
@@ -197,24 +205,28 @@ toScript::toScript(QWidget *parent,toConnection &connection)
   Report->hide();
 
   DropList=new toListView(hsplitter);
-  DropList->addColumn("Dropped");
+  DropList->addColumn(tr("Dropped"));
   DropList->setRootIsDecorated(true);
   DropList->setSorting(0);
   CreateList=new toListView(hsplitter);
-  CreateList->addColumn("Created");
+  CreateList->addColumn(tr("Created"));
   CreateList->setRootIsDecorated(true);
   CreateList->setSorting(0);
   ScriptUI->Tabs->setTabEnabled(ScriptUI->ResultTab,false);
   ScriptUI->Tabs->setTabEnabled(ScriptUI->DifferenceTab,false);
 
+  connect(SearchList,SIGNAL(clicked(QListViewItem *)),this,SLOT(keepOn(QListViewItem *)));
+  connect(DropList,SIGNAL(clicked(QListViewItem *)),this,SLOT(keepOn(QListViewItem *)));
+  connect(CreateList,SIGNAL(clicked(QListViewItem *)),this,SLOT(keepOn(QListViewItem *)));
+
   QGridLayout *layout=new QGridLayout(ScriptUI->ResultTab);
   layout->addWidget(box,0,0);
   layout=new QGridLayout(ScriptUI->DifferenceTab);
   layout->addWidget(hsplitter,0,0);
 
-  ScriptUI->Initial->setTitle("&Initial");
-  ScriptUI->Limit->setTitle("&Limit");
-  ScriptUI->Next->setTitle("&Next");
+  ScriptUI->Initial->setTitle(tr("&Initial"));
+  ScriptUI->Limit->setTitle(tr("&Limit"));
+  ScriptUI->Next->setTitle(tr("&Next"));
   connect(ScriptUI->ModeGroup,SIGNAL(clicked(int)),this,SLOT(changeMode(int)));
   ScriptUI->Tabs->setTabEnabled(ScriptUI->ResizeTab,false);
   ScriptUI->SourceObjects->setSorting(0);
@@ -250,10 +262,11 @@ toScript::toScript(QWidget *parent,toConnection &connection)
   connect(ScriptUI->DestinationSchema,SIGNAL(activated(int)),this,SLOT(changeDestinationSchema(int)));
   connect(ScriptUI->SourceObjects,SIGNAL(clicked(QListViewItem *)),this,SLOT(objectClicked(QListViewItem *)));
   connect(ScriptUI->DestinationObjects,SIGNAL(clicked(QListViewItem *)),this,SLOT(objectClicked(QListViewItem *)));
+  connect(ScriptUI->Browse,SIGNAL(clicked()),this,SLOT(browseFile()));
 
   connect(ScriptUI->SourceObjects,SIGNAL(expanded(QListViewItem *)),
 	  this,SLOT(expandSource(QListViewItem *)));
-  connect(ScriptUI->DestinationObjects,SIGNAL(clicked(QListViewItem *)),
+  connect(ScriptUI->DestinationObjects,SIGNAL(expanded(QListViewItem *)),
 	  this,SLOT(expandDestination(QListViewItem *)));
 
   connect(toMainWidget(),SIGNAL(addedConnection(const QString &)),
@@ -315,18 +328,18 @@ std::list<QString> toScript::createObjectList(QListView *source)
       if (!user.isEmpty()) {
 	if (chk->isOn()&&chk->isEnabled()) {
 	  QString line;
-	  if (type=="TABLE") {
+	  if (type==QString::fromLatin1("TABLE")) {
 	    line=user;
-	    line+=".";
+	    line+=QString::fromLatin1(".");
 	    line+=name;
 	    toPush(tables,line);
 	  } else {
 	    line=type;
-	    line+=":";
+	    line+=QString::fromLatin1(":");
 	    line+=user;
-	    line+=".";
+	    line+=QString::fromLatin1(".");
 	    line+=name;
-	    if (type=="VIEW")
+	    if (type==QString::fromLatin1("VIEW"))
 	      toPush(userViews,line);
 	    else
 	      toPush(userOther,line);
@@ -335,15 +348,15 @@ std::list<QString> toScript::createObjectList(QListView *source)
       } else if (!type.isEmpty()) {
 	if (chk->isOn()&&chk->isEnabled()) {
 	  QString line=type;
-	  line+=":";
+	  line+=QString::fromLatin1(":");
 	  line+=name;
-	  if (type=="TABLESPACE")
+	  if (type==QString::fromLatin1("TABLESPACE"))
 	    toPush(tableSpace,line);
-	  else if (type=="PROFILE")
+	  else if (type==QString::fromLatin1("PROFILE"))
 	    toPush(profiles,line);
-	  else if (type=="ROLE")
+	  else if (type==QString::fromLatin1("ROLE"))
 	    toPush(roles,name);
-	  else if (type=="USER")
+	  else if (type==QString::fromLatin1("USER"))
 	    toPush(users,name);
 	  else
 	    toPush(otherGlobal,line);
@@ -380,21 +393,21 @@ std::list<QString> toScript::createObjectList(QListView *source)
     }
     {
       for(std::list<QString>::iterator i=roles.begin();i!=roles.end();i++) {
-	QString line="ROLE:";
+	QString line=QString::fromLatin1("ROLE:");
 	line+=*i;
 	toPush(lst,line);
       }
     }
     {
       for(std::list<QString>::iterator i=users.begin();i!=users.end();i++) {
-	QString line="USER:";
+	QString line=QString::fromLatin1("USER:");
 	line+=*i;
 	toPush(lst,line);
       }
     }
     {
       for(std::list<QString>::iterator i=tables.begin();i!=tables.end();i++) {
-	QString line="TABLE FAMILY:";
+	QString line=QString::fromLatin1("TABLE FAMILY:");
 	line+=*i;
 	toPush(lst,line);
       }
@@ -409,28 +422,28 @@ std::list<QString> toScript::createObjectList(QListView *source)
     }
   }
   for(std::list<QString>::iterator i=tables.begin();i!=tables.end();i++) {
-    QString line="TABLE CONTENTS:";
+    QString line=QString::fromLatin1("TABLE CONTENTS:");
     line+=*i;
     toPush(lst,line);
   }
   if (ScriptUI->IncludeDDL->isChecked()) {
     {
       for(std::list<QString>::iterator i=tables.begin();i!=tables.end();i++) {
-	QString line="TABLE REFERENCES:";
+	QString line=QString::fromLatin1("TABLE REFERENCES:");
 	line+=*i;
 	toPush(lst,line);
       }
     }
     {
       for(std::list<QString>::iterator i=roles.begin();i!=roles.end();i++) {
-	QString line="ROLE GRANTS:";
+	QString line=QString::fromLatin1("ROLE GRANTS:");
 	line+=*i;
 	toPush(lst,line);
       }
     }
     {
       for(std::list<QString>::iterator i=users.begin();i!=users.end();i++) {
-	QString line="USER GRANTS:";
+	QString line=QString::fromLatin1("USER GRANTS:");
 	line+=*i;
 	toPush(lst,line);
       }
@@ -454,7 +467,7 @@ void toScript::execute(void)
     else if (ScriptUI->Report->isChecked())
       mode=4;
     else {
-      toStatusMessage("No mode selected");
+      toStatusMessage(tr("No mode selected"));
       return;
     }
 
@@ -468,7 +481,79 @@ void toScript::execute(void)
     setupExtract(source);
     switch(mode) {
     case 1:
-      script+=source.create(sourceObjects);
+      if (ScriptUI->OutputTab->isChecked())
+	script+=source.create(sourceObjects);
+      else if (ScriptUI->OutputFile->isChecked()) {
+	if (ScriptUI->Filename->text().isEmpty())
+	  throw tr("No filename specified");
+
+	QFile file(ScriptUI->Filename->text());
+	file.open(IO_WriteOnly);
+
+	if (file.status()!=IO_Ok)
+	  throw tr("Couldn't open file %1").arg(file.name());
+
+	QTextStream stream(&file);
+	source.create(stream,sourceObjects);
+
+	if (file.status()!=IO_Ok)
+	  throw tr("Error writing to file %1").arg(file.name());
+
+	script=tr("-- Script generated to file %1 successfully").arg(ScriptUI->Filename->text());
+      } else if (ScriptUI->OutputDir->isChecked()) {
+	if (ScriptUI->Filename->text().isEmpty())
+	  throw tr("No filename specified");
+
+	QFile file(ScriptUI->Filename->text()+QDir::separator()+"script.sql");
+	file.open(IO_WriteOnly);
+
+	if (file.status()!=IO_Ok)
+	  throw QString(tr("Couldn't open file %1")).arg(file.name());
+
+	QTextStream stream(&file);
+
+	stream<<tr("rem Master script for DDL reverse engineering by TOra\n"
+		   "\n");
+
+	QFile pfile(ScriptUI->Filename->text()+QDir::separator()+"script.tpr");
+	pfile.open(IO_WriteOnly);
+
+	if (pfile.status()!=IO_Ok)
+	  throw QString(tr("Couldn't open file %1")).arg(pfile.name());
+
+	QTextStream pstream(&pfile);
+
+	QRegExp repl("\\W+");
+	for(std::list<QString>::iterator i=sourceObjects.begin();i!=sourceObjects.end();i++) {
+	  std::list<QString> t;
+	  t.insert(t.end(),*i);
+	  QString fn=*i;
+	  fn.replace(repl,"_");
+	  fn+=".sql";
+	  stream<<"@"<<fn<<"\n";
+
+	  QFile tf(ScriptUI->Filename->text()+QDir::separator()+fn);
+	  tf.open(IO_WriteOnly);
+	  pstream<<tf.name()<<"\n";
+
+	  if (tf.status()!=IO_Ok)
+	    throw QString(tr("Couldn't open file %1")).arg(tf.name());
+
+	  QTextStream ts(&tf);
+	  source.create(ts,t);
+
+	  if (tf.status()!=IO_Ok)
+	    throw QString(tr("Error writing to file %1")).arg(tf.name());
+
+
+	  script=tr("-- Scripts generate to directory %1 successfully").arg(ScriptUI->Filename->text());;
+	}
+
+	if (file.status()!=IO_Ok)
+	  throw QString(tr("Error writing to file %1")).arg(file.name());
+	if (pfile.status()!=IO_Ok)
+	  throw QString(tr("Error writing to file %1")).arg(pfile.name());
+      }
       break;
     case 0:
     case 2:
@@ -476,7 +561,6 @@ void toScript::execute(void)
     case 4:
       sourceDescription=source.describe(sourceObjects);
       break;
-      break;
     }
 
     if (ScriptUI->Destination->isEnabled()) {
@@ -492,7 +576,7 @@ void toScript::execute(void)
 	break;
       case 1:
       case 3:
-	throw QString ("Destination shouldn't be enabled now, internal error");
+	throw tr("Destination shouldn't be enabled now, internal error");
       }
 
       std::list<QString> drop;
@@ -515,7 +599,7 @@ void toScript::execute(void)
       Report->hide();
       SearchList->show();
       QRegExp re(ScriptUI->SearchWord->text(),false);
-      QStringList words(QStringList::split(QRegExp(" "),
+      QStringList words(QStringList::split(QRegExp(QString::fromLatin1(" ")),
 						   ScriptUI->SearchWord->text().
 						   upper().simplifyWhiteSpace()));
       QString word=ScriptUI->SearchWord->text().upper();
@@ -532,7 +616,7 @@ void toScript::execute(void)
       for(std::list<QString>::iterator i=sourceDescription.begin();
 	  i!=sourceDescription.end();
 	  i++) {
-	QStringList ctx=QStringList::split("\01",(*i).upper());
+	QStringList ctx=QStringList::split(QString::fromLatin1("\01"),(*i).upper());
 	switch(searchMode) {
 	case 1:
 	case 2:
@@ -562,7 +646,26 @@ void toScript::execute(void)
       Worksheet->hide();
       SearchList->hide();
       Report->show();
-      Report->setText(toGenerateReport(source.connection(),sourceDescription));
+      QString res=toGenerateReport(source.connection(),sourceDescription);
+      Report->setText(res);
+      if (ScriptUI->OutputFile->isChecked()) {
+	if (ScriptUI->Filename->text().isEmpty())
+	  toStatusMessage(tr("No filename specified"));
+	else {
+	  QFile file(ScriptUI->Filename->text());
+	  file.open(IO_WriteOnly);
+
+	  if (file.status()!=IO_Ok)
+	    toStatusMessage(tr("Couldn't open file %1").arg(file.name()));
+	  else {
+	    QTextStream stream(&file);
+	    stream<<res;
+
+	    if (file.status()!=IO_Ok)
+	      toStatusMessage(tr("Error writing to file %1").arg(file.name()));
+	  }
+	}
+      }
     } else {
       Worksheet->show();
       SearchList->hide();
@@ -585,7 +688,7 @@ void toScript::fillDifference(std::list<QString> &objects,QListView *view)
   QStringList lstCtx;
   for(std::list<QString>::iterator i=objects.begin();i!=objects.end();i++) {
     //    printf("Adding %s\n",(const char *)*i);
-    QStringList ctx=QStringList::split("\01",*i);
+    QStringList ctx=QStringList::split(QString::fromLatin1("\01"),*i);
     if (last) {
       while(last&&lastLevel>=int(ctx.count())) {
 	last=last->parent();
@@ -597,7 +700,7 @@ void toScript::fillDifference(std::list<QString> &objects,QListView *view)
       }
     }
     if (lastLevel<0)
-      throw QString("Internal error, lastLevel < 0");
+      throw tr("Internal error, lastLevel < 0");
     while(lastLevel<int(ctx.count())-1) {
       if (last)
 	last=new toResultViewMLine(last,NULL,ctx[lastLevel]);
@@ -612,7 +715,6 @@ void toScript::fillDifference(std::list<QString> &objects,QListView *view)
       item=new toResultViewMLCheck(view,ctx[lastLevel],QCheckListItem::CheckBox);
     last=item;
     item->setOn(true);
-    item->setEnabled(false);
     lstCtx=ctx;
     lastLevel++;
   }
@@ -669,7 +771,7 @@ void toScript::changeConnection(int,bool source)
     }
     (source?ScriptUI->SourceObjects:ScriptUI->DestinationObjects)->clear();
     (source?ScriptUI->SourceSchema:ScriptUI->DestinationSchema)->clear();
-    (source?ScriptUI->SourceSchema:ScriptUI->DestinationSchema)->insertItem("All");
+    (source?ScriptUI->SourceSchema:ScriptUI->DestinationSchema)->insertItem(tr("All"));
     toConnection &conn=toMainWidget()->connection((source?
 						   ScriptUI->SourceConnection:
 						   ScriptUI->DestinationConnection)
@@ -700,7 +802,7 @@ void toScript::changeConnection(int,bool source)
 				      top,QCheckListItem::CheckBox);
 	lastTop->setExpandable(true);
 	if (!second.isEmpty()||first.isEmpty())
-	  lastTop->setText(1,"USER");
+	  lastTop->setText(1,QString::fromLatin1("USER"));
       }
       if (first!=(lastFirst?lastFirst->text(0):QString::null)&&!first.isEmpty()) {
 	lastFirst=new toResultViewCheck(lastTop,first,QCheckListItem::CheckBox);
@@ -716,7 +818,7 @@ void toScript::changeConnection(int,bool source)
   } TOCATCH
 }
 
-void toScript::readOwnerObjects(QListView *list,QListViewItem *item,toConnection &conn)
+void toScript::readOwnerObjects(QListViewItem *item,toConnection &conn)
 {
   if (!item->parent()&&!item->firstChild()) {
     try {
@@ -740,13 +842,13 @@ void toScript::readOwnerObjects(QListView *list,QListViewItem *item,toConnection
 	}
       }
 
-      if (top=="PUBLIC") {
+      if (top==QString::fromLatin1("PUBLIC")) {
 	object=toQuery::readQueryNull(conn,SQLPublicSynonymList);
-	QListViewItem *topItem=new toResultViewCheck(item,"SYNONYM",QCheckListItem::CheckBox);
+	QListViewItem *topItem=new toResultViewCheck(item,QString::fromLatin1("SYNONYM"),QCheckListItem::CheckBox);
 	while(object.size()>0) {
 	  QListViewItem *item=new toResultViewCheck(topItem,toShift(object),
 						    QCheckListItem::CheckBox);
-	  item->setText(1,"SYNONYM");
+	  item->setText(1,QString::fromLatin1("SYNONYM"));
 	  item->setText(2,top);
 	}
       }
@@ -756,7 +858,7 @@ void toScript::readOwnerObjects(QListView *list,QListViewItem *item,toConnection
 
 void toScript::changeMode(int mode)
 {
-  if (mode<0||mode>3)
+  if (mode<0||mode>4)
     return;
 
   if (mode==0||mode==2)
@@ -769,10 +871,8 @@ void toScript::changeMode(int mode)
   else if (mode==0||mode==3||mode==4)
     ScriptUI->Tabs->setTabEnabled(ScriptUI->ResizeTab,false);
 
-  if (mode==1)
-    ScriptUI->IncludeContent->setEnabled(true);
-  else if (mode==0||mode==2||mode==3||mode==4)
-    ScriptUI->IncludeContent->setEnabled(false);
+  ScriptUI->IncludeContent->setEnabled(mode==1);
+  ScriptUI->CommitDistance->setEnabled(mode==1);
 
   if (mode==1||mode==2) {
     ScriptUI->IncludeHeader->setEnabled(true);
@@ -788,6 +888,8 @@ void toScript::changeMode(int mode)
   } else if (mode==1)
     ScriptUI->IncludeDDL->setEnabled(true);
 
+  ScriptUI->OutputGroup->setEnabled(mode==1||mode==2||mode==4);
+
   ScriptUI->IncludeConstraints->setEnabled(ScriptUI->IncludeDDL->isChecked());
   ScriptUI->IncludeIndexes->setEnabled(ScriptUI->IncludeDDL->isChecked());
   ScriptUI->IncludeGrants->setEnabled(ScriptUI->IncludeDDL->isChecked());
@@ -798,6 +900,16 @@ void toScript::changeMode(int mode)
   ScriptUI->IncludeComment->setEnabled(ScriptUI->IncludeDDL->isChecked());
 }
 
+void toScript::keepOn(QListViewItem *parent)
+{
+  if (!parent)
+    return;
+  toResultViewCheck *pchk=dynamic_cast<toResultViewCheck *>(parent);
+  if (!pchk)
+    return;
+  pchk->setOn(true);
+}
+
 void toScript::objectClicked(QListViewItem *parent)
 {
   if (!parent)
@@ -845,7 +957,7 @@ void toScript::changeSchema(int,bool source)
       parent=parent->nextSibling()) {
     toResultViewCheck *chk=dynamic_cast<toResultViewCheck *>(parent);
     if (chk) {
-      bool ena=((src==chk->text(0))||(src=="All"));
+      bool ena=((src==chk->text(0))||(src==QString::fromLatin1("All")));
 
       QListViewItem *next=NULL;
       for (QListViewItem *item=parent;item;item=next) {
@@ -887,7 +999,7 @@ void toScript::newSize(void)
 
   for (QListViewItem *item=ScriptUI->Sizes->firstChild();item;item=item->nextSibling())
     if (max==item->text(0)) {
-      toStatusMessage("Replacing existing size with new",false,false);
+      toStatusMessage(tr("Replacing existing size with new"),false,false);
       delete item;
       break;
     }
@@ -912,7 +1024,8 @@ void toScript::setupExtract(toExtract &extr)
   extr.setConstraints(ScriptUI->IncludeConstraints->isEnabled()&&
 		      ScriptUI->IncludeConstraints->isChecked());
   extr.setContents   (ScriptUI->IncludeContent->isEnabled()    &&
-		      ScriptUI->IncludeContent->isChecked()    );
+		      ScriptUI->IncludeContent->isChecked()    ,
+		      ScriptUI->CommitDistance->value()        );
   extr.setGrants     (ScriptUI->IncludeGrants->isEnabled()     &&
 		      ScriptUI->IncludeGrants->isChecked()     );
   extr.setHeading    (ScriptUI->IncludeHeader->isEnabled()     &&
@@ -928,9 +1041,9 @@ void toScript::setupExtract(toExtract &extr)
   extr.setStorage    (ScriptUI->IncludeStorage->isEnabled()    &&
 		      ScriptUI->IncludeStorage->isChecked()    );
 
-  if (ScriptUI->Schema->currentText()=="Same")
-    extr.setSchema("1");
-  else if (ScriptUI->Schema->currentText()=="None")
+  if (ScriptUI->Schema->currentText()==tr("Same"))
+    extr.setSchema(QString::fromLatin1("1"));
+  else if (ScriptUI->Schema->currentText()==tr("None"))
     extr.setSchema(QString::null);
   else
     extr.setSchema(ScriptUI->Schema->currentText());
@@ -938,17 +1051,17 @@ void toScript::setupExtract(toExtract &extr)
   if (ScriptUI->DontResize->isChecked())
     extr.setResize(QString::null);
   else if (ScriptUI->AutoResize->isChecked())
-    extr.setResize("1");
+    extr.setResize(QString::fromLatin1("1"));
   else {
     QString siz;
     for (QListViewItem *item=ScriptUI->Sizes->firstChild();item;item=item->nextSibling()) {
       siz+=item->text(0);
-      siz+=":";
+      siz+=QString::fromLatin1(":");
       siz+=item->text(1);
-      siz+=":";
+      siz+=QString::fromLatin1(":");
       siz+=item->text(2);
       if (item->nextSibling())
-	siz+=":";
+	siz+=QString::fromLatin1(":");
     }
     extr.setResize(siz);
   }
@@ -956,14 +1069,31 @@ void toScript::setupExtract(toExtract &extr)
 
 void toScript::expandSource(QListViewItem *item)
 {
-  if (item)
-    readOwnerObjects(ScriptUI->SourceObjects,item,
-		     toMainWidget()->connection(ScriptUI->SourceConnection->currentText()));
+  try {
+    if (item)
+      readOwnerObjects(item,
+		       toMainWidget()->connection(ScriptUI->SourceConnection->currentText()));
+  } TOCATCH
 }
 
 void toScript::expandDestination(QListViewItem *item)
 {
-  if (item)
-    readOwnerObjects(ScriptUI->DestinationObjects,item,
-		     toMainWidget()->connection(ScriptUI->DestinationConnection->currentText()));
+  try {
+    if (item)
+      readOwnerObjects(item,
+		       toMainWidget()->connection(ScriptUI->DestinationConnection->currentText()));
+  } TOCATCH
+}
+
+void toScript::browseFile(void)
+{
+  if (ScriptUI->OutputFile->isChecked()) {
+    QString f=toOpenFilename(QString::null,QString::null,this);
+    if (!f.isEmpty())
+      ScriptUI->Filename->setText(f);
+  } else if (ScriptUI->OutputDir->isChecked()) {
+    QString f=TOFileDialog::getExistingDirectory(QString::null,this);
+    if (!f.isEmpty())
+      ScriptUI->Filename->setText(f);
+  }
 }
