@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -39,14 +39,17 @@
 #include "toconnection.h"
 #include "tomain.h"
 #include "toresultbar.h"
+#include "toresultcombo.h"
 #include "toresultlock.h"
 #include "toresultlong.h"
 #include "toresultstats.h"
 #include "toresultview.h"
 #include "tosession.h"
 #include "tosgastatement.h"
+#include "tosgatrace.h"
 #include "tosql.h"
 #include "totool.h"
+#include "towaitevents.h"
 
 #ifdef TO_KDE
 #  include <kmenubar.h>
@@ -61,16 +64,20 @@
 #include <qtimer.h>
 #include <qtoolbar.h>
 #include <qtoolbutton.h>
+#include <qtooltip.h>
 #include <qworkspace.h>
 #include <qworkspace.h>
 
 #include "tosession.moc"
 
+#include "icons/add.xpm"
 #include "icons/clock.xpm"
 #include "icons/disconnect.xpm"
+#include "icons/minus.xpm"
 #include "icons/noclock.xpm"
 #include "icons/refresh.xpm"
 #include "icons/tosession.xpm"
+#include "icons/filter.xpm"
 
 class toSessionTool : public toTool {
 protected:
@@ -175,94 +182,136 @@ static toSQL SQLSessions("toSession:ListSession",
 			 "       b.Consistent_Changes \"-Consistent Changes\",\n"
 			 "       c.Value*10 \"-CPU (ms)\",\n"
 			 "       a.last_call_et \"Last SQL\",\n"
-			 "       a.Process \"-Process\",\n"
+			 "       a.Process \"-Client PID\",\n"
+			 "       e.SPid \"-Server PID\",\n"
 			 "       d.sql_text \"Current statement\",\n"
-			 "       a.SQL_Address||':'||SQL_Hash_Value \" SQL Address\",\n"
-			 "       a.Prev_SQL_Addr||':'||Prev_Hash_Value \" Prev SQl Address\"\n"
+			 "       a.SQL_Address||':'||a.SQL_Hash_Value \" SQL Address\",\n"
+			 "       a.Prev_SQL_Addr||':'||a.Prev_Hash_Value \" Prev SQl Address\"\n"
 			 "  FROM v$session a,\n"
 			 "       v$sess_io b,\n"
 			 "       v$sesstat c,\n"
-			 "       v$sql d\n"
+			 "       v$sql d,\n"
+			 "       v$process e\n"
 			 " WHERE a.sid = b.sid(+)\n"
 			 "   AND a.sid = c.sid(+) AND (c.statistic# = 12 OR c.statistic# IS NULL)\n"
 			 "   AND a.sql_address = d.address(+) AND a.sql_hash_value = d.hash_value(+)\n"
 			 "   AND (d.child_number = 0 OR d.child_number IS NULL)\n"
-			 " ORDER BY a.Sid",
+			 "   AND a.paddr = e.addr(+)\n"
+			 "%1 ORDER BY a.Sid",
 			 "List sessions, must have same number of culumns and the first and last 2 must be "
 			 "the same");
 
 toSession::toSession(QWidget *main,toConnection &connection)
   : toToolWidget(SessionTool,"session.html",main,connection)
 {
-  QToolBar *toolbar=toAllocBar(this,"Session manager",connection.description());
+  QToolBar *toolbar=toAllocBar(this,tr("Session manager"));
 
   new QToolButton(QPixmap((const char **)refresh_xpm),
-		  "Update sessionlist",
-		  "Update sessionlist",
+		  tr("Update sessionlist"),
+		  tr("Update sessionlist"),
 		  this,SLOT(refresh(void)),
 		  toolbar);
+  toolbar->addSeparator();
+  Select=new toResultCombo(toolbar,TO_KDE_TOOLBAR_WIDGET);
+  Select->setSelected(tr("Only active users"));
+  Select->additionalItem(tr("Only active users"));
+  Select->additionalItem(tr("All"));
+  Select->additionalItem(tr("No background"));
+  Select->additionalItem(tr("No system"));
+  Select->query(toSQL::sql(toSQL::TOSQL_USERLIST));
+
+  connect(Select,SIGNAL(activated(int)),this,SLOT(refresh()));
+  
   toolbar->addSeparator();
   new QToolButton(QPixmap((const char **)clock_xpm),
-		  "Enable timed statistics",
-		  "Enable timed statistics",
+		  tr("Enable timed statistics"),
+		  tr("Enable timed statistics"),
 		  this,SLOT(enableStatistics(void)),
 		  toolbar);
   new QToolButton(QPixmap((const char **)noclock_xpm),
-		  "Disable timed statistics",
-		  "Disable timed statistics",
+		  tr("Disable timed statistics"),
+		  tr("Disable timed statistics"),
 		  this,SLOT(disableStatistics(void)),
 		  toolbar);
   toolbar->addSeparator();
   new QToolButton(QPixmap((const char **)disconnect_xpm),
-		  "Disconnect selected session",
-		  "Disconnect selected session",
+		  tr("Disconnect selected session"),
+		  tr("Disconnect selected session"),
 		  this,SLOT(disconnectSession(void)),
 		  toolbar);
   toolbar->addSeparator();
-  new QLabel("Refresh ",toolbar);
-  connect(Refresh=toRefreshCreate(toolbar),SIGNAL(activated(const QString &)),this,SLOT(changeRefresh(const QString &)));
+  new QLabel(tr("Refresh")+" ",toolbar,TO_KDE_TOOLBAR_WIDGET);
+  connect(Refresh=toRefreshCreate(toolbar,TO_KDE_TOOLBAR_WIDGET),SIGNAL(activated(const QString &)),this,SLOT(changeRefresh(const QString &)));
+  toolbar->addSeparator();
 
-  toolbar->setStretchableWidget(new QLabel("",toolbar));
-  new toChangeConnection(toolbar);
+  QToolButton *btn=new QToolButton(toolbar);
+  btn->setToggleButton(true);
+  btn->setIconSet(QPixmap((const char **)filter_xpm));
+  connect(btn,SIGNAL(toggled(bool)),this,SLOT(excludeSelection(bool)));
+  QToolTip::add(btn,tr("Exclude selected sessions"));
+  new QToolButton(QPixmap((const char **)add_xpm),
+		  tr("Select all sessions"),
+		  tr("Select all sessions"),
+		  this,SLOT(selectAll(void)),
+		  toolbar);
+  new QToolButton(QPixmap((const char **)minus_xpm),
+		  tr("Deselect all sessions"),
+		  tr("Deselect all sessions"),
+		  this,SLOT(selectNone(void)),
+		  toolbar);
+
+  toolbar->setStretchableWidget(Total=new QLabel(toolbar,TO_KDE_TOOLBAR_WIDGET));
+  Total->setAlignment(AlignRight|AlignVCenter|ExpandTabs);
+  new toChangeConnection(toolbar,TO_KDE_TOOLBAR_WIDGET);
 
   QSplitter *splitter=new QSplitter(Vertical,this);
-  Sessions=new toResultLong(false,false,toQuery::Background,splitter);
-  Sessions->setSQL(SQLSessions);
+  Sessions=new toSessionList(splitter);
   Sessions->setReadAll(true);
   connect(Sessions,SIGNAL(done()),this,SLOT(done()));
 
   ResultTab=new QTabWidget(splitter);
+
+  CurrentStatement=new toSGAStatement(ResultTab);
+  ResultTab->addTab(CurrentStatement,tr("Current Statement"));
+
+  QString sql=toSQL::string(TOSQL_LONGOPS,connection);
+  sql+=" AND b.sid = :sid<char[101]> AND b.serial# = :ser<char[101]>";
+  LongOps=new toResultLong(true,false,toQuery::Background,ResultTab);
+  LongOps->setSQL(sql);
+  ResultTab->addTab(LongOps,tr("Long ops"));
+
   StatisticSplitter=new QSplitter(Horizontal,ResultTab);
   SessionStatistics=new toResultStats(false,0,StatisticSplitter);
   WaitBar=new toResultBar(StatisticSplitter);
   WaitBar->setSQL(SQLSessionWait);
-  WaitBar->setTitle("Session wait states");
-  WaitBar->setYPostfix("ms/s");
+  WaitBar->setTitle(tr("Session wait states"));
+  WaitBar->setYPostfix(QString::fromLatin1("ms/s"));
   IOBar=new toResultBar(StatisticSplitter);
   IOBar->setSQL(SQLSessionIO);
-  IOBar->setTitle("Session I/O");
-  IOBar->setYPostfix("blocks/s");
-  ResultTab->addTab(StatisticSplitter,"Statistics");
+  IOBar->setTitle(tr("Session I/O"));
+  IOBar->setYPostfix(QString::fromLatin1("blocks/s"));
+  ResultTab->addTab(StatisticSplitter,tr("Statistics"));
+
+  Waits=new toWaitEvents(0,ResultTab,"waits");
+  ResultTab->addTab(Waits,tr("Wait events"));
 
   ConnectInfo=new toResultLong(true,false,toQuery::Background,ResultTab);
   ConnectInfo->setSQL(SQLConnectInfo);
-  ResultTab->addTab(ConnectInfo,"Connect Info");
+  ResultTab->addTab(ConnectInfo,tr("Connect Info"));
   PendingLocks=new toResultLock(ResultTab);
-  ResultTab->addTab(PendingLocks,"Pending Locks");
+  ResultTab->addTab(PendingLocks,tr("Pending Locks"));
   LockedObjects=new toResultLong(false,false,toQuery::Background,ResultTab);
-  ResultTab->addTab(LockedObjects,"Locked Objects");
+  ResultTab->addTab(LockedObjects,tr("Locked Objects"));
   LockedObjects->setSQL(SQLLockedObject);
-  CurrentStatement=new toSGAStatement(ResultTab);
-  ResultTab->addTab(CurrentStatement,"Current Statement");
   AccessedObjects=new toResultLong(false,false,toQuery::Background,ResultTab);
   AccessedObjects->setSQL(SQLAccessedObjects);
-  ResultTab->addTab(AccessedObjects,"Accessing");
+  ResultTab->addTab(AccessedObjects,tr("Accessing"));
 
   PreviousStatement=new toSGAStatement(ResultTab);
-  ResultTab->addTab(PreviousStatement,"Previous Statement");
+  ResultTab->addTab(PreviousStatement,tr("Previous Statement"));
 
   OpenSplitter=new QSplitter(Horizontal,ResultTab);
-  ResultTab->addTab(OpenSplitter,"Open Cursors");
+  ResultTab->addTab(OpenSplitter,tr("Open Cursors"));
   OpenCursors=new toResultLong(false,true,toQuery::Background,OpenSplitter);
   OpenCursors->setSQL(SQLOpenCursors);
   OpenStatement=new toSGAStatement(OpenSplitter);
@@ -276,8 +325,10 @@ toSession::toSession(QWidget *main,toConnection &connection)
   connect(ResultTab,SIGNAL(currentChanged(QWidget *)),
 	  this,SLOT(changeTab(QWidget *)));
 
-  connect(timer(),SIGNAL(timeout(void)),this,SLOT(refreshTabs(void)));
-  toRefreshParse(timer());
+  try {
+    connect(timer(),SIGNAL(timeout(void)),this,SLOT(refreshTabs(void)));
+    toRefreshParse(timer());
+  } TOCATCH
   CurrentTab=StatisticSplitter;
 
   ToolMenu=NULL;
@@ -288,25 +339,101 @@ toSession::toSession(QWidget *main,toConnection &connection)
   setFocusProxy(Sessions);
 }
 
+void toSession::excludeSelection(bool tgl)
+{
+  toSessionList::sessionFilter *filt=dynamic_cast<toSessionList::sessionFilter *>(Sessions->filter());
+  if (filt) {
+    filt->setShow(!tgl);
+    refresh();
+  }
+}
+
+void toSession::selectAll(void)
+{
+  for (QListViewItem *item=Sessions->firstChild();item;item=item->nextSibling()) {
+    toResultViewCheck *chk=dynamic_cast<toResultViewCheck *>(item);
+    if (chk)
+      chk->setOn(true);
+  }
+}
+
+void toSession::selectNone(void)
+{
+  for (QListViewItem *item=Sessions->firstChild();item;item=item->nextSibling()) {
+    toResultViewCheck *chk=dynamic_cast<toResultViewCheck *>(item);
+    if (chk)
+      chk->setOn(false);
+  }
+}
+
+QListViewItem *toSessionList::createItem(QListViewItem *last,const QString &str)
+{
+  sessionFilter *filt=dynamic_cast<sessionFilter *>(filter());
+  if (filt&&filt->show())
+    return new toResultViewCheck(this,last,str,QCheckListItem::CheckBox);
+  else
+    return new toResultViewItem(this,last,str);
+}
+
+void toSessionList::updateFilter()
+{
+  sessionFilter *filt=dynamic_cast<sessionFilter *>(filter());
+  if (filt)
+    filt->updateList(this);
+}
+
+bool toSessionList::sessionFilter::check(const QListViewItem *item)
+{
+  sessionID serial(item->text(0).toInt(),item->text(1).toInt());
+  bool checked=false;
+  for(std::list<sessionID>::iterator i=Serials.begin();i!=Serials.end();i++)
+    if ((*i)==serial) {
+      checked=true;
+      break;
+    }
+  const toResultViewCheck *chk=dynamic_cast<const toResultViewCheck *>(item);
+  if (chk) {
+    const_cast<toResultViewCheck *>(chk)->setOn(checked);
+    return true;
+  }
+  return !checked;
+}
+
+void toSessionList::sessionFilter::updateList(toResultLong *lst)
+{
+  bool first=true;
+  for (QListViewItem *item=lst->firstChild();item;item=item->nextSibling()) {
+    toResultViewCheck *chk=dynamic_cast<toResultViewCheck *>(item);
+    if (chk) {
+      if (first) {
+	Serials.clear();
+	first=false;
+      }
+      if (chk->isOn())
+	Serials.insert(Serials.end(),sessionID(item->text(0).toInt(),item->text(1).toInt()));
+    }
+  }
+}
+
 void toSession::windowActivated(QWidget *widget)
 {
   if (widget==this) {
     if (!ToolMenu) {
       ToolMenu=new QPopupMenu(this);
-      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),"&Refresh",
+      ToolMenu->insertItem(QPixmap((const char **)refresh_xpm),tr("&Refresh"),
 			   this,SLOT(refresh(void)),Key_F5);
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem(QPixmap((const char **)clock_xpm),"Enable timed statistics",
+      ToolMenu->insertItem(QPixmap((const char **)clock_xpm),tr("Enable timed statistics"),
 			   this,SLOT(enableStatistics(void)));
-      ToolMenu->insertItem(QPixmap((const char **)noclock_xpm),"Disable timed statistics",
+      ToolMenu->insertItem(QPixmap((const char **)noclock_xpm),tr("Disable timed statistics"),
 			   this,SLOT(disableStatistics(void)));
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem(QPixmap((const char **)disconnect_xpm),"Disconnect session",
+      ToolMenu->insertItem(QPixmap((const char **)disconnect_xpm),tr("Disconnect session"),
 			   this,SLOT(disconnectSession(void)));
       ToolMenu->insertSeparator();
-      ToolMenu->insertItem("&Change Refresh",Refresh,SLOT(setFocus(void)),
+      ToolMenu->insertItem(tr("&Change Refresh"),Refresh,SLOT(setFocus(void)),
 			   Key_R+ALT);
-      toMainWidget()->menuBar()->insertItem("&Session",ToolMenu,-1,toToolMenuIndex());
+      toMainWidget()->menuBar()->insertItem(tr("&Session"),ToolMenu,-1,toToolMenuIndex());
     }
   } else {
     delete ToolMenu;
@@ -316,36 +443,58 @@ void toSession::windowActivated(QWidget *widget)
 
 void toSession::refresh(void)
 {
-  QListViewItem *item=Sessions->selectedItem();
-  if (item) {
-    Session=item->text(0);
-    Serial=item->text(1);
-  } else
-    Session=Serial=QString::null;
-  Sessions->refresh();
+  try {
+    QListViewItem *item=Sessions->selectedItem();
+    if (item) {
+      Session=item->text(0);
+      Serial=item->text(1);
+    } else
+      Session=Serial=QString::null;
+    QString sql=toSQL::string(SQLSessions,connection());
+    QString extra;
+    if (Select->currentItem()==0)
+      extra="   AND a.Type != 'BACKGROUND' AND a.Status != 'INACTIVE'\n";
+    else if (Select->currentItem()==1)
+      ; // Do nothing
+    else if (Select->currentItem()==2)
+      extra="   AND a.Type != 'BACKGROUND'\n";
+    else if (Select->currentItem()==3)
+      extra="   AND a.SchemaName NOT IN ('SYS','SYSTEM')\n";
+    else
+      extra="   AND a.SchemaName = '"+Select->currentText()+"'\n";
+    Sessions->setSQL(sql.arg(extra));
+    Sessions->refresh();
+  } TOCATCH
 }
 
 void toSession::done(void)
 {
-  for (QListViewItem *item=Sessions->firstChild();item;item=item->nextSibling())
+  int system=0,total=0,active=0;
+  for (QListViewItem *item=Sessions->firstChild();item;item=item->nextSibling()) {
     if (item->text(0)==Session&&
 	item->text(1)==Serial) {
       Sessions->setSelected(item,true);
-      break;
     }
+    total++;
+    if (item->text(8)!="USER")
+      system++;
+    else if (item->text(3)=="ACTIVE")
+      active++;
+  }
+  Total->setText(QString("Total <B>%1</B> (Active <B>%3</B>, System <B>%2</B>)").arg(total).arg(system).arg(active));
 }
 
 void toSession::enableStatistics(bool enable)
 {
   QString sql;
   if (enable)
-    sql="ALTER SYSTEM SET TIMED_STATISTICS = TRUE";
+    sql=QString::fromLatin1("ALTER SYSTEM SET TIMED_STATISTICS = TRUE");
   else
-    sql="ALTER SYSTEM SET TIMED_STATISTICS = FALSE";
+    sql=QString::fromLatin1("ALTER SYSTEM SET TIMED_STATISTICS = FALSE");
   try {
     connection().execute(sql);
   } catch (...) {
-    toStatusMessage("No access to timed statistics flags");
+    toStatusMessage(tr("No access to timed statistics flags"));
   }
 }
 
@@ -357,16 +506,24 @@ void toSession::changeTab(QWidget *tab)
     if (item) {
       if (CurrentTab==StatisticSplitter) {
 	int ses=item->text(0).toInt();
-	SessionStatistics->changeSession(ses);
-      } else if (CurrentTab==ConnectInfo)
+	try {
+	  SessionStatistics->changeSession(ses);
+	} TOCATCH
+      } else if (CurrentTab==ConnectInfo) {
+	ConnectInfo->clearParams();
 	ConnectInfo->changeParams(item->text(0));
-      else if (CurrentTab==PendingLocks)
+      } else if (CurrentTab==LongOps) {
+	LongOps->clearParams();
+	LongOps->changeParams(item->text(0),item->text(1));
+      } else if (CurrentTab==PendingLocks) {
+	PendingLocks->clearParams();
 	PendingLocks->query(item->text(0));
-      else if (CurrentTab==OpenSplitter) {
+      } else if (CurrentTab==OpenSplitter) {
 	QListViewItem *openitem=OpenCursors->currentItem();
 	QString address;
 	if (openitem)
 	  address=openitem->text(2);
+	OpenCursors->clearParams();
 	OpenCursors->changeParams(item->text(0));
 	if (!address.isEmpty())
 	  for (openitem=OpenCursors->firstChild();
@@ -375,14 +532,17 @@ void toSession::changeTab(QWidget *tab)
 	      OpenCursors->setSelected(item,true);
 	      break;
 	    }
-      } else if (CurrentTab==CurrentStatement)
+      } else if (CurrentTab==CurrentStatement) {
 	CurrentStatement->changeAddress(item->text(Sessions->columns()+0));
-      else if (CurrentTab==AccessedObjects)
+      } else if (CurrentTab==AccessedObjects) {
+	AccessedObjects->clearParams();
         AccessedObjects->changeParams(item->text(0));
-      else if (CurrentTab==LockedObjects)
+      } else if (CurrentTab==LockedObjects) {
+	LockedObjects->clearParams();
 	LockedObjects->changeParams(item->text(0));
-      else if (CurrentTab==PreviousStatement)
+      } else if (CurrentTab==PreviousStatement) {
 	PreviousStatement->changeAddress(item->text(Sessions->columns()+1));
+      }
     }
   }
 }
@@ -397,21 +557,21 @@ void toSession::disconnectSession(void)
 {
   QListViewItem *item=Sessions->selectedItem();
   if (item) {
-    QString sess="'";
+    QString sess=QString::fromLatin1("'");
     sess.append(item->text(0));
-    sess.append(",");
+    sess.append(QString::fromLatin1(","));
     sess.append(item->text(1));
-    sess.append("'");
-    QString str("Let current transaction finnish before disconnecting session?");
+    sess.append(QString::fromLatin1("'"));
+    QString str(tr("Let current transaction finish before disconnecting session?"));
     QString sql;
-    switch(TOMessageBox::warning(this,"Commit work?",str,"&Yes","&No","&Cancel")) {
+    switch(TOMessageBox::warning(this,tr("Commit work?"),str,tr("&Yes"),tr("&No"),tr("Cancel"))) {
     case 0:
-      sql="ALTER SYSTEM DISCONNECT SESSION ";
+      sql=QString::fromLatin1("ALTER SYSTEM DISCONNECT SESSION ");
       sql.append(sess);
-      sql.append(" POST_TRANSACTION");
+      sql.append(QString::fromLatin1(" POST_TRANSACTION"));
       break;
     case 1:
-      sql="ALTER SYSTEM KILL SESSION ";
+      sql=QString::fromLatin1("ALTER SYSTEM KILL SESSION ");
       sql.append(sess);
       break;
     case 2:
@@ -425,7 +585,9 @@ void toSession::disconnectSession(void)
 
 void toSession::changeRefresh(const QString &str)
 {
-  toRefreshParse(timer(),str);
+  try {
+    toRefreshParse(timer(),str);
+  } TOCATCH
 }
 
 void toSession::changeItem(QListViewItem *item)
@@ -434,6 +596,7 @@ void toSession::changeItem(QListViewItem *item)
     if (!item->text(0).isEmpty()) {
       WaitBar->changeParams(item->text(0));
       IOBar->changeParams(item->text(0));
+      Waits->setSession(item->text(0).toInt());
     }
     LastSession=item->text(0);
   }
