@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -49,6 +49,7 @@
 
 #include <qfiledialog.h>
 #include <qfileinfo.h>
+#include <qheader.h>
 #include <qlabel.h>
 #include <qlineedit.h>
 #include <qmessagebox.h>
@@ -65,21 +66,26 @@
 
 #include "icons/totemplate.xpm"
 
-static std::map<QString,QString> DefaultText(void)
+static std::map<QCString,QString> DefaultText(void)
 {
-  std::map<QString,QString> def;
+  std::map<QCString,QString> def;
   QString file=toPluginPath();
-  file+="/sqlfunctions.tpl";
+#ifndef OAS
+  file+=QString::fromLatin1("/sqlfunctions.tpl");
   def["PL/SQL Functions"]=file;
   file=toPluginPath();
-  file+="/hints.tpl";
+  file+=QString::fromLatin1("/hints.tpl");
   def["Optimizer Hints"]=file;
+  file=toPluginPath();
+  file+=QString::fromLatin1("/log4plsql.tpl");
+  def["Log4PL/SQL"]=file;
+#endif
   return def;
 }
 
 class toTemplateEdit : public toTemplateEditUI, public toHelpContext {
-  std::map<QString,QString> &TemplateMap;
-  std::map<QString,QString>::iterator LastTemplate;
+  std::map<QCString,QString> &TemplateMap;
+  std::map<QCString,QString>::iterator LastTemplate;
   void connectList(bool conn)
   {
     if (conn)
@@ -94,7 +100,7 @@ class toTemplateEdit : public toTemplateEditUI, public toHelpContext {
   }
   void allocateItem(void)
   {
-    QStringList lst=QStringList::split(":",Name->text());
+    QStringList lst=QStringList::split(QString::fromLatin1(":"),Name->text());
     unsigned int li=0;
     QListViewItem *parent=NULL;
     for(QListViewItem *item=Templates->firstChild();item&&li<lst.count();) {
@@ -113,15 +119,15 @@ class toTemplateEdit : public toTemplateEditUI, public toHelpContext {
       li++;
     }
   }
-  bool clearUnused(QListViewItem *first,const QString &pre)
+  bool clearUnused(QListViewItem *first,const QCString &pre)
   {
     bool ret=false;
     while(first) {
       QListViewItem *delitem=first;
-      QString str=pre;
+      QCString str=pre;
       if (!str.isEmpty())
 	str+=":";
-      str+=first->text(0);
+      str+=first->text(0).latin1();
       if (first->firstChild()&&clearUnused(first->firstChild(),str))
 	delitem=NULL;
       if (delitem&&TemplateMap.find(str)!=TemplateMap.end())
@@ -137,47 +143,52 @@ class toTemplateEdit : public toTemplateEditUI, public toHelpContext {
 public:
   virtual void updateFromMap(void)
   {
-    while(Templates->firstChild())
-      delete Templates->firstChild();
-    QListViewItem *last=NULL;
-    int lastLevel=0;
-    QStringList lstCtx;
-    for(std::map<QString,QString>::iterator i=TemplateMap.begin();i!=TemplateMap.end();i++) {
-      QStringList ctx=QStringList::split(":",(*i).first);
-      if (last) {
-	while(last&&lastLevel>=int(ctx.count())) {
-	  last=last->parent();
-	  lastLevel--;
+    try {
+      while(Templates->firstChild())
+	delete Templates->firstChild();
+      QListViewItem *last=NULL;
+      int lastLevel=0;
+      QStringList lstCtx;
+      for(std::map<QCString,QString>::iterator i=TemplateMap.begin();i!=TemplateMap.end();i++) {
+	QStringList ctx=QStringList::split(QString::fromLatin1(":"),QString::fromLatin1((*i).first));
+	if (last) {
+	  while(last&&lastLevel>=int(ctx.count())) {
+	    last=last->parent();
+	    lastLevel--;
+	  }
+	  while(last&&lastLevel>=0&&!toCompareLists(lstCtx,ctx,(unsigned int)lastLevel)) {
+	    last=last->parent();
+	    lastLevel--;
+	  }
 	}
-	while(last&&lastLevel>=0&&!toCompareLists(lstCtx,ctx,(unsigned int)lastLevel)) {
-	  last=last->parent();
-	  lastLevel--;
+	if (lastLevel<0)
+	  throw qApp->translate("toTemplateEdit","Internal error, lastLevel < 0");
+	while(lastLevel<int(ctx.count())-1) {
+	  if (last)
+	    last=new QListViewItem(last,ctx[lastLevel]);
+	  else
+	    last=new QListViewItem(Templates,ctx[lastLevel]);
+	  last->setOpen(true);
+	  lastLevel++;
 	}
-      }
-      if (lastLevel<0)
-	throw QString("Internal error, lastLevel < 0");
-      while(lastLevel<int(ctx.count())-1) {
 	if (last)
 	  last=new QListViewItem(last,ctx[lastLevel]);
 	else
 	  last=new QListViewItem(Templates,ctx[lastLevel]);
 	last->setOpen(true);
+	if (i==LastTemplate)
+	  last->setSelected(true);
+	lstCtx=ctx;
 	lastLevel++;
       }
-      if (last)
-	last=new QListViewItem(last,ctx[lastLevel]);
-      else
-	last=new QListViewItem(Templates,ctx[lastLevel]);
-      last->setOpen(true);
-      if (i==LastTemplate)
-	last->setSelected(true);
-      lstCtx=ctx;
-      lastLevel++;
+    } catch(const QString &str) {
+      toStatusMessage(str);
+      reject();
     }
   }
-  toTemplateEdit(std::map<QString,QString> &pairs,QWidget *parent,const char *name=0)
+  toTemplateEdit(std::map<QCString,QString> &pairs,QWidget *parent,const char *name=0)
     : toTemplateEditUI(parent,name,true,WStyle_Maximize),
-      toHelpContext("template.html#editor"),
+      toHelpContext(QString::fromLatin1("template.html#editor")),
       TemplateMap(pairs)
   {
     toHelp::connectDialog(this);
@@ -191,8 +202,8 @@ public:
       QListViewItem *item=findLast();
       TemplateMap.erase(LastTemplate);
       LastTemplate=TemplateMap.end();
-      Name->setText("");
-      Description->setText("");
+      Name->setText(QString::null);
+      Description->setText(QString::null);
       if (item) {
 	connectList(false);
 	clearUnused(Templates->firstChild(),"");
@@ -204,12 +215,12 @@ public:
   {
     Preview->setText(Description->text());
   }
-  QString name(QListViewItem *item)
+  QCString name(QListViewItem *item)
   {
-    QString str=item->text(0);
+    QCString str=item->text(0).latin1();
     for(item=item->parent();item;item=item->parent()) {
       str.prepend(":");
-      str.prepend(item->text(0));
+      str.prepend(item->text(0).latin1());
     }
     return str;
   }
@@ -217,7 +228,7 @@ public:
   {
     changeSelection();
     LastTemplate=TemplateMap.end();
-    Description->setText("");
+    Description->setText(QString::null);
     QListViewItem *item=Templates->selectedItem();
     if (item) {
       connectList(false);
@@ -225,7 +236,7 @@ public:
       connectList(true);
       item=item->parent();
     }
-    QString str;
+    QCString str;
     if (item) {
       str=name(item);
       str+=":";
@@ -236,15 +247,15 @@ public:
   {
     bool update=false;
     if (LastTemplate!=TemplateMap.end()) {
-      if (Name->text()!=(*LastTemplate).first||
+      if (Name->text().latin1()!=(*LastTemplate).first||
 	  Description->text()!=(*LastTemplate).second) {
 	TemplateMap.erase(LastTemplate);
-	TemplateMap[Name->text()]=Description->text();
+	TemplateMap[Name->text().latin1()]=Description->text();
 	allocateItem();
 	update=true;
       }
     } else if (!Name->text().isEmpty()) {
-      TemplateMap[Name->text()]=Description->text();
+      TemplateMap[Name->text().latin1()]=Description->text();
       allocateItem();
       update=true;
     }
@@ -252,16 +263,16 @@ public:
 
     QListViewItem *item=Templates->selectedItem();
     if (item) {
-      QString str=name(item);
+      QCString str=name(item);
       LastTemplate=TemplateMap.find(str);
       if (LastTemplate!=TemplateMap.end()) {
-	Name->setText((*LastTemplate).first);
+	Name->setText(QString::fromLatin1((*LastTemplate).first));
 	Description->setText((*LastTemplate).second);
 	Preview->setText((*LastTemplate).second);
       } else {
-	Name->setText("");
+	Name->setText(QString::null);
 	Description->clear();
-	Preview->setText("");
+	Preview->setText(QString::null);
       }
     } else
       LastTemplate=TemplateMap.end();
@@ -280,7 +291,7 @@ public:
   virtual void browse(void)
   {
     QFileInfo file(Filename->text());
-    QString filename=toOpenFilename(file.dirPath(),"*.tpl",this);
+    QString filename=toOpenFilename(file.dirPath(),QString::fromLatin1("*.tpl"),this);
     if (!filename.isEmpty())
       Filename->setText(filename);
   }
@@ -300,13 +311,13 @@ public:
   toTemplatePrefs(toTool *tool,QWidget *parent,const char *name=0)
     : toTemplateSetupUI(parent,name),toSettingTab("template.html#setup"),Tool(tool)
   {
-    std::map<QString,QString> def=DefaultText();
+    std::map<QCString,QString> def=DefaultText();
 
-    int tot=Tool->config("Number",QString::number(-1)).toInt();
+    int tot=Tool->config("Number","-1").toInt();
     {
       for(int i=0;i<tot;i++) {
-	QString num=QString::number(i);
-        QString root=Tool->config(num,"");
+	QCString num=QString::number(i).latin1();
+        QCString root=Tool->config(num,"").latin1();
         num+="file";
         QString file=Tool->config(num,"");
 	new QListViewItem(FileList,root,file);
@@ -314,14 +325,14 @@ public:
 	  def.erase(def.find(root));
       }
     }
-    for (std::map<QString,QString>::iterator i=def.begin();i!=def.end();i++)
+    for (std::map<QCString,QString>::iterator i=def.begin();i!=def.end();i++)
       new QListViewItem(FileList,(*i).first,(*i).second);
   }
   virtual void saveSetting(void)
   {
     int i=0;
     for(QListViewItem *item=FileList->firstChild();item;item=item->nextSibling()) {
-      QString nam=QString::number(i);
+      QCString nam=QString::number(i).latin1();
       Tool->setConfig(nam,item->text(0));
       nam+="file";
       Tool->setConfig(nam,item->text(1));
@@ -341,26 +352,28 @@ public:
     if (item) {
       try {
 	QString file=item->text(1);
-	std::map<QString,QString> pairs;
-	if (!toTool::loadMap(file,pairs)) {
+	std::map<QCString,QString> pairs;
+	try {
+	  toTool::loadMap(file,pairs);
+	} catch(...) {
 	  if (TOMessageBox::warning(this,
-				    "Couldn't open file.",
-				    "Couldn't open file. Start on new file?",
-				    "&Ok",
-				    "Cancel")==1)
+				    qApp->translate("toTemplatePrefs","Couldn't open file."),
+				    qApp->translate("toTemplatePrefs","Couldn't open file. Start on new file?"),
+				    qApp->translate("toTemplatePrefs","&Ok"),
+				    qApp->translate("toTemplatePrefs","Cancel"))==1)
 	    return;
 	}
 	toTemplateEdit edit(pairs,this);
 	if (edit.exec()) {
 	  edit.changeSelection();
 	  if (!toTool::saveMap(file,pairs))
-	    throw QString("Couldn't write file");
+	    throw qApp->translate("toTemplatePrefs","Couldn't write file");
 	}
       } catch (const QString &str) {
 	TOMessageBox::warning(this,
-			      "Couldn't open file",
+			      qApp->translate("toTemplatePrefs","Couldn't open file"),
 			      str,
-			      "&Ok");
+			      qApp->translate("toTemplatePrefs","&Ok"));
       }
     }
   }
@@ -372,46 +385,67 @@ public:
 
 class toTemplateTool : public toTool {
   TODock *Dock;
+  toTemplate *Window;
 protected:
   virtual char **pictureXPM(void)
   { return totemplate_xpm; }
 public:
   toTemplateTool()
     : toTool(410,"SQL Template")
-  { Dock=NULL; }
+  { Dock=NULL; Window=NULL; toTemplateProvider::setToolKey(key()); }
   virtual const char *menuItem()
   { return "SQL Template"; }
-  virtual QWidget *toolWindow(QWidget *parent,toConnection &connection)
+  virtual QWidget *toolWindow(QWidget *,toConnection &)
   {
-    if (Dock) {
-      if (Dock->isHidden()) {
-	delete Dock;
-	Dock=NULL;
-      } else {
-	delete Dock;
-	Dock=NULL;
-	return NULL;
-      }
+    if (!Dock||!Window) {
+      Dock=toAllocDock(qApp->translate("toTemplateTool","Template"),QString::null,*toolbarImage());
+      Window=new toTemplate(Dock);
+    } else if (Dock->isHidden()) {
+#if TO_KDE
+      toAttachDock(Dock,Window,QMainWindow::Left);
+#else
+      Dock->show();
+#endif
+      Window->showResult(true);
+    } else {
+#if TO_KDE
+      toAttachDock(Dock,Window,QMainWindow::Minimized);
+#else
+      Dock->hide();
+#endif
+      Window->showResult(false);
     }
-    Dock=toAllocDock("Template",QString::null,*toolbarImage());
-    toTemplate *window=new toTemplate(Dock);
-    toAttachDock(Dock,window,QMainWindow::Left);
-    window->attachResult();
-    return Dock;
+    return NULL;
   }
-  void closeWindow(toConnection &connection)
-  { Dock=NULL; }
+  void closeWindow()
+  { Dock=NULL; Window=NULL; toTemplateProvider::setShown(false); }
   virtual QWidget *configurationTab(QWidget *parent)
   { return new toTemplatePrefs(this,parent); }
-  virtual bool canHandle(toConnection &conn)
+  virtual bool canHandle(toConnection &)
   { return true; }
 };
 
 static toTemplateTool TemplateTool;
 
+void toTemplate::hideEvent(QHideEvent *e)
+{
+  toTemplateProvider::setShown(false);
+  QVBox::hideEvent(e);
+}
+
+void toTemplate::showEvent(QShowEvent *e)
+{
+  toTemplateProvider::setShown(true);
+  QVBox::showEvent(e);
+}
+
 QWidget *toTemplate::parentWidget(QListViewItem *item)
 {
-  return templateWidget(item)->frame();
+  try {
+    return templateWidget(item)->frame();
+  } catch(...) {
+    return NULL;
+  }
 }
 
 toTemplate *toTemplate::templateWidget(QListView *obj)
@@ -423,7 +457,7 @@ toTemplate *toTemplate::templateWidget(QListView *obj)
       return tpl;
     lst=lst->parent();
   }
-  throw("Not a toTemplate parent");
+  throw tr("Not a toTemplate parent");
 }
 
 toTemplate *toTemplate::templateWidget(QListViewItem *item)
@@ -441,20 +475,21 @@ public:
   { Template->closeFrame(); }
 };
 
-toTemplate::toTemplate(QWidget *parent)
-  : QVBox(parent),toHelpContext("template.html")
+toTemplate::toTemplate(TODock *parent)
+  : QVBox(parent),toHelpContext(QString::fromLatin1("template.html"))
 {
-  Toolbar=toAllocBar(this,"Template Toolbar",QString::null);
+  Toolbar=toAllocBar(this,tr("Template Toolbar"));
 
   List=new toListView(this);
-  List->addColumn("Template");
+  List->addColumn(tr("Template"));
   List->setRootIsDecorated(true);
   List->setSorting(0);
   List->setShowSortIndicator(false);
   List->setTreeStepSize(10);
   List->setSelectionMode(QListView::Single);
+  List->setResizeMode(QListView::AllColumns);
   TODock *dock;
-  dock=Result=toAllocDock("Template result",
+  dock=Result=toAllocDock(tr("Template result"),
 			  QString::null,
 			  *TemplateTool.toolbarImage());
   Frame=new toTemplateResult(dock,this);
@@ -470,22 +505,40 @@ toTemplate::toTemplate(QWidget *parent)
 	 i++)
       (*i)->insertItems(List,Toolbar);
 
-  Toolbar->setStretchableWidget(new QLabel("",Toolbar));
+  Toolbar->setStretchableWidget(new QLabel(Toolbar,TO_KDE_TOOLBAR_WIDGET));
 
   WidgetExtra=NULL;
   setWidget(NULL);
 
   setFocusProxy(List);
+  toAttachDock(parent,this,QMainWindow::Left);
+  toAttachDock((TODock *)Result,Frame,QMainWindow::Bottom);
 }
 
 toTemplate::~toTemplate()
 {
+  TemplateTool.closeWindow();
   delete Result;
 }
 
-void toTemplate::attachResult(void)
+void toTemplate::showResult(bool show)
 {
-  toAttachDock((TODock *)Result,Frame,QMainWindow::Bottom);
+  if (!Result)
+    return;
+
+  if (show) {
+#if TO_KDE
+    toAttachDock((TODock *)Result,Frame,QMainWindow::Bottom);
+#else
+    Result->show();
+#endif
+  } else {
+#if TO_KDE
+    toAttachDock((TODock *)Result,Frame,QMainWindow::Minimized);
+#else
+    Result->hide();
+#endif
+  }
 }
 
 void toTemplate::closeFrame(void)
@@ -517,7 +570,7 @@ void toTemplate::selected(QListViewItem *item)
 QWidget *toTemplate::frame(void)
 {
   if (Result->isHidden())
-    attachResult();
+    showResult(true);
   return Frame;
 }
 
@@ -528,14 +581,22 @@ void toTemplate::collapse(QListViewItem *item)
     ti->collapse();
 }
 
+
+QWidget *toTemplateItem::selectedWidget(QWidget *)
+{
+  return NULL;
+}
+
 void toTemplateItem::setSelected(bool sel)
 {
-  toTemplate *temp=toTemplate::templateWidget(this);
-  if (sel&&temp) {
-    QWidget *frame=toTemplate::parentWidget(this);
-    if (frame)
-      temp->setWidget(selectedWidget(frame));
-  }
+  try {
+    toTemplate *temp=toTemplate::templateWidget(this);
+    if (sel&&temp) {
+      QWidget *frame=toTemplate::parentWidget(this);
+      if (frame)
+	temp->setWidget(selectedWidget(frame));
+    }
+  } TOCATCH
   toResultViewItem::setSelected(sel);
 }
 
@@ -569,13 +630,13 @@ public:
 
 void toTextTemplate::insertItems(QListView *parent,QToolBar *)
 {
-  int tot=TemplateTool.config("Number",QString::number(-1)).toInt();
-  std::map<QString,QString> def=DefaultText();
+  int tot=TemplateTool.config("Number","-1").toInt();
+  std::map<QCString,QString> def=DefaultText();
 
   {
     for(int i=0;i<tot;i++) {
-      QString num=QString::number(i);
-      QString root=TemplateTool.config(num,"");
+      QCString num=QString::number(i).latin1();
+      QCString root=TemplateTool.config(num,"").latin1();
       num+="file";
       QString file=TemplateTool.config(num,"");
       addFile(parent,root,file);
@@ -583,48 +644,58 @@ void toTextTemplate::insertItems(QListView *parent,QToolBar *)
 	def.erase(def.find(root));
     }
   }
-  for (std::map<QString,QString>::iterator i=def.begin();i!=def.end();i++)
+  for (std::map<QCString,QString>::iterator i=def.begin();i!=def.end();i++)
     addFile(parent,(*i).first,(*i).second);
 }
 
 void toTextTemplate::addFile(QListView *parent,const QString &root,const QString &file)
 {
-  std::map<QString,QString> pairs;
-  toTool::loadMap(file,pairs);
-  toTemplateItem *last=new toTemplateItem(*this,parent,root);
-  int lastLevel=0;
-  QStringList lstCtx;
-  for(std::map<QString,QString>::iterator i=pairs.begin();i!=pairs.end();i++) {
-    QStringList ctx=QStringList::split(":",(*i).first);
-    if (last) {
-      while(last&&lastLevel>=int(ctx.count())) {
-	last=dynamic_cast<toTemplateItem *>(last->parent());
-	lastLevel--;
+  std::map<QCString,QString> pairs;
+  try {
+    toTool::loadMap(file,pairs);
+    toTemplateItem *last=new toTemplateItem(*this,parent,root);
+    int lastLevel=0;
+    QStringList lstCtx;
+    for(std::map<QCString,QString>::iterator i=pairs.begin();i!=pairs.end();i++) {
+      QStringList ctx=QStringList::split(QString::fromLatin1(":"),(*i).first);
+      if (last) {
+	while(last&&lastLevel>=int(ctx.count())) {
+	  last=dynamic_cast<toTemplateItem *>(last->parent());
+	  lastLevel--;
+	}
+	while(last&&lastLevel>=0&&!toCompareLists(lstCtx,ctx,(unsigned int)lastLevel)) {
+	  last=dynamic_cast<toTemplateItem *>(last->parent());
+	  lastLevel--;
+	}
       }
-      while(last&&lastLevel>=0&&!toCompareLists(lstCtx,ctx,(unsigned int)lastLevel)) {
-	last=dynamic_cast<toTemplateItem *>(last->parent());
-	lastLevel--;
+      if (lastLevel<0)
+	throw qApp->translate("toTemplate","Internal error, lastLevel < 0");
+      while(lastLevel<int(ctx.count())-1) {
+	last=new toTemplateItem(last,ctx[lastLevel]);
+	lastLevel++;
       }
-    }
-    if (lastLevel<0)
-      throw QString("Internal error, lastLevel < 0");
-    while(lastLevel<int(ctx.count())-1) {
-      last=new toTemplateItem(last,ctx[lastLevel]);
+      last=new toTemplateText(last,ctx[lastLevel],(*i).second);
+      lstCtx=ctx;
       lastLevel++;
     }
-    last=new toTemplateText(last,ctx[lastLevel],(*i).second);
-    lstCtx=ctx;
-    lastLevel++;
-  }
+  } TOCATCH
 }
 
 toTemplateSQL::toTemplateSQL(toConnection &conn,toTemplateItem *parent,
 			     const QString &name,const QString &sql)
-  : toTemplateItem(parent,name),Object(this),Connection(conn),SQL(sql)
+  : toTemplateItem(parent,name),Object(this),Connection(&conn),SQL(sql)
 {
   setExpandable(true);
 }
 
+toTemplateSQL::toTemplateSQL(toTemplateItem *parent,
+			     const QString &name,const QString &sql)
+  : toTemplateItem(parent,name),Object(this),SQL(sql)
+{
+  Connection=NULL;
+  setExpandable(true);
+}
+
 toTemplateSQLObject::toTemplateSQLObject(toTemplateSQL *parent)
   : Parent(parent)
 {
@@ -658,9 +729,9 @@ void toTemplateSQLObject::poll(void)
     if (Query&&Query->poll()) {
       toQDescList desc=Query->describe();
       while(Query->poll()&&!Query->eof()) {
-	Parent->createChild(Query->readValue());
+	QListViewItem *item=Parent->createChild(Query->readValue());
 	for (unsigned int j=1;j<desc.size();j++)
-	  Query->readValue();
+	  item->setText(j,Query->readValue());
       }
       if (Query->eof()) {
 	delete Query;
