@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -59,6 +59,11 @@
 
 #include "totool.moc"
 
+// A little magic to get lrefresh to work and get a check on qApp
+
+#undef QT_TRANSLATE_NOOP
+#define QT_TRANSLATE_NOOP(x,y) QTRANS(x,y)
+
 toToolWidget::toToolWidget(toTool &tool,const QString &ctx,QWidget *parent,toConnection &conn,const char *name)
   : QVBox(parent,name,WDestructiveClose),toHelpContext(ctx),Tool(tool)
 {
@@ -79,7 +84,9 @@ toToolWidget::toToolWidget(toTool &tool,const QString &ctx,QWidget *parent,toCon
 
 void toToolWidget::parentConnection(void)
 {
-  setConnection(toCurrentConnection(parentWidget()));
+  try {
+    setConnection(toCurrentConnection(parentWidget()));
+  } TOCATCH
 }
 
 toToolWidget::~toToolWidget()
@@ -113,43 +120,45 @@ toTimer *toToolWidget::timer(void)
   return Timer;
 }
 
-void toToolWidget::exportData(std::map<QString,QString> &data,const QString &prefix)
+void toToolWidget::exportData(std::map<QCString,QString> &data,const QCString &prefix)
 {
   QWidget *par=parentWidget();
   if (!par)
     par=this;
-  if (par->isMaximized()||par->width()>toMainWidget()->workspace()->width())
-    data[prefix+":State"]="Maximized";
-  else if (par->isMinimized())
-    data[prefix+":State"]="Minimized";
-  else {
-    data[prefix+":X"]=QString::number(par->x());
-    data[prefix+":Y"]=QString::number(par->y());
-    data[prefix+":Width"]=QString::number(par->width());
-    data[prefix+":Height"]=QString::number(par->height());
-  }
+  if (isMaximized()||par->width()>=toMainWidget()->workspace()->width())
+    data[prefix+":State"]=QString::fromLatin1("Maximized");
+  else if (isMinimized())
+    data[prefix+":State"]=QString::fromLatin1("Minimized");
+
+  data[prefix+":X"]=QString::number(par->x());
+  data[prefix+":Y"]=QString::number(par->y());
+  data[prefix+":Width"]=QString::number(par->width());
+  data[prefix+":Height"]=QString::number(par->height());
 }
 
-void toToolWidget::importData(std::map<QString,QString> &data,const QString &prefix)
+void toToolWidget::importData(std::map<QCString,QString> &data,const QCString &prefix)
 {
   QWidget *par=parentWidget();
   if (!par)
     par=this;
-  if (data[prefix+":State"]=="Maximized")
+  if (data[prefix+":State"]==QString::fromLatin1("Maximized")) {
     par->showMaximized();
-  else if (data[prefix+":State"]=="Minimized")
+    showMaximized();
+  } else if (data[prefix+":State"]==QString::fromLatin1("Minimized")) {
     par->showMinimized();
-  else {
+    showMinimized();
+  } else {
     par->showNormal();
     par->setGeometry(data[prefix+":X"].toInt(),
 		     data[prefix+":Y"].toInt(),
 		     data[prefix+":Width"].toInt(),
 		     data[prefix+":Height"].toInt());
+
   }
 }
 
-std::map<QString,toTool *> *toTool::Tools;
-std::map<QString,QString> *toTool::Configuration;
+std::map<QCString,toTool *> *toTool::Tools;
+std::map<QCString,QString> *toTool::Configuration;
 
 #define CONFIG_FILE "/.torarc"
 #define DEF_CONFIG_FILE "/etc/torarc"
@@ -163,7 +172,7 @@ toTool::toTool(int priority,const char *name)
   : Name(name)
 {
   if (!Tools)
-    Tools=new std::map<QString,toTool *>;
+    Tools=new std::map<QCString,toTool *>;
   Priority=priority;
   Key.sprintf("%05d%s",priority,name);
   (*Tools)[Key]=this;
@@ -189,6 +198,8 @@ void toTool::createWindow(void)
 {
   toMain *main=(toMain *)qApp->mainWidget();
   try {
+    if (!canHandle(main->currentConnection()))
+      throw QString("The tool %1 doesn't support the current database").arg(name());
     QWidget *newWin=toolWindow(main->workspace(),main->currentConnection());
 
     if (newWin) {
@@ -214,7 +225,7 @@ void toTool::createWindow(void)
 	  newWin->showMaximized();
       }
 
-#if 1
+#if QT_VERSION < 0x030100
       // This is a really ugly workaround for a Qt layout bug
       QWidget *tmp=NULL;
       QWidget *tmp2=NULL;
@@ -236,24 +247,23 @@ void toTool::createWindow(void)
   } TOCATCH
 }
 
-bool toTool::saveMap(const QString &file,std::map<QString,QString> &pairs)
+bool toTool::saveMap(const QString &file,std::map<QCString,QString> &pairs)
 {
   QCString data;
 
   {
-    QRegExp newline("\n");
-    QRegExp backslash("\\");
-    for (std::map<QString,QString>::iterator i=pairs.begin();i!=pairs.end();i++) {
-      QString line=(*i).first;
-      line.append("=");
-      line.replace(backslash,"\\\\");
-      line.replace(newline,"\\n");
-      QCString str=line.latin1();
-      line=(*i).second;
-      line.replace(backslash,"\\\\");
-      line.replace(newline,"\\n");
+    QRegExp newline(QString::fromLatin1("\n"));
+    QRegExp backslash(QString::fromLatin1("\\"));
+    for (std::map<QCString,QString>::iterator i=pairs.begin();i!=pairs.end();i++) {
+      QCString str=(*i).first;
+      str.append(QString::fromLatin1("="));
+      str.replace(backslash,QString::fromLatin1("\\\\"));
+      str.replace(newline,QString::fromLatin1("\\n"));
+      QString line=(*i).second;
+      line.replace(backslash,QString::fromLatin1("\\\\"));
+      line.replace(newline,QString::fromLatin1("\\n"));
       str+=line.utf8();
-      str+="\n";
+      str+=QString::fromLatin1("\n");
       data+=str;
     }
   }
@@ -261,7 +271,8 @@ bool toTool::saveMap(const QString &file,std::map<QString,QString> &pairs)
 }
 
 #ifdef WIN32
-#  define APPLICATION_NAME "SOFTWARE\\GlobeCom\\tora\\"
+#  define APPLICATION_NAME "SOFTWARE\\Quest\\tora\\"
+#  define FALLBACK_NAME    "SOFTWARE\\Underscore\\tora\\"
 
 static char *toKeyPath(const QString &str,CRegistry &registry)
 {
@@ -269,7 +280,7 @@ static char *toKeyPath(const QString &str,CRegistry &registry)
   for(int pos=str.length()-1;pos>=0&&str.at(pos)!='\\';pos--)
     ;
   if (pos<0)
-    throw QString("Couldn't find \\ in path");
+    throw QT_TRANSLATE_NOOP("toKeyPath","Couldn't find \\ in path");
   QString ret=str.mid(0,pos);
   if (buf)
     free(buf);
@@ -284,7 +295,7 @@ static char *toKeyValue(const QString &str)
   for(int pos=str.length()-1;pos>=0&&str.at(pos)!='\\';pos--)
     ;
   if (pos<0)
-    throw QString("Couldn't find \\ in path");
+    throw QT_TRANSLATE_NOOP("toKeyValue","Couldn't find \\ in path");
   if (buf)
     free(buf);
   buf=strdup(str.mid(pos+1));
@@ -295,65 +306,70 @@ static char *toKeyValue(const QString &str)
 
 void toTool::saveConfig(void)
 {
+  try {
 #ifdef WIN32
-  CRegistry registry;
-  QRegExp re(":");
-  for (std::map<QString,QString>::iterator i=Configuration->begin();i!=Configuration->end();i++) {
-    QString path=(*i).first;
-    QString value=(*i).second;
-    path.prepend(APPLICATION_NAME);
-    path.replace(re,"\\");
-    if (value.isNull())
-      registry.SetStringValue(HKEY_CURRENT_USER,
-			      toKeyPath(path,registry),
-			      toKeyValue(path),
-			      "");
-    else {
-      char *t=strdup(value.utf8());
-      registry.SetStringValue(HKEY_CURRENT_USER,
-			      toKeyPath(path,registry),
-			      toKeyValue(path),
-			      t);
-      free(t);
+    CRegistry registry;
+    QRegExp re(":");
+    for (std::map<QCString,QString>::iterator i=Configuration->begin();i!=Configuration->end();i++) {
+      QCString path=(*i).first;
+      QString value=(*i).second;
+      path.prepend(APPLICATION_NAME);
+      path.replace(re,"\\");
+      if (value.isEmpty()) {
+	if (!registry.SetStringValue(HKEY_CURRENT_USER,
+				     toKeyPath(path,registry),
+				     toKeyValue(path),
+				     ""))
+	  toStatusMessage(QT_TRANSLATE_NOOP("toTool","Couldn't save empty value at key %1").arg(path));
+      } else {
+	char *t=strdup(value.utf8());
+	if (!registry.SetStringValue(HKEY_CURRENT_USER,
+				     toKeyPath(path,registry),
+				     toKeyValue(path),
+				     t))
+	  toStatusMessage(QT_TRANSLATE_NOOP("toTool","Couldn't save %1 value at key %2").arg(value).arg(path));
+	free(t);
+      }
     }
-  }
 #else
-  if (!Configuration)
-    return;
-  QString conf;
-  if (getenv("HOME")) {
-    conf=getenv("HOME");
-  }
-  conf.append(CONFIG_FILE);
-  saveMap(conf,*Configuration);
+    if (!Configuration)
+      return;
+    QCString conf;
+    if (getenv("HOME")) {
+      conf=getenv("HOME");
+    }
+    conf.append(CONFIG_FILE);
+    saveMap(conf,*Configuration);
 #endif
+  } TOCATCH
 }
 
 void toTool::loadConfig(void)
 {
   if (Configuration)
     delete Configuration;
-  Configuration=new std::map<QString,QString>;
+  Configuration=new std::map<QCString,QString>;
 
 #ifndef WIN32
   QString conf;
   if (getenv("HOME")) {
-    conf=getenv("HOME");
+    conf=QString::fromLatin1(getenv("HOME"));
+  }
+  conf.append(QString::fromLatin1(CONFIG_FILE));
+  try {
+    loadMap(conf,*Configuration);
+  } catch(...) {
+    try {
+      loadMap(QString::fromLatin1(DEF_CONFIG_FILE),*Configuration);
+    } catch(...) {
+    }
   }
-  conf.append(CONFIG_FILE);
-  if (!loadMap(conf,*Configuration))
-    loadMap(DEF_CONFIG_FILE,*Configuration);
 #endif
 }
 
-bool toTool::loadMap(const QString &filename,std::map<QString,QString> &pairs)
+void toTool::loadMap(const QString &filename,std::map<QCString,QString> &pairs)
 {
-  QCString data;
-  try {
-    data=toReadFile(filename);
-  } catch (...) {
-    return false;
-  }
+  QCString data=toReadFile(filename);
 
   int pos=0;
   int bol=0;
@@ -363,13 +379,13 @@ bool toTool::loadMap(const QString &filename,std::map<QString,QString> &pairs)
   while(pos<size) {
     switch(data[pos]) {
     case '\n':
-      if (endtag==-1)
-	throw QString("Malformed tag in config file. Missing = on row. (%1)").
-	  arg(((const char *)data)+bol);
       data[wpos]=0;
+      if (endtag==-1)
+	throw QT_TRANSLATE_NOOP("toTool","Malformed tag in config file. Missing = on row. (%1)").
+	  arg(data.mid(bol,wpos-bol));
       {
-	QString tag=((const char *)data)+bol;
-	QString val=((const char *)data)+endtag+1;
+	QCString tag=((const char *)data)+bol;
+	QCString val=((const char *)data)+endtag+1;
 	pairs[tag]=QString::fromUtf8(val);
       }
       bol=pos+1;
@@ -397,7 +413,7 @@ bool toTool::loadMap(const QString &filename,std::map<QString,QString> &pairs)
 	  data[wpos]=':';
 	break;
       default:
-	throw QString("Unknown escape character in string (Only \\\\ and \\n recognised)");
+	throw QT_TRANSLATE_NOOP("toTool","Unknown escape character in string (Only \\\\ and \\n recognised)");
       }
       break;
     default:
@@ -406,36 +422,44 @@ bool toTool::loadMap(const QString &filename,std::map<QString,QString> &pairs)
     wpos++;
     pos++;
   }
-  return true;
+  return;
 }
 
-const QString &toTool::config(const QString &tag,const QString &def)
+const QString &toTool::config(const QCString &tag,const QCString &def)
 {
-  QString str=name();
+  QCString str=name();
   str.append(":");
   str.append(tag);
   return globalConfig(str,def);
 }
 
-void toTool::setConfig(const QString &tag,const QString &def)
+void toTool::eraseConfig(const QCString &tag)
 {
-  QString str=name();
+  QCString str=name();
+  str.append(":");
+  str.append(tag);
+  globalEraseConfig(str);
+}
+
+void toTool::setConfig(const QCString &tag,const QString &def)
+{
+  QCString str=name();
   str.append(":");
   str.append(tag);
   globalSetConfig(str,def);
 }
 
-const QString &toTool::globalConfig(const QString &tag,const QString &def)
+const QString &toTool::globalConfig(const QCString &tag,const QCString &def)
 {
   if (!Configuration)
     loadConfig();
 
-  std::map<QString,QString>::iterator i=Configuration->find(tag);
+  std::map<QCString,QString>::iterator i=Configuration->find(tag);
   if (i==Configuration->end()) {
 #if defined(WIN32)
     CRegistry registry;
-    QRegExp re(":");
-    QString path=tag;
+    QRegExp re(QString::fromLatin1(":"));
+    QCString path=tag;
     path.prepend(APPLICATION_NAME);
     path.replace(re,"\\");
     DWORD siz=1024;
@@ -454,14 +478,51 @@ const QString &toTool::globalConfig(const QString &tag,const QString &def)
 	return (*Configuration)[tag];
       }
     } catch (...) {
+      try {
+	path=tag;
+	path.prepend(FALLBACK_NAME);
+	path.replace(re,"\\");
+	if (registry.GetStringValue(HKEY_CURRENT_USER,
+				    toKeyPath(path,registry),
+				    toKeyValue(path),
+				    buffer,siz)) {
+	  if (siz>0) {
+	    QString ret=QString::fromUtf8(buffer);
+	    (*Configuration)[tag]=ret;
+	  } else {
+	    (*Configuration)[tag]="";
+	  }
+	  return (*Configuration)[tag];
+	}
+      } catch (...) {
+      }
     }
 #endif
-    return def;
+    (*Configuration)[tag]=QString::fromLatin1(def);
+    return (*Configuration)[tag];
   }
   return (*i).second;
 }
 
-void toTool::globalSetConfig(const QString &tag,const QString &value)
+void toTool::globalEraseConfig(const QCString &tag)
+{
+  if (!Configuration)
+    loadConfig();
+  std::map<QCString,QString>::iterator i=Configuration->find(tag);
+  if (i!=Configuration->end()) {
+    Configuration->erase(i);
+#if defined(WIN32)
+    CRegistry registry;
+    QRegExp re(QString::fromLatin1(":"));
+    QCString path=tag;
+    path.prepend(APPLICATION_NAME);
+    path.replace(re,"\\"); 
+    registry.DeleteKey(HKEY_CURRENT_USER,path); // Don't really care if it works.
+#endif
+  }
+}
+
+void toTool::globalSetConfig(const QCString &tag,const QString &value)
 {
   if (!Configuration)
     loadConfig();
@@ -471,16 +532,29 @@ void toTool::globalSetConfig(const QString &tag,const QString &value)
 
 bool toTool::canHandle(toConnection &conn)
 {
-  return (conn.provider()=="Oracle");
+  return (toIsOracle(conn));
 }
 
-toTool *toTool::tool(const QString &key)
+void toTool::customSetup(int)
+{
+}
+
+QWidget *toTool::configurationTab(QWidget *)
+{
+  return NULL;
+}
+
+toTool *toTool::tool(const QCString &key)
 {
   if (!Tools)
-    Tools=new std::map<QString,toTool *>;
-  std::map<QString,toTool *>::iterator i=Tools->find(key);
+    Tools=new std::map<QCString,toTool *>;
+  std::map<QCString,toTool *>::iterator i=Tools->find(key);
   if (i==Tools->end())
     return NULL;
   
   return (*i).second;
 }
+
+void toTool::about(QWidget *)
+{
+}
