@@ -1,7 +1,7 @@
 //***************************************************************************
 /*
  * TOra - An Oracle Toolkit for DBA's and developers
- * Copyright (C) 2000-2001,2001 Underscore AB
+ * Copyright (C) 2003 Quest Software, Inc
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,8 @@
  *      Specifically you are not permitted to link this program with the
  *      Qt/UNIX, Qt/Windows or Qt Non Commercial products of TrollTech.
  *      And you are not permitted to distribute binaries compiled against
- *      these libraries without written consent from Underscore AB. Observe
- *      that this does not disallow linking to the Qt Free Edition.
+ *      these libraries without written consent from Quest Software, Inc.
+ *      Observe that this does not disallow linking to the Qt Free Edition.
  *
  * All trademarks belong to their respective owners.
  *
@@ -59,6 +59,7 @@
 
 #include <qapplication.h>
 #include <qcombobox.h>
+#include <qcstring.h>
 #include <qcursor.h>
 #include <qfile.h>
 #include <qfiledialog.h>
@@ -81,24 +82,40 @@
 #include "windows/cregistry.h"
 #endif
 
+#if defined(Q_OS_MACX)
+#include <sys/param.h>
+#include <CoreServices/CoreServices.h>
+#endif // Q_OS_MACX
+
 #define CHUNK_SIZE 31
 
+// A little magic to get lrefresh to work and get a check on qApp
+
+#undef QT_TRANSLATE_NOOP
+#define QT_TRANSLATE_NOOP(x,y) QTRANS(x,y)
+
 static toSQL SQLUserNames(toSQL::TOSQL_USERLIST,
 			  "SELECT UserName FROM sys.All_Users ORDER BY UserName",
 			  "List users in the database");
 
 static toSQL SQLUserNamesMySQL(toSQL::TOSQL_USERLIST,
 			       "SHOW DATABASES",
-			       QString::null,
+			       "",
 			       "3.0",
 			       "MySQL");
 
 static toSQL SQLUserNamesPgSQL(toSQL::TOSQL_USERLIST,
 			       "SELECT usename AS UserName FROM pg_user ORDER BY UserName",
-			       QString::null,
+			       "",
 			       "7.1",
 			       "PostgreSQL");
 
+static toSQL SQLUserNamesSapDB(toSQL::TOSQL_USERLIST,
+			       "SELECT username \"UserName\" FROM users ORDER BY username",
+			       "",
+			       "",
+			       "SapDB");
+
 static toSQL SQLTextPiece("Global:SQLText",
 			  "SELECT SQL_Text\n"
 			  "  FROM V$SQLText_With_Newlines\n"
@@ -116,7 +133,7 @@ QString toSQLString(toConnection &conn,const QString &address)
     sql.append(*i);
   }
   if (sql.isEmpty())
-    throw QString("SQL Address not found in SGA");
+    throw qApp->translate("toSQLString","SQL Address not found in SGA");
   return sql;
 }
 
@@ -126,12 +143,12 @@ static toSQL SQLNow("Global:Now",
 
 static toSQL SQLNowMySQL("Global:Now",
 			 "SELECT now()",
-			 QString::null,
+			 "",
 			 "3.0",
 			 "MySQL");
 static toSQL SQLNowPgSQL("Global:Now",
 			 "SELECT now()",
-			 QString::null,
+			 "",
 			 "7.1",
 			 "PostgreSQL");
 
@@ -141,7 +158,7 @@ QString toNow(toConnection &conn)
     toQList vals=toQuery::readQuery(conn,SQLNow);
     return toPop(vals);
   } catch(...) {
-    return "Unexpected error";
+    return qApp->translate("toNow","Unexpected error");
   }
 }
 
@@ -150,35 +167,38 @@ QString toSQLStripSpecifier(const QString &sql)
   QString ret;
   char inString=0;
   for(unsigned int i=0;i<sql.length();i++) {
-    QChar c=sql.at(i);
+    QChar rc=sql.at(i);
+    char c=rc.latin1();
     if (inString) {
-      if (char(c)==inString) {
+      if (c==inString) {
 	inString=0;
       }
       ret+=c;
     } else {
-      switch(char(c)) {
+      switch(c) {
       case '\'':
 	inString='\'';
-	ret+=c;
+	ret+=rc;
 	break;
       case '\"':
 	inString='\"';
-	ret+=c;
+	ret+=rc;
 	break;
       case ':':
-	ret+=c;
+	ret+=rc;
 	for (i++;i<sql.length();i++) {
-	  c=sql.at(i);
-	  if (!c.isLetterOrNumber())
+	  rc=sql.at(i);
+	  c=rc.latin1();
+	  if (!rc.isLetterOrNumber())
 	    break;
-	  ret+=c;
+	  ret+=rc;
 	}
 	if (c=='<') {
-	  ret+=" ";
+	  ret+=QString::fromLatin1(" ");
 	  for (i++;i<sql.length();i++) {
-	    c=sql.at(i);
-	    ret+=" ";
+	    rc=sql.at(i);
+	    c=rc.latin1();
+	    ret+=QString::fromLatin1(" ");
 	    if (c=='>') {
 	      i++;
 	      break;
@@ -188,7 +208,7 @@ QString toSQLStripSpecifier(const QString &sql)
 	i--;
 	break;
       default:
-	ret+=c;
+	ret+=rc;
       }
     }
   }
@@ -200,32 +220,35 @@ QString toSQLStripBind(const QString &sql)
   QString ret;
   char inString=0;
   for(unsigned int i=0;i<sql.length();i++) {
-    QChar c=sql.at(i);
+    QChar rc=sql.at(i);
+    char c=rc.latin1();
     if (inString) {
-      if (char(c)==inString) {
+      if (c==inString) {
 	inString=0;
       }
-      ret+=c;
+      ret+=rc;
     } else {
       switch(char(c)) {
       case '\'':
 	inString='\'';
-	ret+=c;
+	ret+=rc;
 	break;
       case '\"':
 	inString='\"';
-	ret+=c;
+	ret+=rc;
 	break;
       case ':':
-	ret+="''";
+	ret+=QString::fromLatin1("''");
 	for (i++;i<sql.length();i++) {
-	  c=sql.at(i);
-	  if (!c.isLetterOrNumber())
+	  rc=sql.at(i);
+	  c=rc.latin1();
+	  if (!rc.isLetterOrNumber())
 	    break;
 	}
 	if (c=='<') {
 	  for (i++;i<sql.length();i++) {
-	    c=sql.at(i);
+	    rc=sql.at(i);
+	    c=rc.latin1();
 	    if (c=='>') {
 	      i++;
 	      break;
@@ -235,7 +258,7 @@ QString toSQLStripBind(const QString &sql)
 	i--;
 	break;
       default:
-	ret+=c;
+	ret+=rc;
       }
     }
   }
@@ -258,13 +281,15 @@ QString toSQLToAddress(toConnection &conn,const QString &sql)
     if (search==toSQLString(conn,*i))
       return *i;
   }
-  throw QString("SQL Query not found in SGA");
+  throw qApp->translate("toSQLToAddress","SQL Query not found in SGA");
 }
 
 static std::list<QString> LastMessages;
 
 void toStatusMessage(const QString &str,bool save,bool log)
 {
+  if (!toThread::mainThread())
+    return;
   toMain *main=dynamic_cast<toMain *>(qApp->mainWidget());
   if (main) {
     int sec=toTool::globalConfig(CONF_STATUS_MESSAGE,DEFAULT_STATUS_MESSAGE).toInt();
@@ -300,16 +325,16 @@ QComboBox *toRefreshCreate(QWidget *parent,const char *name,const QString &def,Q
   if (item)
     refresh=item;
   else
-    refresh=new QComboBox(false,parent);
+    refresh=new QComboBox(false,parent,name);
 
-  refresh->insertItem("None");
-  refresh->insertItem("2 seconds");
-  refresh->insertItem("5 seconds");
-  refresh->insertItem("10 seconds");
-  refresh->insertItem("30 seconds");
-  refresh->insertItem("1 min");
-  refresh->insertItem("5 min");
-  refresh->insertItem("10 min");
+  refresh->insertItem(qApp->translate("toRefreshCreate","None"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","2 seconds"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","5 seconds"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","10 seconds"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","30 seconds"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","1 min"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","5 min"));
+  refresh->insertItem(qApp->translate("toRefreshCreate","10 min"));
   QString str;
   if (def)
     str=def;
@@ -340,24 +365,24 @@ void toRefreshParse(toTimer *timer,const QString &str)
   if (t.isEmpty())
     t=toTool::globalConfig(CONF_REFRESH,DEFAULT_REFRESH);
 
-  if (t=="None")
+  if (t==qApp->translate("toRefreshCreate","None")||t=="None")
     timer->stop();
-  else if (t=="2 seconds")
+  else if (t==qApp->translate("toRefreshCreate","2 seconds")||t=="2 seconds")
     timer->start(2*1000);
-  else if (t=="5 seconds")
+  else if (t==qApp->translate("toRefreshCreate","5 seconds")||t=="5 seconds")
     timer->start(5*1000);
-  else if (t=="10 seconds")
+  else if (t==qApp->translate("toRefreshCreate","10 seconds")||t=="10 seconds")
     timer->start(10*1000);
-  else if (t=="30 seconds")
+  else if (t==qApp->translate("toRefreshCreate","30 seconds")||t=="30 seconds")
     timer->start(30*1000);
-  else if (t=="1 min")
+  else if (t==qApp->translate("toRefreshCreate","1 min")||t=="1 min")
     timer->start(60*1000);
-  else if (t=="5 min")
+  else if (t==qApp->translate("toRefreshCreate","5 min")||t=="5 min")
     timer->start(300*1000);
-  else if (t=="10 min")
+  else if (t==qApp->translate("toRefreshCreate","10 min")||t=="10 min")
     timer->start(600*1000);
   else
-    throw QString("Unknown timer value");
+    throw qApp->translate("toRefreshParse","Unknown timer value");
 }
 
 QString toDeepCopy(const QString &str)
@@ -366,12 +391,13 @@ QString toDeepCopy(const QString &str)
 }
 
 #ifdef ENABLE_STYLE
-#include <qmotifstyle.h>
-#include <qmotifplusstyle.h>
-#include <qsgistyle.h>
-#include <qcdestyle.h>
-#include <qwindowsstyle.h>
-#include <qplatinumstyle.h>
+#  if QT_VERSION < 300
+#    include <qmotifstyle.h>
+#    include <qmotifplusstyle.h>
+#    include <qsgistyle.h>
+#    include <qcdestyle.h>
+#    include <qwindowsstyle.h>
+#    include <qplatinumstyle.h>
 
 void toSetSessionType(const QString &str)
 {
@@ -388,7 +414,7 @@ void toSetSessionType(const QString &str)
   else if (str=="Platinum")
     qApp->setStyle(new QPlatinumStyle());
   else
-    throw QString("Failed to find style match");
+    toStatusMessage(qApp->translate("toSetSessionType","Failed to find style %1").arg(str));
 }
 
 QString toGetSessionType(void)
@@ -406,15 +432,76 @@ QString toGetSessionType(void)
     return "Platinum";
   else if (style->isA("QWindowsStyle"))
     return "Windows";
-  throw QString("Failed to find style match");
+  toStatusMessage(qApp->translate("toGetSessionType","Failed to find style match"));
+  return DEFAULT_STYLE;
 }
+
+QStringList toGetSessionTypes(void)
+{
+  QStringList ret;
+  ret<<"Motif Plus";
+  ret<<"SGI";
+  ret<<"CDI";
+  ret<<"Motif";
+  ret<<"Platinum";
+  ret<<"Windows";
+  return ret;
+}
+
+#  else
+#    include <qstylefactory.h>
+#    include <qstyle.h>
+
+QStringList toGetSessionTypes(void)
+{
+  return QStyleFactory::keys();
+}
+
+QString toGetSessionType(void)
+{
+  QStyle *style=&qApp->style();
+
+  // This is probably really slow, but who cares.
+
+  QStringList keys=QStyleFactory::keys();
+  for(unsigned int i=0;i<keys.size();i++) {
+    QString name=keys[i];
+    QStyle *t=QStyleFactory::create(name);
+    if (!t)
+      continue;
+    if (t->className()==style->className()) {
+      delete t;
+      return name;
+    }
+    delete t;
+  }
+
+  // Weird should never get here.
+  return style->className();
+}
+
+void toSetSessionType(const QString &str)
+{
+  QStyle *style=QStyleFactory::create(str);
+  if (style)
+    qApp->setStyle(style);
+  else
+    toStatusMessage(qApp->translate("toSetSessionType","Failed to find style %1").arg(str));
+}
+
+#  endif
 #endif
 
-QToolBar *toAllocBar(QWidget *parent,const QString &str,const QString &db)
+QToolBar *toAllocBar(QWidget *parent,const QString &str)
 {
+  QString db;
+  try {
+    db=toCurrentConnection(parent).description(false);
+  } catch(...) {
+  }
   QString name=str;
   if (!db.isEmpty()&&!toTool::globalConfig(CONF_DB_TITLE,"Yes").isEmpty()) {
-    name+=" ";
+    name+=QString::fromLatin1(" ");
     name+=db;
   }
   QToolBar *tool;
@@ -440,11 +527,16 @@ QToolBar *toAllocBar(QWidget *parent,const QString &str,const QString &db)
 
 TODock *toAllocDock(const QString &name,
 		    const QString &db,
-		    const QPixmap &icon)
+#ifdef TO_KDE
+		    const QPixmap &icon
+#else
+		    const QPixmap &
+#endif
+		    )
 {
   QString str=name;
   if (!db.isEmpty()&&!toTool::globalConfig(CONF_DB_TITLE,"Yes").isEmpty()) {
-    str+=" ";
+    str+=QString::fromLatin1(" ");
     str+=db;
   }
 #ifdef TO_KDE
@@ -457,7 +549,7 @@ TODock *toAllocDock(const QString &name,
     frm->setCaption(str);
     return frm;
   } else {
-    QToolBar *toolbar=toAllocBar(toMainWidget(),name,db);
+    QToolBar *toolbar=toAllocBar(toMainWidget(),name);
     return toolbar;
   }
 #  else
@@ -473,7 +565,7 @@ void toAttachDock(TODock *dock,QWidget *container,QMainWindow::ToolBarDock place
 #ifdef TO_KDE
   KDockMainWindow *main=dynamic_cast<KDockMainWindow *>(toMainWidget());
   if (main) {
-    KDockWidget::DockPosition pos;
+    KDockWidget::DockPosition pos=KDockWidget::DockLeft;
     int pct=20;
     switch (place) {
     case QMainWindow::Top:
@@ -483,23 +575,29 @@ void toAttachDock(TODock *dock,QWidget *container,QMainWindow::ToolBarDock place
       pct=80;
       pos=KDockWidget::DockBottom;
       break;
-    case QMainWindow::Left:
-      pos=KDockWidget::DockLeft;
+    default:
+      toStatusMessage(qApp->translate("toAttachDock","Unknown dock position"));
       break;
     case QMainWindow::Right:
       pct=80;
       pos=KDockWidget::DockRight;
       break;
-    default:
-      throw QString("Unknown dock position");
+    case QMainWindow::Left:
+    case QMainWindow::Minimized:
+      break;
     }
     KDockWidget *dw=(KDockWidget *)(dock);
     if (dw) {
       dw->setWidget(container);
-      dw->manualDock(main->getMainDockWidget(),pos,pct);
+      if (place==QMainWindow::Minimized)
+	main->makeDockInvisible(dw);
+      else
+	dw->manualDock(main->getMainDockWidget(),pos,pct);
     }
-  } else
-    throw QString("Main widget not KDockMainWindow");
+  } else {
+    toStatusMessage(qApp->translate("toAttachDock","Main widget not KDockMainWindow"));
+    return;
+  }
 #else
 #  if QT_VERSION < 300
   if (!toTool::globalConfig(CONF_DOCK_TOOLBAR,"Yes").isEmpty()) {
@@ -543,7 +641,7 @@ QString toFontToString(const QFont &fnt)
 QFont toStringToFont(const QString &str)
 {
   if (str.isEmpty())
-    return QFont("Courier",12);
+    return QFont(QString::fromLatin1("Courier"),12);
 #if QT_VERSION >= 300
   QFont fnt;
   if (fnt.fromString(str))
@@ -554,9 +652,9 @@ QFont toStringToFont(const QString &str)
   fnt.setRawName(str);
   return fnt;
 #else
-  QStringList lst=QStringList::split("/",str);
+  QStringList lst=QStringList::split(QString::fromLatin1("/"),str);
   if (lst.count()!=5)
-    return QFont("Courier",12);
+    return QFont(QString::fromLatin1("Courier"),12);
   return QFont(lst[0],lst[1].toInt(),lst[2].toInt(),
 	       bool(lst[3].toInt())
 #  if QT_VERSION < 300
@@ -568,9 +666,9 @@ QFont toStringToFont(const QString &str)
 
 int toSizeDecode(const QString &str)
 {
-  if (str=="KB")
+  if (str==QString::fromLatin1("KB"))
     return 1024;
-  if (str=="MB")
+  if (str==QString::fromLatin1("MB"))
     return 1024*1024;
   return 1;
 }
@@ -578,15 +676,17 @@ int toSizeDecode(const QString &str)
 QString toPluginPath(void)
 {
   QString str;
-#ifndef WIN32
-  str=toTool::globalConfig(CONF_PLUGIN_DIR,DEFAULT_PLUGIN_DIR);
-#else
+#ifdef WIN32
   CRegistry registry;
   DWORD siz=1024;
   char buffer[1024];
   try {
     if (registry.GetStringValue(HKEY_LOCAL_MACHINE,
+#ifdef OAS
+				"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\oas.exe",
+#else
 				"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\tora.exe",
+#endif
 				"Path",
 				buffer,siz)) {
       if (siz>0)
@@ -594,6 +694,34 @@ QString toPluginPath(void)
     }
   } catch(...) {
   }
+#elif defined( Q_OS_MACX )
+  { // MacOS
+  char resourcePath[MAXPATHLEN];
+  memset( &resourcePath[0], 0, MAXPATHLEN );
+  CFBundleRef appBundle = ::CFBundleGetMainBundle();
+  if ( appBundle ) {
+    CFURLRef urlRef = CFBundleCopyResourcesDirectoryURL( appBundle );
+    if ( urlRef ) {
+      UInt8* _p = (UInt8*) &resourcePath[0];
+      bool isOK = CFURLGetFileSystemRepresentation(
+          urlRef, TRUE, _p, MAXPATHLEN );
+      if ( !isOK ) {
+        // QMessageBox::warning( 0, "File error",
+        //     QString( "Unexpected: no file system representation") );
+      }
+    } else {
+      // QMessageBox::warning( 0, "File error",
+      //     QString( "Unexpected: unable to get resource directory") );
+    }
+    CFRelease( urlRef );
+    str = &resourcePath[0];
+  } else {
+    // QMessageBox::warning( 0, "File error",
+    //     QString( "Unexpected: unable to get main bundle") );
+  }
+  } // MacOS
+#else
+  str=toTool::globalConfig(CONF_PLUGIN_DIR,DEFAULT_PLUGIN_DIR);
 #endif
   return str;
 }
@@ -603,7 +731,7 @@ QString toHelpPath(void)
   QString str=toTool::globalConfig(CONF_HELP_PATH,"");
   if (str.isEmpty()) {
     str=toPluginPath();
-    str+="/help/toc.htm";
+    str+=QString::fromLatin1("/help/toc.htm");
   }
   return str;
 }
@@ -633,7 +761,7 @@ QString toExpandFile(const QString &file)
   else
     home=homet;
 #endif
-  ret.replace(QRegExp("\\$HOME"),home);
+  ret.replace(QRegExp(QString::fromLatin1("\\$HOME")),home);
   return ret;
 }
 
@@ -648,36 +776,36 @@ QCString toReadFile(const QString &filename)
       QFile file(tmpFile);
       if (!file.open(IO_ReadOnly)) {
 	KIO::NetAccess::removeTempFile(tmpFile);
-	throw QString("Couldn't open file %1.").arg(filename);
+	throw QT_TRANSLATE_NOOP("toReadFile","Couldn't open file %1.").arg(filename);
       }
 
       int size=file.size();
 
       char *buf=new char[size+1];
       if (file.readBlock(buf,size)==-1) {
-	delete buf;
+	delete[] buf;
 	KIO::NetAccess::removeTempFile(tmpFile);
-	throw QString("Encountered problems read configuration");
+	throw QT_TRANSLATE_NOOP("toReadFile","Encountered problems read configuration");
       }
       buf[size]=0;
       QCString ret(buf,size+1);
-      delete buf;
+      delete[] buf;
       KIO::NetAccess::removeTempFile(tmpFile);
       return ret;
     }
-    throw QString("Couldn't download file");
+    throw QT_TRANSLATE_NOOP("toReadFile","Couldn't download file");
   }
 #endif
   QFile file(expanded);
   if (!file.open(IO_ReadOnly))
-    throw QString("Couldn't open file %1.").arg(filename);
+    throw QT_TRANSLATE_NOOP("toReadFile","Couldn't open file %1.").arg(filename);
   
   int size=file.size();
   
   char *buf=new char[size+1];
   if (file.readBlock(buf,size)==-1) {
-    delete buf;
-    throw QString("Encountered problems read configuration");
+    delete[] buf;
+    throw QT_TRANSLATE_NOOP("toReadFile","Encountered problems read configuration");
   }
   buf[size]=0;
   QCString ret(buf,size+1);
@@ -694,33 +822,37 @@ bool toWriteFile(const QString &filename,const QCString &data)
     KTempFile file;
     file.file()->writeBlock(data,data.length());
     if (file.status()!=IO_Ok) {
-      TOMessageBox::warning(toMainWidget(),"File error","Couldn't write data to tempfile");
+      TOMessageBox::warning(toMainWidget(),QT_TRANSLATE_NOOP("toWriteFile","File error"),
+			    QT_TRANSLATE_NOOP("toWriteFile","Couldn't write data to tempfile"));
       file.unlink();
       return false;
     }
     file.close();
     if (!KIO::NetAccess::upload(file.name(),url)) {
       file.unlink();
-      TOMessageBox::warning(toMainWidget(),"File error","Couldn't upload data to URL");
+      TOMessageBox::warning(toMainWidget(),QT_TRANSLATE_NOOP("toWriteFile","File error"),
+			    QT_TRANSLATE_NOOP("toWriteFile","Couldn't upload data to URL"));
       return false;
     }
     file.unlink();
-    toStatusMessage("File saved successfully",false,false);
+    toStatusMessage(QT_TRANSLATE_NOOP("toWriteFile","File saved successfully"),false,false);
     return true;
   }
 #endif
 
   QFile file(expanded);
   if (!file.open(IO_WriteOnly)) {
-    TOMessageBox::warning(toMainWidget(),"File error","Couldn't open file for writing");
+    TOMessageBox::warning(toMainWidget(),QT_TRANSLATE_NOOP("toWriteFile","File error"),
+			  QT_TRANSLATE_NOOP("toWriteFile","Couldn't open file for writing"));
     return false;
   }
   file.writeBlock(data,data.length());
   if (file.status()!=IO_Ok) {
-    TOMessageBox::warning(toMainWidget(),"File error","Couldn't write data to file");
+    TOMessageBox::warning(toMainWidget(),QT_TRANSLATE_NOOP("toWriteFile","File error"),
+			  QT_TRANSLATE_NOOP("toWriteFile","Couldn't write data to file"));
     return false;
   }
-  toStatusMessage("File saved successfully",false,false);
+  toStatusMessage(QT_TRANSLATE_NOOP("toWriteFile","File saved successfully"),false,false);
   return true;
 }
 
@@ -741,25 +873,31 @@ bool toCompareLists(QStringList &lst1,QStringList &lst2,unsigned int len)
 
 static QString GetExtensions(void)
 {
-  static QRegExp repl("\\s*,\\s*");
+  static QRegExp repl(QString::fromLatin1("\\s*,\\s*"));
   QString t=toTool::globalConfig(CONF_EXTENSIONS,DEFAULT_EXTENSIONS);
-  t.replace(repl,"\n");
+  t.replace(repl,QString::fromLatin1("\n"));
   return t;
 }
 
 static QString AddExt(QString t,const QString &filter)
 {
-  static QRegExp hasext("\\.[^\\/]*$");
+  static QRegExp hasext(QString::fromLatin1("\\.[^\\/]*$"));
   if (t.isEmpty())
     return t;
+
+  toTool::globalSetConfig(CONF_LAST_DIR,t);
+
   if (hasext.match(t)<0) {
-    static QRegExp findext("\\.[^ \t\r\n\\)\\|]*");
+    static QRegExp findext(QString::fromLatin1("\\.[^ \t\r\n\\)\\|]*"));
     int len=0;
     int pos=findext.match(filter,0,&len);
     if (pos>=0)
       t+=filter.mid(pos,len);
-    else
-      t+=".sql";
+    else {
+      QFile file(t);
+      if (!file.exists())
+	t+=QString::fromLatin1(".sql");
+    }
   }
   return t;
 }
@@ -769,15 +907,20 @@ QString toOpenFilename(const QString &filename,const QString &filter,QWidget *pa
   QString t=filter;
   if (t.isEmpty())
     t=GetExtensions();
+  
+  QString dir=filename;
+  if (dir.isNull())
+    dir=toTool::globalConfig(CONF_LAST_DIR,"");
+
 #ifdef TO_KDE
-  KURL url=TOFileDialog::getOpenURL(filename,t,parent);
+  KURL url=TOFileDialog::getOpenURL(dir,t,parent);
   if (url.isEmpty())
     return QString::null;
   if (url.isLocalFile())
     return AddExt(url.path(),t);
   return AddExt(url.url(),t);
 #else
-  return AddExt(TOFileDialog::getOpenFileName(filename,t,parent),t);
+  return AddExt(TOFileDialog::getOpenFileName(dir,t,parent),t);
 #endif
 }
 
@@ -786,17 +929,22 @@ QString toSaveFilename(const QString &filename,const QString &filter,QWidget *pa
   QString t=filter;
   if (t.isEmpty())
     t=GetExtensions();
+
+  QString dir=filename;
+  if (dir.isNull())
+    dir=toTool::globalConfig(CONF_LAST_DIR,"");
+
 #ifdef TO_KDE
-  KURL url=TOFileDialog::getSaveURL(filename,t,parent);
+  KURL url=TOFileDialog::getSaveURL(dir,t,parent);
   if (url.hasPass())
-    TOMessageBox::warning(toMainWidget(),"File open password",url.pass());
+    TOMessageBox::warning(toMainWidget(),qApp->translate("toSaveFilename","File open password"),url.pass());
   if (url.isEmpty())
     return QString::null;
   if (url.isLocalFile())
     return AddExt(url.path(),t);
   return AddExt(url.url(),t);
 #else
-  return AddExt(TOFileDialog::getSaveFileName(filename,t,parent),t);
+  return AddExt(TOFileDialog::getSaveFileName(dir,t,parent),t);
 #endif
 }
 
@@ -872,7 +1020,7 @@ QBrush toChartBrush(int index)
   QBrush::BrushStyle rets;
 
   switch(style) {
-  case 4:
+  default:
     rets=QBrush::SolidPattern;
     break;
   case 3:
@@ -905,7 +1053,7 @@ toToolWidget *toCurrentTool(QObject *cur)
       return tool;
     cur=cur->parent();
   }
-  throw QString("Couldn't find parent tool. Internal error.");
+  throw qApp->translate("toCurrentTool","Couldn't find parent tool. Internal error.");
 }
 
 toConnection &toCurrentConnection(QObject *cur)
@@ -940,9 +1088,8 @@ toBusy::~toBusy()
 {
   BusyLock.lock();
   if (toThread::mainThread()) {
-    Count--;
-    if (Count<0)
-      Count=0;
+    if (Count>0)
+      Count--;
     if (!Count)
       qApp->restoreOverrideCursor();
   }
@@ -973,6 +1120,21 @@ bool toIsOracle(const toConnection &conn)
   return conn.provider()=="Oracle";
 }
 
+bool toIsSapDB(const toConnection &conn)
+{
+  return conn.provider()=="SapDB";
+}
+
+bool toIsMySQL(const toConnection &conn)
+{
+  return conn.provider()=="MySQL";
+}
+
+bool toIsPostgreSQL(const toConnection &conn)
+{
+  return conn.provider()=="PostgreSQL";
+}
+
 static QListViewItem *FindItem(QListView *lst,QListViewItem *first,const QString &str)
 {
   while(first) {
@@ -980,7 +1142,7 @@ static QListViewItem *FindItem(QListView *lst,QListViewItem *first,const QString
     if (tmp==str)
       return first;
     else {
-      tmp+=":";
+      tmp+=QString::fromLatin1(":");
       if (str.startsWith(tmp)) {
 	QListViewItem *ret=FindItem(lst,first->firstChild(),str.mid(tmp.length()));
 	if (ret)
@@ -1001,8 +1163,11 @@ void toToolCaption(toToolWidget *widget,const QString &caption)
 {
   QString title;
   if (!toTool::globalConfig(CONF_DB_TITLE,"Yes").isEmpty()) {
-    title=widget->connection().description();
-    title+=" ";
+    try {
+      title=widget->connection().description();
+      title+=QString::fromLatin1(" ");
+    } catch(...) {
+    }
   }
   title+=caption;
 
@@ -1010,13 +1175,13 @@ void toToolCaption(toToolWidget *widget,const QString &caption)
   toMainWidget()->windowsMenu();
 }
 
-void toMapExport(std::map<QString,QString> &data,const QString &prefix,
-		 std::map<QString,QString> &src)
+void toMapExport(std::map<QCString,QString> &data,const QCString &prefix,
+		 std::map<QCString,QString> &src)
 {
-  std::map<QString,QString>::iterator i=src.begin();
+  std::map<QCString,QString>::iterator i=src.begin();
   if (i!=src.end()) {
-    data[prefix+":First"]=(*i).first;
-    QString key=prefix+":d:";
+    data[prefix+":First"]=QString::fromLatin1((*i).first);
+    QCString key=prefix+":d:";
     do {
       data[key+(*i).first]=(*i).second;
       i++;
@@ -1024,22 +1189,26 @@ void toMapExport(std::map<QString,QString> &data,const QString &prefix,
   }
 }
 
-void toMapImport(std::map<QString,QString> &data,const QString &prefix,
-		 std::map<QString,QString> &dst)
+void toMapImport(std::map<QCString,QString> &data,const QCString &prefix,
+		 std::map<QCString,QString> &dst)
 {
   dst.clear();
-  std::map<QString,QString>::iterator i=data.find(prefix+":First");
+  std::map<QCString,QString>::iterator i=data.find(prefix+":First");
   if (i!=data.end()) {
-    QString key=prefix+":d:";
-    i=data.find(key+(*i).second);
-    while(i!=data.end()&&(*i).first.startsWith(key)) {
-      QString t=(*i).first.mid(key.length());
+    QCString key=prefix+":d:";
+    i=data.find(key+(*i).second.latin1());
+    while(i!=data.end()&&(*i).first.mid(0,key.length())==key) {
+      QCString t=(*i).first.mid(key.length());
       if (t.isNull())
 	t="";
       dst[t]=(*i).second;
       i++;
     }
   }
+#if 0
+  for(std::map<QCString,QString>::iterator j=dst.begin();j!=dst.end();j++)
+    printf("%s=%s\n",(const char *)((*j).first),(const char *)((*j).second));
+#endif
 }
 
 bool toCheckModal(QWidget *widget)
@@ -1061,28 +1230,43 @@ void toUpdateIndicateEmpty(void)
   IndicateEmpty=!toTool::globalConfig(CONF_INDICATE_EMPTY,"").isEmpty();
 }
 
-QString toNull(const QString &str)
+toQValue toNull(const toQValue &str)
 {
   if (IndicateEmpty) {
     if (str.isNull())
       return str;
-    if (str.isEmpty())
-      return "''";
+    if (str==QString(""))
+      return QString::fromLatin1("''");
   } else if (str.isNull())
-    return "{null}";
+    return QString::fromLatin1("{null}");
   return str;
 }
 
-QString toUnnull(const QString &str)
+toQValue toUnnull(const toQValue &str)
 {
   if (IndicateEmpty) {
-    if (str=="''")
-      return "";
-  } else if (str=="{null}")
+    if (QString(str)==QString::fromLatin1("''"))
+      return QString::fromLatin1("");
+  } else if (QString(str)==QString::fromLatin1("{null}"))
     return QString::null;
   return str;
 }
 
+QString toTranslateMayby(const QString &ctx,const QString &text)
+{
+  if (ctx.contains(QString::fromLatin1(" "))||ctx.latin1()!=ctx.utf8()||text.latin1()!=text.utf8()||ctx.isEmpty()||text.isEmpty())
+    return text;
+#ifdef TODEBUG_TRANSLATION
+  static std::map<QString,QString> Context;
+  QString t=ctx+QString::fromLatin1(" ")+text;
+  if (Context[t].isEmpty()) {
+    Context[t]=text;
+    printf("QT_TRANSLATE_NOOP(\"%s\",\"%s\"),\n",(const char *)ctx.latin1(),(const char *)text.latin1());
+  }
+#endif
+  return QT_TRANSLATE_NOOP(ctx.latin1(),text.latin1());
+}
+
 toPopupButton::toPopupButton(const QIconSet &iconSet,const QString &textLabel,
 			     const QString &grouptext,QToolBar *parent,const char *name)
   : QToolButton(iconSet,textLabel,grouptext,NULL,NULL,parent,name)
@@ -1109,11 +1293,55 @@ void toPopupButton::click(void)
 #endif
 }
 
+QString toObfuscate(const QString &str)
+{
+  if (str.isEmpty())
+    return str;
+
+#if QT_VERSION >= 0x030100
+  QByteArray arr=qCompress(str.utf8());
+  QString ret="\002";
+#else
+  QByteArray arr=str.utf8();
+  QString ret="\001";
+#endif
+  char buf[100]; // Just to be on the safe side
+  for(unsigned int i=0;i<arr.size();i++) {
+    sprintf(buf,"%02x",((unsigned int)arr.at(i))%0xff);
+    ret+=buf;
+  }
+  return ret;
+}
+
+QString toUnobfuscate(const QString &str)
+{
+  if (str.isEmpty())
+    return str;
+
+  if (str.at(0)!='\001'&&str.at(0)!='\002')
+    return str;
+
+  QByteArray arr(int(str.length()/2));
+  for(unsigned int i=1;i<str.length();i+=2)
+    arr.at(i/2)=str.mid(i,2).toInt(0,16);
+  if (str.at(0)=='\002') {
+#if QT_VERSION >= 0x030100
+    QByteArray ret=qUncompress(arr);
+    return QString::fromUtf8(ret);
+#else
+    toStatusMessage("Tried to unobfuscate obfuscated text from a Qt of version 3.1\n"
+		    "from one with Qt version 3.0.x which won't work");
+    return QString::null;
+#endif
+  } else
+    return QString::fromUtf8(arr);
+}
+
 #ifndef TO_LICENSE
 
 QString toCheckLicense(bool)
 {
-  return "Welcome to TOra";
+  return qApp->translate("toCheckLicense","Welcome to " TOAPPNAME);
 }
 
 bool toFreeware(void)
@@ -1126,26 +1354,3 @@ bool toFreeware(void)
 #include "license/tolicense.cpp"
 #include "license/tolicenseui.cpp"
 #endif
-
-void toCheckCommercial(const QString &reason)
-{
-  if (!toFreeware()) {
-    time_t t=toTool::globalConfig("FirstInstall","").toInt();
-    if (time(NULL)-t>31*24*3600) {
-      if (toCheckLicense(false).contains("Unregistered")) {
-	TOMessageBox::critical(toMainWidget(),
-			       "Need registration",
-			       "You are probably not using this software in a non\n"
-			       "commercial setting since:\n\n"+reason+
-			       "\n\n"
-			       "If you do please contact tora@henrik.org and explain what you\n"
-			       "are using it for and I'll send you a non commercial license so\n"
-			       "you can continue using TOra. Otherwise you need to buy a license\n"
-			       "at http://www.globecom.se/tora/register",
-			       "Ok");
-	exit(2);
-      }
-    }
-  }
-}
-
